@@ -21,6 +21,7 @@
 #define ExpandingDelegate_H
 
 #include <QtGui/QItemDelegate>
+#include <QItemDelegate>
 #include <QtGui/QTextLine>
 #include <QModelIndex>
 #include <QPoint>
@@ -31,6 +32,7 @@ class KateDocument;
 class KateTextLine;
 class ExpandingWidgetModel;
 class QVariant;
+class QStyleOptionViewItem;
 
 /**
  * This is a delegate that cares, together with ExpandingWidgetModel, about embedded widgets in tree-view.
@@ -52,12 +54,12 @@ class ExpandingDelegate : public QItemDelegate
     
     ExpandingWidgetModel* model() const;
   protected:
-    //column may be -1 if unknown
-    void changeBackground( int row, int column, QStyleOptionViewItem & option ) const;
+    //Called right before paint to allow last-minute changes to the style
+    virtual void adjustStyle( const QModelIndex& index, QStyleOptionViewItem & option ) const;
     virtual void drawDisplay ( QPainter * painter, const QStyleOptionViewItem & option, const QRect & rect, const QString & text ) const;
     virtual QSize sizeHint ( const QStyleOptionViewItem & option, const QModelIndex & index ) const;
     virtual bool editorEvent ( QEvent * event, QAbstractItemModel * model, const QStyleOptionViewItem & option, const QModelIndex & index );
-
+    virtual void drawBackground ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
     //option can be changed
     virtual QList<QTextLayout::FormatRange> createHighlighting(const QModelIndex& index, QStyleOptionViewItem& option) const;
 
@@ -69,6 +71,9 @@ class ExpandingDelegate : public QItemDelegate
     //Called when an item was expanded/unexpanded and the height changed
     virtual void heightChanged() const;
   
+    //Initializes the style options from the index
+    void initStyleOption(QStyleOptionViewItem *option, const QModelIndex &index) const;
+    
     mutable int m_currentColumnStart; //Text-offset for custom highlighting, will be applied to m_cachedHighlights(Only highlights starting after this will be used). Shoult be zero of the highlighting is not taken from kate.
     mutable QList<int> m_currentColumnStarts;
     mutable QList<QTextLayout::FormatRange> m_cachedHighlights;
