@@ -36,14 +36,14 @@
 KateCompletionTree::KateCompletionTree(KateCompletionWidget* parent)
   : ExpandingTree(parent), m_needResize(false)
 {
-  setUniformRowHeights(false);
   header()->hide();
   setRootIsDecorated(false);
   setIndentation(0);
   setFrameStyle(QFrame::NoFrame);
   setAllColumnsShowFocus(true);
   setAlternatingRowColors(true);
-  setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
+  //We need ScrollPerItem, because ScrollPerPixel is too slow with a very large competion-list(see KDevelop).
+  setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
 
   m_resizeTimer = new QTimer(this);
   m_resizeTimer->setSingleShot(true);
@@ -62,7 +62,7 @@ KateCompletionTree::KateCompletionTree(KateCompletionWidget* parent)
 
 void KateCompletionTree::currentChanged ( const QModelIndex & current, const QModelIndex & previous ) {
   widget()->model()->rowSelected(current);
-  QTreeView::currentChanged(current, previous);
+  ExpandingTree::currentChanged(current, previous);
 }
 
 void KateCompletionTree::scrollContentsBy( int dx, int dy )
@@ -111,7 +111,6 @@ void KateCompletionTree::resizeColumns(bool fromResizeEvent, bool firstShow)
   int numColumns = model()->columnCount();
   
   QVector<int> columnSize(numColumns, 5);
-  columnSize[0] = 50;
 
   QModelIndex current = indexAt(QPoint(1,1));
 
@@ -130,7 +129,7 @@ void KateCompletionTree::resizeColumns(bool fromResizeEvent, bool firstShow)
       if( s.width() > columnSize[a] && s.width() < 2000 )
         columnSize[a] = s.width();
       else if( s.width() > 2000 )
-        kDebug() << "got invalid size-hint of width " << s.width();
+        kDebug( 13035 ) << "got invalid size-hint of width " << s.width();
     }
 
     QModelIndex oldCurrent = current;
@@ -145,6 +144,8 @@ void KateCompletionTree::resizeColumns(bool fromResizeEvent, bool firstShow)
   }
 
   int totalColumnsWidth = 0;
+  
+  int maxWidth = (QApplication::desktop()->screenGeometry(widget()->view()).width()*3) / 4;
 
   ///Step 2: Update column-sizes
   if( changed ) {
@@ -161,6 +162,28 @@ void KateCompletionTree::resizeColumns(bool fromResizeEvent, bool firstShow)
         maximumResize = diff;
     }
 
+    int noReduceTotalWidth = 0; //The total width of the widget of no columns are reduced
+    for( int n = 0; n < numColumns; n++ ) {
+      if(columnSize[n] < columnWidth(n))
+        noReduceTotalWidth += columnWidth(n);
+      else
+        noReduceTotalWidth += columnSize[n];
+    }
+
+    //Check whether we can afford to reduce none of the columns
+    //Only reduce size if we widget would else be too wide.
+  bool noReduce = noReduceTotalWidth < maxWidth;
+
+    if(noReduce) {
+      totalColumnsWidth = 0;
+      for( int n = 0; n < numColumns; n++ ) {
+      	if(columnSize[n] < columnWidth(n))
+          columnSize[n] = columnWidth(n);
+
+        totalColumnsWidth += columnSize[n];
+      }
+    }
+
     if( minimumResize > -40 && maximumResize == 0 ) {
       //No column needs to be exanded, and no column needs to be reduced by more then 40 pixels.
       //To prevent flashing, do not resize at all.
@@ -181,24 +204,23 @@ void KateCompletionTree::resizeColumns(bool fromResizeEvent, bool firstShow)
   int newIndentWidth = columnViewportPosition(modelIndexOfName);
 
   int scrollBarWidth = verticalScrollBar()->width();
-  int newMinWidth = totalColumnsWidth /*+ scrollBarWidth*/;
+  int newMinWidth = totalColumnsWidth;
 
-  int minWidth = qMax(500, newMinWidth);
+  int targetWidth = qMax(100, newMinWidth);
 
-  //kDebug() << "New min width: " << minWidth << " Old min: " << minimumWidth() << " width " << width();
-  setMinimumWidth(minWidth);
+  //kDebug( 13035 ) << "New min width: " << minWidth << " Old min: " << minimumWidth() << " width " << width();
+  setMinimumWidth(targetWidth);
 
   if (!fromResizeEvent && (firstShow || oldIndentWidth != newIndentWidth))
   {
     //Never allow a completion-widget to be wider than 2/3 of the screen
-    int maxWidth = (QApplication::desktop()->screenGeometry(widget()->view()).width()*3) / 4;
-    int newWidth = qMin(maxWidth, minWidth); 
-    //kDebug() << "fromResize " << fromResizeEvent << " indexOfName " << modelIndexOfName << " oldI " << oldIndentWidth << " newI " << newIndentWidth << " minw " << minWidth << " w " << widget()->width() << " newW " << newWidth;
-    widget()->resize(newWidth + scrollBarWidth, widget()->height());
+    int newWidth = qMin(maxWidth, targetWidth); 
+    //kDebug( 13035 ) << "fromResize " << fromResizeEvent << " indexOfName " << modelIndexOfName << " oldI " << oldIndentWidth << " newI " << newIndentWidth << " minw " << minWidth << " w " << widget()->width() << " newW " << newWidth;
+    widget()->resize(newWidth + (verticalScrollBar()->isVisible() ? scrollBarWidth : 0), widget()->height());
   }
 
-  if( totalColumnsWidth ) //Set the size of the last column to fill the whole rest of the widget
-    setColumnWidth(numColumns-1, width() - totalColumnsWidth + columnSize[numColumns-1]);
+  //if( totalColumnsWidth ) //Set the size of the last column to fill the whole rest of the widget
+  setColumnWidth(numColumns-1, viewport()->width() - columnViewportPosition(numColumns-1));
   
   if (oldIndentWidth != newIndentWidth)
     widget()->updatePosition();
