@@ -52,80 +52,45 @@
 //#include "modeltest.h"
 
 KateCompletionWidget::KateCompletionWidget(KateView* parent)
-  : QFrame(parent, Qt::Tool | Qt::FramelessWindowHint)
+  : QFrame(parent, Qt::ToolTip)
   , m_presentationModel(new KateCompletionModel(this))
   , m_completionRange(0L)
   , m_entryList(new KateCompletionTree(this))
   , m_argumentHintModel(new KateArgumentHintModel(this))
   , m_argumentHintTree(new KateArgumentHintTree(this))
-  , m_automaticInvocation(false)
   , m_automaticInvocationDelay(300)
   , m_filterInstalled(false)
   , m_configWidget(new KateCompletionConfig(m_presentationModel, view()))
   , m_inCompletionList(false)
   , m_isSuspended(false)
   , m_dontShowArgumentHints(false)
+  , m_needShow(false)
   , m_expandedAddedHeightBase(0)
   , m_expandingAddedHeight(0)
 {
-  //new ModelTest(m_presentationModel, this);
 
   setFrameStyle( QFrame::Box | QFrame::Plain );
   setLineWidth( 1 );
   //setWindowOpacity(0.8);
 
   m_entryList->setModel(m_presentationModel);
+  m_entryList->setColumnWidth(0, 0); //These will be determined automatically in KateCompletionTree::resizeColumns
+  m_entryList->setColumnWidth(1, 0);
+  m_entryList->setColumnWidth(2, 0);
+
+  m_argumentHintTree->setParent(0, Qt::ToolTip);
   m_argumentHintTree->setModel(m_argumentHintModel);
 
-  m_statusBar = new QWidget(this);
-  m_statusBar->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
-
-  m_sortButton = new QToolButton(m_statusBar);
-  m_sortButton->setIcon(KIcon("sort"));
-  m_sortButton->setCheckable(true);
-  connect(m_sortButton, SIGNAL(toggled(bool)), m_presentationModel, SLOT(setSortingEnabled(bool)));
-
-  m_sortText = new QLabel(i18n("Sort: None"), m_statusBar);
-
-  m_filterButton = new QToolButton(m_statusBar);
-  m_filterButton->setIcon(KIcon("view-filter"));
-  m_filterButton->setCheckable(true);
-  connect(m_filterButton, SIGNAL(toggled(bool)), m_presentationModel, SLOT(setFilteringEnabled(bool)));
-
-  m_filterText = new QLabel(i18n("Filter: None"), m_statusBar);
-
-  m_configButton = new QPushButton(KIcon("configure"), i18n("Setup"), m_statusBar);
-  connect(m_configButton, SIGNAL(pressed()), SLOT(showConfig()));
+  connect(m_entryList->verticalScrollBar(), SIGNAL(valueChanged(int)), m_presentationModel, SLOT(placeExpandingWidgets()));
+  connect(m_argumentHintTree->verticalScrollBar(), SIGNAL(valueChanged(int)), m_argumentHintModel, SLOT(placeExpandingWidgets()));
+  connect(view(), SIGNAL(focusOut(KTextEditor::View*)), this, SLOT(viewFocusOut()));
 
   m_automaticInvocationTimer = new QTimer(this);
   m_automaticInvocationTimer->setSingleShot(true);
   connect(m_automaticInvocationTimer, SIGNAL(timeout()), this, SLOT(automaticInvocation()));
 
-  m_updateFocusTimer = new QTimer(this);
-  m_updateFocusTimer->setSingleShot(true);
-  connect(m_updateFocusTimer, SIGNAL(timeout()), this, SLOT(updateFocus()));
-
-  QSizeGrip* sg = new QSizeGrip(m_statusBar);
-
-  QHBoxLayout* statusLayout = new QHBoxLayout(m_statusBar);
-  statusLayout->addWidget(m_sortButton);
-  statusLayout->addWidget(m_sortText);
-  statusLayout->addSpacing(8);
-  statusLayout->addWidget(m_filterButton);
-  statusLayout->addWidget(m_filterText);
-  statusLayout->addSpacing(8);
-  statusLayout->addStretch();
-  QVBoxLayout* gripLayout = new QVBoxLayout();
-  gripLayout->addStretch();
-  statusLayout->addWidget(m_configButton);
-  gripLayout->addWidget(sg);
-  statusLayout->addLayout(gripLayout);
-  statusLayout->setMargin(0);
-  statusLayout->setSpacing(2);
-
   QVBoxLayout* vl = new QVBoxLayout(this);
   vl->addWidget(m_entryList);
-  vl->addWidget(m_statusBar);
   vl->setMargin(0);
 
   // Keep branches expanded
@@ -135,14 +100,13 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
 
   connect(view(), SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), SLOT(cursorPositionChanged()));
   connect(view()->doc()->history(), SIGNAL(editDone(KateEditInfo*)), SLOT(editDone(KateEditInfo*)), Qt::QueuedConnection);
-  connect(view(), SIGNAL(focusOut(KTextEditor::View*)), this, SLOT(viewFocusOut()));
-  connect(view(), SIGNAL(focusIn(KTextEditor::View*)), this, SLOT(viewFocusIn()));
   connect(view(), SIGNAL(verticalScrollPositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)), this, SLOT(updatePositionSlot()));
 
   // This is a non-focus widget, it is passed keyboard input from the view
 
   //We need to do this, because else the focus goes to nirvana without any control when the completion-widget is clicked.
   setFocusPolicy(Qt::ClickFocus);
+  m_argumentHintTree->setFocusPolicy(Qt::ClickFocus);
 
   foreach (QWidget* childWidget, findChildren<QWidget*>())
     childWidget->setFocusPolicy(Qt::NoFocus);
@@ -151,50 +115,31 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
 KateCompletionWidget::~KateCompletionWidget() {
 }
 
+void KateCompletionWidget::viewFocusOut() {
+  abortCompletion();
+}
+
 void KateCompletionWidget::modelContentChanged() {
   int realItemCount = 0;
   foreach (KTextEditor::CodeCompletionModel* model, m_presentationModel->completionModels())
     realItemCount += model->rowCount();
-  //kDebug() << "content changed, item count " << realItemCount << "\n" << kBacktrace();
-  if( !m_isSuspended && !isVisible() && realItemCount != 0 ) {
-    //kDebug() << "showing";
+  if( !m_isSuspended && (!isVisible() || m_needShow) && realItemCount != 0 ) {
+    m_needShow = false;
     updateAndShow();
   }
-}
 
-void KateCompletionWidget::focusInEvent ( QFocusEvent * event ) {
-  //Instantly give the focus back
-  view()->activateWindow(); //When the completion-widget is clicked, the application-window is deactivated, so re-activate it
-  view()->setFocus(Qt::OtherFocusReason);
-  event->accept();
-  QFrame::focusInEvent(event);
-}
-
-void KateCompletionWidget::updateFocus() {
-  if( isCompletionActive() ) {
-    bool childHasFocus = QApplication::focusWidget() ? isAncestorOf(QApplication::focusWidget()) : false;
-    if(!childHasFocus && !view()->hasFocus()) {
-      if(isVisible()) {
-        hide();
-        m_isSuspended = true;
-      }
-    }else if(childHasFocus) {
-      //Whenever the completion-widget gets the focus by clicking, move it back to the text-view.
-      view()->activateWindow(); //When the completion-widget is clicked, the application-window is deactivated, so re-activate it
-      view()->setFocus(Qt::OtherFocusReason);
-    }
+  if(m_presentationModel->rowCount(QModelIndex()) == 0 && m_argumentHintModel->rowCount(QModelIndex()) == 0) {
+    kDebug( 13035 ) << "hiding because no content";
+    hide();
+    return;
   }
-}
 
-void KateCompletionWidget::viewFocusOut() {
-    //Check whether the view is still visible in a short time. If it isn't, hide the completion-widget.
-    m_updateFocusTimer->start(100);
-}
-
-void KateCompletionWidget::viewFocusIn() {
-  if( m_isSuspended ) {
-    show();
-    m_isSuspended = false;
+  //With each filtering items can be added or removed, so we have to reset the current index here so we always have a selected item
+  m_entryList->setCurrentIndex(model()->index(0,0));
+  if(!model()->indexIsItem(m_entryList->currentIndex())) {
+    QModelIndex firstIndex = model()->index(0,0, m_entryList->currentIndex());
+    m_entryList->setCurrentIndex(firstIndex);
+    //m_entryList->scrollTo(firstIndex, QAbstractItemView::PositionAtTop);
   }
 }
 
@@ -240,7 +185,8 @@ void KateCompletionWidget::startCompletion( const KTextEditor::Range & word, KTe
 {
   m_isSuspended = false;
   m_inCompletionList = true; //Always start at the top of the completion-list
-  
+  m_needShow = true;
+
   disconnect(this->model(), SIGNAL(contentGeometryChanged()), this, SLOT(modelContentChanged()));
 
   m_dontShowArgumentHints = true;
@@ -277,15 +223,19 @@ void KateCompletionWidget::startCompletion( const KTextEditor::Range & word, KTe
 
   cursorPositionChanged();
 
+  if (model)
+    model->completionInvoked(view(), word, invocationType);
+  else
+    foreach (KTextEditor::CodeCompletionModel* model, m_sourceModels)
+      model->completionInvoked(view(), word, invocationType);
+
+  kDebug( 13035 ) << "msdofjdsoifdsflkdsjf";
   if (model)
     m_presentationModel->setCompletionModel(model);
   else
     m_presentationModel->setCompletionModels(m_sourceModels);
-  
-  if (!m_presentationModel->completionModels().isEmpty()) {
-    foreach (KTextEditor::CodeCompletionModel* model, m_presentationModel->completionModels())
-      model->completionInvoked(view(), word, invocationType);
 
+  if (!m_presentationModel->completionModels().isEmpty()) {
     m_presentationModel->setCurrentCompletion(view()->doc()->text(KTextEditor::Range(m_completionRange->start(), view()->cursorPosition())));
   }
   
@@ -363,8 +313,6 @@ void KateCompletionWidget::updateArgumentHintGeometry()
 
 void KateCompletionWidget::updateHeight()
 {
-  kDebug() << "updateHeight(), height: " << geometry().height() << " current added-height: " << m_expandingAddedHeight;
-
   QRect geom = geometry();
 
   int baseHeight = geom.height() - m_expandingAddedHeight;
@@ -380,32 +328,32 @@ void KateCompletionWidget::updateHeight()
   if( baseHeight < 300 ) {
     baseHeight = 300; //Here we enforce a minimum desirable height
     m_expandingAddedHeight = 0;
-//     kDebug() << "Resetting baseHeight and m_expandingAddedHeight";
+//     kDebug( 13035 ) << "Resetting baseHeight and m_expandingAddedHeight";
   }
 
   int newExpandingAddedHeight = 0;
 
-//   kDebug() << "baseHeight: " << baseHeight;
+//   kDebug( 13035 ) << "baseHeight: " << baseHeight;
 
   newExpandingAddedHeight = model()->expandingWidgetsHeight();
 
-//   kDebug() << "new newExpandingAddedHeight: " << newExpandingAddedHeight;
+//   kDebug( 13035 ) << "new newExpandingAddedHeight: " << newExpandingAddedHeight;
 
   int screenBottom = QApplication::desktop()->screenGeometry(view()).bottom();
 
   int bottomPosition = baseHeight + newExpandingAddedHeight + geometry().top();
 //  int targetHeight = baseHeight + newExpandingAddedHeight;
-//   kDebug() << "targetHeight: " << targetHeight;
+//   kDebug( 13035 ) << "targetHeight: " << targetHeight;
 
-//   kDebug() << "screen-bottom: " << screenBottom << " bottomPosition: " << bottomPosition;
+//   kDebug( 13035 ) << "screen-bottom: " << screenBottom << " bottomPosition: " << bottomPosition;
 
   if( bottomPosition > screenBottom-50 ) {
     newExpandingAddedHeight -= bottomPosition - (screenBottom-50);
-//     kDebug() << "Too high, moved bottomPosition to: " << baseHeight + newExpandingAddedHeight + geometry().top() << " changed newExpandingAddedHeight to " << newExpandingAddedHeight;
+//     kDebug( 13035 ) << "Too high, moved bottomPosition to: " << baseHeight + newExpandingAddedHeight + geometry().top() << " changed newExpandingAddedHeight to " << newExpandingAddedHeight;
   }
 
   int finalHeight = baseHeight+newExpandingAddedHeight;
-//   kDebug() << "finalHeight: " << finalHeight;
+//   kDebug( 13035 ) << "finalHeight: " << finalHeight;
   if( finalHeight < 50 ) {
     return;
   }
@@ -422,7 +370,7 @@ void KateCompletionWidget::updateHeight()
 void KateCompletionWidget::cursorPositionChanged( )
 {
   if (!isCompletionActive())  {
-    m_presentationModel->setCurrentCompletion(QString());
+//    m_presentationModel->setCurrentCompletion(QString());
     return;
   }
 
@@ -443,11 +391,7 @@ void KateCompletionWidget::cursorPositionChanged( )
 
 bool KateCompletionWidget::isCompletionActive( ) const
 {
-  bool ret = !isHidden();
-  if (ret)
-    Q_ASSERT(m_completionRange && m_completionRange->isValid());
-
-  return ret;
+  return m_completionRange && isVisible();
 }
 
 void KateCompletionWidget::abortCompletion( )
@@ -456,17 +400,18 @@ void KateCompletionWidget::abortCompletion( )
 
   m_isSuspended = false;
 
-  if (!isCompletionActive())
-    return;
-
-  hide();
+  bool wasActive = isCompletionActive();
 
   clear();
 
+  if(isVisible())
+    hide();
+
   delete m_completionRange;
   m_completionRange = 0L;
 
-  view()->sendCompletionAborted();
+  if (wasActive)
+    view()->sendCompletionAborted();
 }
 
 void KateCompletionWidget::clear() {
@@ -511,13 +456,21 @@ void KateCompletionWidget::execute(bool shift)
   KTextEditor::CodeCompletionModel* model = static_cast<KTextEditor::CodeCompletionModel*>(const_cast<QAbstractItemModel*>(toExecute.model()));
   Q_ASSERT(model);
 
-  model->executeCompletionItem(view()->document(), *m_completionRange, toExecute.row());
+  KTextEditor::CodeCompletionModel2* model2 = dynamic_cast<KTextEditor::CodeCompletionModel2*>(model);
+
+  if(model2)
+    model2->executeCompletionItem2(view()->document(), *m_completionRange, toExecute);
+  else if(toExecute.parent().isValid())
+    //The normale CodeCompletionInterface cannot handle feedback for hierarchical models, so just do the replacement
+    view()->document()->replaceText(*m_completionRange, model->data(toExecute.sibling(toExecute.row(), KTextEditor::CodeCompletionModel::Name)).toString());
+  else
+    model->executeCompletionItem(view()->document(), *m_completionRange, toExecute.row());
 
   view()->doc()->editEnd();
 
   hide();
 
-  view()->sendCompletionExecuted(start, model, toExecute.row());
+  view()->sendCompletionExecuted(start, model, toExecute);
 }
 
 void KateCompletionWidget::resizeEvent( QResizeEvent * event )
@@ -540,7 +493,6 @@ void KateCompletionWidget::showEvent ( QShowEvent * event )
 void KateCompletionWidget::hideEvent( QHideEvent * event )
 {
   QWidget::hideEvent(event);
-
   m_argumentHintTree->hide();
 
   if (isCompletionActive())
@@ -770,16 +722,6 @@ void KateCompletionWidget::unregisterCompletionModel(KTextEditor::CodeCompletion
   m_sourceModels.removeAll(model);
 }
 
-bool KateCompletionWidget::isAutomaticInvocationEnabled() const
-{
-  return m_automaticInvocation;
-}
-
-void KateCompletionWidget::setAutomaticInvocationEnabled(bool enabled)
-{
-  m_automaticInvocation = enabled;
-}
-
 int KateCompletionWidget::automaticInvocationDelay() const {
   return m_automaticInvocationDelay;
 }
@@ -791,7 +733,7 @@ void KateCompletionWidget::setAutomaticInvocationDelay(int delay) {
 
 void KateCompletionWidget::editDone(KateEditInfo * edit)
 {
-  if (!isAutomaticInvocationEnabled()
+  if (!view()->config()->automaticCompletionInvocation()
        || (edit->editSource() != Kate::UserInputEdit)
        || edit->isRemoval()
        || isCompletionActive()
