@@ -73,9 +73,6 @@ class KateCompletionWidget : public QFrame
     void registerCompletionModel(KTextEditor::CodeCompletionModel* model);
     void unregisterCompletionModel(KTextEditor::CodeCompletionModel* model);
     
-    bool isAutomaticInvocationEnabled() const;
-    void setAutomaticInvocationEnabled(bool enabled = true);
-
     int automaticInvocationDelay() const;
     void setAutomaticInvocationDelay(int delay);
     
@@ -108,19 +105,19 @@ class KateCompletionWidget : public QFrame
   public Q_SLOTS:
     void abortCompletion();
     void showConfig();
-    void viewFocusIn();
-    void viewFocusOut();
+/*    void viewFocusIn();
+    void viewFocusOut();*/
     void updatePositionSlot();
     void automaticInvocation();
 
-    void updateFocus();
+/*    void updateFocus();*/
     void argumentHintsChanged(bool hasContent);
     
   protected:
     virtual void showEvent ( QShowEvent * event );
     virtual void resizeEvent ( QResizeEvent * event );
     virtual void hideEvent ( QHideEvent * event );
-    virtual void focusInEvent ( QFocusEvent * event );
+//    virtual void focusInEvent ( QFocusEvent * event );
 
   private Q_SLOTS:
     void modelContentChanged();
@@ -129,7 +126,7 @@ class KateCompletionWidget : public QFrame
     void modelReset();
     void startCharacterDeleted(KTextEditor::SmartCursor* cursor, bool deletedBefore);
     void rowsInserted(const QModelIndex& parent, int row, int rowEnd);
-
+    void viewFocusOut();
   private:
     void updateAndShow();
     void updateArgumentHintGeometry();
@@ -151,7 +148,7 @@ class KateCompletionWidget : public QFrame
     KateArgumentHintTree* m_argumentHintTree;
 
     QTimer* m_automaticInvocationTimer;
-    QTimer* m_updateFocusTimer;
+    //QTimer* m_updateFocusTimer;
     QWidget* m_statusBar;
     QToolButton* m_sortButton;
     QLabel* m_sortText;
@@ -160,7 +157,6 @@ class KateCompletionWidget : public QFrame
     QPushButton* m_configButton;
 
     QString m_automaticInvocationLine;
-    bool m_automaticInvocation;
     int m_automaticInvocationDelay;
     bool m_filterInstalled;
 
@@ -168,6 +164,7 @@ class KateCompletionWidget : public QFrame
     bool m_inCompletionList; //Are we in the completion-list? If not, we're in the argument-hint list
     bool m_isSuspended;
     bool m_dontShowArgumentHints; //Used temporarily to prevent flashing
+    bool m_needShow;
 
     int m_expandedAddedHeightBase;
     int m_expandingAddedHeight;
