@@ -80,6 +80,8 @@ namespace Ui
   class EditConfigWidget;
   class IndentationConfigWidget;
   class OpenSaveConfigWidget;
+  class OpenSaveConfigAdvWidget;
+  class CompletionConfigTab;
 }
 
 class KateConfigPage : public KTextEditor::ConfigPage
@@ -145,6 +147,27 @@ class KateIndentConfigTab : public KateConfigPage
     void showWhatsThis(const QString& text);
 };
 
+class KateCompletionConfigTab : public KateConfigPage
+{
+  Q_OBJECT
+
+  public:
+    KateCompletionConfigTab(QWidget *parent);
+    ~KateCompletionConfigTab();
+
+  protected:
+    Ui::CompletionConfigTab *ui;
+
+  public Q_SLOTS:
+    void apply ();
+    void reload ();
+    void reset () {}
+    void defaults () {}
+
+  private Q_SLOTS:
+    void showWhatsThis(const QString& text);
+};
+
 class KateSelectConfigTab : public KateConfigPage
 {
   Q_OBJECT
@@ -183,6 +206,7 @@ public Q_SLOTS:
 private:
   KateSelectConfigTab *selectConfigTab;
   KateIndentConfigTab *indentConfigTab;
+  KateCompletionConfigTab *completionConfigTab;
 };
 
 class KateViewDefaultsConfig : public KateConfigPage
@@ -257,6 +281,7 @@ class KateSaveConfigTab : public KateConfigPage
 
   private:
     Ui::OpenSaveConfigWidget* ui;
+    Ui::OpenSaveConfigAdvWidget* uiadv;
     ModeConfigPage *modeConfigPage;
 };
 
