@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2005 Hamish Rodda <rodda@kde.org>
+   Copyright (C) 2008 Dominik Haumann <dhaumann kde org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,6 +20,7 @@
 #include "katelayoutcache.h"
 
 #include <QtCore/QMutex>
+#include <QtAlgorithms>
 
 #include "katerenderer.h"
 #include "kateview.h"
@@ -29,6 +31,111 @@
 
 static bool enableLayoutCache = false;
 
+//BEGIN KateLineLayoutMap
+KateLineLayoutMap::KateLineLayoutMap()
+{
+}
+
+KateLineLayoutMap::~KateLineLayoutMap()
+{
+}
+
+bool lessThan(const KateLineLayoutMap::LineLayoutPair& lhs,
+              const KateLineLayoutMap::LineLayoutPair& rhs)
+{
+  return lhs.first < rhs.first;
+}
+
+void KateLineLayoutMap::clear()
+{
+}
+
+bool KateLineLayoutMap::contains(int i) const
+{
+  LineLayoutMap::const_iterator it =
+    qBinaryFind(m_lineLayouts.constBegin(), m_lineLayouts.constEnd(), LineLayoutPair(i,KateLineLayoutPtr()), lessThan);
+  return (it != m_lineLayouts.constEnd());
+}
+
+void KateLineLayoutMap::insert(int realLine, const KateLineLayoutPtr& lineLayoutPtr)
+{
+  LineLayoutMap::iterator it =
+    qBinaryFind(m_lineLayouts.begin(), m_lineLayouts.end(), LineLayoutPair(realLine,KateLineLayoutPtr()), lessThan);
+  if (it != m_lineLayouts.end()) {
+    (*it).second = lineLayoutPtr;
+  } else {
+    it = qUpperBound(m_lineLayouts.begin(), m_lineLayouts.end(), LineLayoutPair(realLine,KateLineLayoutPtr()), lessThan);
+    m_lineLayouts.insert(it, LineLayoutPair(realLine, lineLayoutPtr));
+  }
+}
+
+void KateLineLayoutMap::viewWidthIncreased()
+{
+  LineLayoutMap::iterator it = m_lineLayouts.begin();
+  for ( ; it != m_lineLayouts.end(); ++it) {
+    if ((*it).second->isValid() && (*it).second->viewLineCount() > 1)
+      (*it).second->invalidateLayout();
+  }
+}
+
+void KateLineLayoutMap::viewWidthDecreased(int newWidth)
+{
+  LineLayoutMap::iterator it = m_lineLayouts.begin();
+  for ( ; it != m_lineLayouts.end(); ++it) {
+    if ((*it).second->isValid()
+        && ((*it).second->viewLineCount() > 1 || (*it).second->width() > newWidth))
+      (*it).second->invalidateLayout();
+  }
+}
+
+void KateLineLayoutMap::relayoutLines(int startRealLine, int endRealLine)
+{
+  LineLayoutMap::iterator start =
+      qLowerBound(m_lineLayouts.begin(), m_lineLayouts.end(), LineLayoutPair(startRealLine, KateLineLayoutPtr()), lessThan);
+  LineLayoutMap::iterator end =
+      qUpperBound(start, m_lineLayouts.end(), LineLayoutPair(endRealLine, KateLineLayoutPtr()), lessThan);
+
+  while (start != end) {
+    (*start).second->setLayoutDirty();
+    ++start;
+  }
+}
+
+void KateLineLayoutMap::slotEditDone(int fromLine, int toLine, int shiftAmount)
+{
+  LineLayoutMap::iterator start =
+      qLowerBound(m_lineLayouts.begin(), m_lineLayouts.end(), LineLayoutPair(fromLine, KateLineLayoutPtr()), lessThan);
+  LineLayoutMap::iterator end =
+      qUpperBound(start, m_lineLayouts.end(), LineLayoutPair(toLine, KateLineLayoutPtr()), lessThan);
+  LineLayoutMap::iterator it;
+
+  if (shiftAmount != 0) {
+    for (it = end; it != m_lineLayouts.end(); ++it) {
+      (*it).first += shiftAmount;
+      (*it).second->setLine((*it).second->line() + shiftAmount);
+    }
+
+    for (it = start; it != end; ++it) {
+      (*it).second->clear();
+    }
+
+    m_lineLayouts.erase(start, end);
+  } else {
+    for (it = start; it != end; ++it) {
+      (*it).second->setLayoutDirty();
+    }
+  }
+}
+
+
+KateLineLayoutPtr& KateLineLayoutMap::operator[](int i)
+{
+  LineLayoutMap::iterator it =
+    qBinaryFind(m_lineLayouts.begin(), m_lineLayouts.end(), LineLayoutPair(i, KateLineLayoutPtr()), lessThan);
+  return (*it).second;
+}
+//END KateLineLayoutMap
+
 KateLayoutCache::KateLayoutCache(KateRenderer* renderer, QObject* parent)
   : QObject(parent)
   , m_renderer(renderer)
@@ -43,7 +150,7 @@ KateLayoutCache::KateLayoutCache(KateRenderer* renderer, QObject* parent)
 
 void KateLayoutCache::updateViewCache(const KTextEditor::Cursor& startPos, int newViewLineCount, int viewLinesScrolled)
 {
-  //kDebug() << startPos << " nvlc " << newViewLineCount << " vls " << viewLinesScrolled;
+  //kDebug( 13033 ) << startPos << " nvlc " << newViewLineCount << " vls " << viewLinesScrolled;
 
   int oldViewLineCount = m_textLayouts.count();
   if (newViewLineCount == -1)
@@ -114,10 +221,12 @@ void KateLayoutCache::updateViewCache(const KTextEditor::Cursor& startPos, int n
   KateLineLayoutPtr l = line(realLine);
   for (int i = 0; i < newViewLineCount; ++i) {
     if (!l) {
-      if (i < m_textLayouts.count())
-        m_textLayouts[i] = KateTextLayout::invalid();
-      else
+      if (i < m_textLayouts.count()) {
+        if (m_textLayouts[i].isValid())
+          m_textLayouts[i] = KateTextLayout::invalid();
+      } else {
         m_textLayouts.append(KateTextLayout::invalid());
+      }
       continue;
     }
 
@@ -126,7 +235,7 @@ void KateLayoutCache::updateViewCache(const KTextEditor::Cursor& startPos, int n
 
     if (i < m_textLayouts.count()) {
       bool dirty = false;
-      if (m_textLayouts[i].line() != realLine || m_textLayouts[i].viewLine() != _viewLine || !m_textLayouts[i].isValid())
+      if (m_textLayouts[i].line() != realLine || m_textLayouts[i].viewLine() != _viewLine || (!m_textLayouts[i].isValid() && l->viewLine(_viewLine).isValid()))
         dirty = true;
       m_textLayouts[i] = l->viewLine(_viewLine);
       if (dirty)
@@ -136,7 +245,7 @@ void KateLayoutCache::updateViewCache(const KTextEditor::Cursor& startPos, int n
       m_textLayouts.append(l->viewLine(_viewLine));
     }
 
-    //kDebug() << "Laid out line " << realLine << " (" << l << "), viewLine " << _viewLine << " (" << m_textLayouts[i].kateLineLayout().data() << ")";
+    //kDebug( 13033 ) << "Laid out line " << realLine << " (" << l << "), viewLine " << _viewLine << " (" << m_textLayouts[i].kateLineLayout().data() << ")";
     //m_textLayouts[i].debugOutput();
 
     _viewLine++;
@@ -156,13 +265,18 @@ KateLineLayoutPtr KateLayoutCache::line( int realLine, int virtualLine ) const
 {
   if (m_lineLayouts.contains(realLine)) {
     KateLineLayoutPtr l = m_lineLayouts[realLine];
+
     if (virtualLine != -1)
       l->setVirtualLine(virtualLine);
+
     if (!l->isValid())
       m_renderer->layoutLine(l, wrap() ? m_viewWidth : -1, enableLayoutCache);
-    else if (l->isLayoutDirty())
+
+    else if (l->isLayoutDirty() && !acceptDirtyLayouts())
       m_renderer->layoutLine(l, wrap() ? m_viewWidth : -1, enableLayoutCache);
-    Q_ASSERT(l->isValid() && !l->isLayoutDirty());
+
+    Q_ASSERT(l->isValid() && (!l->isLayoutDirty() || acceptDirtyLayouts()));
+
     return l;
   }
 
@@ -174,6 +288,10 @@ KateLineLayoutPtr KateLayoutCache::line( int realLine, int virtualLine ) const
   m_renderer->layoutLine(l, wrap() ? m_viewWidth : -1, enableLayoutCache);
   Q_ASSERT(l->isValid());
 
+  if (acceptDirtyLayouts())
+    // Mark it dirty, because it may not have the syntax highlighting applied
+    l->setLayoutDirty(true);
+
   m_lineLayouts.insert(realLine, l);
   return l;
 }
@@ -255,6 +373,9 @@ int KateLayoutCache::displayViewLine(const KTextEditor::Cursor& virtualCursor, b
   KTextEditor::Cursor work = viewCacheStart();
   work.setLine(m_renderer->doc()->getVirtualLine(work.line()));
 
+  if (!work.isValid())
+    return virtualCursor.line();
+
   int limit = m_textLayouts.count();
 
   // Efficient non-word-wrapped path
@@ -318,13 +439,13 @@ int KateLayoutCache::viewLineCount(int realLine) const
 
 void KateLayoutCache::viewCacheDebugOutput( ) const
 {
-  kDebug() << "Printing values for " << m_textLayouts.count() << " lines:";
+  kDebug( 13033 ) << "Printing values for " << m_textLayouts.count() << " lines:";
   if (m_textLayouts.count())
   foreach (const KateTextLayout& t, m_textLayouts)
     if (t.isValid())
       t.debugOutput();
     else
-      kDebug() << "Line Invalid.";
+      kDebug( 13033 ) << "Line Invalid.";
 }
 
 void KateLayoutCache::slotEditDone(KateEditInfo* edit)
@@ -333,35 +454,7 @@ void KateLayoutCache::slotEditDone(KateEditInfo* edit)
   int toLine = edit->oldRange().end().line();
   int shiftAmount = edit->translate().line();
 
-  if (shiftAmount) {
-    QMap<int, KateLineLayoutPtr> oldMap = m_lineLayouts;
-    m_lineLayouts.clear();
-
-    QMapIterator<int, KateLineLayoutPtr> it = oldMap;
-    while (it.hasNext()) {
-      it.next();
-
-      if (it.key() > toLine) {
-        KateLineLayoutPtr layout = it.value();
-        layout->setLine(layout->line() + shiftAmount);
-        m_lineLayouts.insert(it.key() + shiftAmount, layout);
-
-      } else if (it.key() < fromLine) {
-        m_lineLayouts.insert(it.key(), it.value());
-
-      } else {
-        //gets deleted when oldMap goes but invalidate it just in case
-        const_cast<KateLineLayoutPtr&>(it.value())->invalidateLayout();
-      }
-    }
-
-  } else {
-    for (int i = fromLine; i <= toLine; ++i)
-      if (m_lineLayouts.contains(i)) {
-        const_cast<KateLineLayoutPtr&>(m_lineLayouts[i])->setLayoutDirty();
-        m_lineLayouts.remove(i);
-      }
-  }
+  m_lineLayouts.slotEditDone(fromLine, toLine, shiftAmount);
 }
 
 void KateLayoutCache::clear( )
@@ -382,22 +475,9 @@ void KateLayoutCache::setViewWidth( int width )
 
   // Only get rid of layouts that we have to
   if (wider) {
-    QMapIterator<int, KateLineLayoutPtr> it = m_lineLayouts;
-    while (it.hasNext()) {
-      it.next();
-
-      if (it.value()->viewLineCount() > 1)
-        const_cast<KateLineLayoutPtr&>(it.value())->invalidateLayout();
-    }
-
+    m_lineLayouts.viewWidthIncreased();
   } else {
-    QMapIterator<int, KateLineLayoutPtr> it = m_lineLayouts;
-    while (it.hasNext()) {
-      it.next();
-
-      if (it.value()->viewLineCount() > 1 || it.value()->width() > m_viewWidth)
-        const_cast<KateLineLayoutPtr&>(it.value())->invalidateLayout();
-    }
+    m_lineLayouts.viewWidthDecreased(width);
   }
 }
 
@@ -414,9 +494,28 @@ void KateLayoutCache::setWrap( bool wrap )
 
 void KateLayoutCache::relayoutLines( int startRealLine, int endRealLine )
 {
-  for (int i = startRealLine; i <= endRealLine; ++i)
-    if (m_lineLayouts.contains(i))
-      m_lineLayouts[i]->setLayoutDirty();
+  if (startRealLine > endRealLine)
+    kWarning() << "start" << startRealLine << "before end" << endRealLine;
+
+  m_lineLayouts.relayoutLines(startRealLine, endRealLine);
+}
+
+bool KateLayoutCache::acceptDirtyLayouts() const
+{
+  if (m_acceptDirtyLayouts.hasLocalData())
+    return *m_acceptDirtyLayouts.localData();
+
+  return false;
+}
+
+void KateLayoutCache::setAcceptDirtyLayouts(bool accept)
+{
+  if (!m_acceptDirtyLayouts.hasLocalData())
+    m_acceptDirtyLayouts.setLocalData(new bool);
+
+  *m_acceptDirtyLayouts.localData() = accept;
 }
 
 #include "katelayoutcache.moc"
+
+// kate: space-indent on; indent-width 2; replace-tabs on;
