@@ -78,7 +78,7 @@ KateSearchBar::KateSearchBar(KateViewBar * viewBar, bool initAsPower)
     // Init highlight
     m_topRange = m_view->doc()->newSmartRange(m_view->doc()->documentRange());
     static_cast<KateSmartRange*>(m_topRange)->setInternal();
-    m_topRange->setInsertBehavior(SmartRange::ExpandRight);
+    m_topRange->setInsertBehavior(SmartRange::ExpandLeft | SmartRange::ExpandRight);
     enableHighlights(true);
 
 
@@ -936,7 +936,7 @@ void KateSearchBar::onForAll(const QString & pattern, Range inputRange,
         KateDocument::repairPattern(patternCopy, multiLinePattern);
     }
 
-    // Clear backwards flag, this algorithm is for foward mode
+    // Clear backwards flag, this algorithm is for forward mode
     if (enabledOptions.testFlag(Search::Backwards)) {
         enabledOptions &= ~Search::SearchOptions(Search::Backwards);
     }
@@ -1148,7 +1148,7 @@ QVector<QString> KateSearchBar::getCapturePatterns(const QString & pattern) {
 void KateSearchBar::addMenuEntry(QMenu * menu, QVector<QString> & insertBefore, QVector<QString> & insertAfter,
         uint & walker, const QString & before, const QString after, const QString description,
         const QString & realBefore, const QString & realAfter) {
-    QAction * const action = menu->addAction(before + after + "\t" + description);
+    QAction * const action = menu->addAction(before + after + '\t' + description);
     insertBefore[walker] = QString(realBefore.isEmpty() ? before : realBefore);
     insertAfter[walker] = QString(realAfter.isEmpty() ? after : realAfter);
     action->setData(QVariant(walker++));
@@ -1172,10 +1172,10 @@ void KateSearchBar::showAddMenu(bool forPattern) {
             popupMenu->addSeparator();
             addMenuEntry(popupMenu, insertBefore, insertAfter, walker, ".", "", i18n("Any single character (excluding line breaks)"));
             popupMenu->addSeparator();
-            addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "+", "", i18n("One or more occurences"));
-            addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "*", "", i18n("Zero or more occurences"));
-            addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "?", "", i18n("Zero or one occurences"));
-            addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "{a", ",b}", i18n("<a> through <b> occurences"), "{", ",}");
+            addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "+", "", i18n("One or more occurrences"));
+            addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "*", "", i18n("Zero or more occurrences"));
+            addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "?", "", i18n("Zero or one occurrences"));
+            addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "{a", ",b}", i18n("<a> through <b> occurrences"), "{", ",}");
             popupMenu->addSeparator();
             addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "(", ")", i18n("Group, capturing"));
             addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "|", "", i18n("Or"));
@@ -1197,7 +1197,7 @@ void KateSearchBar::showAddMenu(bool forPattern) {
                         ? (QString(" = (") + capturePatterns[i - 1].left(30)) + QString(")")
                         : QString();
                 addMenuEntry(popupMenu, insertBefore, insertAfter, walker, "\\" + number, "",
-                        i18n("Reference") + " " + number + captureDetails);
+                        i18n("Reference") + ' ' + number + captureDetails);
             }
 
             popupMenu->addSeparator();
@@ -1429,21 +1429,39 @@ void KateSearchBar::onPowerModeChanged(int index, bool invokedByUserAction) {
 
 
 void KateSearchBar::onMutatePower() {
-    // Coming from power search?
-    const bool fromReplace = (m_powerUi != NULL) && (m_widget->isVisible());
-    if (fromReplace) {
-        QLineEdit * const patternLineEdit = m_powerUi->pattern->lineEdit();
-        Q_ASSERT(patternLineEdit != NULL);
-        patternLineEdit->selectAll();
-        m_powerUi->pattern->setFocus(Qt::MouseFocusReason);
-        return;
+    QString initialPattern;
+    bool selectionOnly = false;
+
+    // Guess settings from context: init pattern with current selection
+    const bool selected = m_view->selection();
+    if (selected) {
+        const Range & selection = m_view->selectionRange();
+        if (selection.onSingleLine()) {
+            // ... with current selection
+            initialPattern = m_view->selectionText();
+        } else {
+            // Enable selection only
+            selectionOnly = true;
+        }
     }
 
-    // Coming from incremental search?
-    const bool fromIncremental = (m_incUi != NULL) && (m_widget->isVisible());
-    QString initialPattern;
-    if (fromIncremental) {
-        initialPattern = m_incUi->pattern->displayText();
+    // If there's no new selection, we'll use the existing pattern
+    if (initialPattern.isNull()) {
+        // Coming from power search?
+        const bool fromReplace = (m_powerUi != NULL) && (m_widget->isVisible());
+        if (fromReplace) {
+            QLineEdit * const patternLineEdit = m_powerUi->pattern->lineEdit();
+            Q_ASSERT(patternLineEdit != NULL);
+            patternLineEdit->selectAll();
+            m_powerUi->pattern->setFocus(Qt::MouseFocusReason);
+            return;
+        }
+
+        // Coming from incremental search?
+        const bool fromIncremental = (m_incUi != NULL) && (m_widget->isVisible());
+        if (fromIncremental) {
+            initialPattern = m_incUi->pattern->displayText();
+        }
     }
 
     // Create dialog
@@ -1468,7 +1486,7 @@ void KateSearchBar::onMutatePower() {
         }
 
         // Add power widget
-        m_widget = new QWidget;
+        m_widget = new QWidget(this);
         m_powerUi = new Ui::PowerSearchBar;
         m_powerUi->setupUi(m_widget);
         m_layout->addWidget(m_widget);
@@ -1500,30 +1518,8 @@ void KateSearchBar::onMutatePower() {
         QLineEdit * const replacementLineEdit = m_powerUi->pattern->lineEdit();
         Q_ASSERT(replacementLineEdit != NULL);
         replacementLineEdit->completer()->setCaseSensitivity(Qt::CaseSensitive);
-
-        // Shortcut hack
-        #if QT_VERSION < KDE_MAKE_VERSION(4,4,0) // Remove this once Qt 4.4 becomes necessary
-        foreach (QWidget* widget, m_widget->findChildren<QWidget*>())
-            m_view->actionCollection ()->addAssociatedWidget(widget);
-        #endif
     }
 
-    // Guess settings from context
-    bool selectionOnly = false;
-    if (!fromIncremental) {
-        // Init pattern with current selection
-        const bool selected = m_view->selection();
-        if (selected) {
-            const Range & selection = m_view->selectionRange();
-            if (selection.onSingleLine()) {
-                // ... with current selection
-                initialPattern = m_view->selectionText();
-            } else {
-                // Enable selection only
-                selectionOnly = true;
-            }
-        }
-    }
     setChecked(m_powerUi->selectionOnly, selectionOnly);
 
     // Restore previous settings
@@ -1582,19 +1578,33 @@ void KateSearchBar::onMutatePower() {
 
 
 void KateSearchBar::onMutateIncremental() {
-    // Coming from incremental search?
-    const bool fromIncremental = (m_incUi != NULL) && (m_widget->isVisible());
     QString initialPattern;
-    if (fromIncremental) {
-        m_incUi->pattern->selectAll();
-        m_incUi->pattern->setFocus(Qt::MouseFocusReason);
-        return;
+
+    // Guess settings from context: init pattern with current selection
+    const bool selected = m_view->selection();
+    if (selected) {
+        const Range & selection = m_view->selectionRange();
+        if (selection.onSingleLine()) {
+            // ... with current selection
+            initialPattern = m_view->selectionText();
+        }
     }
 
-    // Coming from power search?
-    const bool fromReplace = (m_powerUi != NULL) && (m_widget->isVisible());
-    if (fromReplace) {
-        initialPattern = m_powerUi->pattern->currentText();
+    // If there's no new selection, we'll use the existing pattern
+    if (initialPattern.isNull()) {
+        // Coming from incremental search?
+        const bool fromIncremental = (m_incUi != NULL) && (m_widget->isVisible());
+        if (fromIncremental) {
+            m_incUi->pattern->selectAll();
+            m_incUi->pattern->setFocus(Qt::MouseFocusReason);
+            return;
+        }
+
+        // Coming from power search?
+        const bool fromReplace = (m_powerUi != NULL) && (m_widget->isVisible());
+        if (fromReplace) {
+            initialPattern = m_powerUi->pattern->currentText();
+        }
     }
 
     // Create dialog
@@ -1614,7 +1624,7 @@ void KateSearchBar::onMutateIncremental() {
         }
 
         // Add incremental widget
-        m_widget = new QWidget;
+        m_widget = new QWidget(this);
         m_incUi = new Ui::IncrementalSearchBar;
         m_incUi->setupUi(m_widget);
         m_layout->addWidget(m_widget);
@@ -1636,25 +1646,6 @@ void KateSearchBar::onMutateIncremental() {
 
         // Focus proxy
         centralWidget()->setFocusProxy(m_incUi->pattern);
-
-        // Shortcut hack
-        #if QT_VERSION < KDE_MAKE_VERSION(4,4,0) // Remove this once Qt 4.4 becomes necessary
-        foreach (QWidget* widget, m_widget->findChildren<QWidget*>())
-            m_view->actionCollection ()->addAssociatedWidget(widget);
-        #endif
-    }
-
-    // Guess settings from context
-    if (!fromReplace) {
-        // Init pattern with current selection
-        const bool selected = m_view->selection();
-        if (selected) {
-            const Range & selection = m_view->selectionRange();
-            if (selection.onSingleLine()) {
-                // ... with current selection
-                initialPattern = m_view->selectionText();
-            }
-        }
     }
 
     // Restore previous settings
