@@ -24,6 +24,7 @@
 
 #include "katecmd.h"
 #include <ktexteditor/attribute.h>
+#include <ktexteditor/annotationinterface.h>
 #include <ktexteditor/rangefeedback.h>
 #include "katecodefolding.h"
 #include "kateconfig.h"
@@ -33,6 +34,7 @@
 #include "kateviewinternal.h"
 #include "katelayoutcache.h"
 #include "katetextlayout.h"
+#include "katesmartrange.h"
 
 #include <kapplication.h>
 #include <kglobalsettings.h>
@@ -56,6 +58,8 @@
 #include <QtGui/QPen>
 #include <QtGui/QBoxLayout>
 #include <QtGui/QToolButton>
+#include <QtGui/QToolTip>
+#include <QtGui/QAction>
 
 #include <math.h>
 
@@ -277,7 +281,7 @@ KateCmdLine::KateCmdLine (KateView *view, KateViewBar *viewBar)
     topLayout->setMargin(0);
     m_lineEdit = new KateCmdLineEdit (this, view);
     topLayout->addWidget (m_lineEdit);
-    
+
     setFocusProxy (m_lineEdit);
 }
 
@@ -603,104 +607,6 @@ void KateCmdLineEdit::fromHistory( bool up )
 //BEGIN KateIconBorder
 using namespace KTextEditor;
 
-static const char * bookmark_xpm[] = {
-"14 13 82 1",
-"   c None",
-".  c #F27D01",
-"+  c #EF7901",
-"@  c #F3940F",
-"#  c #EE8F12",
-"$  c #F9C834",
-"%  c #F5C33A",
-"&  c #F09110",
-"*  c #FCEE3E",
-"=  c #FBEB3F",
-"-  c #E68614",
-";  c #FA8700",
-">  c #F78703",
-",  c #F4920E",
-"'  c #F19113",
-")  c #F6C434",
-"!  c #FDF938",
-"~  c #FDF839",
-"{  c #F1BC3A",
-"]  c #E18017",
-"^  c #DA7210",
-"/  c #D5680B",
-"(  c #CA5404",
-"_  c #FD8F06",
-":  c #FCB62D",
-"<  c #FDE049",
-"[  c #FCE340",
-"}  c #FBE334",
-"|  c #FDF035",
-"1  c #FEF834",
-"2  c #FCEF36",
-"3  c #F8DF32",
-"4  c #F7DC3D",
-"5  c #F5CE3E",
-"6  c #DE861B",
-"7  c #C64C03",
-"8  c #F78C07",
-"9  c #F8B019",
-"0  c #FDE12D",
-"a  c #FEE528",
-"b  c #FEE229",
-"c  c #FBD029",
-"d  c #E18814",
-"e  c #CB5605",
-"f  c #EF8306",
-"g  c #F3A00E",
-"h  c #FBC718",
-"i  c #FED31C",
-"j  c #FED11D",
-"k  c #F8B91C",
-"l  c #E07D0D",
-"m  c #CB5301",
-"n  c #ED8A0E",
-"o  c #F7A90D",
-"p  c #FEC113",
-"q  c #FEC013",
-"r  c #F09B0E",
-"s  c #D35E03",
-"t  c #EF9213",
-"u  c #F9A208",
-"v  c #FEAA0C",
-"w  c #FCA10B",
-"x  c #FCA70B",
-"y  c #FEAF0B",
-"z  c #F39609",
-"A  c #D86203",
-"B  c #F08C0D",
-"C  c #FA9004",
-"D  c #F17F04",
-"E  c #E36D04",
-"F  c #E16F03",
-"G  c #EE8304",
-"H  c #F88C04",
-"I  c #DC6202",
-"J  c #E87204",
-"K  c #E66A01",
-"L  c #DC6001",
-"M  c #D15601",
-"N  c #DA5D01",
-"O  c #D25200",
-"P  c #DA5F00",
-"Q  c #BC3C00",
-"      .+      ",
-"      @#      ",
-"      $%      ",
-"     &*=-     ",
-" ;>,')!~{]^/( ",
-"_:<[}|11234567",
-" 890aaaaabcde ",
-"  fghiiijklm  ",
-"   nopqpqrs   ",
-"   tuvwxyzA   ",
-"   BCDEFGHI   ",
-"   JKL  MNO   ",
-"   P      Q   "};
-
 const int halfIPW = 8;
 
 KateIconBorder::KateIconBorder ( KateViewInternal* internalView, QWidget *parent )
@@ -712,11 +618,13 @@ KateIconBorder::KateIconBorder ( KateViewInternal* internalView, QWidget *parent
   , m_lineNumbersOn( false )
   , m_foldingMarkersOn( false )
   , m_dynWrapIndicatorsOn( false )
+  , m_annotationBorderOn( false )
   , m_dynWrapIndicators( 0 )
   , m_cachedLNWidth( 0 )
   , m_maxCharWidth( 0 )
   , iconPaneWidth (16)
-  , m_blockRange(0)
+  , m_annotationBorderWidth (6)
+  , m_foldingRange(0)
   , m_lastBlockLine(-1)
 {
 
@@ -764,12 +672,12 @@ KateIconBorder::KateIconBorder ( KateViewInternal* internalView, QWidget *parent
   setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
   setMouseTracking(true);
   m_doc->setMarkDescription( MarkInterface::markType01, i18n("Bookmark") );
-  m_doc->setMarkPixmap( MarkInterface::markType01, QPixmap((const char**)bookmark_xpm) );
+  m_doc->setMarkPixmap( MarkInterface::markType01, KIcon("bookmarks").pixmap(16, 16) );
 
   updateFont();
 }
 
-KateIconBorder::~KateIconBorder() {delete m_blockRange;}
+KateIconBorder::~KateIconBorder() {delete m_foldingRange;}
 
 void KateIconBorder::setIconBorderOn( bool enable )
 {
@@ -783,6 +691,29 @@ void KateIconBorder::setIconBorderOn( bool enable )
   QTimer::singleShot( 0, this, SLOT(update()) );
 }
 
+void KateIconBorder::setAnnotationBorderOn( bool enable )
+{
+  if( enable == m_annotationBorderOn )
+    return;
+
+  m_annotationBorderOn = enable;
+
+  emit m_view->annotationBorderVisibilityChanged(m_view, enable);
+
+  updateGeometry();
+}
+
+void KateIconBorder::removeAnnotationHovering()
+{
+  // remove hovering if it's still there
+  if (m_annotationBorderOn && !m_hoveredAnnotationText.isEmpty())
+  {
+    m_hoveredAnnotationText = QString();
+    hideAnnotationTooltip();
+    QTimer::singleShot( 0, this, SLOT(update()) );
+  }
+}
+
 void KateIconBorder::setLineNumbersOn( bool enable )
 {
   if( enable == m_lineNumbersOn )
@@ -828,12 +759,17 @@ QSize KateIconBorder::sizeHint() const
   if (m_iconBorderOn)
     w += iconPaneWidth + 1;
 
+  if (m_annotationBorderOn)
+  {
+    w += m_annotationBorderWidth + 1;
+  }
+
   if (m_lineNumbersOn || (m_view->dynWordWrap() && m_dynWrapIndicatorsOn)) {
-    w += lineNumberWidth();
+    w += lineNumberWidth() + 1;
   }
 
   if (m_foldingMarkersOn)
-    w += iconPaneWidth;
+    w += iconPaneWidth + 1;
 
   w += 4;
 
@@ -934,6 +870,8 @@ void KateIconBorder::paintEvent(QPaintEvent* e)
 
 static void paintTriangle (QPainter &painter, const QColor &baseColor, int xOffset, int yOffset, int width, int height, bool open)
 {
+  painter.setRenderHint(QPainter::Antialiasing);
+
   qreal size = qMin (width, height);
 
   QColor c = baseColor.dark ();
@@ -967,6 +905,8 @@ static void paintTriangle (QPainter &painter, const QColor &baseColor, int xOffs
     QPointF points[3] = { middle+QPointF(-halfSizeP, -halfSize), middle+QPointF(-halfSizeP, halfSize), middle+QPointF(halfSizeP, 0) };
     painter.drawConvexPolygon(points, 3);
   }
+
+  painter.setRenderHint(QPainter::Antialiasing, false);
 }
 
 void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
@@ -1002,7 +942,7 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
   int w( this->width() );                     // sane value/calc only once
 
   QPainter p ( this );
-  p.setRenderHints (QPainter::Antialiasing);
+  p.setRenderHints (QPainter::TextAntialiasing);
   p.setFont ( m_view->renderer()->config()->font() ); // for line numbers
   // the line number color is for the line numbers, vertical separator lines
   // and for for the code folding lines.
@@ -1018,6 +958,9 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
       m_doc->lineInfo(&oldInfo,m_viewInternal->cache()->viewLine(startz).line()-1);
   }
 
+  KTextEditor::AnnotationModel *model = m_view->annotationModel() ?
+      m_view->annotationModel() : m_doc->annotationModel();
+
   for (uint z=startz; z <= endz; z++)
   {
     int y = h * z;
@@ -1026,7 +969,7 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
     if (z < lineRangesSize)
       realLine = m_viewInternal->cache()->viewLine(z).line();
 
-    int lnX ( 0 );
+    int lnX = 0;
 
     p.fillRect( 0, y, w-4, h, m_view->renderer()->config()->iconBarColor() );
     p.fillRect( w-4, y, 4, h, m_view->renderer()->config()->backgroundColor() );
@@ -1036,8 +979,7 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
     {
       p.setPen ( m_view->renderer()->config()->lineNumberColor() );
       p.setBrush ( m_view->renderer()->config()->lineNumberColor() );
-
-      p.drawLine(lnX+iconPaneWidth, y, lnX+iconPaneWidth, y+h);
+      p.drawLine(lnX+iconPaneWidth+1, y, lnX+iconPaneWidth+1, y+h);
 
       if( (realLine > -1) && (m_viewInternal->cache()->viewLine(z).startCol() == 0) )
       {
@@ -1070,23 +1012,94 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
         }
       }
 
-      lnX += iconPaneWidth + 1;
+      lnX += iconPaneWidth + 2;
+    }
+
+    // annotation information
+    if( m_annotationBorderOn )
+    {
+      // Draw a border line between annotations and the line numbers
+      p.setPen ( m_view->renderer()->config()->lineNumberColor() );
+      p.setBrush ( m_view->renderer()->config()->lineNumberColor() );
+
+      int borderWidth = m_annotationBorderWidth;
+      p.drawLine(lnX+borderWidth+1, y, lnX+borderWidth+1, y+h);
+
+      if( (realLine > -1) && model )
+      {
+        // Fetch data from the model
+        QVariant text = model->data( realLine, Qt::DisplayRole );
+        QVariant foreground = model->data( realLine, Qt::ForegroundRole );
+        QVariant background = model->data( realLine, Qt::BackgroundRole );
+        // Fill the background
+        if( background.isValid() )
+        {
+          p.fillRect( lnX, y, borderWidth + 1, h, background.value<QBrush>() );
+        }
+        // Set the pen for drawing the foreground
+        if( foreground.isValid() )
+        {
+          p.setBrush( foreground.value<QBrush>() );
+        }
+
+        // Draw a border around all adjacent entries that have the same text as the currently hovered one
+        if( m_hoveredAnnotationText == text.toString() )
+        {
+          p.drawLine( lnX, y, lnX, y+h );
+          p.drawLine( lnX+borderWidth, y, lnX+borderWidth, y+h );
+
+          QVariant beforeText = model->data( realLine-1, Qt::DisplayRole );
+          QVariant afterText = model->data( realLine+1, Qt::DisplayRole );
+          if( (beforeText.isValid() && beforeText.canConvert<QString>()
+              && text.isValid() && text.canConvert<QString>()
+              && beforeText.toString() != text.toString() || realLine == 0)
+              && m_viewInternal->cache()->viewLine(z).viewLine() == 0)
+          {
+            p.drawLine( lnX+1, y, lnX+borderWidth, y );
+          }
+
+          if( ((afterText.isValid() && afterText.canConvert<QString>()
+              && text.isValid() && text.canConvert<QString>()
+              && afterText.toString() != text.toString())
+                || realLine == m_view->doc()->lines() - 1)
+              && m_viewInternal->cache()->viewLine(z).viewLine() == m_viewInternal->cache()->viewLineCount(realLine)-1)
+          {
+            p.drawLine( lnX+1, y+h-1, lnX+borderWidth, y+h-1 );
+          }
+        }
+        if( foreground.isValid() )
+        {
+          QPen pen = p.pen();
+          pen.setWidth( 1 );
+          p.setPen( pen );
+        }
+
+        // Now draw the normal text
+        if( text.isValid() && text.canConvert<QString>() && (m_viewInternal->cache()->viewLine(z).startCol() == 0)  )
+        {
+          p.drawText( lnX+3, y, borderWidth-3, h, Qt::AlignLeft|Qt::AlignVCenter, text.toString() );
+        }
+      }
+
+      // adjust current X position and reset the pen and brush
+      lnX += borderWidth + 2;
+
+      p.setPen ( m_view->renderer()->config()->lineNumberColor() );
+      p.setBrush ( m_view->renderer()->config()->lineNumberColor() );
     }
 
     // line number
     if( m_lineNumbersOn || (m_view->dynWordWrap() && m_dynWrapIndicatorsOn) )
     {
-      lnX +=2;
-
       if (realLine > -1)
         if (m_viewInternal->cache()->viewLine(z).startCol() == 0) {
           if (m_lineNumbersOn)
-            p.drawText( lnX + 1, y, lnWidth-4, h, Qt::AlignRight|Qt::AlignVCenter, QString("%1").arg( realLine + 1 ) );
+            p.drawText( lnX, y, lnWidth-4, h, Qt::AlignRight|Qt::AlignVCenter, QString("%1").arg( realLine + 1 ) );
         } else if (m_view->dynWordWrap() && m_dynWrapIndicatorsOn) {
-          p.drawPixmap(lnX + lnWidth - m_arrow.width() - 4, y, m_arrow);
+          p.drawPixmap(lnX + lnWidth - m_arrow.width() - 2, y, m_arrow);
         }
 
-      lnX += lnWidth;
+      lnX += lnWidth + 2;
     }
 
     // folding markers
@@ -1122,7 +1135,7 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
         oldInfo = info;
       }
 
-      lnX += iconPaneWidth;
+      lnX += iconPaneWidth + 2;
     }
   }
 }
@@ -1135,6 +1148,11 @@ KateIconBorder::BorderArea KateIconBorder::positionToArea( const QPoint& p ) con
     if( p.x() <= x )
       return IconBorder;
   }
+  if( this->m_annotationBorderOn ) {
+    x += m_annotationBorderWidth;
+    if( p.x() <= x )
+      return AnnotationBorder;
+  }
   if( m_lineNumbersOn || m_dynWrapIndicators ) {
     x += lineNumberWidth();
     if( p.x() <= x )
@@ -1153,7 +1171,7 @@ void KateIconBorder::mousePressEvent( QMouseEvent* e )
   const KateTextLayout& t = m_viewInternal->yToKateTextLayout(e->y());
   if (t.isValid()) {
     m_lastClickedLine = t.line();
-    if ( positionToArea( e->pos() ) != IconBorder )
+    if ( positionToArea( e->pos() ) != IconBorder && positionToArea( e->pos() ) != AnnotationBorder )
     {
       QMouseEvent forward( QEvent::MouseButtonPress,
         QPoint( 0, e->y() ), e->button(), e->buttons(),e->modifiers() );
@@ -1165,49 +1183,53 @@ void KateIconBorder::mousePressEvent( QMouseEvent* e )
   QWidget::mousePressEvent(e);
 }
 
-void KateIconBorder::showBlock(int line) {
-  //kDebug()<<"showBlock: 1";
-  if (line==m_lastBlockLine) return;
-  //kDebug()<<"showBlock: 2";
-  m_lastBlockLine=line;
-  delete m_blockRange;
-  m_blockRange=0;
-  KateCodeFoldingTree *tree=m_doc->foldingTree();
+void KateIconBorder::showBlock(int line)
+{
+  if (line == m_lastBlockLine) return;
+  m_lastBlockLine = line;
+
+  // get the new range, that should be highlighted
+  KTextEditor::Range newRange = KTextEditor::Range::invalid();
+  KateCodeFoldingTree *tree = m_doc->foldingTree();
   if (tree) {
-    //kDebug()<<"showBlock: 3";
     KateCodeFoldingNode *node = tree->findNodeForLine(line);
     KTextEditor::Cursor beg;
     KTextEditor::Cursor end;
-    if (node != tree->rootNode () && node->getBegin(tree,&beg) && node->getEnd(tree,&end)) {
-      kDebug()<<"BEGIN"<<beg<<"END"<<end;
-      m_blockRange=m_doc->newSmartRange(KTextEditor::Range(beg,end));
-      KTextEditor::Attribute::Ptr attr(new KTextEditor::Attribute());
-      attr->setBackground(foldingColor(0,line,false));
-      m_blockRange->setAttribute(attr);
-      m_doc->addHighlightToView(m_view,m_blockRange,false);
-
-      // (dh) In order to prevent a dangling pointer on document reload (F5) set the smart range
-      // pointer to null, if the range is deleted (http://bugs.kde.org/show_bug.cgi?id=160527)
-      KTextEditor::SmartRangeNotifier *notifier = m_blockRange->primaryNotifier();
-      connect(notifier, SIGNAL(rangeDeleted(KTextEditor::SmartRange*)), this, SLOT(blockRangeDeleted()));
+    if (node != tree->rootNode () && node->getBegin(tree, &beg) && node->getEnd(tree, &end)) {
+      newRange = KTextEditor::Range(beg, end);
     }
   }
-}
 
-void KateIconBorder::blockRangeDeleted()
-{
-  m_blockRange = 0;
+  if (newRange.isValid() && m_foldingRange && *m_foldingRange == newRange) {
+    // new range equals the old one, nothing to do.
+    return;
+  } else { // the ranges differ, delete the old, if it exists
+    delete m_foldingRange;
+    m_foldingRange = 0;
+  }
+
+  if (newRange.isValid()) {
+    kDebug(13025) << "new folding hl-range:" << newRange;
+    m_foldingRange = m_doc->newSmartRange(newRange);
+    static_cast<KateSmartRange*>(m_foldingRange)->setInternal();
+    KTextEditor::Attribute::Ptr attr(new KTextEditor::Attribute());
+    attr->setBackground(foldingColor(0, line, false));
+    m_foldingRange->setAttribute(attr);
+    m_doc->addHighlightToView(m_view, m_foldingRange, false);
+  }
 }
 
 void KateIconBorder::hideBlock() {
   m_lastBlockLine=-1;
-  delete m_blockRange;
-  m_blockRange=0;
+  delete m_foldingRange;
+  m_foldingRange = 0;
 }
 
 void KateIconBorder::leaveEvent(QEvent *event)
 {
   hideBlock();
+  removeAnnotationHovering();
+
   QWidget::leaveEvent(event);
 }
 
@@ -1217,6 +1239,23 @@ void KateIconBorder::mouseMoveEvent( QMouseEvent* e )
   if (t.isValid()) {
     if ( positionToArea( e->pos() ) == FoldingMarkers) showBlock(t.line());
     else hideBlock();
+    if ( positionToArea( e->pos() ) == AnnotationBorder )
+    {
+      KTextEditor::AnnotationModel *model = m_view->annotationModel() ?
+        m_view->annotationModel() : m_doc->annotationModel();
+      if (model)
+      {
+        m_hoveredAnnotationText = model->data( t.line(), Qt::DisplayRole ).toString();
+        showAnnotationTooltip( t.line(), e->globalPos() );
+        QTimer::singleShot( 0, this, SLOT(update()) );
+      }
+    }
+    else
+    {
+      m_hoveredAnnotationText = QString();
+      hideAnnotationTooltip();
+      QTimer::singleShot( 0, this, SLOT(update()) );
+    }
     if ( positionToArea( e->pos() ) != IconBorder )
     {
       QMouseEvent forward( QEvent::MouseMove,
@@ -1224,6 +1263,11 @@ void KateIconBorder::mouseMoveEvent( QMouseEvent* e )
       m_viewInternal->mouseMoveEvent( &forward );
     }
   }
+  else
+  {
+    // remove hovering if it's still there
+    removeAnnotationHovering();
+  }
 
   QWidget::mouseMoveEvent(e);
 }
@@ -1260,6 +1304,14 @@ void KateIconBorder::mouseReleaseEvent( QMouseEvent* e )
         emit toggleRegionVisibility(cursorOnLine);
       }
     }
+
+    if ( area == AnnotationBorder ) {
+      if( e->button() == Qt::LeftButton && KGlobalSettings::singleClick() ) {
+        emit m_view->annotationActivated( m_view, cursorOnLine );
+      } else if ( e->button() == Qt::RightButton ) {
+        showAnnotationMenu( cursorOnLine, e->globalPos() );
+      }
+    }
   }
 
   QMouseEvent forward( QEvent::MouseButtonRelease,
@@ -1269,6 +1321,16 @@ void KateIconBorder::mouseReleaseEvent( QMouseEvent* e )
 
 void KateIconBorder::mouseDoubleClickEvent( QMouseEvent* e )
 {
+  int cursorOnLine = m_viewInternal->yToKateTextLayout(e->y()).line();
+
+  if (cursorOnLine == m_lastClickedLine &&
+      cursorOnLine <= m_doc->lastLine() )
+  {
+    BorderArea area = positionToArea( e->pos() );
+    if( area == AnnotationBorder && !KGlobalSettings::singleClick() ) {
+      emit m_view->annotationActivated( m_view, cursorOnLine );
+    }
+  }
   QMouseEvent forward( QEvent::MouseButtonDblClick,
     QPoint( 0, e->y() ), e->button(), e->buttons(),e->modifiers() );
   m_viewInternal->mouseDoubleClickEvent( &forward );
@@ -1336,8 +1398,99 @@ void KateIconBorder::showMarkMenu( uint line, const QPoint& pos )
     }
   }
 }
+
+void KateIconBorder::showAnnotationTooltip( int line, const QPoint& pos )
+{
+  KTextEditor::AnnotationModel *model = m_view->annotationModel() ?
+    m_view->annotationModel() : m_doc->annotationModel();
+
+  if( model )
+  {
+    QVariant data = model->data( line, Qt::ToolTipRole );
+    QString tip = data.toString();
+    if (!tip.isEmpty())
+      QToolTip::showText( pos, data.toString(), this );
+  }
+}
+
+
+int KateIconBorder::annotationLineWidth( int line )
+{
+  KTextEditor::AnnotationModel *model = m_view->annotationModel() ?
+    m_view->annotationModel() : m_doc->annotationModel();
+
+  if( model )
+  {
+    QVariant data = model->data( line, Qt::DisplayRole );
+    return data.toString().length() * m_maxCharWidth + 8;
+  }
+  return 8;
+}
+
+void KateIconBorder::updateAnnotationLine( int line )
+{
+  if( annotationLineWidth(line) > m_annotationBorderWidth )
+  {
+    m_annotationBorderWidth = annotationLineWidth(line);
+    updateGeometry();
+
+    QTimer::singleShot( 0, this, SLOT(update()) );
+  }
+}
+
+void KateIconBorder::showAnnotationMenu( int line, const QPoint& pos)
+{
+  KMenu menu;
+  QAction a("Disable Annotation Bar", &menu);
+  menu.addAction(&a);
+  emit m_view->annotationContextMenuAboutToShow( m_view, &menu, line  );
+  if (menu.exec(pos) == &a)
+    m_view->setAnnotationBorderVisible(false);
+}
+
+void KateIconBorder::hideAnnotationTooltip()
+{
+  QToolTip::hideText();
+}
+
+void KateIconBorder::updateAnnotationBorderWidth( )
+{
+  m_annotationBorderWidth = 6;
+  KTextEditor::AnnotationModel *model = m_view->annotationModel() ?
+    m_view->annotationModel() : m_doc->annotationModel();
+
+  if( model ) {
+    for( int i = 0; i < m_view->doc()->lines(); i++ ) {
+      int curwidth = annotationLineWidth( i );
+      if( curwidth > m_annotationBorderWidth )
+        m_annotationBorderWidth = curwidth;
+    }
+  }
+
+  updateGeometry();
+
+  QTimer::singleShot( 0, this, SLOT(update()) );
+}
+
+
+
+void KateIconBorder::annotationModelChanged( KTextEditor::AnnotationModel * oldmodel, KTextEditor::AnnotationModel * newmodel )
+{
+  if( oldmodel )
+  {
+    oldmodel->disconnect( this );
+  }
+  if( newmodel )
+  {
+    connect( newmodel, SIGNAL(reset()), this, SLOT(updateAnnotationBorderWidth()) );
+    connect( newmodel, SIGNAL(lineChanged( int )), this, SLOT(updateAnnotationLine( int )) );
+  }
+  updateAnnotationBorderWidth();
+}
+
 //END KateIconBorder
 
+//BEGIN KateViewEncodingAction
 KateViewEncodingAction::KateViewEncodingAction(KateDocument *_doc, KateView *_view, const QString& text, QObject *parent)
        : KCodecAction(text, parent,true), doc(_doc), view (_view)
 {
@@ -1378,13 +1531,12 @@ void KateViewEncodingAction::setScriptForEncodingAutoDetection (KEncodingDetecto
     doc->setScriptForEncodingAutoDetection(script);
   view->reloadFile();
 }
-
-
+//END KateViewEncodingAction
 
 //BEGIN KateViewBar related classes
 
 KateViewBarWidget::KateViewBarWidget (KateViewBar *viewBar)
- : QWidget (), m_viewBar (viewBar)
+ : QWidget (viewBar), m_viewBar (viewBar)
 {
   m_viewBar->addBarWidget (this);
 
@@ -1396,7 +1548,7 @@ KateViewBarWidget::KateViewBarWidget (KateViewBar *viewBar)
   // hide button
   QToolButton *hideButton = new QToolButton(this);
   hideButton->setAutoRaise(true);
-  hideButton->setIcon(KIcon("process-stop"));
+  hideButton->setIcon(KIcon("dialog-close"));
   connect(hideButton, SIGNAL(clicked()), this, SLOT(hideBar()));
   layout->addWidget(hideButton);
   layout->setAlignment( hideButton, Qt::AlignLeft|Qt::AlignTop );
@@ -1481,6 +1633,7 @@ void KateViewBar::keyPressEvent(QKeyEvent* event)
     return;
   }
   QWidget::keyPressEvent(event);
+
 }
 
 void KateViewBar::hideEvent(QHideEvent* event)
@@ -1492,3 +1645,6 @@ void KateViewBar::hideEvent(QHideEvent* event)
 //END KateViewBar related classes
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
+
+
+
