@@ -509,10 +509,8 @@ void CustomFileDialog::bookmarkChosen( int i )
 
 void CustomFileDialog::goHome()
 {
-  if ( getenv( "HOME" ) )
-    setDir( getenv( "HOME" ) );
-  else
-    setDir( "/" );
+  const QByteArray home = qgetenv( "HOME" );
+  setDir( home.isEmpty() ? QLatin1String( "/" ) : QString(home) );
 }
 
 void CustomFileDialog::done( int i )
@@ -646,7 +644,7 @@ void Directory::setOpen( bool o )
 
     listView()->setUpdatesEnabled( false );
     QFileInfoList files = thisDir.entryInfoList();
-    foreach ( QFileInfo f, files ){
+    foreach ( const QFileInfo &f, files ){
       if ( f.fileName() != "." && f.fileName() != ".." && f.isDir() )
         (void)new Directory( this, f.fileName() );
     }
