@@ -57,6 +57,10 @@ public:
     void putData(const QByteArray& group, const char* key, const QByteArray& value,
                  KConfigBase::WriteConfigFlags flags, bool expand=false);
     QStringList groupList(const QByteArray& group) const;
+    // copies the entries from @p source to @p otherGroup changing all occurrences
+    // of @p source with @p destination
+    void copyGroup(const QByteArray& source, const QByteArray& destination,
+                   KConfigGroup *otherGroup, KConfigBase::WriteConfigFlags flags) const;
 
 protected:
     KSharedPtr<KConfigBackend> mBackend;
@@ -64,7 +68,7 @@ protected:
     KConfigPrivate(const KComponentData &componentData_, KConfig::OpenFlags flags,
            const char* resource);
 
-    ~KConfigPrivate()
+    virtual ~KConfigPrivate()
     {
     }
 
@@ -100,8 +104,6 @@ private:
     void parseConfigFiles();
     void initCustomized(KConfig*);
     bool lockLocal();
-
-    void setDirty(bool b);
 };
 
 #endif // KCONFIG_P_H
