@@ -32,6 +32,7 @@
 #include "kcomponentdata.h"
 #include "kstandarddirs.h"
 #include "kconfigdata.h"
+#include "kde_file.h"
 #include <kdebug.h>
 
 #include <QtCore/QDate>
@@ -60,15 +61,19 @@ class KConfigGroupPrivate : public QSharedData
     }
 
     KConfigGroupPrivate(KConfigGroup* parent, bool isImmutable, bool isConst, const QByteArray& name)
-        : sOwner(parent->d->sOwner), mOwner(parent->d->mOwner), mParent(parent->d), mName(name),
+        : sOwner(parent->d->sOwner), mOwner(parent->d->mOwner), mName(name),
           bImmutable(isImmutable), bConst(isConst)
     {
+        if (!parent->d->mName.isEmpty())
+            mParent = parent->d;
     }
 
     KConfigGroupPrivate(const KConfigGroupPrivate* other, bool isImmutable, const QByteArray &name)
-        : sOwner(other->sOwner), mOwner(other->mOwner), mParent(other->mParent), mName(name),
+        : sOwner(other->sOwner), mOwner(other->mOwner), mName(name),
           bImmutable(isImmutable), bConst(other->bConst)
     {
+        if (!other->mName.isEmpty())
+            mParent = const_cast<KConfigGroupPrivate *>(other);
     }
 
     KSharedConfig::Ptr sOwner;
@@ -97,6 +102,8 @@ class KConfigGroupPrivate : public QSharedData
 
     QByteArray fullName(const QByteArray& aGroup) const
     {
+        if (mName.isEmpty())
+            return aGroup;
         return fullName() + '\x1d' + aGroup;
     }
 
@@ -191,13 +198,17 @@ static QList<qreal> asRealList(const QByteArray& string)
     return list;
 }
 
+static QString errString( const char * pKey, const QByteArray & value, const QVariant & aDefault ) {
+    return QString::fromLatin1("\"%1\" - conversion of \"%3\" to %2 failed")
+            .arg(pKey).arg(QVariant::typeToName(aDefault.type())).arg(value.constData());
+}
+
+static QString formatError( int expected, int got ) {
+    return QString::fromLatin1(" (wrong format: expected %1 items, got %2)").arg( expected ).arg( got );
+}
+
 QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& value, const QVariant& aDefault)
 {
-    const QString errString = QString::fromLatin1("\"%1\" - conversion of \"%3\" to %2 failed")
-            .arg(pKey).arg(QVariant::typeToName(aDefault.type())).arg(value.constData());
-    const QString formatError = QString::fromLatin1(" (wrong format: expected %1 items, got %2)");
-    QVariant tmp = aDefault;
-
     // if a type handler is added here you must add a QVConversions definition
     // to conversion_check.h, or ConversionCheck::to_QVariant will not allow
     // readEntry<T> to convert to QVariant.
@@ -224,17 +235,18 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& val
         case QVariant::Int:
         case QVariant::UInt:
         case QVariant::LongLong:
-        case QVariant::ULongLong:
-            tmp = value;
+        case QVariant::ULongLong: {
+            QVariant tmp = value;
             if ( !tmp.convert(aDefault.type()) )
                 tmp = aDefault;
             return tmp;
+        }
         case QVariant::Point: {
             const QList<int> list = asIntList(value);
 
             if ( list.count() != 2 ) {
-                kError() << errString
-                         << formatError.arg(2).arg(list.count());
+                kError() << errString( pKey, value, aDefault )
+                         << formatError( 2, list.count() );
                 return aDefault;
             }
             return QPoint(list.at( 0 ), list.at( 1 ));
@@ -243,8 +255,8 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& val
             const QList<qreal> list = asRealList(value);
 
             if ( list.count() != 2 ) {
-                kError() << errString
-                         << formatError.arg(2).arg(list.count());
+                kError() << errString( pKey, value, aDefault )
+                         << formatError( 2, list.count() );
                 return aDefault;
             }
             return QPointF(list.at( 0 ), list.at( 1 ));
@@ -253,13 +265,13 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& val
             const QList<int> list = asIntList(value);
 
             if ( list.count() != 4 ) {
-                kError() << errString
-                         << formatError.arg(4).arg(list.count());
+                kError() << errString( pKey, value, aDefault )
+                         << formatError( 4, list.count() );
                 return aDefault;
             }
             const QRect rect(list.at( 0 ), list.at( 1 ), list.at( 2 ), list.at( 3 ));
             if ( !rect.isValid() ) {
-                kError() << errString;
+                kError() << errString( pKey, value, aDefault );
                 return aDefault;
             }
             return rect;
@@ -268,13 +280,13 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& val
             const QList<qreal> list = asRealList(value);
 
             if ( list.count() != 4 ) {
-                kError() << errString
-                         << formatError.arg(4).arg(list.count());
+                kError() << errString( pKey, value, aDefault )
+                         << formatError( 4, list.count() );
                 return aDefault;
             }
             const QRectF rect(list.at( 0 ), list.at( 1 ), list.at( 2 ), list.at( 3 ));
             if ( !rect.isValid() ) {
-                kError() << errString;
+                kError() << errString( pKey, value, aDefault );
                 return aDefault;
             }
             return rect;
@@ -283,13 +295,13 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& val
             const QList<int> list = asIntList(value);
 
             if ( list.count() != 2 ) {
-                kError() << errString
-                         << formatError.arg(2).arg(list.count());
+                kError() << errString( pKey, value, aDefault )
+                         << formatError( 2, list.count() );
                 return aDefault;
             }
             const QSize size(list.at( 0 ), list.at( 1 ));
             if ( !size.isValid() ) {
-                kError() << errString;
+                kError() << errString( pKey, value, aDefault );
                 return aDefault;
             }
             return size;
@@ -298,13 +310,13 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& val
             const QList<qreal> list = asRealList(value);
 
             if ( list.count() != 2 ) {
-                kError() << errString
-                         << formatError.arg(2).arg(list.count());
+                kError() << errString( pKey, value, aDefault )
+                         << formatError( 2, list.count() );
                 return aDefault;
             }
             const QSizeF size(list.at( 0 ), list.at( 1 ));
             if ( !size.isValid() ) {
-                kError() << errString;
+                kError() << errString( pKey, value, aDefault );
                 return aDefault;
             }
             return size;
@@ -312,15 +324,15 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& val
         case QVariant::DateTime: {
             const QList<int> list = asIntList(value);
             if ( list.count() != 6 ) {
-                kError() << errString
-                         << formatError.arg(6).arg(list.count());
+                kError() << errString( pKey, value, aDefault )
+                         << formatError( 6, list.count() );
                 return aDefault;
             }
             const QDate date( list.at( 0 ), list.at( 1 ), list.at( 2 ) );
             const QTime time( list.at( 3 ), list.at( 4 ), list.at( 5 ) );
             const QDateTime dt( date, time );
             if ( !dt.isValid() ) {
-                kError() << errString;
+                kError() << errString( pKey, value, aDefault );
                 return aDefault;
             }
             return dt;
@@ -330,13 +342,13 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray& val
             if ( list.count() == 6 )
                 list = list.mid(0, 3); // don't break config files that stored QDate as QDateTime
             if ( list.count() != 3 ) {
-                kError() << errString
-                         << formatError.arg(3).arg(list.count());
+                kError() << errString( pKey, value, aDefault )
+                         << formatError( 3, list.count() );
                 return aDefault;
             }
             const QDate date( list.at( 0 ), list.at( 1 ), list.at( 2 ) );
             if ( !date.isValid() ) {
-                kError() << errString;
+                kError() << errString( pKey, value, aDefault );
                 return aDefault;
             }
             return date;
@@ -420,8 +432,8 @@ QString KConfigGroupPrivate::expandString(const QString& value)
                 else
 #endif
                 {
-                    const char* pEnv = getenv( aVarName.toAscii() );
-                    if( pEnv )
+                    QByteArray pEnv = qgetenv( aVarName.toAscii() );
+                    if( !pEnv.isEmpty() )
                     // !!! Sergey A. Sukiyazov <corwin@micom.don.ru> !!!
                     // An environment variable may contain values in 8bit
                     // locale specified encoding or UTF8 encoding
@@ -492,7 +504,7 @@ static QString translatePath( QString path ) // krazy:exclude=passbyvalue
    // since it would not recognize paths without a trailing '/'.
    // All of the 3 following functions to return the user's home directory
    // can return different paths. We have to test all them.
-   const QString homeDir0 = QFile::decodeName(getenv("HOME"));
+   const QString homeDir0 = QFile::decodeName(qgetenv("HOME"));
    const QString homeDir1 = QDir::homePath();
    const QString homeDir2 = QDir(homeDir1).canonicalPath();
    if (cleanHomeDirPath(path, homeDir0) ||
@@ -583,7 +595,6 @@ KConfigGroup::~KConfigGroup()
 KConfigGroup KConfigGroup::groupImpl(const QByteArray& aGroup)
 {
     Q_ASSERT_X(isValid(), "KConfigGroup::groupImpl", "accessing an invalid group");
-    Q_ASSERT_X(!d->mName.isEmpty(), "KConfigGroup::groupImpl", "the default group is not allowed to have children");
     Q_ASSERT_X(!aGroup.isEmpty(), "KConfigGroup::groupImpl", "can not have an unnamed child group");
 
     KConfigGroup newGroup;
@@ -596,7 +607,6 @@ KConfigGroup KConfigGroup::groupImpl(const QByteArray& aGroup)
 const KConfigGroup KConfigGroup::groupImpl(const QByteArray& aGroup) const
 {
     Q_ASSERT_X(isValid(), "KConfigGroup::groupImpl", "accessing an invalid group");
-    Q_ASSERT_X(!d->mName.isEmpty(), "KConfigGroup::groupImpl", "the default group is not allowed to have children");
     Q_ASSERT_X(!aGroup.isEmpty(), "KConfigGroup::groupImpl", "can not have an unnamed child group");
 
     KConfigGroup newGroup;
@@ -607,6 +617,23 @@ const KConfigGroup KConfigGroup::groupImpl(const QByteArray& aGroup) const
     return newGroup;
 }
 
+KConfigGroup KConfigGroup::parent() const
+{
+    Q_ASSERT_X(isValid(), "KConfigGroup::parent", "accessing an invalid group");
+
+    KConfigGroup parentGroup;
+
+    if (d->mParent) {
+        parentGroup.d = d->mParent;
+    } else {
+        parentGroup.d = new KConfigGroupPrivate(d->mOwner, d->mOwner->isImmutable(), d->bConst, "");
+        // make sure we keep the refcount up on the KConfig object
+        parentGroup.d->sOwner = d->sOwner;
+    }
+
+    return parentGroup;
+}
+
 void KConfigGroup::deleteGroup(WriteConfigFlags flags)
 {
     Q_ASSERT_X(isValid(), "KConfigGroup::deleteGroup", "accessing an invalid group");
@@ -624,9 +651,10 @@ void KConfigGroup::changeGroup( const char *group )
 {
     Q_ASSERT_X(isValid(), "KConfigGroup::changeGroup", "accessing an invalid group");
 
+    KConfigGroup pnt(parent());
     // detach (QExplicitlySharedDataPointer takes care of deleting the old d if necessary)
     // ### temporary solution until QExplicitlySharedDataPointer has detach()
-    d = new KConfigGroupPrivate(d.data(), config()->isGroupImmutable(group), group);
+    d = new KConfigGroupPrivate(&pnt, pnt.isGroupImmutable(group), d->bConst, group);
 }
 
 QString KConfigGroup::name() const
@@ -759,7 +787,7 @@ QVariant KConfigGroup::readEntry( const char* key, const QVariant &aDefault ) co
 
     QVariant value;
     if (!readEntryGui( data, key, aDefault, value ))
-        value = convertToQVariant(key, data, aDefault);
+        return convertToQVariant(key, data, aDefault);
 
     return value;
 }
@@ -1270,3 +1298,32 @@ bool KConfigGroup::isGroupImmutableImpl(const QByteArray& b) const
 
     return config()->isGroupImmutable(d->fullName(b));
 }
+
+void KConfigGroup::copyTo(KConfigBase* other, WriteConfigFlags pFlags) const
+{
+    Q_ASSERT_X(isValid(), "KConfigGroup::copyTo", "accessing an invalid group");
+    Q_ASSERT(other != 0);
+
+    if (KConfigGroup *otherGroup = dynamic_cast<KConfigGroup*>(other)) {
+        config()->d_func()->copyGroup(d->fullName(), otherGroup->d->fullName(), otherGroup, pFlags);
+    } else if (KConfig* otherConfig = dynamic_cast<KConfig*>(other)) {
+        KConfigGroup newGroup = otherConfig->group(d->fullName());
+        otherConfig->d_func()->copyGroup(d->fullName(), d->fullName(), &newGroup, pFlags);
+    } else {
+        Q_ASSERT_X(false, "KConfigGroup::copyTo", "unknown type of KConfigBase");
+    }
+}
+
+void KConfigGroup::reparent(KConfigBase* parent, WriteConfigFlags pFlags)
+{
+    Q_ASSERT_X(isValid(), "KConfigGroup::reparent", "accessing an invalid group");
+    Q_ASSERT_X(!d->bConst, "KConfigGroup::reparent", "reparenting a read-only group");
+    Q_ASSERT_X(!d->bImmutable, "KConfigGroup::reparent", "reparenting an immutable group");
+    Q_ASSERT(parent != 0);
+
+    KConfigGroup oldGroup(*this);
+
+    d = KConfigGroupPrivate::create(parent, d->mName, false, false);
+    oldGroup.copyTo(this, pFlags);
+    oldGroup.deleteGroup(); // so that the entries with the old group name are deleted on sync
+}
