@@ -39,6 +39,8 @@ template <typename T> class KSharedPtr;
 typedef KSharedPtr<KSharedConfig> KSharedConfigPtr;
 
 /**
+ * \class KConfigGroup kconfiggroup.h <KConfigGroup>
+ * 
  * A class for one specific group in a KConfig object.
  * 
  * If you want to access the top-level entries of a KConfig
@@ -66,9 +68,6 @@ public:
      * @p group is the group name encoded in UTF-8.
      */
     KConfigGroup(KConfigBase *master, const QString &group);
-    /**
-       \overload
-    */
     KConfigGroup(KConfigBase *master, const char *group);
 
     /**
@@ -78,18 +77,9 @@ public:
      * This allows to create subgroups, by passing an existing group as @p master.
      */
     KConfigGroup(const KConfigBase *master, const QString &group);
-    /**
-       \overload
-    */
     KConfigGroup(const KConfigBase *master, const char *group);
 
-    /**
-       \overload
-    */
     KConfigGroup(const KSharedConfigPtr &master, const QString& group);
-    /**
-       \overload
-    */
     KConfigGroup(const KSharedConfigPtr &master, const char* group);
 
     KConfigGroup(const KConfigGroup &);
@@ -130,9 +120,6 @@ public:
      * Return the config object that this group belongs to.
      */
     KConfig* config();
-    /**
-       \overload
-    */
     const KConfig* config() const;
 
     /**
@@ -141,11 +128,34 @@ public:
      * groups. A subgroup can only change to another subgroup of the parent.
      */
     KDE_DEPRECATED void changeGroup( const QString &group );
-    /**
-       \overload
-    */
     KDE_DEPRECATED void changeGroup( const char *group);
 
+    /**
+     * Copies the entries in this group to another config object.
+     * @param other The other config object to copy this group's entries to. @note @p other
+     *        @em can be either another group or a different file.
+     *
+     * @since 4.1
+     */
+    void copyTo(KConfigBase *other, WriteConfigFlags pFlags = Normal) const;
+
+    /**
+     * Changes the group that this group belongs to.
+     *
+     * @param parent the config object to place this group under. If @p parent is a KConfig it will be
+     *               promoted to a top-level group.
+     *
+     * @since 4.1
+     */
+    void reparent(KConfigBase* parent, WriteConfigFlags pFlags = Normal);
+
+    /**
+     * Returns the group that this group belongs to, can be invalid if this is a top-level group.
+     *
+     * @since 4.1
+     */
+    KConfigGroup parent() const;
+
     /** 
      * @reimp
      */
@@ -185,9 +195,6 @@ public:
     template <typename T>
         inline T readEntry(const QString& key, const T& aDefault) const
             { return readCheck(key.toUtf8().constData(), aDefault); }
-    /**
-       \overload
-    */
     template <typename T>
         inline T readEntry(const char *key, const T& aDefault) const
             { return readCheck(key, aDefault); }
@@ -197,16 +204,9 @@ public:
      *
      * @param key The key to search for.
      * @param aDefault A default value returned if the key was not found.
-     * @return The value for this key.
-     *
-     * @note The return value will always have the same QVariant::Type
-     * as \a aDefault. This implies that the only possible return
-     * value for \c aDefault == \c QVariant() is \c QVariant().
+     * @return The value for this key. Can be QVariant() if aDefault is null.
      */
     QVariant readEntry(const QString& key, const QVariant &aDefault) const;
-    /**
-       \overload
-    */
     QVariant readEntry(const char* key, const QVariant &aDefault) const;
 
     /**
@@ -218,9 +218,6 @@ public:
      * @return The value for this key. Can be QString() if aDefault is null.
      */
     QString readEntry(const char* key, const QString& aDefault ) const;
-    /**
-       \overload
-    */
     QString readEntry(const QString& key, const QString& aDefault) const;
 
     /**
@@ -231,9 +228,6 @@ public:
      * @return The value for this key. Can be QString() if aDefault is null.
      */
     QString readEntry(const QString &key, const char * aDefault = 0) const;
-    /**
-       \overload
-    */
     QString readEntry(const char *key, const char *aDefault = 0 ) const;
 
     /**
@@ -247,9 +241,6 @@ public:
      *          readEntry(const char*, const QList<T>&) const
      */
     QVariantList readEntry( const QString &key, const QVariantList& aDefault ) const;
-    /**
-       \overload
-    */
     QVariantList readEntry( const char* key, const QVariantList& aDefault ) const;
 
     /**
@@ -259,9 +250,6 @@ public:
      * @return The list. Contains @p aDefault if @p key does not exist.
      */
     QStringList readEntry(const QString &key, const QStringList& aDefault) const;
-    /**
-       \overload
-    */
     QStringList readEntry(const char* key, const QStringList& aDefault) const;
 
     /**
@@ -273,9 +261,6 @@ public:
     template<typename T>
         inline QList<T> readEntry(const QString& key, const QList<T> &aDefault) const
             { return readListCheck(key.toUtf8().constData(), aDefault); }
-    /**
-       \overload
-    */
     template<typename T>
         inline QList<T> readEntry(const char* key, const QList<T> &aDefault) const
             { return readListCheck(key, aDefault); }
@@ -288,9 +273,6 @@ public:
      * @return The list. Contains @p aDefault if @p pKey does not exist.
      */
     QStringList readXdgListEntry(const QString& pKey, const QStringList& aDefault = QStringList()) const;
-    /**
-       \overload
-    */
     QStringList readXdgListEntry(const char* pKey, const QStringList& aDefault = QStringList()) const;
 
     /**
@@ -305,9 +287,6 @@ public:
      * @return The value for this key. Can be QString() if @p aDefault is null.
      */
     QString readPathEntry( const QString& pKey, const QString & aDefault ) const;
-    /**
-       \overload
-    */
     QString readPathEntry( const char *key, const QString & aDefault ) const;
 
     /**
@@ -322,9 +301,6 @@ public:
      * @return The list. Empty if the entry does not exist.
      */
     QStringList readPathEntry( const QString& pKey, const QStringList& aDefault ) const;
-    /**
-       \overload
-    */
     QStringList readPathEntry( const char *key, const QStringList& aDefault ) const;
 
     /**
@@ -337,9 +313,6 @@ public:
      */
     QString readEntryUntranslated( const QString& pKey,
                                    const QString& aDefault = QString() ) const;
-    /**
-       \overload
-    */
     QString readEntryUntranslated( const char *key,
                                    const QString& aDefault = QString() ) const;
 
@@ -352,9 +325,6 @@ public:
      */
     void writeEntry( const QString& key, const QVariant& value,
                      WriteConfigFlags pFlags = Normal );
-    /**
-       \overload
-    */
     void writeEntry( const char *key, const QVariant& value,
                      WriteConfigFlags pFlags = Normal );
 
@@ -367,9 +337,6 @@ public:
      */
     void writeEntry( const QString& key, const QString& value,
                      WriteConfigFlags pFlags = Normal );
-    /**
-       \overload
-    */
     void writeEntry( const char *key, const QString& value,
                      WriteConfigFlags pFlags = Normal );
 
@@ -382,9 +349,6 @@ public:
      */
     void writeEntry( const QString& key, const QByteArray& value,
                      WriteConfigFlags pFlags = Normal );
-    /**
-       \overload
-    */
     void writeEntry( const char *key, const QByteArray& value,
                      WriteConfigFlags pFlags = Normal );
 
@@ -396,9 +360,6 @@ public:
      * @param pFlags The flags to use when writing this entry.
      */
     void writeEntry(const QString &key, const char *value, WriteConfigFlags pFlags = Normal);
-    /**
-       \overload
-    */
     void writeEntry(const char *key, const char *value, WriteConfigFlags pFlags = Normal);
 
     /**
@@ -412,9 +373,6 @@ public:
         inline void writeEntry( const char *key, const T& value, WriteConfigFlags pFlags = Normal )
             { writeCheck( key, value, pFlags ); }
 
-    /**
-       \overload
-    */
     template <typename T>
         inline void writeEntry( const QString& key, const T& value, WriteConfigFlags pFlags = Normal )
             { writeCheck( key.toUtf8().constData(), value, pFlags ); }
@@ -431,9 +389,6 @@ public:
     void writeEntry( const QString& key, const QStringList &value,
                      WriteConfigFlags pFlags = Normal );
 
-    /**
-       \overload
-    */
     void writeEntry( const char* key, const QStringList &value,
                      WriteConfigFlags pFlags = Normal );
 
@@ -449,9 +404,6 @@ public:
     void writeEntry( const QString& key, const QVariantList &value,
                      WriteConfigFlags pFlags = Normal );
 
-    /**
-       \overload
-    */
     void writeEntry( const char* key, const QVariantList &value,
                      WriteConfigFlags pFlags = Normal );
 
@@ -466,9 +418,6 @@ public:
         inline void writeEntry(const QString& key, const QList<T> &value, WriteConfigFlags pFlags = Normal)
             { writeListCheck( key.toUtf8().constData(), value, pFlags ); }
 
-    /**
-       \overload
-    */
     template <typename T>
         inline void writeEntry(const char* key, const QList<T> &value, WriteConfigFlags pFlags = Normal)
             { writeListCheck( key, value, pFlags ); }
@@ -485,9 +434,6 @@ public:
      */
     void writeXdgListEntry( const QString& pKey, const QStringList &value,
                             WriteConfigFlags pFlags = Normal );
-    /**
-       \overload
-    */
     void writeXdgListEntry( const char *pKey, const QStringList &value,
                             WriteConfigFlags pFlags = Normal );
 
@@ -504,9 +450,6 @@ public:
      */
     void writePathEntry( const QString& pKey, const QString & path,
                          WriteConfigFlags pFlags = Normal );
-    /**
-       \overload
-    */
     void writePathEntry( const char *pKey, const QString & path,
                          WriteConfigFlags pFlags = Normal );
 
@@ -525,9 +468,6 @@ public:
      */
     void writePathEntry( const QString& pKey, const QStringList &value,
                          WriteConfigFlags pFlags = Normal );
-    /**
-       \overload
-    */
     void writePathEntry( const char *pKey, const QStringList &value,
                          WriteConfigFlags pFlags = Normal );
 
@@ -539,9 +479,6 @@ public:
      * @param pFlags       The flags to use when deleting this entry.
      */
     void deleteEntry(const QString& pKey, WriteConfigFlags pFlags = Normal);
-    /**
-       \overload
-    */
     void deleteEntry(const char *pKey, WriteConfigFlags pFlags = Normal);
 
     /**
@@ -553,9 +490,6 @@ public:
      * @return If true, the key is available.
      */
     bool hasKey(const QString &key) const;
-    /**
-       \overload
-    */
     bool hasKey(const char *key) const;
 
     /**
@@ -571,9 +505,6 @@ public:
      * in this configuration file.
      */
     bool isEntryImmutable(const QString &key) const;
-    /**
-       \overload
-    */
     bool isEntryImmutable(const char *key) const;
 
     /**
@@ -587,9 +518,6 @@ public:
      * @param key The key of the entry to revert.
      */
     void revertToDefault(const QString &key);
-    /**
-       \overload
-    */
     void revertToDefault(const char* key);
 
     /**
@@ -618,9 +546,6 @@ public:
      * @param key The key of the entry to check.
      */
     bool hasDefault(const QString &key) const;
-    /**
-       \overload
-    */
     bool hasDefault(const char* key) const;
 
     /**
