@@ -1,11 +1,12 @@
 /*
    This file is part of the KDE libraries
-   Copyright (c) 2005-2007 David Jarvie <software@astrojar.org.uk>
+   Copyright (c) 2005-2007 David Jarvie <djarvie@kde.org>
    Copyright (c) 2005 S.R.Haque <srhaque@iee.org>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +21,7 @@
 
 /** @file
  * Time zone functions
- * @author David Jarvie <software@astrojar.org.uk>.
+ * @author David Jarvie <djarvie@kde.org>.
  * @author S.R.Haque <srhaque@iee.org>.
  */
 
@@ -301,7 +302,7 @@ class KTimeZoneLeapSecondsPrivate;
  *
  * @short Represents a time zone database or collection
  * @ingroup timezones
- * @author David Jarvie <software@astrojar.org.uk>.
+ * @author David Jarvie <djarvie@kde.org>.
  * @author S.R.Haque <srhaque@iee.org>.
  */
 class KDECORE_EXPORT KTimeZones
@@ -408,7 +409,7 @@ private:
  * @short Base class representing a time zone
  * @see KTimeZoneBackend, KTimeZoneSource, KTimeZoneData
  * @ingroup timezones
- * @author David Jarvie <software@astrojar.org.uk>.
+ * @author David Jarvie <djarvie@kde.org>.
  * @author S.R.Haque <srhaque@iee.org>.
  */
 class KDECORE_EXPORT KTimeZone  //krazy:exclude=dpointer (has non-const d-pointer to Backend class)
@@ -422,7 +423,7 @@ public:
      * UTC offset and time zone abbreviation (e.g. EST, GMT).
      *
      * @short Time zone phase
-     * @author David Jarvie <software@astrojar.org.uk>.
+     * @author David Jarvie <djarvie@kde.org>.
      */
     class KDECORE_EXPORT Phase
     {
@@ -502,7 +503,7 @@ public:
      * standard time, including the UTC time of the change.
      *
      * @short Time zone transition
-     * @author David Jarvie <software@astrojar.org.uk>.
+     * @author David Jarvie <djarvie@kde.org>.
      */
     class KDECORE_EXPORT Transition
     {
@@ -549,7 +550,7 @@ public:
      * @short Leap seconds adjustment for a time zone
      * @see KTimeZone, KTimeZoneData
      * @ingroup timezones
-     * @author David Jarvie <software@astrojar.org.uk>.
+     * @author David Jarvie <djarvie@kde.org>.
      */
     class KDECORE_EXPORT LeapSeconds
     {
@@ -1109,7 +1110,7 @@ private:
  * @short Base backend class for KTimeZone classes
  * @see KTimeZone
  * @ingroup timezones
- * @author David Jarvie <software@astrojar.org.uk>.
+ * @author David Jarvie <djarvie@kde.org>.
  */
 class KDECORE_EXPORT KTimeZoneBackend  //krazy:exclude=dpointer (non-const d-pointer for KTimeZoneBackend-derived classes)
 {
@@ -1216,7 +1217,7 @@ private:
  * @short Base class representing a source of time zone information
  * @see KTimeZone, KTimeZoneData
  * @ingroup timezones
- * @author David Jarvie <software@astrojar.org.uk>.
+ * @author David Jarvie <djarvie@kde.org>.
  * @author S.R.Haque <srhaque@iee.org>.
  */
 class KDECORE_EXPORT KTimeZoneSource
@@ -1227,9 +1228,13 @@ public:
 
     /**
      * Extracts detail information for one time zone from the source database.
+     *
      * In this base class, the method always succeeds and returns an empty data
      * instance. Derived classes should reimplement this method to return an
-     * appropriate data class derived from KTimeZoneData.
+     * appropriate data class derived from KTimeZoneData. Some source databases
+     * may not be compatible with this method of parsing. In these cases, they
+     * should use the constructor KTimeZoneSource(false) and calling this method
+     * will produce a fatal error.
      *
      * @param zone the time zone for which data is to be extracted.
      * @return an instance of a class derived from KTimeZoneData containing
@@ -1239,6 +1244,35 @@ public:
      */
     virtual KTimeZoneData *parse(const KTimeZone &zone) const;
 
+    /**
+     * Return whether the source database supports the ad hoc extraction of data for
+     * individual time zones using parse(const KTimeZone&).
+     *
+     * @return true if parse(const KTimeZone&) works, false if parsing must be
+     *         performed by other methods
+     */
+    bool useZoneParse() const;
+
+protected:
+    /**
+     * Constructor for use by derived classes, which specifies whether the
+     * source database supports the ad hoc extraction of data for individual
+     * time zones using parse(const KTimeZone&).
+     *
+     * If parse(const KTimeZone&) cannot be used, KTimeZone derived classes
+     * which use this KTimeZoneSource derived class must create a
+     * KTimeZoneData object at construction time so that KTimeZone::data()
+     * will always return a data object.
+     *
+     * Note the default constructor is equivalent to KTimeZoneSource(true), i.e.
+     * it is only necessary to use this constructor if parse(const KTimeZone&)
+     * does not work.
+     *
+     * @param useZoneParse true if parse(const KTimeZone&) works, false if
+     *                     parsing must be performed by other methods
+     */
+    explicit KTimeZoneSource(bool useZoneParse);
+
 private:
     KTimeZoneSourcePrivate * const d;
 };
@@ -1256,7 +1290,7 @@ private:
  * @short Base class for parsed time zone data
  * @see KTimeZone, KTimeZoneSource
  * @ingroup timezones
- * @author David Jarvie <software@astrojar.org.uk>.
+ * @author David Jarvie <djarvie@kde.org>.
  */
 class KDECORE_EXPORT KTimeZoneData
 {
