@@ -118,6 +118,43 @@ static QString cleanpath( const QString &_path, bool cleanDirSeparator, bool dec
   return result;
 }
 
+#ifdef Q_WS_WIN
+
+// returns true if provided arguments desinate letter+colon or double slash
+#define IS_DRIVE_OR_DOUBLESLASH(isletter, char1, char2, colon, slash) \
+  ((isletter && char2 == colon) || (char1 == slash && char2 == slash))
+
+// Removes file:/// or file:// or file:/ or / prefix assuming that str 
+// is (nonempty) Windows absolute path with a drive letter or double slash.
+// If there was file protocol, the path is decoded from percent encoding
+static QString removeSlashOrFilePrefix(const QString& str)
+{
+  // FIXME this should maybe be replaced with some (faster?)/nicer logic
+  const int len = str.length();
+  if (str[0]=='f') {
+    if ( len > 10 && str.startsWith( QLatin1String( "file:///" ) ) 
+         && IS_DRIVE_OR_DOUBLESLASH(str[8].isLetter(), str[8], str[9], QLatin1Char(':'), QLatin1Char('/')) )
+      return QUrl::fromPercentEncoding( str.toLatin1() ).mid(8);
+    else if ( len > 9 && str.startsWith( QLatin1String( "file://" ) )
+              && IS_DRIVE_OR_DOUBLESLASH(str[7].isLetter(), str[7], str[8], QLatin1Char(':'), QLatin1Char('/')) )
+      return QUrl::fromPercentEncoding( str.toLatin1() ).mid(7);
+    else if ( len > 8 && str.startsWith( QLatin1String( "file:/" ) )
+              && IS_DRIVE_OR_DOUBLESLASH(str[6].isLetter(), str[6], str[7], QLatin1Char(':'), QLatin1Char('/')) )
+      return QUrl::fromPercentEncoding( str.toLatin1() ).mid(6);
+  }
+  /* No 'else' here since there can be "f:/" path. */
+
+  /* '/' + drive letter or // */
+  if ( len > 2 && str[0] == QLatin1Char('/')
+       && IS_DRIVE_OR_DOUBLESLASH(str[1].isLetter(), str[1], str[2], QLatin1Char(':'), QLatin1Char('/')) )
+    return str.mid(1);
+  /* drive letter or // */
+  else if ( len >= 2 && IS_DRIVE_OR_DOUBLESLASH(str[0].isLetter(), str[0], str[1], QLatin1Char(':'), QLatin1Char('/')) )
+    return str;
+  return QString();
+}
+#endif
+
 bool KUrl::isRelativeUrl(const QString &_url)
 {
 #if 0
@@ -204,8 +241,13 @@ void KUrl::List::populateMimeData( QMimeData* mimeData,
     if ( ( flags & KUrl::NoTextExport ) == 0 )
     {
         QStringList prettyURLsList;
-        for ( uit = begin(); uit != uEnd ; ++uit )
-            prettyURLsList.append( (*uit).prettyUrl() );
+        for ( uit = begin(); uit != uEnd ; ++uit ) {
+            QString prettyURL = (*uit).prettyUrl();
+            if ( (*uit).protocol() == "mailto" ) {
+                prettyURL = (*uit).path(); // remove mailto: when pasting into konsole
+            }
+            prettyURLsList.append( prettyURL );
+        }
 
         QByteArray plainTextData = prettyURLsList.join( "\n" ).toLocal8Bit();
         if( count() > 1 ) // terminate last line, unless it's the only line
@@ -295,7 +337,7 @@ KUrl::List::operator QVariant() const
   return qVariantFromValue(*this);
 }
 
-////
+///
 
 KUrl::KUrl()
     : QUrl(), d(0)
@@ -312,17 +354,18 @@ KUrl::KUrl( const QString &str )
 {
   if ( !str.isEmpty() ) {
 #ifdef Q_WS_WIN
-    int len = str.length();
-    QString pathToSet;
     kDebug(126) << "KUrl::KUrl ( const QString &str = " << str.toAscii().data() << " )";
-    if ( len > 9 && str.startsWith( QLatin1String( "file://" ) ) && str[7].isLetter() && str[8] == QLatin1Char(':') )
-      pathToSet = str.mid(7);
-    else if ( len > 2 && str[0] == QLatin1Char('/') && str[1].isLetter() && str[2] == QLatin1Char(':') )
-      pathToSet = str.mid(1);
-    else if ( len >= 2 && str[0].isLetter() && str[1] == QLatin1Char(':') )
-      pathToSet = str;
+    QString pathToSet( removeSlashOrFilePrefix( QDir::fromNativeSeparators(str) ) );
     if ( !pathToSet.isEmpty() ) {
-      setPath( QDir::fromNativeSeparators( pathToSet ) );
+      // we have a prefix indicating this is a local URL
+      // remember the possible query using _setEncodedUrl(), then set up the correct path without query protocol part
+      int index = pathToSet.lastIndexOf('?');
+      if (index == -1)
+        setPath( QDir::fromNativeSeparators( pathToSet ) );
+      else {
+        setPath( QDir::fromNativeSeparators( pathToSet.left( index ) ) );
+        _setQuery( pathToSet.mid( index + 1 ) );
+      }
     }
 #else
     if ( str[0] == QLatin1Char('/') || str[0] == QLatin1Char('~') )
@@ -337,15 +380,29 @@ KUrl::KUrl( const QString &str )
 KUrl::KUrl( const char * str )
   : QUrl(), d(0)
 {
-  if ( str && str[0] ) {
 #ifdef Q_WS_WIN
-    kDebug(126) << "KUrl::KUrl " << " " << str;
+  // true if @a c is letter
+  #define IS_LETTER(c) \
+    ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'))
 
-    if ( str[0] == '/' && (str[1] >= 'A' && str[1] <= 'Z' || str[1] >= 'a' && str[1] <= 'z') && str[2] == ':' )
-    	setPath( QString::fromUtf8( str+1 ) );
-    else if ( (str[0] >= 'A' && str[0] <= 'Z' || str[0] >= 'a' && str[0] <= 'z')  &&  str[1] == ':' )
+  // like IS_DRIVE_OR_DOUBLESLASH, but slash is prepended
+  #define IS_SLASH_AND_DRIVE_OR_DOUBLESLASH_0 \
+    ( str[0] == '/' && IS_DRIVE_OR_DOUBLESLASH(IS_LETTER(str[1]), str[1], str[2], ':', '/') )
+
+  // like IS_DRIVE_OR_DOUBLESLASH, with characters == str[0] and str[1]
+  #define IS_DRIVE_OR_DOUBLESLASH_0 \
+    ( IS_DRIVE_OR_DOUBLESLASH(IS_LETTER(str[0]), str[0], str[1], ':', '/') )
+
+#if defined(DEBUG_KURL)
+  kDebug(126) << "KUrl::KUrl " << " " << str;
+#endif
+  if ( str && str[0] && str[1] && str[2] ) {
+    if ( IS_SLASH_AND_DRIVE_OR_DOUBLESLASH_0 )
+      setPath( QString::fromUtf8( str+1 ) );
+    else if ( IS_DRIVE_OR_DOUBLESLASH_0 )
       setPath( QString::fromUtf8( str ) );
 #else
+  if ( str && str[0] ) {
     if ( str[0] == '/' || str[0] == '~' )
       setPath( QString::fromUtf8( str ) );
 #endif
@@ -359,11 +416,13 @@ KUrl::KUrl( const QByteArray& str )
 {
   if ( !str.isEmpty() ) {
 #ifdef Q_WS_WIN
+#ifdef DEBUG_KURL
     kDebug(126) << "KUrl::KUrl " << " " << str.data();
-    if ( str[0] == '/' && (str[1] >= 'A' && str[1] <= 'Z' || str[1] >= 'a' && str[1] <= 'z') && str[2] == ':' )
-    	  setPath( QString::fromUtf8( str.mid( 1 ) ) );
-    else if ( (str[0] >= 'A' && str[0] <= 'Z' || str[0] >= 'a' && str[0] <= 'z')  &&  str[1] == ':' )
-        setPath( QString::fromUtf8( str ) );
+#endif
+    if ( IS_SLASH_AND_DRIVE_OR_DOUBLESLASH_0 )
+      setPath( QString::fromUtf8( str.mid( 1 ) ) );
+    else if ( IS_DRIVE_OR_DOUBLESLASH_0 )
+      setPath( QString::fromUtf8( str ) );
 #else
     if ( str[0] == '/' || str[0] == '~' )
       setPath( QString::fromUtf8( str ) );
@@ -376,7 +435,7 @@ KUrl::KUrl( const QByteArray& str )
 KUrl::KUrl( const KUrl& _u )
     : QUrl( _u ), d(0)
 {
-#ifdef Q_WS_WIN
+#if defined(Q_WS_WIN) && defined(DEBUG_KURL)
     kDebug(126) << "KUrl::KUrl(KUrl) " << " path " << _u.path() << " toLocalFile " << _u.toLocalFile();
 #endif
 }
@@ -384,7 +443,7 @@ KUrl::KUrl( const KUrl& _u )
 KUrl::KUrl( const QUrl &u )
     : QUrl( u ), d(0)
 {
-#ifdef Q_WS_WIN
+#if defined(Q_WS_WIN) && defined(DEBUG_KURL)
     kDebug(126) << "KUrl::KUrl(Qurl) " << " path " << u.path() << " toLocalFile " << u.toLocalFile();
 #endif
 }
@@ -392,7 +451,7 @@ KUrl::KUrl( const QUrl &u )
 KUrl::KUrl( const KUrl& _u, const QString& _rel_url )
    : QUrl(), d(0)
 {
-#ifdef Q_WS_WIN
+#if defined(Q_WS_WIN) && defined(DEBUG_KURL)
     kDebug(126) << "KUrl::KUrl(KUrl,QString rel_url) " << " path " << _u.path() << " toLocalFile " << _u.toLocalFile();
 #endif
 #if 0
@@ -521,6 +580,15 @@ bool KUrl::equals( const KUrl &_u, const EqualsOptions& options ) const
   {
     QString path1 = path((options & CompareWithoutTrailingSlash) ? RemoveTrailingSlash : LeaveTrailingSlash);
     QString path2 = _u.path((options & CompareWithoutTrailingSlash) ? RemoveTrailingSlash : LeaveTrailingSlash);
+#ifdef Q_WS_WIN
+    const bool bLocal1 = isLocalFile();
+    const bool bLocal2 = _u.isLocalFile();
+    if ( !bLocal1 && bLocal2 || bLocal1 && !bLocal2 )
+      return false;
+    // local files are case insensitive
+    if ( bLocal1 && bLocal2 && 0 != QString::compare( path1, path2, Qt::CaseInsensitive ) )
+      return false;
+#endif
     if ( path1 != path2 )
       return false;
 
@@ -604,7 +672,11 @@ void KUrl::setFileName( const QString& _txt )
   //QString path = m_strPath_encoded.isEmpty() ? m_strPath : m_strPath_encoded;
   QString path = this->path();
   if ( path.isEmpty() )
+#ifdef Q_OS_WIN
+    path = isLocalFile() ? QDir::rootPath() : QLatin1String("/");
+#else
     path = QDir::rootPath();
+#endif
   else
   {
     int lastSlash = path.lastIndexOf( QLatin1Char('/') );
@@ -771,12 +843,14 @@ QString KUrl::toLocalFile( AdjustPathOption trailing ) const
   return trailingSlash( trailing, QUrl::toLocalFile() );
 }
 
-bool KUrl::isLocalFile() const
+inline static bool hasSubUrl( const QUrl& url );
+
+static inline bool isLocalFile( const QUrl& url )
 {
-  if ( ( scheme() != QLatin1String("file") ) || hasSubUrl() )
+  if ( ( url.scheme() != QLatin1String("file") ) || hasSubUrl( url ) )
      return false;
 
-  if (host().isEmpty() || (host() == QLatin1String("localhost")))
+  if (url.host().isEmpty() || (url.host() == QLatin1String("localhost")))
      return true;
 
   char hostname[ 256 ];
@@ -787,7 +861,12 @@ bool KUrl::isLocalFile() const
   for(char *p = hostname; *p; p++)
      *p = tolower(*p);
 
-  return (host() == QString::fromLatin1( hostname ));
+  return (url.host() == QString::fromLatin1( hostname ));
+}
+
+bool KUrl::isLocalFile() const
+{
+  return ::isLocalFile( *this );
 }
 
 void KUrl::setFileEncoding(const QString &encoding)
@@ -832,7 +911,7 @@ QString KUrl::fileEncoding() const
   if (q[0] == '?')
      q = q.mid(1);
 
-  QStringList args = q.split('&', QString::SkipEmptyParts);
+  const QStringList args = q.split('&', QString::SkipEmptyParts);
   for(QStringList::ConstIterator it = args.begin();
       it != args.end();
       ++it)
@@ -844,32 +923,49 @@ QString KUrl::fileEncoding() const
   return QString();
 }
 
-bool KUrl::hasSubUrl() const
+inline static bool hasSubUrl( const QUrl& url )
 {
   // The isValid call triggers QUrlPrivate::validate which needs the full encoded url,
   // all this takes too much time for isLocalFile()
-  if ( scheme().isEmpty() /*|| !isValid()*/ )
+  if ( url.scheme().isEmpty() /*|| !isValid()*/ )
     return false;
-  const QString ref = fragment();
+  const QByteArray ref( url.fragment().toLatin1() );
   if (ref.isEmpty())
      return false;
-  if (ref.startsWith("gzip:"))
-     return true;
-  if (ref.startsWith("bzip:"))
-     return true;
-  if (ref.startsWith("bzip2:"))
-     return true;
-  if (ref.startsWith("tar:"))
-     return true;
-  if (ref.startsWith("ar:"))
-     return true;
-  if (ref.startsWith("zip:"))
-     return true;
-  if ( scheme() == "error" ) // anything that starts with error: has suburls
+  switch ( ref.data()[0] ) {
+  case 'g':
+    if ( ref.startsWith("gzip:") )
+      return true;
+    break;
+  case 'b':
+    if ( ref.startsWith("bzip:") || ref.startsWith("bzip2:") )
+      return true;
+    break;
+  case 't':
+    if ( ref.startsWith("tar:") )
+      return true;
+    break;
+  case 'a':
+    if ( ref.startsWith("ar:") )
+      return true;
+    break;
+  case 'z':
+    if ( ref.startsWith("zip:") )
+      return true;
+    break;
+  default:
+    break;
+  }
+  if ( url.scheme() == "error" ) // anything that starts with error: has suburls
      return true;
   return false;
 }
 
+bool KUrl::hasSubUrl() const
+{
+  return ::hasSubUrl( *this );
+}
+
 QString KUrl::url( AdjustPathOption trailing ) const
 {
   if ( trailing == AddTrailingSlash && !path().endsWith( QLatin1Char('/') ) ) {
@@ -984,6 +1080,7 @@ QString KUrl::pathOrUrl() const
   }
 }
 
+// Used for text/uri-list in the mime data
 QString KUrl::toMimeDataString() const // don't fold this into populateMimeData, it's also needed by other code like konqdrag
 {
   if ( isLocalFile() )
@@ -1010,10 +1107,6 @@ QString KUrl::toMimeDataString() const // don't fold this into populateMimeData,
 #endif
   }
 
-  if ( protocol() == "mailto" ) {
-      return path();
-  }
-
   return url(/*0 , 106*/); // 106 is mib enum for utf8 codec
 }
 
@@ -1438,6 +1531,10 @@ bool urlcmp( const QString& _url1, const QString& _url2, const KUrl::EqualsOptio
         options |= QUrl::StripTrailingSlash;
     if ( _options & KUrl::CompareWithoutFragment )
         options |= QUrl::RemoveFragment;
+#ifdef Q_WS_WIN
+    if ( ::isLocalFile( u1 ) && ::isLocalFile( u2 ) )
+      return 0 == QString::compare( u1.toString( options ), u2.toString( options ), Qt::CaseInsensitive );
+#endif
     return u1.toString( options ) == u2.toString( options );
 
 #if 0
@@ -1500,8 +1597,8 @@ static QString _relativePath(const QString &base_dir, const QString &path, bool
    if (_base_dir[_base_dir.length()-1] != QLatin1Char('/'))
       _base_dir.append(QLatin1Char('/') );
 
-   QStringList list1 = _base_dir.split(QLatin1Char('/'), QString::SkipEmptyParts);
-   QStringList list2 = _path.split(QLatin1Char('/'), QString::SkipEmptyParts);
+   const QStringList list1 = _base_dir.split(QLatin1Char('/'), QString::SkipEmptyParts);
+   const QStringList list2 = _path.split(QLatin1Char('/'), QString::SkipEmptyParts);
 
    // Find where they meet
    int level = 0;
@@ -1578,8 +1675,15 @@ void KUrl::setPath( const QString& _path )
     kDebug(126) << "KUrl::setPath " << " " << _path.toAscii().data();
 #endif
     if ( scheme().isEmpty() )
-        setScheme( "file" );
+        setScheme( QLatin1String( "file" ) );
     QString path = KShell::tildeExpand( _path );
+#ifdef Q_WS_WIN
+    //This is necessary because QUrl has the "path" part including the first slash
+    //Without this QUrl doesn't understand that this is a path, and some operations fail
+    //e.g. C:/blah needs to become /C:/blah
+    if( !path.isEmpty() && path[0] != QLatin1Char('/') && scheme() == QLatin1String( "file" ) )
+        path = QLatin1Char('/') + path;
+#endif
     QUrl::setPath( path );
 }
 
