@@ -1,5 +1,6 @@
 /*
    This file is part of the KDE libraries
+   Copyright (C) 2004 Jaroslaw Staniek <js@iidea.pl>
    Copyright (C) 2007 Christian Ehrlicher <ch.ehrlicher@gmx.de>
    Copyright (C) 2007 Bernhard Loos <nhuh.put@web.de>
 
@@ -22,9 +23,12 @@
 
 #include <config.h>
 #include <QtCore/QBool>
+#include <QtCore/QTextCodec>
 
 #ifdef Q_OS_WIN
-#include <kdebug.h>
+
+#include "kglobal.h"
+#include <klocale.h>
 
 #include <QtCore/QDir>
 #include <QtCore/QString>
@@ -41,58 +45,79 @@
 
 static void kMessageOutput(QtMsgType type, const char *msg);
 
-static class kGlobalClass {
-    public: 
-        kGlobalClass() {
+static class kMessageOutputInstaller {
+    public:
+        kMessageOutputInstaller() {
             qInstallMsgHandler(kMessageOutput);
         }
-
-        QString kde4prefix;
-} myGlobals; 
+} kMessageOutputInstallerInstance;
 
 static HINSTANCE kdecoreDllInstance = NULL;
+static wchar_t kde4prefixUtf16[MAX_PATH + 2];
+static QString *kde4Prefix = NULL;
 
-/**
- * The dll entry point - get the instance handle for GetModuleFleNameW
- * Maybe also some special initialization / cleanup can be done here
- **/
-BOOL WINAPI DllMain ( HINSTANCE hinstDLL,DWORD fdwReason,LPVOID )
+void initKde4prefixUtf16()
 {
-    switch ( fdwReason ) {
-    case DLL_PROCESS_ATTACH:
-        kdecoreDllInstance = hinstDLL;
-        break;
-    default:
-        break;
+    //the path is C:\some\path\kde4\bin\kdecore.dll
+    GetModuleFileNameW(kdecoreDllInstance, kde4prefixUtf16, MAX_PATH + 1);
+    int bs1 = 0, bs2 = 0;
+
+    //we convert \ to / and remove \bin\kdecore.dll from the string
+    int pos;
+    for (pos = 0; pos < MAX_PATH + 1 && kde4prefixUtf16[pos] != 0; ++pos) {
+        if (kde4prefixUtf16[pos] == '\\') {
+            bs1 = bs2;
+            bs2 = pos;
+            kde4prefixUtf16[pos] = '/';
+        }
     }
-    return true;
+    Q_ASSERT(bs1);
+    Q_ASSERT(pos < MAX_PATH + 1);
+    kde4prefixUtf16[bs1] = '/';
+    kde4prefixUtf16[bs1+1] = 0;
 }
 
 // can't use QCoreApplication::applicationDirPath() because sometimes we
 // don't have an instantiated QCoreApplication
 QString getKde4Prefix()
 {
-    if (myGlobals.kde4prefix.isEmpty()) {
-        wchar_t kde4prefix[MAX_PATH + 1];
-        //the path is C:\some\path\kde4\bin\kdecore.dll
-        GetModuleFileNameW(kdecoreDllInstance, kde4prefix, MAX_PATH + 1);
-        int bs1 = 0, bs2 = 0;
+  // we can get called after DLL_PROCESS_DETACH!
+  return kde4Prefix ? *kde4Prefix : QString::fromUtf16((ushort*) kde4prefixUtf16);
+}
 
-        //we convert \ to / and remove \bin\kdecore.dll from the string
-        int pos;
-        for (pos = 0; pos < MAX_PATH + 1 && kde4prefix[pos] != 0; ++pos) {
-            if (kde4prefix[pos] == '\\') {
-                bs1 = bs2;
-                bs2 = pos;
-                kde4prefix[pos] = '/';
-            }
-        }
-        Q_ASSERT(bs1);
-        Q_ASSERT(pos < MAX_PATH + 1);
-        kde4prefix[bs1] = 0;
-        myGlobals.kde4prefix = QString::fromUtf16((ushort *) kde4prefix);
+/**
+ * The dll entry point - get the instance handle for GetModuleFleNameW
+ * Maybe also some special initialization / cleanup can be done here
+ **/
+extern "C"
+BOOL WINAPI DllMain ( HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpReserved)
+{
+    switch ( fdwReason ) {
+    case DLL_PROCESS_ATTACH:
+        kdecoreDllInstance = hinstDLL;
+        initKde4prefixUtf16();
+        kde4Prefix = new QString( QString::fromUtf16((ushort*) kde4prefixUtf16) );
+        break;
+    case DLL_PROCESS_DETACH:
+        /* msdn:
+           When handling DLL_PROCESS_DETACH, a DLL should free resources such
+           as heap memory only if the DLL is being unloaded dynamically (the
+           lpReserved parameter is NULL). If the process is terminating (the
+           lpvReserved parameter is non-NULL), all threads in the process except
+           the current thread either have exited already or have been explicitly
+           terminated by a call to the ExitProcess function, which might leave
+           some process resources such as heaps in an inconsistent state. In this
+           case, it is not safe for the DLL to clean up the resources. Instead,
+           the DLL should allow the operating system to reclaim the memory.
+         */
+        if( lpReserved == NULL )
+            delete kde4Prefix;
+        kde4Prefix = 0;
+        break;
+    default:
+        break;
     }
-    return myGlobals.kde4prefix;
+    return true;
 }
 
 /**
@@ -153,6 +178,8 @@ bool showWin32FilePropertyDialog ( const QString& fileName )
     return ShellExecuteExW ( &execInfo );
 }
 
+// note: QLocale().name().left(2).toLatin1() returns the same
+
 QByteArray getWin32LocaleName()
 {
     bool ok;
