@@ -126,7 +126,7 @@ void KMimeType::checkEssentialMimeTypes()
     // Note that this messagebox is queued, so it will only be shown once getting back to the event loop
 
     // No mimetypes installed? Are you setting XDG_DATA_DIRS without including /usr/share in it?
-    KMessage::message( KMessage::Error, i18n( "No mime types installed. Check that shared-mime-info is installed, and that XDG_DATA_DIRS is not set, or includes /usr/share" ) );
+    KMessage::message( KMessage::Error, i18n( "No mime types installed. Check that shared-mime-info is installed, and that XDG_DATA_DIRS is not set, or includes /usr/share." ) );
     return; // no point in going any further
   }
 
@@ -322,7 +322,7 @@ KMimeType::Ptr KMimeType::findByUrlHelper( const KUrl& _url, mode_t mode,
                 // "for glob_match in glob_matches:"
                 // "if glob_match is subclass or equal to sniffed_type, use glob_match"
                 const QString sniffedMime = mime->name();
-                foreach(KMimeType::Ptr mimeFromPattern, mimeList) {
+                foreach(const KMimeType::Ptr &mimeFromPattern, mimeList) {
                     //kDebug(7009) << "sniffedMime=" << sniffedMime << "mimeFromPattern=" << mimeFromPattern->name();
                     if (mimeFromPattern->is(sniffedMime)) {
                         // We have magic + pattern pointing to this, so it's a pretty good match
@@ -600,22 +600,16 @@ QString KMimeTypePrivate::fallbackParent() const
 bool KMimeTypePrivate::inherits(KMimeType::Ptr mime) const
 {
     if (mime && m_strName == mime->d_func()->m_strName) {
-      return true;
+        return true;
     }
-    QStringList parents = m_parentMimeTypes;
-    if (parents.isEmpty()) {
-        const QString myParent = fallbackParent();
-        if (!myParent.isEmpty())
-            parents.append(myParent);
-    }
-    foreach( const QString& parent, parents ) {
-        KMimeType::Ptr parentMime = KMimeType::mimeType(parent);
+    foreach( const QString& parent, parentMimeTypes() ) {
+        KMimeType::Ptr parentMime = KMimeTypeFactory::self()->findMimeTypeByName(parent);
         if (!parentMime) // error
             return false;
         if (parentMime->d_func()->inherits(mime)) // recurse
-          return true;
-  }
-  return false;
+            return true;
+    }
+    return false;
 }
 
 bool KMimeType::is( const QString& mimeTypeName ) const
@@ -627,6 +621,49 @@ bool KMimeType::is( const QString& mimeTypeName ) const
     return d->inherits(mime);
 }
 
+QStringList KMimeType::parentMimeTypes() const
+{
+    Q_D(const KMimeType);
+    return d->parentMimeTypes();
+}
+
+QStringList KMimeTypePrivate::parentMimeTypes() const
+{
+    QStringList parents = m_parentMimeTypes;
+    if (parents.isEmpty()) {
+        const QString myParent = fallbackParent();
+        if (!myParent.isEmpty())
+            parents.append(myParent);
+    }
+    return parents;
+}
+
+void KMimeTypePrivate::collectParentMimeTypes(QStringList& allParents) const
+{
+    QStringList parents = parentMimeTypes();
+    Q_FOREACH(const QString& parent, parents) {
+        // I would use QSet, but since order matters I better not
+        if (!allParents.contains(parent))
+            allParents.append(parent);
+    }
+    // We want a breadth-first search, so that the least-specific parent (octet-stream) is last
+    // This means iterating twice, unfortunately.
+    Q_FOREACH(const QString& parent, parents) {
+        KMimeType::Ptr parentMime = KMimeTypeFactory::self()->findMimeTypeByName(parent);
+        if (parentMime)
+            parentMime->d_func()->collectParentMimeTypes(allParents);
+    }
+}
+
+QStringList KMimeType::allParentMimeTypes() const
+{
+    QStringList allParents;
+    const QString canonical = KMimeTypeFactory::self()->resolveAlias(name());
+    if (!canonical.isEmpty())
+        allParents.append(canonical);
+    d_func()->collectParentMimeTypes(allParents);
+    return allParents;
+}
 
 QString KMimeType::defaultMimeType()
 {
