@@ -244,6 +244,15 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     m_bAllowAsDefault = desktopGroup.readEntry("AllowDefault", true);
     entryMap.remove("AllowDefault");
 
+    // allow plugin users to translate categories without needing a separate key
+    QMap<QString,QString>::Iterator entry = entryMap.find("X-KDE-PluginInfo-Category");
+    if (entry != entryMap.end()) {
+        const QString& key = entry.key();
+        m_mapProps.insert(key, QVariant(desktopGroup.readEntryUntranslated(key)));
+        m_mapProps.insert(key + "-Translated", QVariant(*entry));
+        entryMap.erase(entry);
+    }
+
     // Store all additional entries in the property map.
     // A QMap<QString,QString> would be easier for this but we can't
     // break BC, so we have to store it in m_mapProps.
@@ -619,7 +628,7 @@ QString KService::username() const {
     QVariant v = property("X-KDE-Username", QVariant::String);
     user = v.isValid() ? v.toString() : QString();
     if (user.isEmpty())
-        user = ::getenv("ADMIN_ACCOUNT");
+        user = QString::fromLocal8Bit(qgetenv("ADMIN_ACCOUNT"));
     if (user.isEmpty())
         user = "root";
     return user;
@@ -728,10 +737,9 @@ QString KService::locateLocal() const
 QString KService::newServicePath(bool showInMenu, const QString &suggestedName,
                                  QString *menuId, const QStringList *reservedMenuIds)
 {
-    QString base = suggestedName;
-    if (!showInMenu)
-        base.prepend("kde4-");
+    Q_UNUSED(showInMenu); // TODO KDE5: remove argument
 
+    QString base = suggestedName;
     QString result;
     for(int i = 1; true; i++)
     {
@@ -748,30 +756,15 @@ QString KService::newServicePath(bool showInMenu, const QString &suggestedName,
         if (s)
             continue;
 
-        if (showInMenu)
-        {
-            if (!KStandardDirs::locate("xdgdata-apps", result).isEmpty())
-                continue;
-        }
-        else
-        {
-            QString file = result.mid(5); // Strip "kde4-"
-            if (!KStandardDirs::locate("apps", ".hidden/"+file).isEmpty())
-                continue;
-        }
+        if (!KStandardDirs::locate("xdgdata-apps", result).isEmpty())
+            continue;
 
         break;
     }
     if (menuId)
         *menuId = result;
 
-    if (showInMenu)
-    {
-        return KStandardDirs::locateLocal("xdgdata-apps", result);
-    }
-
-    QString file = result.mid(5); // Strip "kde4-"
-    return KStandardDirs::locateLocal("apps", ".hidden/"+file);
+    return KStandardDirs::locateLocal("xdgdata-apps", result);
 }
 
 bool KService::isApplication() const
