@@ -128,7 +128,7 @@ KPluginLoader::KPluginLoader(const KService &service, const KComponentData &comp
     Q_ASSERT(service.isValid());
 
     if (service.isApplication()) {
-        d->errorString = i18n("The service '%1' provides no library or the Library key is missing in the .desktop file", service.name());
+        d->errorString = i18n("The service '%1' provides no library or the Library key is missing in the .desktop file.", service.name());
         return;
     }
     load();
@@ -146,11 +146,9 @@ KPluginFactory *KPluginLoader::factory()
     if (!isLoaded())
         return 0;
 
-    KPluginFactory *factory;
 
     if (d->lib) {
-        factory = d->lib->factory(d->name.toUtf8());
-        return factory;
+        return d->lib->factory(d->name.toUtf8());
     }
 
     QObject *obj = instance();
@@ -158,7 +156,7 @@ KPluginFactory *KPluginLoader::factory()
     if (!obj)
         return 0;
 
-    factory = qobject_cast<KPluginFactory *>(obj);
+    KPluginFactory *factory = qobject_cast<KPluginFactory *>(obj);
 
     if (factory == 0) {
         delete obj;
@@ -186,7 +184,7 @@ bool KPluginLoader::load()
     d->verificationData = (KDEPluginVerificationData *) lib.resolve("kde_plugin_verification_data");
     if (d->verificationData) {
         if ((d->verificationData->KDEVersion > KDE_VERSION) || (KDE_VERSION_MAJOR << 16 != (d->verificationData->KDEVersion & 0xFF0000))) {
-            d->errorString = i18n("The plugin '%1' uses incompatible KDE library. (%2)", d->name, d->verificationData->KDEVersionString);
+            d->errorString = i18n("The plugin '%1' uses an incompatible KDE library (%2).", d->name, d->verificationData->KDEVersionString);
             lib.unload();
             unload();
             return false;
