@@ -18,8 +18,8 @@
    Boston, MA 02110-1301, USA.
 */
 
-#ifndef K_BUILD_SERVICE_FACTORY_H
-#define K_BUILD_SERVICE_FACTORY_H
+#ifndef KDED_KBUILD_SERVICE_FACTORY_H
+#define KDED_KBUILD_SERVICE_FACTORY_H
 
 #include <QtCore/QStringList>
 
@@ -86,6 +86,7 @@ public:
 
 private:
     void saveOfferList(QDataStream &str);
+    void collectInheritedServices();
 
     QHash<QString, KService::Ptr> m_nameMemoryHash; // m_nameDict is not useable while building ksycoca
     QHash<QString, KService::Ptr> m_relNameMemoryHash; // m_relNameDict is not useable while building ksycoca
