@@ -15,7 +15,7 @@ namespace KDESu {
 
 const int defTimeout = 120*60;
 const int defEchoMode = 0;
-const int defKeep = false;
+const bool defKeep = false;
 
 }
 
