@@ -41,13 +41,17 @@ class QString;
 /**
  * \short A container for a set of QAction objects.
  *
- * KActionCollection acts as the owning QObject for a set of QAction objects.  It
+ * KActionCollection manages a set of QAction objects.  It
  * allows them to be grouped for organized presentation of configuration to the user,
  * saving + loading of configuration, and optionally for automatic plugging into
  * specified widget(s).
  *
  * Additionally, KActionCollection provides several convenience functions for locating
  * named actions, and actions grouped by QActionGroup.
+ *
+ * \note If you create your own action collection and need to assign shortcuts
+ * to the actions within, you have to call associateWidget() or
+ * addAssociatedWidget() to have them working.
  */
 class KDEUI_EXPORT KActionCollection : public QObject
 {
@@ -147,11 +151,35 @@ public:
   void readSettings( KConfigGroup* config = 0 );
 
   /**
-    * Write the current configurable key associations to @p config,
-    * or (if @p config is zero) to the application's
+    * Import from @p config all configurable global key associations.
+    *
+    * \since 4.1
+    *
+    * \param config Config object to read from
+    */
+  void importGlobalShortcuts( KConfigGroup* config );
+
+  /**
+    * Export the current configurable global key associations to @p config.
+    *
+    * \since 4.1
+    *
+    * \param config Config object to save to
+    * \param writeDefaults set to true to write settings which are already at defaults.
+    */
+  void exportGlobalShortcuts( KConfigGroup* config, bool writeDefaults = false ) const;
+
+  /**
+    * Write the current configurable key associations to @a config. What the
+    * function does if @a config is zero depends. If this action collection
+    * belongs to a KXMLGuiClient the setting are saved to the kxmlgui
+    * definition file. If not the settings are written to the applications
+    * config file.
+    *
+    * \note oneAction() and writeDefaults() have no meaning for the kxmlgui
     * configuration file.
     *
-    * \param config Config object to save to, or null to use the application's config object.
+    * \param config Config object to save to, or null (see above)
     * \param writeDefaults set to true to write settings which are already at defaults.
     * \param oneAction pass an action here if you just want to save the values for one action, eg.
     *                  if you know that action is the only one which has changed.
@@ -178,10 +206,10 @@ public:
   QAction *action(int index) const;
 
   /**
-   * Find the first action with a given \a name in the action collection.
+   * Get the action with the given \a name from the action collection.
    *
-   * @param name Name of the KAction, or null to match all actions
-   * @return A pointer to the first KAction in the collection which matches the parameters or
+   * @param name Name of the KAction
+   * @return A pointer to the KAction in the collection which matches the parameters or
    * null if nothing matches.
    */
   QAction* action( const QString& name ) const;
@@ -204,6 +232,9 @@ public:
   /**
    * Set the \a componentData associated with this action collection.
    *
+   * \warning Don't call this method on a KActionCollection that contains
+   * actions. This is not supported.
+   *
    * \param componentData the KComponentData which is to be associated with this action collection,
    * or an invalid KComponentData instance to indicate the default KComponentData.
    */
@@ -225,13 +256,20 @@ Q_SIGNALS:
 
   /**
    * Indicates that \a action was removed from this action collection.
+   * @deprecated
    */
-  void removed( QAction* action );
+  QT_MOC_COMPAT void removed( QAction* action );
 
   /**
-   * Indicates that \a action was highlighted
+   * Indicates that \a action was highlighted (hovered over).
+   * @deprecated Replaced by actionHovered(QAction* action);
    */
-  void actionHighlighted(QAction* action);
+  QT_MOC_COMPAT void actionHighlighted(QAction* action);
+  
+  /**
+   * Indicates that \a action was hovered.
+   */
+  void actionHovered(QAction* action);
 
   /**
    * Indicates that \a action was triggered
@@ -243,7 +281,16 @@ protected Q_SLOTS:
   virtual void connectNotify ( const char * signal );
 
   virtual void slotActionTriggered();
-  virtual void slotActionHighlighted();
+  
+  /**
+   * @internal
+   * @deprecated Replaced by slotActionHovered();
+   */
+  QT_MOC_COMPAT virtual void slotActionHighlighted();
+
+private Q_SLOTS:
+  void slotActionHovered();
+  
 
 public:
   /**
@@ -328,6 +375,7 @@ private:
 
   KActionCollection( const KXMLGUIClient* parent ); // used by KXMLGUIClient
 
+  friend class KActionCollectionPrivate;
   class KActionCollectionPrivate* const d;
 };
 
