@@ -29,13 +29,13 @@
 #include "kfontaction.h"
 
 #include <QtGui/QToolBar>
-#include <QtGui/QFontComboBox>
 
 #include <kdebug.h>
 #include <kfontdialog.h>
 #include <kicon.h>
 #include <klocale.h>
 #include <kfontchooser.h>
+#include <kfontcombobox.h>
 
 class KFontAction::KFontActionPrivate
 {
@@ -48,7 +48,7 @@ class KFontAction::KFontActionPrivate
 
         void _k_slotFontChanged(const QFont &font)
         {
-            kDebug(129) << "QFontComboBox - slotFontChanged("
+            kDebug(129) << "KFontComboBox - slotFontChanged("
                         << font.family() << ") settingFont=" << settingFont;
             if (settingFont)
                 return;
@@ -124,7 +124,7 @@ QWidget* KFontAction::createWidget(QWidget* _parent)
     // This is the visual element on the screen.  This method overrides
     // the KSelectAction one, preventing KSelectAction from creating its
     // regular KComboBox.
-    QFontComboBox *cb = new QFontComboBox( parent );
+    KFontComboBox *cb = new KFontComboBox( parent );
 
     kDebug(129) << "\tset=" << font();
     // Do this before connecting the signal so that nothing will fire.
@@ -137,18 +137,18 @@ QWidget* KFontAction::createWidget(QWidget* _parent)
 }
 
 /*
- * Maintenance note: Keep in sync with QFontComboBox::setCurrentFont()
+ * Maintenance note: Keep in sync with KFontComboBox::setCurrentFont()
  */
 void KFontAction::setFont( const QString &family )
 {
     kDebug(129) << "KFontAction::setFont(" << family << ")";
 
-    // Supress triggered(QString) signal and prevent recursive call to ourself.
+    // Suppress triggered(QString) signal and prevent recursive call to ourself.
     d->settingFont++;
 
     foreach(QWidget *w, createdWidgets())
     {
-        QFontComboBox *cb = qobject_cast<QFontComboBox *>(w);
+        KFontComboBox *cb = qobject_cast<KFontComboBox *>(w);
         kDebug(129) << "\tw=" << w << "cb=" << cb;
 
         if(!cb) continue;
@@ -178,7 +178,7 @@ void KFontAction::setFont( const QString &family )
     if (setCurrentAction(lowerName, Qt::CaseInsensitive))
       return;
 
-    // TODO: Inconsistent state if QFontComboBox::setCurrentFont() succeeded
+    // TODO: Inconsistent state if KFontComboBox::setCurrentFont() succeeded
     //       but setCurrentAction() did not and vice-versa.
     kDebug(129) << "Font not found " << family.toLower();
 }
