@@ -43,13 +43,13 @@
 
 namespace KStandardAction
 {
-AutomaticAction::AutomaticAction(const KIcon &icon, const QString &text, const QKeySequence &cut, const char *slot,
+AutomaticAction::AutomaticAction(const KIcon &icon, const QString &text, const KShortcut &shortcut, const char *slot,
                                  QObject *parent)
     : KAction(parent)
 {
     setText(text);
     setIcon(icon);
-    setShortcut(cut);
+    setShortcut(shortcut);
     connect(this, SIGNAL(triggered()), this, slot);
 }
 
@@ -107,17 +107,15 @@ KAction *create(StandardAction id, const QObject *recvr, const char *slot, QObje
     }
 
     if ( QApplication::isRightToLeft() ) {
-      if ( id == Prior )
-        iconName = "go-next";
-
-      if ( id == Next )
-        iconName = "go-previous";
-
-      if ( id == DocumentBack )
-        iconName = "go-next";
-
-      if ( id == DocumentForward )
-        iconName = "go-previous";
+     switch ( id ) {
+      case Prior:           iconName = "go-next-page"; break;
+      case Next:            iconName = "go-previous-page"; break;
+      case FirstPage:       iconName = "go-last-page"; break;
+      case LastPage:        iconName = "go-first-page"; break;
+      case DocumentBack:    iconName = "go-next"; break;
+      case DocumentForward: iconName = "go-previous"; break;
+      default: break;
+     }
     }
 
     QIcon icon = iconName.isEmpty() ? KIcon() : KIcon(iconName);
@@ -441,18 +439,23 @@ KAction *spelling( const QObject *recvr, const char *slot, QObject *parent )
   return KStandardAction::create( Spelling, recvr, slot, parent );
 }
 
-KAction *buildAutomaticAction( QObject* parent, StandardAction id, const char* slot )
+static KAction *buildAutomaticAction( QObject* parent, StandardAction id, const char* slot )
 {
   const KStandardActionInfo* p = infoPtr( id );
   if ( !p )
     return 0;
 
-  AutomaticAction *action = new AutomaticAction(KIcon( p->psIconName ), p->psLabel, KStandardShortcut::shortcut( p->idAccel ).primary(),
-                                                slot, parent);
+  AutomaticAction *action = new AutomaticAction( KIcon( p->psIconName ), p->psLabel,
+      KStandardShortcut::shortcut( p->idAccel ), slot, parent);
+
   action->setObjectName(p->psName);
   action->setWhatsThis( p->psWhatsThis );
 
-  return action;
+  KActionCollection *collection = qobject_cast<KActionCollection *>(parent);
+  if (collection)
+      collection->addAction(action->objectName(), action);
+
+    return action;
 }
 
 KAction *cut( QObject* parent )
