@@ -24,9 +24,9 @@
 #include <QtGui/QGroupBox>
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
-#include <QtGui/QTextEdit>
 #include <QtGui/QListWidget>
 #include <QtGui/QScrollArea>
+#include <QtGui/QTextDocumentFragment>
 
 #include <kapplication.h>
 #include <kconfig.h>
@@ -37,10 +37,12 @@
 #include <knotification.h>
 #include <kiconloader.h>
 #include <kconfiggroup.h>
+#include <ktextedit.h>
 
 #ifdef Q_WS_X11
 #include <qx11info_x11.h>
 #include <X11/Xlib.h>
+#include <fixx11h.h>
 #endif
 
 // Some i18n filters, that standard button texts are piped through
@@ -75,15 +77,6 @@
 #define I18N_POST_BUTTON_FILTER \
     i18nc( "@action:button post-filter", "." );
 
- /**
-  * Easy MessageBox Dialog.
-  *
-  * Provides convenience functions for some i18n'ed standard dialogs,
-  * as well as audible notification via @ref KNotification
-  *
-  * @author Waldo Bastian (bastian@kde.org)
-  */
-
 static bool KMessageBox_queue = false;
 KConfig* KMessageBox_againConfig = 0;
 
@@ -94,7 +87,7 @@ static QIcon themedMessageBoxIcon(QMessageBox::Icon icon)
 
     switch (icon) {
     case QMessageBox::NoIcon:
-        return QPixmap();
+        return QIcon();
         break;
     case QMessageBox::Information:
         icon_name = "dialog-information";
@@ -109,7 +102,7 @@ static QIcon themedMessageBoxIcon(QMessageBox::Icon icon)
         break;
     }
 
-   QIcon ret = KIconLoader::global()->loadIcon(icon_name, KIconLoader::NoGroup, KIconLoader::SizeLarge, KIconLoader::DefaultState, QStringList(), 0, true);
+   QIcon ret = KIconLoader::global()->loadIcon(icon_name, KIconLoader::NoGroup, KIconLoader::SizeHuge, KIconLoader::DefaultState, QStringList(), 0, true);
 
    if (ret.isNull()) {
        return QMessageBox::standardIcon(icon);
@@ -161,102 +154,123 @@ int KMessageBox::createKMessageBox(KDialog *dialog, QMessageBox::Icon icon,
                       ask, checkboxReturn, options, details, icon);
 }
 
+static int longest_line( const QFontMetrics & fm, const QString & text ) {
+    const QStringList lines = QTextDocumentFragment::fromHtml( text ).toPlainText().split( QLatin1String( "\n" ) );
+    int len = 0;
+    Q_FOREACH( const QString & line, lines )
+        len = qMax( len, fm.width( line ) );
+    return len;
+}
+
 int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
                              const QString &text, const QStringList &strlist,
                              const QString &ask, bool *checkboxReturn, Options options,
                              const QString &details, QMessageBox::Icon notifyType)
 {
-    QWidget *topcontents = new QWidget (dialog);
-    QVBoxLayout* toplayout = new QVBoxLayout( topcontents );
-    toplayout->setSpacing( KDialog::spacingHint()*2 );
-    toplayout->setMargin(0);
+    QWidget *mainWidget = new QWidget(dialog);
+    QVBoxLayout *mainLayout = new QVBoxLayout(mainWidget);
+    mainLayout->setSpacing(KDialog::spacingHint() * 2);
+    mainLayout->setMargin(0);
 
-    QWidget *contents = new QWidget(topcontents);
-    QHBoxLayout * lay = new QHBoxLayout(contents);
-    lay->setMargin(0);
-    lay->setSpacing(KDialog::spacingHint());
-    toplayout->addWidget(contents);
-    toplayout->addStretch();
+    QHBoxLayout *hLayout = new QHBoxLayout();
+    hLayout->setMargin(0);
+    hLayout->setSpacing(KDialog::spacingHint());
+    mainLayout->addLayout(hLayout);
+    mainLayout->addStretch();
 
-    QLabel *label1 = new QLabel( contents);
+    QLabel *iconLabel = new QLabel(mainWidget);
 
     if (!icon.isNull()) {
-       int size = IconSize(KIconLoader::Dialog);
-       label1->setPixmap(icon.pixmap(size, size));
+       iconLabel->setPixmap(icon.pixmap(KIconLoader::SizeHuge));
     }
 
-    lay->addWidget( label1, 0, Qt::AlignCenter );
-    lay->addSpacing(KDialog::spacingHint());
+    QVBoxLayout *iconLayout = new QVBoxLayout();
+    iconLayout->addStretch(1);
+    iconLayout->addWidget(iconLabel);
+    iconLayout->addStretch(5);
 
-    QLabel *label2 = new QLabel( text, 0 );
-    label2->setOpenExternalLinks(options & KMessageBox::AllowLink);
-    label2->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::TextSelectableByKeyboard);
-    QPalette messagePal(label2->palette());
+    hLayout->addLayout(iconLayout);
+    hLayout->addSpacing(KDialog::spacingHint());
+
+    QLabel *messageLabel = new QLabel(text, mainWidget);
+    messageLabel->setOpenExternalLinks(options & KMessageBox::AllowLink );
+    Qt::TextInteractionFlags flags = Qt::TextSelectableByMouse | Qt::TextSelectableByKeyboard;
+    if ( options & KMessageBox::AllowLink )
+        flags |= Qt::LinksAccessibleByMouse | Qt::LinksAccessibleByKeyboard;;
+    messageLabel->setTextInteractionFlags(flags);
+    QPalette messagePal(messageLabel->palette());
     messagePal.setColor(QPalette::Window, Qt::transparent);
-    label2->setPalette(messagePal);
+    messageLabel->setPalette(messagePal);
 
     QRect desktop = KGlobalSettings::desktopGeometry(dialog);
-    if (desktop.width() / 3 < label2->fontMetrics().width(text)) {
-        // do only enable automatic wrapping for messages which are longer than one third of the current screen width
-        label2->setWordWrap(true);
+    if (desktop.width() / 3 < longest_line(messageLabel->fontMetrics(), text)) {
+        // do only enable automatic wrapping of messages which are longer than one third of the current screen
+        messageLabel->setWordWrap(true);
     }
-    QScrollArea* textArea = new QScrollArea( contents );
-    textArea->setWidget( label2 );
-    textArea->setFrameShape( QFrame::NoFrame );
-    textArea->setWidgetResizable(true);
-    QPalette scrollPal(textArea->palette());
+
+    QScrollArea* messageScrollArea = new QScrollArea(mainWidget);
+    messageScrollArea->setWidget(messageLabel);
+    messageScrollArea->setFrameShape(QFrame::NoFrame);
+    messageScrollArea->setWidgetResizable(true);
+    QPalette scrollPal(messageScrollArea->palette());
     scrollPal.setColor(QPalette::Window, Qt::transparent);
-    textArea->setPalette(scrollPal);
+    messageScrollArea->setPalette(scrollPal);
 
-    lay->addWidget( textArea );
+    hLayout->addWidget(messageScrollArea);
 
-    QListWidget *listwidget = 0;
+    QListWidget *listWidget = 0;
     if (!strlist.isEmpty()) {
         // enable automatic wrapping since the listwidget has already a good initial width
-        label2->setWordWrap(true);
-        listwidget = new QListWidget(topcontents);
-        toplayout->addWidget(listwidget);
-        listwidget->addItems(strlist);
-        listwidget->setSelectionMode(QListWidget::NoSelection);
-        toplayout->setStretchFactor(listwidget, 1);
+        messageLabel->setWordWrap(true);
+        listWidget = new QListWidget(mainWidget);
+        mainLayout->addWidget(listWidget);
+        listWidget->addItems(strlist);
+        listWidget->setSelectionMode(QListWidget::NoSelection);
+        mainLayout->setStretchFactor(listWidget, 1);
     }
 
     QPointer<QCheckBox> checkbox = 0;
     if (!ask.isEmpty()) {
-        checkbox = new QCheckBox(ask, topcontents);
-        toplayout->addWidget(checkbox);
+        checkbox = new QCheckBox(ask, mainWidget);
+        mainLayout->addWidget(checkbox);
         if (checkboxReturn) {
             checkbox->setChecked(*checkboxReturn);
         }
     }
 
     if (!details.isEmpty()) {
-        QGroupBox *detailsGroup = new QGroupBox( i18n("Details"), dialog);
-        QVBoxLayout *layout = new QVBoxLayout;
-        if ( details.length() < 512 ) {
-            QLabel *label3 = new QLabel( details );
-            label3->setOpenExternalLinks(options & KMessageBox::AllowLink);
-            label3->setTextInteractionFlags(Qt::TextInteractionFlags(label3->style()->styleHint(QStyle::SH_MessageBox_TextInteractionFlags)));
-            //label3->setMinimumSize(label3->sizeHint());
-            label3->setWordWrap(true);
-            layout->addWidget( label3 );
+        QGroupBox *detailsGroup = new QGroupBox(i18n("Details"));
+        QVBoxLayout *detailsLayout = new QVBoxLayout(detailsGroup);
+        if (details.length() < 512) {
+            QLabel *detailsLabel = new QLabel(details);
+            detailsLabel->setOpenExternalLinks(options & KMessageBox::AllowLink);
+            Qt::TextInteractionFlags flags = Qt::TextSelectableByMouse | Qt::TextSelectableByKeyboard;
+            if ( options & KMessageBox::AllowLink )
+                flags |= Qt::LinksAccessibleByMouse | Qt::LinksAccessibleByKeyboard;;
+            detailsLabel->setTextInteractionFlags(flags);
+            detailsLabel->setWordWrap(true);
+            detailsLayout->addWidget(detailsLabel);
         } else {
-            QTextEdit* te = new QTextEdit(details);
-            te->setReadOnly( true );
-            te->setMinimumHeight( te->fontMetrics().lineSpacing() * 11 );
-            layout->addWidget( te );
+            KTextEdit *detailTextEdit = new KTextEdit(details);
+            detailTextEdit->setReadOnly(true);
+            detailTextEdit->setMinimumHeight(detailTextEdit->fontMetrics().lineSpacing() * 11);
+            detailsLayout->addWidget(detailTextEdit);
         }
-        detailsGroup->setLayout( layout );
         dialog->setDetailsWidget(detailsGroup);
     }
 
-    dialog->setMainWidget(topcontents);
-    dialog->showButtonSeparator(false);
-    if (!listwidget) {
-        dialog->layout()->setSizeConstraint( QLayout::SetFixedSize );
+    dialog->setMainWidget(mainWidget);
+    dialog->showButtonSeparator(true);
+    if (!listWidget) {
+        int hfw = messageLabel->heightForWidth(messageScrollArea->sizeHint().width() - 2);
+        if (hfw != messageScrollArea->sizeHint().height() && hfw < desktop.height() / 2) {
+            messageScrollArea->setMinimumHeight(hfw);
+        }
+        if (details.isEmpty())
+            dialog->setFixedSize(dialog->sizeHint() + QSize( 10, 10 ));
     }
 
-    if ( (options & KMessageBox::Dangerous) ) {
+    if ((options & KMessageBox::Dangerous)) {
         if (dialog->isButtonEnabled(KDialog::Cancel))
             dialog->setDefaultButton(KDialog::Cancel);
         else if (dialog->isButtonEnabled(KDialog::No))
@@ -264,26 +278,28 @@ int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
     }
 
     KDialog::ButtonCode defaultCode = dialog->defaultButton();
-    if ( defaultCode != KDialog::NoDefault ) {
-        dialog->setButtonFocus( defaultCode );
+    if (defaultCode != KDialog::NoDefault) {
+        dialog->setButtonFocus(defaultCode);
     }
 
-    if ( (options & KMessageBox::Notify) ) {
-        sendNotification( text, strlist, notifyType, dialog->topLevelWidget()->winId());
+#ifndef Q_WS_WIN // FIXME problems with KNotify on Windows
+    if ((options & KMessageBox::Notify)) {
+        sendNotification(text, strlist, notifyType, dialog->topLevelWidget()->winId());
     }
+#endif
 
     if (KMessageBox_queue) {
         KDialogQueue::queueDialog(dialog);
         return KMessageBox::Cancel; // We have to return something.
     }
 
-    if ( (options & KMessageBox::NoExec) ) {
+    if ((options & KMessageBox::NoExec)) {
         return KMessageBox::Cancel; // We have to return something.
     }
 
-    // We use a QGuardedPtr because the dialog may get deleted
+    // We use a QPointer because the dialog may get deleted
     // during exec() if the parent of the dialog gets deleted.
-    // In that case the guarded ptr will reset to 0.
+    // In that case the QPointer will reset to 0.
     QPointer<KDialog> guardedDialog = dialog;
 
     int result = guardedDialog->exec();
@@ -387,7 +403,7 @@ int KMessageBox::questionYesNoList(QWidget *parent, const QString &text,
                            const QString &dontAskAgainName,
                            Options options)
 { // in order to avoid code duplication, convert to WId, it will be converted back
-    return questionYesNoListWId( parent ? parent->winId() : 0, text, strlist,
+    return questionYesNoListWId( parent ? parent->effectiveWinId() : 0, text, strlist,
         caption, buttonYes, buttonNo, dontAskAgainName, options );
 }
 
@@ -449,7 +465,7 @@ int KMessageBox::questionYesNoCancel(QWidget *parent,
                           const QString &dontAskAgainName,
                           Options options)
 {
-    return questionYesNoCancelWId( parent ? parent->winId() : 0, text, caption, buttonYes, buttonNo, buttonCancel,
+    return questionYesNoCancelWId( parent ? parent->effectiveWinId() : 0, text, caption, buttonYes, buttonNo, buttonCancel,
         dontAskAgainName, options );
 }
 
@@ -542,7 +558,7 @@ int KMessageBox::warningYesNoList(QWidget *parent, const QString &text,
                               const QString &dontAskAgainName,
                               Options options)
 {
-    return warningYesNoListWId( parent ? parent->winId() : 0, text, strlist, caption,
+    return warningYesNoListWId( parent ? parent->effectiveWinId() : 0, text, strlist, caption,
         buttonYes, buttonNo, dontAskAgainName, options );
 }
 
@@ -554,11 +570,6 @@ int KMessageBox::warningYesNoListWId(WId parent_id, const QString &text,
                               const QString &dontAskAgainName,
                               Options options)
 {
-    // warningYesNo and warningYesNoList are always "dangerous"
-    // ### Remove this line for KDE 4, when the 'options' default parameter
-    // takes effects.
-    options |= Dangerous;
-
     ButtonCode res;
     if ( !shouldBeShownYesNo(dontAskAgainName, res) ) {
         return res;
@@ -632,7 +643,7 @@ int KMessageBox::warningContinueCancelList(QWidget *parent, const QString &text,
                              const QString &dontAskAgainName,
                              Options options)
 {
-    return warningContinueCancelListWId( parent ? parent->winId() : 0, text, strlist,
+    return warningContinueCancelListWId( parent ? parent->effectiveWinId() : 0, text, strlist,
         caption, buttonContinue, buttonCancel, dontAskAgainName, options );
 }
 
@@ -718,7 +729,7 @@ int KMessageBox::warningYesNoCancelList(QWidget *parent, const QString &text,
                                     const QString &dontAskAgainName,
                                     Options options)
 {
-    return warningYesNoCancelListWId( parent ? parent->winId() : 0, text, strlist,
+    return warningYesNoCancelListWId( parent ? parent->effectiveWinId() : 0, text, strlist,
         caption, buttonYes, buttonNo, buttonCancel, dontAskAgainName, options );
 }
 
@@ -783,7 +794,7 @@ int KMessageBox::warningYesNoCancelListWId(WId parent_id, const QString &text,
 void KMessageBox::error(QWidget *parent,  const QString &text,
                    const QString &caption, Options options)
 {
-    return errorListWId( parent ? parent->winId() : 0, text, QStringList(), caption, options );
+    return errorListWId( parent ? parent->effectiveWinId() : 0, text, QStringList(), caption, options );
 }
 
 void KMessageBox::errorWId(WId parent_id, const QString &text,
@@ -795,7 +806,7 @@ void KMessageBox::errorWId(WId parent_id, const QString &text,
 void KMessageBox::errorList(QWidget *parent, const QString &text, const QStringList &strlist,
                        const QString &caption, Options options)
 {
-    return errorListWId( parent ? parent->winId() : 0, text, strlist, caption, options );
+    return errorListWId( parent ? parent->effectiveWinId() : 0, text, strlist, caption, options );
 }
 
 void KMessageBox::errorListWId(WId parent_id,  const QString &text, const QStringList &strlist,
@@ -829,7 +840,7 @@ KMessageBox::detailedError(QWidget *parent,  const QString &text,
                    const QString &details,
                    const QString &caption, Options options)
 {
-    return detailedErrorWId( parent ? parent->winId() : 0, text, details, caption, options );
+    return detailedErrorWId( parent ? parent->effectiveWinId() : 0, text, details, caption, options );
 }
 
 void KMessageBox::detailedErrorWId(WId parent_id,  const QString &text,
@@ -862,7 +873,7 @@ void KMessageBox::queuedDetailedError(QWidget *parent,  const QString &text,
                    const QString &details,
                    const QString &caption)
 {
-    return queuedDetailedErrorWId( parent ? parent->winId() : 0, text, details, caption );
+    return queuedDetailedErrorWId( parent ? parent->effectiveWinId() : 0, text, details, caption );
 }
 
 void KMessageBox::queuedDetailedErrorWId(WId parent_id,  const QString &text,
@@ -878,7 +889,7 @@ void KMessageBox::queuedDetailedErrorWId(WId parent_id,  const QString &text,
 void KMessageBox::sorry(QWidget *parent, const QString &text,
                    const QString &caption, Options options)
 {
-    return sorryWId( parent ? parent->winId() : 0, text, caption, options );
+    return sorryWId( parent ? parent->effectiveWinId() : 0, text, caption, options );
 }
 
 void KMessageBox::sorryWId(WId parent_id, const QString &text,
@@ -910,7 +921,7 @@ void KMessageBox::detailedSorry(QWidget *parent, const QString &text,
                    const QString &details,
                    const QString &caption, Options options)
 {
-    return detailedSorryWId( parent ? parent->winId() : 0, text, details, caption, options );
+    return detailedSorryWId( parent ? parent->effectiveWinId() : 0, text, details, caption, options );
 }
 
 void KMessageBox::detailedSorryWId(WId parent_id, const QString &text,
@@ -954,7 +965,7 @@ void KMessageBox::informationWId(WId parent_id,const QString &text,
 void KMessageBox::informationList(QWidget *parent,const QString &text, const QStringList & strlist,
                          const QString &caption, const QString &dontShowAgainName, Options options)
 {
-    return informationListWId( parent ? parent->winId() : 0, text, strlist, caption,
+    return informationListWId( parent ? parent->effectiveWinId() : 0, text, strlist, caption,
         dontShowAgainName, options );
 }
 
@@ -1006,9 +1017,9 @@ void KMessageBox::enableAllMessages()
 
    typedef QMap<QString, QString> configMap;
 
-   configMap map = cg.entryMap();
+   const configMap map = cg.entryMap();
 
-   configMap::Iterator it;
+   configMap::ConstIterator it;
    for (it = map.begin(); it != map.end(); ++it) {
       cg.deleteEntry( it.key() );
    }
@@ -1058,7 +1069,7 @@ int KMessageBox::messageBox( QWidget *parent, DialogType type, const QString &te
                              const KGuiItem &buttonNo, const KGuiItem &buttonCancel,
                              const QString &dontShowAskAgainName, Options options )
 {
-    return messageBoxWId( parent ? parent->winId() : 0, type, text, caption,
+    return messageBoxWId( parent ? parent->effectiveWinId() : 0, type, text, caption,
         buttonYes, buttonNo, buttonCancel, dontShowAskAgainName, options );
 }
 
@@ -1101,7 +1112,7 @@ int KMessageBox::messageBoxWId( WId parent_id, DialogType type, const QString &t
 
 void KMessageBox::queuedMessageBox( QWidget *parent, DialogType type, const QString &text, const QString &caption, Options options )
 {
-    return queuedMessageBoxWId( parent ? parent->winId() : 0, type, text, caption, options );
+    return queuedMessageBoxWId( parent ? parent->effectiveWinId() : 0, type, text, caption, options );
 }
 
 void KMessageBox::queuedMessageBoxWId( WId parent_id, DialogType type, const QString &text, const QString &caption, Options options )
@@ -1114,7 +1125,7 @@ void KMessageBox::queuedMessageBoxWId( WId parent_id, DialogType type, const QSt
 
 void KMessageBox::queuedMessageBox( QWidget *parent, DialogType type, const QString &text, const QString &caption )
 {
-    return queuedMessageBoxWId( parent ? parent->winId() : 0, type, text, caption );
+    return queuedMessageBoxWId( parent ? parent->effectiveWinId() : 0, type, text, caption );
 }
 
 void KMessageBox::queuedMessageBoxWId( WId parent_id, DialogType type, const QString &text, const QString &caption )
