@@ -1,8 +1,4 @@
 /*
-
-Requires the Qt widget libraries, available at no cost at
-http://www.troll.no
-
 Copyright (C) 1996 Bernd Johannes Wuebben  <wuebben@kde.org>
 Copyright (c) 1999 Preston Brown <pbrown@kde.org>
 Copyright (c) 1999 Mario Weilguni <mweilguni@kde.org>
@@ -24,27 +20,22 @@ Boston, MA 02110-1301, USA.
 */
 
 #include "kfontchooser.h"
+#include "fonthelpers_p.h"
 #include "sampleedit_p.h"
 
-
-#include <QtGui/QComboBox>
 #include <QtGui/QCheckBox>
-#include <QtCore/QFile>
-#include <QtGui/QFont>
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
 #include <QtGui/QSplitter>
-#include <QtGui/QLineEdit>
 #include <QtGui/QScrollBar>
-#include <QtCore/QMutableStringListIterator>
 #include <QtGui/QFontDatabase>
-#include <QList>
 #include <QtGui/QGroupBox>
 #include <kcharsets.h>
 #include <kconfig.h>
 #include <kdialog.h>
 #include <kglobal.h>
 #include <kglobalsettings.h>
+#include <klineedit.h>
 #include <klistwidget.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
@@ -81,26 +72,6 @@ static int minimumListHeight( const QListWidget *list, int numVisibleEntry )
     return ( w * numVisibleEntry + 2 * list->frameWidth() );
 }
 
-static bool localeLessThan (const QString &a, const QString &b)
-{
-    return QString::localeAwareCompare(a, b) < 0;
-}
-
-static void splitFontString (const QString &font, QString *family, QString *foundry = NULL)
-{
-    int p1 = font.indexOf('[');
-    if (p1 < 0) {
-        if (family) *family = font.trimmed();
-        if (foundry) foundry->clear();
-    }
-    else {
-        int p2 = font.indexOf(']', p1);
-        p2 = p2 > p1 ? p2 : font.length();
-        if (family) *family = font.left(p1).trimmed();
-        if (foundry) *foundry = font.mid(p1 + 1, p2 - p1 - 1).trimmed();
-    }
-}
-
 class KFontChooser::Private
 {
 public:
@@ -144,7 +115,7 @@ public:
     KIntNumInput *sizeOfFont;
 
     SampleEdit   *sampleEdit;
-    QLineEdit    *xlfdEdit;
+    KLineEdit    *xlfdEdit;
 
     QLabel       *familyLabel;
     QLabel       *styleLabel;
@@ -209,7 +180,6 @@ KFontChooser::KFontChooser( QWidget *parent,
         gridLayout = new QGridLayout( page );
         gridLayout->setMargin( KDialog::marginHint() );
         gridLayout->setSpacing( KDialog::spacingHint() );
-        gridLayout->addItem( new QSpacerItem(0,fontMetrics().lineSpacing()), 0, 0 );
         row = 1;
     }
     else
@@ -237,7 +207,6 @@ KFontChooser::KFontChooser( QWidget *parent,
     } else {
         d->familyCheckbox = 0;
         d->familyLabel = new QLabel( i18nc("@label","Font:"), page );
-        d->familyLabel->setObjectName( "familyLabel" );
         familyLayout->addWidget(d->familyLabel, 1, Qt::AlignLeft);
     }
     gridLayout->addLayout(familyLayout, row, 0 );
@@ -254,7 +223,6 @@ KFontChooser::KFontChooser( QWidget *parent,
     } else {
         d->styleCheckbox = 0;
         d->styleLabel = new QLabel(i18n("Font style:"), page );
-        d->styleLabel->setObjectName( "styleLabel" );
         styleLayout->addWidget(d->styleLabel, 1, Qt::AlignLeft);
     }
     styleLayout->addSpacing( checkBoxGap );
@@ -272,7 +240,6 @@ KFontChooser::KFontChooser( QWidget *parent,
     } else {
         d->sizeCheckbox = 0;
         d->sizeLabel = new QLabel(i18nc("@label:listbox Font size", "Size:"), page );
-        d->sizeLabel->setObjectName( "sizeLabel" );
         sizeLayout->addWidget(d->sizeLabel, 1, Qt::AlignLeft);
     }
     sizeLayout->addSpacing( checkBoxGap );
@@ -285,7 +252,6 @@ KFontChooser::KFontChooser( QWidget *parent,
     // now create the actual boxes that hold the info
     //
     d->familyListBox = new KListWidget( page );
-    d->familyListBox->setObjectName("familyListBox");
     d->familyListBox->setEnabled( flags ^ ShowDifferences );
     gridLayout->addWidget( d->familyListBox, row, 0 );
     QString fontFamilyWhatsThisText (
@@ -313,7 +279,6 @@ KFontChooser::KFontChooser( QWidget *parent,
         minimumListHeight( d->familyListBox, visibleListSize  ) );
 
     d->styleListBox = new KListWidget( page );
-    d->styleListBox->setObjectName("styleListBox");
     d->styleListBox->setEnabled( flags ^ ShowDifferences );
     gridLayout->addWidget(d->styleListBox, row, 1);
     d->styleListBox->setWhatsThis(i18nc("@info:whatsthis","Here you can choose the font style to be used." ));
@@ -338,8 +303,7 @@ KFontChooser::KFontChooser( QWidget *parent,
 
 
     d->sizeListBox = new KListWidget( page );
-    d->sizeListBox->setObjectName("sizeListBox");
-    d->sizeOfFont = new KIntNumInput( page); // "sizeOfFont");
+    d->sizeOfFont = new KIntNumInput(page);
     d->sizeOfFont->setMinimum(4);
     d->sizeOfFont->setMaximum(999);
     d->sizeOfFont->setSliderEnabled(false);
@@ -357,11 +321,9 @@ KFontChooser::KFontChooser( QWidget *parent,
                  "environment (e.g. widget dimensions, paper size)." );
         d->sizeIsRelativeCheckBox = new QCheckBox( sizeIsRelativeCBText,
                                                 page );
-        d->sizeIsRelativeCheckBox->setObjectName( "sizeIsRelativeCheckBox" );
         d->sizeIsRelativeCheckBox->setTristate( flags & ShowDifferences );
         QGridLayout *sizeLayout2 = new QGridLayout();
         sizeLayout2->setSpacing( KDialog::spacingHint()/2 );
-        sizeLayout2->setObjectName( "sizeLayout2" );
         gridLayout->addLayout(sizeLayout2, row, 2);
         sizeLayout2->setColumnStretch( 1, 1 ); // to prevent text from eating the right border
         sizeLayout2->addWidget( d->sizeOfFont, 0, 0, 1, 2);
@@ -374,7 +336,6 @@ KFontChooser::KFontChooser( QWidget *parent,
         d->sizeIsRelativeCheckBox = 0L;
         QGridLayout *sizeLayout2 = new QGridLayout();
         sizeLayout2->setSpacing( KDialog::spacingHint()/2 );
-        sizeLayout2->setObjectName( "sizeLayout2" );
         gridLayout->addLayout(sizeLayout2, row, 2);
         sizeLayout2->addWidget( d->sizeOfFont, 0, 0);
         sizeLayout2->addWidget(d->sizeListBox, 1,0);
@@ -411,7 +372,6 @@ KFontChooser::KFontChooser( QWidget *parent,
     //
     d->sampleEdit = new SampleEdit(page);
     d->sampleEdit->setAcceptRichText(false);
-    d->sampleEdit->setObjectName("sampleEdit");
     QFont tmpFont( KGlobalSettings::generalFont().family(), 64, QFont::Black );
     d->sampleEdit->setFont(tmpFont);
     d->sampleEdit->setMinimumHeight( d->sampleEdit->fontMetrics().lineSpacing() );
@@ -454,8 +414,7 @@ KFontChooser::KFontChooser( QWidget *parent,
         vbox->addWidget( label );
     }
 
-    d->xlfdEdit = new QLineEdit( page );
-    d->xlfdEdit->setObjectName( "xlfdEdit" );
+    d->xlfdEdit = new KLineEdit( page );
     vbox->addWidget( d->xlfdEdit );
     //
     // Finished setting up the chooser layout.
@@ -618,7 +577,10 @@ void KFontChooser::Private::_k_family_chosen_slot(const QString& family)
 
     QString currentFamily;
     if (family.isEmpty()) {
-        currentFamily = qtFamilies[familyListBox->currentItem()->text()];
+        Q_ASSERT( familyListBox->currentItem() );
+        if (familyListBox->currentItem()) {
+          currentFamily = qtFamilies[familyListBox->currentItem()->text()];
+        }
     }
     else {
         currentFamily = qtFamilies[family];
@@ -909,7 +871,7 @@ int KFontChooser::Private::setupSizeListBox (const QString& family, const QStrin
     int bestFitSize = fillSizeList(sizes);
 
     // Set the best fit size as current in the listbox if available.
-    QList<QListWidgetItem*> selectedSizeList =
+    const QList<QListWidgetItem*> selectedSizeList =
         sizeListBox->findItems( QString::number(bestFitSize),
                                 Qt::MatchExactly );
     if ( !selectedSizeList.isEmpty() ) {
@@ -1050,26 +1012,7 @@ void KFontChooser::Private::setFamilyBoxItems(const QStringList &fonts)
 {
     signalsAllowed = false;
 
-    QStringList trfonts;
-    qtFamilies.clear();
-    foreach (const QString &font, fonts) {
-        // Try to split into name and foundry.
-        QString family, foundry;
-        splitFontString(font, &family, &foundry);
-        QString trfont;
-        if (foundry.isEmpty()) {
-            // i18n: Filtering message, so that translators can translate the
-            // font names on their own should they want. May be replaced in
-            // the future with conventional messages in a PO file.
-            trfont = i18nc("@item Font name", "%1", family);
-        }
-        else {
-            trfont = i18nc("@item Font name [foundry]", "%1 [%2]", family, foundry);
-        }
-        trfonts.append(trfont);
-        qtFamilies.insert(trfont, font);
-    }
-    qSort(trfonts.begin(), trfonts.end(), localeLessThan);
+    QStringList trfonts = translateFontNameList(fonts, &qtFamilies);
     familyListBox->clear();
     familyListBox->addItems(trfonts);
 
@@ -1095,6 +1038,5 @@ void KFontChooser::Private::_k_showXLFDArea(bool show)
     }
 }
 
-
 #include "kfontchooser.moc"
 #include "sampleedit_p.moc"
