@@ -57,6 +57,22 @@ void KStatusBarJobTracker::registerJob(KJob *job)
     d->progressWidget.insert(job, vi);
 }
 
+void KStatusBarJobTracker::unregisterJob(KJob *job)
+{
+    KAbstractWidgetJobTracker::unregisterJob(job);
+
+    if (!d->progressWidget.contains(job))
+        return;
+
+    if (d->currentProgressWidget == d->progressWidget[job])
+        d->currentProgressWidget = 0;
+
+    if (!d->progressWidget[job]->beingDeleted)
+        delete d->progressWidget[job];
+
+    d->progressWidget.remove(job);
+}
+
 QWidget *KStatusBarJobTracker::widget(KJob *job)
 {
     if (!d->progressWidget.contains(job)) {
@@ -156,11 +172,13 @@ void KStatusBarJobTracker::Private::ProgressWidget::init(KJob *job, QWidget *par
     stack->insertWidget(2, label);
     setMinimumSize(sizeHint());
 
-    parent->layout()->addWidget(widget);
-
     setMode(KStatusBarJobTracker::LabelOnly);
 
     q->setAutoDelete(job, true);
+
+    QVBoxLayout *layout = new QVBoxLayout;
+    layout->addWidget(widget);
+    setLayout(layout);
 }
 
 void KStatusBarJobTracker::Private::ProgressWidget::setMode(StatusBarModes newMode)
