@@ -74,7 +74,7 @@
 #endif
 
 #include <fcntl.h>
-#include <stdlib.h> // getenv(), srand(), rand()
+#include <stdlib.h> // srand(), rand()
 #include <unistd.h>
 #if defined Q_WS_X11
 //#ifndef Q_WS_QWS //FIXME(E): NetWM should talk to QWS...
@@ -100,7 +100,8 @@
 #undef Status
 #include <Carbon/Carbon.h>
 #include <QImage>
-#include <QSystemTrayIcon>
+#include <ksystemtrayicon.h>
+#include <kkernel_mac.h>
 #endif
 
 #ifdef Q_OS_UNIX
@@ -141,10 +142,6 @@ static int kde_x_errhandler( Display *dpy, XErrorEvent *err )
 void KApplication_init_windows();
 #endif
 
-#ifdef Q_WS_MAC
-void KApplication_early_init_mac();
-#endif
-
 /*
   Private data to make keeping binary compatibility easier
  */
@@ -416,7 +413,6 @@ int KApplication::xioErrhandler( Display* dpy )
 {
     if(kapp)
     {
-        emit aboutToQuit();
 #ifdef Q_WS_X11
         d->oldXIOErrorHandler( dpy );
 #else
@@ -436,7 +432,7 @@ int KApplication::xErrhandler( Display* dpy, void* err_ )
         // add KDE specific stuff here
         d->oldXErrorHandler( dpy, err );
     }
-    const QByteArray fatalXError = getenv("KDE_FATAL_X_ERROR");
+    const QByteArray fatalXError = qgetenv("KDE_FATAL_X_ERROR");
     if (!fatalXError.isEmpty()) {
         abort();
     }
@@ -481,13 +477,13 @@ void KApplicationPrivate::init(bool GUIenabled)
   }
 
 #ifdef Q_WS_MAC
-  KApplication_early_init_mac();
+  mac_initialize_dbus();
 #endif
 
   if ( q->type() == KApplication::GuiClient )
   {
-    QStringList plugins = KGlobal::dirs()->resourceDirs( "qtplugins" );
-    QStringList::Iterator it = plugins.begin();
+    const QStringList plugins = KGlobal::dirs()->resourceDirs( "qtplugins" );
+    QStringList::ConstIterator it = plugins.begin();
     while (it != plugins.end()) {
       q->addLibraryPath( *it );
       ++it;
@@ -568,7 +564,7 @@ void KApplicationPrivate::init(bool GUIenabled)
   (void) KGlobal::locale();
 
   KSharedConfig::Ptr config = componentData.config();
-  QByteArray readOnly = getenv("KDE_HOME_READONLY");
+  QByteArray readOnly = qgetenv("KDE_HOME_READONLY");
   if (readOnly.isEmpty() && q->applicationName() != QLatin1String("kdialog"))
   {
     if (KAuthorized::authorize(QLatin1String("warn_unwritable_config")))
@@ -599,11 +595,12 @@ void KApplicationPrivate::init(bool GUIenabled)
 
 #ifdef Q_WS_MAC
   if (q->type() == KApplication::GuiClient) {
+      // This is a QSystemTrayIcon instead of K* because we can't be sure q is a QWidget
       QSystemTrayIcon *trayIcon;
       if (QSystemTrayIcon::isSystemTrayAvailable())
       {
           trayIcon = new QSystemTrayIcon(q);
-          trayIcon->setIcon(KIcon(KCmdLineArgs::appName()));
+          trayIcon->setIcon(q->windowIcon());
           /* it's counter-intuitive, but once you do setIcon it's already set the
              dock icon... ->show actually shows an icon in the menu bar too  :P */
           // trayIcon->show();
@@ -731,11 +728,10 @@ commitDataRestart:
     d->session_save = false;
 }
 
+#ifdef Q_WS_X11
 static void checkRestartVersion( QSessionManager& sm )
 {
-#if defined Q_WS_X11
     Display* dpy = QX11Info::display();
-
     Atom type;
     int format;
     unsigned long nitems, after;
@@ -751,7 +747,6 @@ static void checkRestartVersion( QSessionManager& sm )
         }
         XFree( data );
     }
-#endif // defined Q_WS_X11
 #define NUM_TO_STRING2( num ) #num
 #define NUM_TO_STRING( num ) NUM_TO_STRING2( num )
     QString wrapper = KStandardDirs::findExe( "kde" NUM_TO_STRING( KDE_VERSION_MAJOR ) ); // "kde4", etc.
@@ -761,6 +756,7 @@ static void checkRestartVersion( QSessionManager& sm )
     restartCommand.prepend( wrapper );
     sm.setRestartCommand( restartCommand );
 }
+#endif // Q_WS_X11
 
 void KApplication::saveState( QSessionManager& sm )
 {
@@ -797,7 +793,7 @@ void KApplication::saveState( QSessionManager& sm )
     // tell the session manager about our new lifecycle
     QStringList restartCommand = sm.restartCommand();
 
-    QByteArray multiHead = getenv("KDE_MULTIHEAD");
+    QByteArray multiHead = qgetenv("KDE_MULTIHEAD");
     if (multiHead.toLower() == "true") {
         // if multihead is enabled, we save our -display argument so that
         // we are restored onto the correct head... one problem with this
@@ -805,7 +801,7 @@ void KApplication::saveState( QSessionManager& sm )
         // to a different display (ie. if we are in a university lab and try,
         // try to restore a multihead session, our apps could be started on
         // someone else's display instead of our own)
-        QByteArray displayname = getenv("DISPLAY");
+        QByteArray displayname = qgetenv("DISPLAY");
         if (! displayname.isNull()) {
             // only store the command if we actually have a DISPLAY
             // environment variable
@@ -815,7 +811,9 @@ void KApplication::saveState( QSessionManager& sm )
         sm.setRestartCommand( restartCommand );
     }
 
+#ifdef Q_WS_X11
     checkRestartVersion( sm );
+#endif
 
     // finally: do session management
     emit saveYourself(); // for compatibility
@@ -870,7 +868,7 @@ void KApplicationPrivate::parseCommandLine( )
             q->setWindowIcon(KIcon(args->getOption("icon")));
         }
         else {
-            q->setWindowIcon(KIcon(componentData.componentName()));
+            q->setWindowIcon(KIcon(componentData.aboutData()->programIconName()));
         }
     }
 
@@ -883,15 +881,14 @@ void KApplicationPrivate::parseCommandLine( )
         componentData.setConfigName(config);
     }
 
-    bool nocrashhandler = (getenv("KDE_DEBUG") != NULL);
+    bool nocrashhandler = (!qgetenv("KDE_DEBUG").isEmpty());
     if (!nocrashhandler && args->isSet("crashhandler"))
     {
-        // set default crash handler / set emergency save function to nothing
+        // set default crash handler
         KCrash::setCrashHandler(KCrash::defaultCrashHandler);
-        KCrash::setEmergencySaveFunction(NULL);
-
-        KCrash::setApplicationName(args->appName());
     }
+    // Always set the app name, can be usefuls for apps that call setEmergencySaveFunction or enable AutoRestart
+    KCrash::setApplicationName(args->appName());
 
 #ifdef Q_WS_X11
     if ( args->isSet( "waitforwm" ) ) {
@@ -915,11 +912,12 @@ void KApplicationPrivate::parseCommandLine( )
     // FIXME(E): Implement for Qt Embedded
 #endif
 
+#ifndef Q_WS_WIN
     if (args->isSet("smkey"))
     {
         sessionKey = args->getOption("smkey");
     }
-
+#endif
 }
 
 extern void kDebugCleanup();
@@ -1076,7 +1074,7 @@ QString KApplication::tempSaveName( const QString& pFilename )
         }
     }
 
-  aFilename.replace( QLatin1String("/"), QLatin1String("\\!") )
+  aFilename.replace( '/', QLatin1String("\\!") )
     .prepend( QLatin1Char('#') )
     .append( QLatin1Char('#') )
     .prepend( QLatin1Char('/') ).prepend( aAutosaveDir.absolutePath() );
@@ -1206,9 +1204,10 @@ void KApplicationPrivate::_k_slot_KToolInvocation_hook(QStringList& envs,QByteAr
     if (QX11Info::display()) {
         QByteArray dpystring(XDisplayString(QX11Info::display()));
         envs << QString::fromLatin1( QByteArray("DISPLAY=") + dpystring );
-    } else if( getenv( "DISPLAY" )) {
-        QByteArray dpystring( getenv( "DISPLAY" ));
-        envs << QString::fromLatin1( QByteArray("DISPLAY=") + dpystring );
+    } else {
+        const QByteArray dpystring( qgetenv( "DISPLAY" ));
+        if(!dpystring.isEmpty())
+            envs << QString::fromLatin1( QByteArray("DISPLAY=") + dpystring );
     }
 
     if(startup_id.isEmpty())
