@@ -37,6 +37,24 @@ namespace Sonnet
 
         bool backgroundCheckingButtonShown() const;
 
+        /**
+         * Sets the language/dictionary that will be selected by default
+         * in this config widget.
+         * This overrides the setting in the config file.
+         *
+         * @param language the language which will be selected by default.
+         * @since 4.1
+         */
+        void setLanguage( const QString &language );
+
+        /**
+         * Get the currently selected language for spell checking.  Returns an empty string if
+         * kdelibs was built without any spellchecking plugins.
+         * @return the language currently selected in the language combobox
+         * @since 4.1
+         */
+        QString language() const;
+
     public Q_SLOTS:
         void save();
         void setBackgroundCheckingButtonShown( bool );
@@ -44,6 +62,13 @@ namespace Sonnet
     protected Q_SLOTS:
         void slotChanged();
 
+    Q_SIGNALS:
+        /**
+         * Signal sends when config was changed
+         * @since 4.1
+         */
+        void configChanged();
+
     private:
         void init(KConfig *config);
         void setFromGui();
@@ -51,6 +76,7 @@ namespace Sonnet
 
     private:
         class Private;
+        friend class Private;
         Private* const d;
     };
 }
