@@ -260,11 +260,7 @@ void KDateTable::initAccels()
     localCollection->readSettings();
     localCollection->addAssociatedWidget( this );
     foreach (QAction* action, localCollection->actions())
-#if QT_VERSION < KDE_MAKE_VERSION(4,4,0)
-        action->setShortcutContext(Qt::WidgetShortcut); // remove after Qt4.4 becomes mandatory
-#else
         action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-#endif
 }
 
 int KDateTable::posFromDate( const QDate &date_ )
