@@ -55,10 +55,11 @@
 #include <QtGui/QStyleOption>
 #include <kconfiggroup.h>
 
-class KLineEdit::KLineEditPrivate
+class KLineEditPrivate
 {
 public:
-    KLineEditPrivate()
+    KLineEditPrivate(KLineEdit* qq)
+        : q(qq)
     {
         completionBox = 0L;
         handleURLDrops = true;
@@ -71,6 +72,7 @@ public:
 
         drawClickMsg = false;
         enableClickMsg = false;
+        threeStars = false;
         if ( !initialized )
         {
             KConfigGroup config( KGlobal::config(), "General" );
@@ -81,6 +83,8 @@ public:
 
         clearButton = 0;
         clickInClear = false;
+        wideEnoughForClear = true;
+        overlap = 0;
     }
 
     ~KLineEditPrivate()
@@ -89,6 +93,24 @@ public:
 //        delete completionBox;
     }
 
+    void _k_slotSettingsChanged(int category)
+    {
+        Q_UNUSED(category);
+
+        if (clearButton) {
+            clearButton->setAnimationsEnabled(KGlobalSettings::graphicEffectsLevel() & KGlobalSettings::SimpleAnimationEffects);
+        }
+    }
+
+    void doCompletion(const QString& txt);
+
+    /**
+     * Checks whether we should/should not consume a key used as a shortcut.
+     * This makes it possible to handle shortcuts in the focused widget before any
+     * window-global QAction is triggered.
+     */
+    bool overrideShortcut(const QKeyEvent* e);
+
     static bool initialized;
     static bool backspacePerformsCompletion; // Configuration option
 
@@ -110,17 +132,22 @@ public:
     QString clickMessage;
     bool enableClickMsg:1;
     bool drawClickMsg:1;
+    bool threeStars:1;
 
     bool possibleTripleClick :1;  // set in mousePressEvent, deleted in tripleClickTimeout
 
     bool clickInClear:1;
+    bool wideEnoughForClear:1;
     KLineEditButton *clearButton;
 
     KCompletionBox *completionBox;
 
+    int overlap;
+
     QAction *noCompletionAction, *shellCompletionAction, *autoCompletionAction, *popupCompletionAction, *shortAutoCompletionAction, *popupAutoCompletionAction, *defaultAction;
 
     QMap<KGlobalSettings::Completion, bool> disableCompletionMap;
+    KLineEdit* q;
 };
 
 // FIXME: Go back to using StyleSheets instead of a proxy style
@@ -129,11 +156,12 @@ public:
 class KLineEditStyle : public KdeUiProxyStyle
 {
 public:
-  KLineEditStyle(KLineEdit *parent) : KdeUiProxyStyle(parent), overlap(0) {}
- 
+  KLineEditStyle(KLineEdit *parent, KLineEditPrivate *lineEditPrivate)
+    : KdeUiProxyStyle(parent), lineEditPrivate(lineEditPrivate) {}
+
   QRect subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const;
-  
-  int overlap;
+
+  KLineEditPrivate* lineEditPrivate;
 };
 
 QRect KLineEditStyle::subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
@@ -142,6 +170,7 @@ QRect KLineEditStyle::subElementRect(SubElement element, const QStyleOption *opt
   {
     QRect rect = style()->subElementRect(SE_LineEditContents, option, widget);
 
+    int overlap = lineEditPrivate->overlap;
     if (option->direction == Qt::LeftToRight) return rect.adjusted(0, 0, -overlap, 0);
     else return rect.adjusted(overlap, 0, 0, 0);
   }
@@ -149,18 +178,18 @@ QRect KLineEditStyle::subElementRect(SubElement element, const QStyleOption *opt
   return KdeUiProxyStyle::subElementRect(element, option, widget);
 }
 
-bool KLineEdit::KLineEditPrivate::backspacePerformsCompletion = false;
-bool KLineEdit::KLineEditPrivate::initialized = false;
+bool KLineEditPrivate::backspacePerformsCompletion = false;
+bool KLineEditPrivate::initialized = false;
 
 
 KLineEdit::KLineEdit( const QString &string, QWidget *parent )
-          :QLineEdit( string, parent ), d(new KLineEditPrivate)
+    : QLineEdit( string, parent ), d(new KLineEditPrivate(this))
 {
     init();
 }
 
 KLineEdit::KLineEdit( QWidget *parent )
-          :QLineEdit( parent ), d(new KLineEditPrivate)
+    : QLineEdit( parent ), d(new KLineEditPrivate(this))
 {
     init();
 }
@@ -186,13 +215,15 @@ void KLineEdit::init()
                       mode == KGlobalSettings::CompletionAuto);
     connect( this, SIGNAL(selectionChanged()), this, SLOT(slotRestoreSelectionColors()));
 
+    connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)), this, SLOT(_k_slotSettingsChanged(int)));
+
     QPalette p = palette();
     if ( !d->previousHighlightedTextColor.isValid() )
       d->previousHighlightedTextColor=p.color(QPalette::Normal,QPalette::HighlightedText);
     if ( !d->previousHighlightColor.isValid() )
       d->previousHighlightColor=p.color(QPalette::Normal,QPalette::Highlight);
-    
-    QStyle *lineEditStyle = new KLineEditStyle(this);
+
+    QStyle *lineEditStyle = new KLineEditStyle(this, d);
     lineEditStyle->setParent(this);
     setStyle(lineEditStyle);
 }
@@ -211,7 +242,7 @@ void KLineEdit::setClearButtonShown(bool show)
 
         d->clearButton = new KLineEditButton(this);
         d->clearButton->setCursor( Qt::ArrowCursor );
-        d->clearButton->setToolTip( i18n( "Clear text" ) );
+        d->clearButton->setToolTip( i18nc( "@action:button Clear current text in the line edit", "Clear text" ) );
 
         updateClearButtonIcon(text());
         updateClearButton();
@@ -221,8 +252,7 @@ void KLineEdit::setClearButtonShown(bool show)
         delete d->clearButton;
         d->clearButton = 0;
         d->clickInClear = false;
-        KLineEditStyle *lestyle = dynamic_cast<KLineEditStyle *>(style());
-        if (lestyle) lestyle->overlap = 0;
+        d->overlap = 0;
     }
 }
 
@@ -231,6 +261,17 @@ bool KLineEdit::isClearButtonShown() const
     return d->clearButton != 0;
 }
 
+QSize KLineEdit::clearButtonUsedSize() const
+{
+    QSize s;
+    if (d->clearButton) {
+        const int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, this);
+        s = d->clearButton->sizeHint();
+        s.rwidth() += frameWidth;
+    }
+    return s;
+}
+
 void KLineEdit::updateClearButtonIcon(const QString& text)
 {
     if (!d->clearButton || isReadOnly()) {
@@ -239,14 +280,9 @@ void KLineEdit::updateClearButtonIcon(const QString& text)
 
     int clearButtonState = KIconLoader::DefaultState;
 
-    if (text.length() > 0) {
+    if (d->wideEnoughForClear && text.length() > 0) {
         d->clearButton->animateVisible(true);
     } else {
-/*        if (d->clickButtonState == KIconLoader::DisabledState) {
-            return;
-        }
-
-        d->clickButtonState = KIconLoader::DisabledState;*/
         d->clearButton->animateVisible(false);
     }
 
@@ -259,6 +295,8 @@ void KLineEdit::updateClearButtonIcon(const QString& text)
     } else {
         d->clearButton->setPixmap(SmallIcon("edit-clear-locationbar-ltr", 0, clearButtonState));
     }
+
+    d->clearButton->setVisible(text.length());
 }
 
 void KLineEdit::updateClearButton()
@@ -267,16 +305,20 @@ void KLineEdit::updateClearButton()
         return;
     }
 
-    QSize geom = size();
-    int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
-    int buttonWidth = d->clearButton->sizeHint().width();
-    QSize newButtonSize(buttonWidth, geom.height());
+    const QSize geom = size();
+    const int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth,0,this);
+    const int buttonWidth = d->clearButton->sizeHint().width();
+    const QSize newButtonSize(buttonWidth, geom.height());
+    const QFontMetrics fm(font());
+    const int em = fm.width("m");
+
+    // make sure we have enough room for the clear button
+    // no point in showing it if we can't also see a few characters as well
+    const bool wideEnough = geom.width() > 4 * em + buttonWidth + frameWidth;
 
     if (newButtonSize != d->clearButton->size()) {
         d->clearButton->resize(newButtonSize);
-        
-        KLineEditStyle *lestyle = dynamic_cast<KLineEditStyle *>(style());
-        if (lestyle) lestyle->overlap = buttonWidth + frameWidth;
+        d->overlap = wideEnough ? buttonWidth + frameWidth : 0;
     }
 
     if (qApp->isLeftToRight()) {
@@ -284,6 +326,14 @@ void KLineEdit::updateClearButton()
     } else {
         d->clearButton->move(frameWidth + 1, 0);
     }
+
+    if (wideEnough != d->wideEnoughForClear) {
+        // we may (or may not) have been showing the button, but now our
+        // positiong on that matter has shifted, so let's ensure that it
+        // is properly visible (or not)
+        d->wideEnoughForClear = wideEnough;
+        updateClearButtonIcon(text());
+    }
 }
 
 void KLineEdit::setCompletionMode( KGlobalSettings::Completion mode )
@@ -362,7 +412,7 @@ void KLineEdit::rotateText( KCompletionBase::KeyBindingType type )
           input = comp->nextMatch();
 
        // Skip rotation if previous/next match is null or the same text
-       if ( input.isNull() || input == displayText() )
+       if ( input.isEmpty() || input == displayText() )
             return;
        setCompletedText( input, hasSelectedText() );
     }
@@ -376,12 +426,12 @@ void KLineEdit::makeCompletion( const QString& text )
     if ( !comp || mode == KGlobalSettings::CompletionNone )
         return;  // No completion object...
 
-    QString match = comp->makeCompletion( text );
+    const QString match = comp->makeCompletion( text );
 
     if ( mode == KGlobalSettings::CompletionPopup ||
          mode == KGlobalSettings::CompletionPopupAuto )
     {
-        if ( match.isNull() )
+        if ( match.isEmpty() )
         {
             if ( d->completionBox )
             {
@@ -396,7 +446,7 @@ void KLineEdit::makeCompletion( const QString& text )
     {
         // all other completion modes
         // If no match or the same match, simply return without completing.
-        if ( match.isNull() || match == text )
+        if ( match.isEmpty() || match == text )
             return;
 
         if ( mode != KGlobalSettings::CompletionShell )
@@ -427,8 +477,7 @@ void KLineEdit::setReadOnly(bool readOnly)
 
         if (d->clearButton) {
             d->clearButton->animateVisible(false);
-            KLineEditStyle *lestyle = dynamic_cast<KLineEditStyle *>(style());
-            if (lestyle) lestyle->overlap = 0;
+            d->overlap = 0;
         }
     }
     else
@@ -443,8 +492,7 @@ void KLineEdit::setReadOnly(bool readOnly)
         if (d->clearButton && !text().isEmpty()) {
             int buttonWidth = d->clearButton->sizeHint().width();
             d->clearButton->animateVisible(true);
-            KLineEditStyle *lestyle = dynamic_cast<KLineEditStyle *>(style());
-            if (lestyle) lestyle->overlap = buttonWidth;
+            d->overlap = buttonWidth;
         }
     }
 }
@@ -599,7 +647,7 @@ void KLineEdit::resizeEvent( QResizeEvent * ev )
 
 void KLineEdit::keyPressEvent( QKeyEvent *e )
 {
-    int key = e->key() | e->modifiers();
+    const int key = e->key() | e->modifiers();
 
     if ( KStandardShortcut::copy().contains( key ) )
     {
@@ -684,7 +732,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
     if ( echoMode() == QLineEdit::Normal &&
          completionMode() != KGlobalSettings::CompletionNone )
     {
-        KeyBindingMap keys = getKeyBindings();
+        const KeyBindingMap keys = getKeyBindings();
         KGlobalSettings::Completion mode = completionMode();
         bool noModifier = (e->modifiers() == Qt::NoButton ||
                            e->modifiers() == Qt::ShiftModifier ||
@@ -782,11 +830,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                     if (e->key() == Qt::Key_Delete )
                         d->autoSuggest=false;
 
-                    if ( emitSignals() )
-                        emit completion( txt );
-
-                    if ( handleSignals() )
-                        makeCompletion( txt );
+                    d->doCompletion(txt);
 
                     if(  (e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete) )
                         d->autoSuggest=true;
@@ -860,12 +904,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                 if ( d->completionBox )
                   d->completionBox->setCancelledText( txt );
 
-                if ( emitSignals() )
-                  emit completion( txt ); // emit when requested...
-
-                if ( handleSignals() ) {
-                  makeCompletion( txt );  // handle when requested...
-                }
+                d->doCompletion(txt);
 
                 if ( (e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete ) &&
                     mode == KGlobalSettings::CompletionPopupAuto )
@@ -896,10 +935,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                 int len = txt.length();
                 if ( cursorPosition() == len && len != 0 )
                 {
-                    if ( emitSignals() )
-                        emit completion( txt );
-                    if ( handleSignals() )
-                        makeCompletion( txt );
+                    d->doCompletion(txt);
                     return;
                 }
             }
@@ -1045,20 +1081,23 @@ void KLineEdit::tripleClickTimeout()
 QMenu* KLineEdit::createStandardContextMenu()
 {
     QMenu *popup = QLineEdit::createStandardContextMenu();
-    
+
     if( !isReadOnly() )
     {
+        // FIXME: This code depends on Qt's action ordering.
         QList<QAction *> actionList = popup->actions();
-        enum { UndoAct, RedoAct, CutAct, CopyAct, PasteAct, ClearAct, SelectAllAct, NCountActs }; 
+        enum { UndoAct, RedoAct, Separator1, CutAct, CopyAct, PasteAct, DeleteAct, ClearAct,
+               Separator2, SelectAllAct, NCountActs };
         QAction *separatorAction = 0L;
-        int idx = actionList.indexOf( actionList[SelectAllAct] ) + 1;
+        // separator we want is right after Delete right now.
+        int idx = actionList.indexOf( actionList[DeleteAct] ) + 1;
         if ( idx < actionList.count() )
-            separatorAction = actionList.at( idx );         
+            separatorAction = actionList.at( idx );
         if ( separatorAction )
         {
             KAction *clearAllAction = KStandardAction::clear( this, SLOT( clear() ), this) ;
             if ( text().isEmpty() )
-                clearAllAction->setEnabled( false );    
+                clearAllAction->setEnabled( false );
             popup->insertAction( separatorAction, clearAllAction );
         }
     }
@@ -1070,19 +1109,19 @@ QMenu* KLineEdit::createStandardContextMenu()
     // menu item.
     if ( compObj() && !isReadOnly() && KAuthorized::authorize("lineedit_text_completion") )
     {
-        QMenu *subMenu = popup->addMenu( KIcon("text-completion"), i18n("Text Completion") );
+        QMenu *subMenu = popup->addMenu( KIcon("text-completion"), i18nc("@title:menu", "Text Completion") );
         connect( subMenu, SIGNAL( triggered ( QAction* ) ),
                  this, SLOT( completionMenuActivated( QAction* ) ) );
 
         popup->addSeparator();
 
         QActionGroup* ag = new QActionGroup( this );
-        d->noCompletionAction = ag->addAction( i18n("None"));
-        d->shellCompletionAction = ag->addAction( i18n("Manual") );
-        d->autoCompletionAction = ag->addAction( i18n("Automatic") );
-        d->popupCompletionAction = ag->addAction( i18n("Dropdown List") );
-        d->shortAutoCompletionAction = ag->addAction( i18n("Short Automatic") );
-        d->popupAutoCompletionAction = ag->addAction( i18n("Dropdown List && Automatic"));
+        d->noCompletionAction = ag->addAction( i18nc("@item:inmenu Text Completion", "None"));
+        d->shellCompletionAction = ag->addAction( i18nc("@item:inmenu Text Completion", "Manual") );
+        d->autoCompletionAction = ag->addAction( i18nc("@item:inmenu Text Completion", "Automatic") );
+        d->popupCompletionAction = ag->addAction( i18nc("@item:inmenu Text Completion", "Dropdown List") );
+        d->shortAutoCompletionAction = ag->addAction( i18nc("@item:inmenu Text Completion", "Short Automatic") );
+        d->popupAutoCompletionAction = ag->addAction( i18nc("@item:inmenu Text Completion", "Dropdown List && Automatic"));
         subMenu->addActions( ag->actions() );
 
         //subMenu->setAccel( KStandardShortcut::completion(), ShellCompletion );
@@ -1104,7 +1143,7 @@ QMenu* KLineEdit::createStandardContextMenu()
         if ( mode != KGlobalSettings::completionMode() )
         {
             subMenu->addSeparator();
-            d->defaultAction = subMenu->addAction( i18n("Default") );
+            d->defaultAction = subMenu->addAction( i18nc("@item:inmenu Text Completion", "Default") );
         }
     }
 
@@ -1173,10 +1212,9 @@ void KLineEdit::completionMenuActivated( QAction  *act)
 
 void KLineEdit::dropEvent(QDropEvent *e)
 {
-    KUrl::List urlList;
     if( d->handleURLDrops )
     {
-        urlList = KUrl::List::fromMimeData( e->mimeData() );
+        const KUrl::List urlList = KUrl::List::fromMimeData( e->mimeData() );
         if ( !urlList.isEmpty() )
         {
             QString dropText = text();
@@ -1205,13 +1243,14 @@ bool KLineEdit::event( QEvent* ev )
     if ( ev->type() == QEvent::ShortcutOverride )
     {
         QKeyEvent *e = static_cast<QKeyEvent *>( ev );
-        if (overrideAccel (e))
-        {
+        if (d->overrideShortcut(e)) {
             ev->accept();
         }
     }
     else if( ev->type() == QEvent::KeyPress )
     {
+        // Hmm -- all this could be done in keyPressEvent too...
+
         QKeyEvent *e = static_cast<QKeyEvent *>( ev );
 
         if( e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter )
@@ -1241,6 +1280,17 @@ bool KLineEdit::event( QEvent* ev )
             // Eat the event if the user asked for it, or if a completionbox was visible
             if (stopEvent)
                 return true;
+        } else if (e->key() == Qt::Key_Tab && e->modifiers() == Qt::NoButton ) {
+            // #65877: Key_Tab should complete using the first (or selected) item, and then offer completions again
+            const bool tabHandling = d->completionBox && d->completionBox->isVisible() && d->completionBox->count() > 0;
+            if (tabHandling) {
+                QListWidgetItem* currentItem = d->completionBox->currentItem();
+                const QString txt = currentItem ? currentItem->text() : d->completionBox->item(0)->text();
+                setTextWorkaround(txt);
+                d->doCompletion(txt);
+                e->accept();
+                return true;
+            }
         }
     }
     return QLineEdit::event( ev );
@@ -1314,33 +1364,33 @@ void KLineEdit::userCancelled(const QString & cancelText)
     }
 }
 
-bool KLineEdit::overrideAccel (const QKeyEvent* e)
+bool KLineEditPrivate::overrideShortcut(const QKeyEvent* e)
 {
     KShortcut scKey;
 
     int key = e->key() | e->modifiers();
-    KeyBindingMap keys = getKeyBindings();
+    const KLineEdit::KeyBindingMap keys = q->getKeyBindings();
 
-    if (keys[TextCompletion].isEmpty())
+    if (keys[KLineEdit::TextCompletion].isEmpty())
         scKey = KStandardShortcut::shortcut(KStandardShortcut::TextCompletion);
     else
-        scKey = keys[TextCompletion];
+        scKey = keys[KLineEdit::TextCompletion];
 
     if (scKey.contains( key ))
         return true;
 
-    if (keys[NextCompletionMatch].isEmpty())
+    if (keys[KLineEdit::NextCompletionMatch].isEmpty())
         scKey = KStandardShortcut::shortcut(KStandardShortcut::NextCompletion);
     else
-        scKey = keys[NextCompletionMatch];
+        scKey = keys[KLineEdit::NextCompletionMatch];
 
     if (scKey.contains( key ))
         return true;
 
-    if (keys[PrevCompletionMatch].isEmpty())
+    if (keys[KLineEdit::PrevCompletionMatch].isEmpty())
         scKey = KStandardShortcut::shortcut(KStandardShortcut::PrevCompletion);
     else
-        scKey = keys[PrevCompletionMatch];
+        scKey = keys[KLineEdit::PrevCompletionMatch];
 
     if (scKey.contains( key ))
         return true;
@@ -1369,9 +1419,9 @@ bool KLineEdit::overrideAccel (const QKeyEvent* e)
     else if (KStandardShortcut::endOfLine().contains( key ))
         return true;
 
-    if (d->completionBox && d->completionBox->isVisible ())
+    if (completionBox && completionBox->isVisible ())
     {
-        int key = e->key();
+        const int key = e->key();
         Qt::KeyboardModifiers modifiers = e->modifiers();
         if ((key == Qt::Key_Backtab || key == Qt::Key_Tab) &&
             (modifiers == Qt::NoModifier || (modifiers & Qt::ShiftModifier)))
@@ -1415,7 +1465,7 @@ void KLineEdit::setCompletedItems( const QStringList& items, bool autoSuggest )
 
             d->completionBox->setItems( items );
 
-            QList<QListWidgetItem*> matchedItems = d->completionBox->findItems( currentSelection , Qt::MatchExactly);
+            const QList<QListWidgetItem*> matchedItems = d->completionBox->findItems( currentSelection , Qt::MatchExactly);
             QListWidgetItem* matchedItem = matchedItems.isEmpty() ? 0 : matchedItems.first();
 
             // If no item is selected, that means the listbox hasn't been manipulated by the user yet,
@@ -1525,7 +1575,7 @@ void KLineEdit::clear()
 
 void KLineEdit::setTextWorkaround( const QString& text )
 {
-    if (!text.isNull())
+    if (!text.isEmpty())
     {
         setText( text );
         end( false ); // force cursor at end
@@ -1547,6 +1597,16 @@ bool KLineEdit::autoSuggest() const
 
 void KLineEdit::paintEvent( QPaintEvent *ev )
 {
+    if (echoMode() == Password && d->threeStars) {
+        QString oldText = text();
+        bool isModifiedState = isModified(); // save modified state because setText resets it
+        setText(text() + text() + text());
+        QLineEdit::paintEvent(ev);
+        setText(oldText);
+        setModified(isModifiedState);
+        return;
+    }
+
     QLineEdit::paintEvent( ev );
 
     if ( d->enableClickMsg && d->drawClickMsg && !hasFocus() && text().isEmpty() ) {
@@ -1616,9 +1676,10 @@ void KLineEdit::setPasswordMode(bool b)
         const QString val = cg.readEntry("EchoMode", "OneStar");
         if (val == "NoEcho")
             setEchoMode(NoEcho);
-        else
+        else {
+            d->threeStars = (val == "ThreeStars");
             setEchoMode(Password);
-       //KDE3 has also a "ThreeStars" mode, but QLineEdit doesn't support it.
+        }
     }
     else
     {
@@ -1631,6 +1692,17 @@ bool KLineEdit::passwordMode() const
     return echoMode() == NoEcho || echoMode() == Password;
 }
 
+void KLineEditPrivate::doCompletion(const QString& txt)
+{
+    if (q->emitSignals()) {
+        emit q->completion(txt); // emit when requested...
+    }
+
+    if (q->handleSignals()) {
+        q->makeCompletion(txt);  // handle when requested...
+    }
+}
+
 #include "klineedit.moc"
 #include "klineedit_p.moc"
 
