@@ -7,6 +7,7 @@
      (C) 1999 Chris Schlaeger (cs@kde.org)
      (C) 2002 Joseph Wenninger (jowenn@kde.org)
      (C) 2005-2006 Hamish Rodda (rodda@kde.org)
+     (C) 2000-2008 David Faure (faure@kde.org)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -58,6 +59,7 @@
 #include <ktoolbar.h>
 #include <kwindowsystem.h>
 #include <kconfiggroup.h>
+#include <kglobalsettings.h>
 
 #if defined Q_WS_X11
 #include <qx11info_x11.h>
@@ -115,6 +117,11 @@ bool DockResizeListener::eventFilter(QObject *watched, QEvent *event)
     if (event->type() == QEvent::Resize) {
         m_win->setSettingsDirty();
     }
+#ifdef Q_WS_WIN
+    if (event->type() == QEvent::Move) {
+        m_win->setSettingsDirty();
+    }
+#endif
     return QObject::eventFilter(watched, event);
 }
 
@@ -217,8 +224,12 @@ KMainWindow::KMainWindow(KMainWindowPrivate &dd, QWidget *parent, Qt::WFlags f)
 
 void KMainWindowPrivate::init(KMainWindow *_q)
 {
+    KGlobal::ref();
+
     q = _q;
 
+    q->setAnimated(KGlobalSettings::graphicEffectsLevel() & KGlobalSettings::SimpleAnimationEffects);
+
     q->setAttribute( Qt::WA_DeleteOnClose );
 
     KWhatsThisManager::init ();
@@ -227,6 +238,8 @@ void KMainWindowPrivate::init(KMainWindow *_q)
 
     //actionCollection()->setWidget( this );
     QObject::connect(qApp, SIGNAL(aboutToQuit()), q, SLOT(_k_shuttingDown()));
+    QObject::connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)),
+                     q, SLOT(_k_slotSettingsChanged(int)));
 
     // force KMWSessionManager creation - someone a better idea?
     ksm->dummyInit();
@@ -529,6 +542,9 @@ void KMainWindow::closeEvent ( QCloseEvent *e )
                 // cancel closing, it's stupid to end up with no windows at all....
                 e->ignore();
             }
+        } else {
+            // not the last window, simple deref
+            KGlobal::deref();
         }
     } else e->ignore(); //if the window should not be closed, don't close it
 }
@@ -608,30 +624,27 @@ void KMainWindow::saveMainWindowSettings(const KConfigGroup &_cg)
            cg.writeEntry("MenuBar", mb->isHidden() ? "Disabled" : "Enabled");
     }
 
-    QString configGroup = cg.name();
-
     // Utilise the QMainWindow::saveState() functionality
     QByteArray state = saveState();
     cg.writeEntry("State", state.toBase64());
     // One day will need to save the version number, but for now, assume 0
 
-    if ( !autoSaveSettings() || configGroup == autoSaveGroup() ) {
+    if ( !autoSaveSettings() || cg.name() == autoSaveGroup() ) { // TODO should be cg == d->autoSaveGroup, to compare both kconfig and group name
         if(!cg.hasDefault("ToolBarsMovable") && !KToolBar::toolBarsLocked())
             cg.revertToDefault("ToolBarsMovable");
         else
-            cg.writeEntry("ToolBarsMovable", KToolBar::toolBarsLocked() ? "Disabled" : "Enabled");    
+            cg.writeEntry("ToolBarsMovable", KToolBar::toolBarsLocked() ? "Disabled" : "Enabled");
     }
 
     int n = 1; // Toolbar counter. toolbars are counted from 1,
     foreach (KToolBar* toolbar, toolBars()) {
-        QString group;
+        QString group("Toolbar");
         // Give a number to the toolbar, but prefer a name if there is one,
         // because there's no real guarantee on the ordering of toolbars
-        group = (toolbar->objectName().isEmpty() ? QString::number(n) : QString(" ")+toolbar->objectName());
-        group.prepend(" Toolbar");
-        group.prepend(configGroup);
-        KConfigGroup groupGrp(cg.config(), group);
-        toolbar->saveSettings(groupGrp);
+        group += (toolbar->objectName().isEmpty() ? QString::number(n) : QString(" ")+toolbar->objectName());
+
+        KConfigGroup toolbarGroup(&cg, group);
+        toolbar->saveSettings(toolbarGroup);
         n++;
     }
 }
@@ -655,7 +668,7 @@ bool KMainWindow::readPropertiesInternal( KConfig *config, int number )
     applyMainWindowSettings(cg); // Menubar, statusbar and toolbar settings.
 
     s.setNum(number);
-    KConfigGroup grp(cg.config(), s);
+    KConfigGroup grp(config, s);
     readProperties(grp);
     return true;
 }
@@ -691,22 +704,10 @@ void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg, bool force)
         state = cg.readEntry("State", state);
         state = QByteArray::fromBase64(state);
         // One day will need to load the version number, but for now, assume 0
-#if QT_VERSION < 0x040400
-        //In older versions of Qt, the call to restoreState is causing bad flicker
-        //in the toolbars and menus.  This happens for instance when switching
-        //between tabs in Konqueror.  This is apparently fixed in Qt 4.4+ since it
-        //uses alien child windows.
-        setUpdatesEnabled(false);
         restoreState(state);
-        setUpdatesEnabled(true);
-#else
-        restoreState(state);
-#endif
     }
 
-    QString configGroup = cg.name();
-
-    if ( !autoSaveSettings() || configGroup == autoSaveGroup() ) {
+    if ( !autoSaveSettings() || cg.name() == autoSaveGroup() ) { // TODO should be cg == d->autoSaveGroup, to compare both kconfig and group name
         QString entry = cg.readEntry ("ToolBarsMovable", "Enabled");
         if ( entry == "Disabled" )
             KToolBar::setToolBarsLocked(true);
@@ -716,64 +717,111 @@ void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg, bool force)
 
     int n = 1; // Toolbar counter. toolbars are counted from 1,
     foreach (KToolBar* toolbar, toolBars()) {
-        QString group;
-        if (!configGroup.isEmpty())
-        {
-           // Give a number to the toolbar, but prefer a name if there is one,
-           // because there's no real guarantee on the ordering of toolbars
-           group = (toolbar->objectName().isEmpty() ? QString::number(n) : QString(" ")+toolbar->objectName());
-           group.prepend(" Toolbar");
-           group.prepend(configGroup);
-        }
-        KConfigGroup configGroup(cg.config(), group);
-        toolbar->applySettings(configGroup, force);
+        QString group("Toolbar");
+        // Give a number to the toolbar, but prefer a name if there is one,
+        // because there's no real guarantee on the ordering of toolbars
+        group += (toolbar->objectName().isEmpty() ? QString::number(n) : QString(" ")+toolbar->objectName());
+
+        KConfigGroup toolbarGroup(&cg, group);
+        toolbar->applySettings(toolbarGroup, force);
         n++;
     }
 
     d->settingsDirty = false;
 }
 
+#ifdef Q_WS_WIN
+
+/*
+ The win32 implementation for restoring/savin windows size differs
+ from the unix/max implementation in three topics:
+
+1. storing and restoring the position, which may not work on x11
+    see http://doc.trolltech.com/4.3/geometry.html#x11-peculiarities
+2. using QWidget::saveGeometry() and QWidget::restoreGeometry()
+    this would probably be usable on x11 and/or on mac, but I'm unable to
+    check this on unix/mac, so I leave this fix to the x11/mac experts.
+3. store geometry separately for each resolution -> on unix/max the size
+    and with of the window are already saved separately on non windows
+    system although not using ...Geometry functions -> could also be
+    fixed by x11/mac experts.
+*/
+void KMainWindow::restoreWindowSize( const KConfigGroup & _cg )
+{
+    K_D(KMainWindow);
+
+    int scnum = QApplication::desktop()->screenNumber(parentWidget());
+    QRect desk = QApplication::desktop()->screenGeometry(scnum);
+
+    // if the desktop is virtual then use virtual screen size
+    if (QApplication::desktop()->isVirtualDesktop())
+      desk = QApplication::desktop()->screenGeometry(QApplication::desktop()->screen());
+
+    QString geometryKey = QString::fromLatin1("geometry-%1-%2").arg(desk.width()).arg(desk.height());
+    QByteArray geometry = _cg.readEntry( geometryKey, QByteArray() );
+    // if first time run, center window
+    if (!restoreGeometry( QByteArray::fromBase64(geometry) ))
+        move( (desk.width()-width())/2, (desk.height()-height())/2 );
+}
+
 void KMainWindow::saveWindowSize( const KConfigGroup & _cg ) const
 {
-  K_D(const KMainWindow);
-  int scnum = QApplication::desktop()->screenNumber(parentWidget());
-  QRect desk = QApplication::desktop()->screenGeometry(scnum);
+    K_D(const KMainWindow);
+    int scnum = QApplication::desktop()->screenNumber(parentWidget());
+    QRect desk = QApplication::desktop()->screenGeometry(scnum);
 
-  // if the desktop is virtual then use virtual screen size
-  if (QApplication::desktop()->isVirtualDesktop())
-    desk = QApplication::desktop()->screenGeometry(QApplication::desktop()->screen());
+    // if the desktop is virtual then use virtual screen size
+    if (QApplication::desktop()->isVirtualDesktop())
+        desk = QApplication::desktop()->screenGeometry(QApplication::desktop()->screen());
 
-  int w, h;
-#if defined Q_WS_X11
-  // save maximalization as desktop size + 1 in that direction
-  KWindowInfo info = KWindowSystem::windowInfo( winId(), NET::WMState );
-  w = info.state() & NET::MaxHoriz ? desk.width() + 1 : width();
-  h = info.state() & NET::MaxVert ? desk.height() + 1 : height();
+    // geometry is saved separately for each resolution
+    QString geometryKey = QString::fromLatin1("geometry-%1-%2").arg(desk.width()).arg(desk.height());
+    QByteArray geometry = saveGeometry();
+    KConfigGroup cg(_cg);
+    cg.writeEntry( geometryKey, geometry.toBase64() );
+}
 #else
-  if (isMaximized()) {
-    w = desk.width() + 1;
-    h = desk.height() + 1;
-  } else {
-    w = width();
-    h = height();
-  }
-  //TODO: add "Maximized" property instead "+1" hack
+void KMainWindow::saveWindowSize( const KConfigGroup & _cg ) const
+{
+    K_D(const KMainWindow);
+    int scnum = QApplication::desktop()->screenNumber(parentWidget());
+    QRect desk = QApplication::desktop()->screenGeometry(scnum);
+
+    // if the desktop is virtual then use virtual screen size
+    if (QApplication::desktop()->isVirtualDesktop())
+        desk = QApplication::desktop()->screenGeometry(QApplication::desktop()->screen());
+
+    int w, h;
+#if defined Q_WS_X11
+    // save maximalization as desktop size + 1 in that direction
+    KWindowInfo info = KWindowSystem::windowInfo( winId(), NET::WMState );
+    w = info.state() & NET::MaxHoriz ? desk.width() + 1 : width();
+    h = info.state() & NET::MaxVert ? desk.height() + 1 : height();
+#else
+    if (isMaximized()) {
+        w = desk.width() + 1;
+        h = desk.height() + 1;
+    } else {
+        w = width();
+        h = height();
+    }
+    //TODO: add "Maximized" property instead "+1" hack
 #endif
-  KConfigGroup cg(_cg);
+    KConfigGroup cg(_cg);
 
-  QRect size( desk.width(), w, desk.height(), h );
-  bool defaultSize = (size == d->defaultWindowSize);
-  QString widthString = QString::fromLatin1("Width %1").arg(desk.width());
-  QString heightString = QString::fromLatin1("Height %1").arg(desk.height());
-  if (!cg.hasDefault(widthString) && defaultSize)
-     cg.revertToDefault(widthString);
-  else
-     cg.writeEntry(widthString, w );
+    QRect size( desk.width(), w, desk.height(), h );
+    bool defaultSize = (size == d->defaultWindowSize);
+    QString widthString = QString::fromLatin1("Width %1").arg(desk.width());
+    QString heightString = QString::fromLatin1("Height %1").arg(desk.height());
+    if (!cg.hasDefault(widthString) && defaultSize)
+        cg.revertToDefault(widthString);
+    else
+        cg.writeEntry(widthString, w );
 
-  if (!cg.hasDefault(heightString) && defaultSize)
-     cg.revertToDefault(heightString);
-  else
-     cg.writeEntry(heightString, h );
+    if (!cg.hasDefault(heightString) && defaultSize)
+        cg.revertToDefault(heightString);
+    else
+        cg.writeEntry(heightString, h );
 }
 
 void KMainWindow::restoreWindowSize( const KConfigGroup & config )
@@ -783,7 +831,7 @@ void KMainWindow::restoreWindowSize( const KConfigGroup & config )
         parseGeometry(true);
     } else {
         // restore the size
-        int scnum = QApplication::desktop()->screenNumber(parentWidget());
+        const int scnum = QApplication::desktop()->screenNumber(parentWidget());
         QRect desk = QApplication::desktop()->screenGeometry(scnum);
 
         // if the desktop is virtual then use virtual screen size
@@ -792,20 +840,8 @@ void KMainWindow::restoreWindowSize( const KConfigGroup & config )
 
         if ( d->defaultWindowSize.isNull() ) // only once
           d->defaultWindowSize = QRect(desk.width(), width(), desk.height(), height()); // store default values
-        QSize size( config.readEntry( QString::fromLatin1("Width %1").arg(desk.width()), 0 ),
+        const QSize size( config.readEntry( QString::fromLatin1("Width %1").arg(desk.width()), 0 ),
                     config.readEntry( QString::fromLatin1("Height %1").arg(desk.height()), 0 ) );
-
-        if (size.isEmpty()) {
-            // try the KDE 2.0 way
-            size = QSize( config.readEntry( QLatin1String("Width"), 0 ),
-                          config.readEntry( QLatin1String("Height"), 0 ) );
-            if (!size.isEmpty()) {
-                KConfigGroup cgw = config;
-                // make sure the other resolutions don't get old settings
-                cgw.writeEntry( QLatin1String("Width"), 0 );
-                cgw.writeEntry( QLatin1String("Height"), 0 );
-            }
-        }
         if ( !size.isEmpty() ) {
 #ifdef Q_WS_X11
             int state = ( size.width() > desk.width() ? NET::MaxHoriz : 0 )
@@ -829,6 +865,7 @@ void KMainWindow::restoreWindowSize( const KConfigGroup & config )
         }
     }
 }
+#endif
 
 bool KMainWindow::initialGeometrySet() const
 {
@@ -869,15 +906,20 @@ bool KMainWindow::settingsDirty() const
 }
 
 void KMainWindow::setAutoSaveSettings( const QString & groupName, bool saveWindowSize )
+{
+    setAutoSaveSettings(KConfigGroup(KGlobal::config(), groupName), saveWindowSize);
+}
+
+void KMainWindow::setAutoSaveSettings( const KConfigGroup & group,
+                                       bool saveWindowSize )
 {
     K_D(KMainWindow);
     d->autoSaveSettings = true;
-    d->autoSaveGroup = groupName;
+    d->autoSaveGroup = group;
     d->autoSaveWindowSize = saveWindowSize;
 
-    KConfigGroup cg(KGlobal::config(), groupName);
     // Now read the previously saved settings
-    applyMainWindowSettings(cg);
+    applyMainWindowSettings(d->autoSaveGroup);
 }
 
 void KMainWindow::resetAutoSaveSettings()
@@ -897,7 +939,13 @@ bool KMainWindow::autoSaveSettings() const
 QString KMainWindow::autoSaveGroup() const
 {
     K_D(const KMainWindow);
-    return d->autoSaveGroup;
+    return d->autoSaveSettings ? d->autoSaveGroup.name() : QString();
+}
+
+KConfigGroup KMainWindow::autoSaveConfigGroup() const
+{
+    K_D(const KMainWindow);
+    return d->autoSaveSettings ? d->autoSaveGroup : KConfigGroup();
 }
 
 void KMainWindow::saveAutoSaveSettings()
@@ -905,9 +953,8 @@ void KMainWindow::saveAutoSaveSettings()
     K_D(KMainWindow);
     Q_ASSERT( d->autoSaveSettings );
     //kDebug(200) << "KMainWindow::saveAutoSaveSettings -> saving settings";
-    KConfigGroup cg(KGlobal::config(), d->autoSaveGroup);
-    saveMainWindowSettings(cg);
-    KGlobal::config()->sync();
+    saveMainWindowSettings(d->autoSaveGroup);
+    d->autoSaveGroup.sync();
     d->settingsDirty = false;
     if ( d->settingsTimer )
         d->settingsTimer->stop();
@@ -917,6 +964,9 @@ bool KMainWindow::event( QEvent* ev )
 {
     K_D(KMainWindow);
     switch( ev->type() ) {
+#ifdef Q_WS_WIN
+    case QEvent::Move:
+#endif
     case QEvent::Resize:
         if ( d->autoSaveWindowSize )
             setSettingsDirty();
@@ -1006,6 +1056,19 @@ void KMainWindowPrivate::_k_shuttingDown()
     }
 }
 
+void KMainWindowPrivate::_k_slotSettingsChanged(int category)
+{
+    Q_UNUSED(category);
+
+    // This slot will be called when the style KCM changes settings that need
+    // to be set on the already running applications.
+
+    // At this level (KMainWindow) the only thing we need to restore is the
+    // animations setting (whether the user wants builtin animations or not).
+
+    q->setAnimated(KGlobalSettings::graphicEffectsLevel() & KGlobalSettings::SimpleAnimationEffects);
+}
+
 KToolBar *KMainWindow::toolBar( const QString& name )
 {
     QString childName = name;
