@@ -158,7 +158,7 @@ public:
     /**
      * Retrieve the standard help menu.
      *
-     * It contains entires for the
+     * It contains entries for the
      * help system (activated by F1), an optional "What's This?" entry
      * (activated by Shift F1), an application specific dialog box,
      * and an "About KDE" dialog box.
@@ -166,7 +166,7 @@ public:
      * Example (adding a standard help menu to your application):
      * \code
      * KMenu *help = helpMenu( <myTextString> );
-     * menuBar()->insertItem( i18n("&Help"), help );
+     * menuBar()->addMenu( help );
      * \endcode
      *
      * @param aboutAppText The string that is used in the application
@@ -196,7 +196,7 @@ public:
      *
      * Example (adding a help menu to your application):
      * \code
-     * menuBar()->insertItem( i18n("&Help"), customHelpMenu() );
+     * menuBar()->addMenu( customHelpMenu() );
      * \endcode
      *
      * @param showWhatsThis Set this to @p false if you do not want to include
@@ -375,6 +375,14 @@ public:
     void setAutoSaveSettings( const QString & groupName = QLatin1String("MainWindow"),
                               bool saveWindowSize = true );
 
+    /**
+     * Overload that lets you specify a KConfigGroup.
+     * This allows the settings to be saved into another file than KGlobal::config().
+     * @since 4.1
+     */
+    void setAutoSaveSettings(const KConfigGroup & group,
+                             bool saveWindowSize = true);
+
     /**
      * Disable the auto-save-settings feature.
      * You don't normally need to call this, ever.
@@ -389,12 +397,23 @@ public:
 
     /**
      * @return the group used for setting-autosaving.
-     * Only meaningful if setAutoSaveSettings() was called.
+     * Only meaningful if setAutoSaveSettings(QString) was called.
      * This can be useful for forcing a save or an apply, e.g. before and after
      * using KEditToolbar.
+     *
+     * NOTE: you should rather use saveAutoSaveSettings() for saving or autoSaveConfigGroup() for loading.
+     * This method doesn't make sense if setAutoSaveSettings(KConfigGroup) was called.
      */
     QString autoSaveGroup() const;
 
+    /**
+     * @return the group used for setting-autosaving.
+     * Only meaningful if setAutoSaveSettings() was called.
+     * This can be useful for forcing an apply, e.g. after using KEditToolbar.
+     * @since 4.1
+     */
+    KConfigGroup autoSaveConfigGroup() const;
+
     /**
      * Read settings for statusbar, menubar and toolbar from their respective
      * groups in the config file and apply them.
@@ -432,7 +451,7 @@ public:
 
 public Q_SLOTS:
     /**
-     * Makes a KDE compliant caption.
+     * Makes a KDE compliant caption (window title).
      *
      * @param caption Your caption. @em Do @em not include the application name
      * in this string. It will be added automatically according to the KDE
@@ -658,7 +677,7 @@ protected Q_SLOTS:
     * void MyMainLevel::setupInterface()
     * {
     *   ..
-    *   menuBar()->insertItem( i18n("&Help"), customHelpMenu() );
+    *   menuBar()->addMenu( customHelpMenu() );
     *   ..
     * }
     *
@@ -699,6 +718,7 @@ protected:
     KMainWindowPrivate * const k_ptr;
 private:
     Q_PRIVATE_SLOT(k_func(), void _k_shuttingDown())
+    Q_PRIVATE_SLOT(k_func(), void _k_slotSettingsChanged(int))
 };
 
 #define RESTORE(type) { int n = 1;\
