@@ -31,6 +31,7 @@ extern "C" int xmlLoadExtDtdDefaultValue;
 #include <QtCore/QTextCodec>
 #include <QtCore/QFileInfo>
 #include <kshell.h>
+#include <kurl.h>
 #include <QtCore/QList>
 
 class MyPair {
@@ -89,13 +90,13 @@ int main(int argc, char **argv) {
     options.add("+xml", ki18n("The file to transform"));
 
     KAboutData aboutData( "meinproc4", "kio_help4", ki18n("XML-Translator" ),
-    "$Revision: 749213 $",
+    "$Revision: 828077 $",
     ki18n("KDE Translator for XML"));
 
-    KCmdLineArgs::init(argc, argv, &aboutData);
+    KCmdLineArgs::init(argc, argv, &aboutData, KCmdLineArgs::CmdLineArgKDE);
     KCmdLineArgs::addCmdLineOptions( options );
 
-    KComponentData ins("meinproc4");
+    KComponentData ins("kio_help4");
     KGlobal::locale();
 
 
@@ -135,12 +136,12 @@ int main(int argc, char **argv) {
         QString pwd_buffer = QDir::currentPath();
         QFileInfo file( args->arg( 0 ) );
 
-        QString catalogs;
-        catalogs += KStandardDirs::locate( "dtd", "customization/catalog" );
+        QByteArray catalogs;
+        catalogs += KUrl::fromLocalFile( KStandardDirs::locate( "dtd", "customization/catalog.xml" ) ).toEncoded();
         catalogs += ' ';
-        catalogs += KStandardDirs::locate( "dtd", "docbook/xml-dtd-4.1.2/docbook.cat" );
+        catalogs += KUrl::fromLocalFile( KStandardDirs::locate( "dtd", "docbook/xml-dtd-4.1.2/catalog.xml" ) ).toEncoded();
 
-        setenv( "SGML_CATALOG_FILES", QFile::encodeName( catalogs ).constData(), 1);
+        setenv( "XML_CATALOG_FILES", catalogs.constData(), 1 );
         QString exe;
 #if defined( XMLLINT )
         exe = XMLLINT;
@@ -153,7 +154,7 @@ int main(int argc, char **argv) {
         if ( QFileInfo( exe ).isExecutable() ) {
             QDir::setCurrent( file.absolutePath() );
             QString cmd = exe;
-            cmd += " --catalogs --valid --noout ";
+            cmd += " --valid --noout ";
 #ifdef Q_OS_WIN
             cmd += file.fileName();
 #else
