@@ -133,7 +133,7 @@ private:
     Q_PRIVATE_SLOT(d, void _k_initDeviceList())
     Q_PRIVATE_SLOT(d, void _k_deviceAdded(const QString&))
     Q_PRIVATE_SLOT(d, void _k_deviceRemoved(const QString&))
-    Q_PRIVATE_SLOT(d, void _k_deviceChanged(const QString&))
+    Q_PRIVATE_SLOT(d, void _k_itemChanged(const QString&))
     Q_PRIVATE_SLOT(d, void _k_reloadBookmarks())
     Q_PRIVATE_SLOT(d, void _k_storageSetupDone(Solid::ErrorType, QVariant))
     Q_PRIVATE_SLOT(d, void _k_storageTeardownDone(Solid::ErrorType, QVariant))
