@@ -6,6 +6,7 @@
                   1999,2000,2001,2002,2003 Carsten Pfeiffer <pfeiffer@kde.org>
                   2003 Clarence Dang <dang@kde.org>
                   2007 David Faure <faure@kde.org>
+                  2008 Rafael Fernández López <ereslibre@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -69,7 +70,9 @@ public:
         : boxLayout(0),
           labeledCustomWidget(0),
           bottomCustomWidget(0),
+          speedBarWidth(-1),
           inAccept(false),
+          dummyAdded(false),
           q(q)
     {
     }
@@ -80,10 +83,12 @@ public:
     void readConfig(const KConfigGroup &configGroup);
     void writeConfig(KConfigGroup &configGroup);
     void setNonExtSelection();
-    void setLocationText(const QString&);
+    void setLocationText(const KUrl&);
+    void setLocationText(const KUrl::List&);
     void appendExtension(KUrl &url);
     void updateLocationEditExtension(const QString &);
     void updateFilter();
+    void updateSplitterSize();
     KUrl::List& parseSelectedUrls();
     /**
      * Parses the string "line" for files. If line doesn't contain any ", the
@@ -105,23 +110,31 @@ public:
      * handles setting the locationEdit.
      */
     void multiSelectionChanged();
-    /**
-     * Returns the URL which represents the directory of \a url. If
-     * \a url already is a directory, then just \a url is returned.
-     */
-    static KUrl directoryUrl(const KUrl& url);
 
     /**
      * Returns the absolute version of the URL specified in locationEdit.
      */
     KUrl getCompleteUrl(const QString&) const;
 
+    /**
+     * Sets the dummy entry on the history combo box. If the dummy entry
+     * already exists, it is overwritten with this information.
+     */
+    void setDummyHistoryEntry(const QString& text, const QPixmap& icon = QPixmap(),
+                              bool usePreviousPixmapIfNull = true);
+
+    /**
+     * Removes the dummy entry of the history combo box.
+     */
+    void removeDummyHistoryEntry();
+
     // private slots
     void _k_slotLocationChanged( const QString& );
     void _k_urlEntered( const KUrl& );
     void _k_enterUrl( const KUrl& );
     void _k_enterUrl( const QString& );
     void _k_locationAccepted( const QString& );
+    void _k_locationActivated( const QString& );
     void _k_slotFilterChanged();
     void _k_fileHighlighted( const KFileItem& );
     void _k_fileSelected( const KFileItem& );
@@ -131,6 +144,7 @@ public:
     void _k_toggleSpeedbar( bool );
     void _k_toggleBookmarks( bool );
     void _k_slotAutoSelectExtClicked();
+    void _k_placesViewSplitterMoved();
 
     void addToRecentDocuments();
 
@@ -173,6 +187,11 @@ public:
 
     QStringList mimetypes; //the list of possible mimetypes to save as
 
+    // caches the speed bar width. This value will be updated when the splitter
+    // is moved. This allows us to properly set a value when the dialog itself
+    // is resized
+    int speedBarWidth;
+
     // indicates if the location edit should be kept or cleared when changing
     // directories
     bool keepLocation;
@@ -184,6 +203,8 @@ public:
     bool hasDefaultFilter; // necessary for the operationMode
     bool autoDirectoryFollowing;
     bool inAccept; // true between beginning and end of accept()
+    bool dummyAdded; // if the dummy item has been added. This prevents the combo from having a
+                     // blank item added when loaded
 
     KFileWidget::OperationMode operationMode;
 
@@ -200,6 +221,8 @@ public:
     KDirOperator *ops;
     KFileFilterCombo *filterWidget;
     KFileWidget* q;
+
+    KFilePlacesModel *model;
 };
 
 K_GLOBAL_STATIC(KUrl, lastDirectory) // to set the start path
@@ -258,8 +281,8 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     d->toolbar->setObjectName("KFileWidget::toolbar");
     d->toolbar->setMovable(false);
 
-    KFilePlacesModel *model = new KFilePlacesModel(this);
-    d->urlNavigator = new KUrlNavigator(model, d->directoryUrl(startDir), d->toolbar);
+    d->model = new KFilePlacesModel(this);
+    d->urlNavigator = new KUrlNavigator(d->model, startDir, d->toolbar);
     d->urlNavigator->setPlacesSelectorVisible(false);
 
     KUrl u;
@@ -346,7 +369,7 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
                                             "For instance, if the current location is file:/home/%1 clicking this "
                                             "button will take you to file:/home.</qt>",  KUser().loginName() ));
 
-        d->toolbar->addAction( coll->action( "back" ) );
+    d->toolbar->addAction( coll->action( "back" ) );
     coll->action( "back" )->setWhatsThis(i18n("Click this button to move backwards one step in the browsing history."));
     d->toolbar->addAction( coll->action( "forward" ) );
     coll->action( "forward" )->setWhatsThis(i18n("Click this button to move forward one step in the browsing history."));
@@ -418,6 +441,10 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     // the Location label/edit
     d->locationLabel = new QLabel(i18n("&Location:"), this);
     d->locationEdit = new KUrlComboBox(KUrlComboBox::Files, true, this);
+    // Properly let the dialog be resized (to smaller). Otherwise we could have
+    // huge dialogs that can't be resized to smaller (it would be as big as the longest
+    // item in this combo box). (ereslibre)
+    d->locationEdit->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLength);
     connect( d->locationEdit, SIGNAL( editTextChanged( const QString& ) ),
              SLOT( _k_slotLocationChanged( const QString& )) );
 
@@ -427,7 +454,7 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     KUrlCompletion *fileCompletionObj = new KUrlCompletion( KUrlCompletion::FileCompletion );
     QString dir = d->url.url(KUrl::AddTrailingSlash);
 
-    d->urlNavigator->setUrl( d->directoryUrl( dir ) );
+    d->urlNavigator->setUrl( dir );
 
     fileCompletionObj->setDir( dir );
     d->locationEdit->setCompletionObject( fileCompletionObj );
@@ -437,6 +464,8 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
 
     connect(d->locationEdit, SIGNAL( returnPressed( const QString&  )),
             this,  SLOT( _k_locationAccepted( const QString& ) ));
+    connect(d->locationEdit, SIGNAL( activated( const QString& )),
+            this,  SLOT( _k_locationActivated( const QString& ) ));
 
     // the Filter label/edit
     whatsThisText = i18n("<qt>This is the filter to apply to the file list. "
@@ -448,6 +477,10 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     d->filterLabel = new QLabel(i18n("&Filter:"), this);
     d->filterLabel->setWhatsThis(whatsThisText);
     d->filterWidget = new KFileFilterCombo(this);
+    // Properly let the dialog be resized (to smaller). Otherwise we could have
+    // huge dialogs that can't be resized to smaller (it would be as big as the longest
+    // item in this combo box). (ereslibre)
+    d->filterWidget->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLength);
     d->filterWidget->setWhatsThis(whatsThisText);
     d->filterLabel->setBuddy(d->filterWidget);
     connect(d->filterWidget, SIGNAL(filterChanged()), SLOT(_k_slotFilterChanged()));
@@ -606,36 +639,28 @@ void KFileWidget::slotOk()
 
     if ( (mode() & KFile::Directory) != KFile::Directory ) {
         if ( locationEditCurrentText.isEmpty() ) {
-            if (items.isEmpty() )
-            {
-                QString msg;
-                if ( d->operationMode == Saving )
-                    msg = i18n("Please specify the filename to save to.");
-                else
-                    msg = i18n("Please select the file to open.");
-                KMessageBox::information(this, msg);
+            // allow directory navigation by entering a path and pressing
+            // enter, by simply returning we will browse the new path
+            if (items.isEmpty())
                 return;
-            }
 
             // weird case: the location edit is empty, but there are
             // highlighted files
-            else {
-
-                bool multi = (mode() & KFile::Files) != 0;
-                QString endQuote = QLatin1String("\" ");
-                QString name, files;
-                foreach (KFileItem fileItem, items) {
-                    name = fileItem.name();
-                    if ( multi ) {
-                        name.prepend( QLatin1Char( '"' ) );
-                        name.append( endQuote );
-                    }
-
-                    files.append( name );
+            bool multi = (mode() & KFile::Files) != 0;
+            QString endQuote = QLatin1String("\" ");
+            QString name;
+            KUrl::List urlList;
+            foreach (const KFileItem &fileItem, items) {
+                name = fileItem.name();
+                if ( multi ) {
+                    name.prepend( QLatin1Char( '"' ) );
+                    name.append( endQuote );
                 }
-                d->setLocationText( files );
-                return;
+
+                urlList << fileItem.url();
             }
+            d->setLocationText( urlList );
+            return;
         }
     }
 
@@ -659,7 +684,7 @@ void KFileWidget::slotOk()
             else { // multi (dirs and/or files)
                 d->url = d->ops->url();
                 KUrl::List urlList;
-                foreach (KFileItem item, items) {
+                foreach (const KFileItem &item, items) {
                     urlList.append(item.url());
                 }
                 d->urlList = urlList;
@@ -667,8 +692,9 @@ void KFileWidget::slotOk()
         }
 
         KUrl url = KIO::NetAccess::mostLocalUrl(d->url,topLevelWidget());
-        if ( (mode() & KFile::LocalOnly) == KFile::LocalOnly &&
-             !url.isLocalFile() ) {
+        if ( ( (mode() & KFile::LocalOnly) == KFile::LocalOnly ) &&
+             !url.isLocalFile() )
+        {
 // ### after message freeze, add message for directories!
             KMessageBox::sorry( this,
                                 i18n("You can only select local files."),
@@ -681,7 +707,6 @@ void KFileWidget::slotOk()
         return;
     }
 
-
     KUrl selectedUrl;
 
     if ( (mode() & KFile::Files) == KFile::Files ) {// multiselection mode
@@ -710,8 +735,9 @@ void KFileWidget::slotOk()
     }
 
     KUrl url = KIO::NetAccess::mostLocalUrl(selectedUrl,topLevelWidget());
-    if ( (mode() & KFile::LocalOnly) == KFile::LocalOnly &&
-         !url.isLocalFile() ) {
+    if ( ( (mode() & KFile::LocalOnly) == KFile::LocalOnly ) &&
+         !url.isLocalFile() )
+    {
         KMessageBox::sorry( this,
                             i18n("You can only select local files."),
                             i18n("Remote Files Not Accepted") );
@@ -912,7 +938,8 @@ void KFileWidget::accept()
 
     KUrl::List list = selectedUrls();
     QList<KUrl>::const_iterator it = list.begin();
-    for ( ; it != list.end(); ++it ) {
+    int atmost = d->locationEdit->maxItems(); //don't add more items than necessary
+    for ( ; it != list.end() && atmost > 0; ++it ) {
         const KUrl& url = *it;
         // we strip the last slash (-1) because KUrlComboBox does that as well
         // when operating in file-mode. If we wouldn't , dupe-finding wouldn't
@@ -926,7 +953,11 @@ void KFileWidget::accept()
                 break;
             }
         }
+        //FIXME I don't think this works correctly when the KUrlComboBox has some default urls.
+        //KUrlComboBox should provide a function to add an url and rotate the existing ones, keeping
+        //track of maxItems, and we shouldn't be able to insert items as we please.
         d->locationEdit->insertItem( 1,file);
+        atmost--;
     }
 
     KSharedConfig::Ptr config = KGlobal::config();
@@ -949,17 +980,25 @@ void KFileWidget::accept()
 
 void KFileWidgetPrivate::_k_fileHighlighted(const KFileItem &i)
 {
-    if (!i.isNull() && i.isDir())
+    const bool modified = locationEdit->lineEdit()->isModified();
+    locationEdit->lineEdit()->setModified( false );
+
+    if ( ( !i.isNull() && i.isDir() ) ||
+         ( locationEdit->hasFocus() && !locationEdit->currentText().isEmpty() ) ) // don't disturb
         return;
 
     if ( (ops->mode() & KFile::Files) != KFile::Files ) {
-        if ( i.isNull() )
+        if ( i.isNull() ) {
+            if ( !modified ) {
+                setLocationText( KUrl() );
+            }
             return;
+        }
 
         url = i.url();
 
         if ( !locationEdit->hasFocus() ) { // don't disturb while editing
-            setLocationText( i.name() );
+            setLocationText( url );
         }
         emit q->fileHighlighted(url.url());
     }
@@ -968,6 +1007,8 @@ void KFileWidgetPrivate::_k_fileHighlighted(const KFileItem &i)
         multiSelectionChanged();
         emit q->selectionChanged();
     }
+
+    locationEdit->lineEdit()->selectAll();
 }
 
 void KFileWidgetPrivate::_k_fileSelected(const KFileItem &i)
@@ -976,11 +1017,12 @@ void KFileWidgetPrivate::_k_fileSelected(const KFileItem &i)
         return;
 
     if ( (ops->mode() & KFile::Files) != KFile::Files ) {
-        if ( i.isNull() )
+        if ( i.isNull() ) {
+            setLocationText( KUrl() );
             return;
+        }
 
-        url = i.url();
-        setLocationText( i.name() );
+        setLocationText( i.url() );
     }
     else {
         multiSelectionChanged();
@@ -994,45 +1036,124 @@ void KFileWidgetPrivate::_k_fileSelected(const KFileItem &i)
 // (d->ops->selectedItems()), but what can we do?
 void KFileWidgetPrivate::multiSelectionChanged()
 {
-    if ( locationEdit->hasFocus() ) // don't disturb
+    if ( locationEdit->hasFocus() && !locationEdit->currentText().isEmpty() ) // don't disturb
         return;
 
-    locationEdit->lineEdit()->setModified( false );
     const KFileItemList list = ops->selectedItems();
+
     if ( list.isEmpty() ) {
-        locationEdit->clearEditText();
+        setLocationText( KUrl() );
         return;
     }
 
     static const QString &begin = KGlobal::staticQString(" \"");
-    QString text;
-    foreach (KFileItem fileItem, list) {
-        text.append( begin ).append( fileItem.name() ).append( QLatin1Char( '"' ) );
+    KUrl::List urlList;
+    foreach (const KFileItem &fileItem, list) {
+        urlList << fileItem.url();
     }
 
-    setLocationText( text.trimmed() );
+    setLocationText( urlList );
 }
 
-KUrl KFileWidgetPrivate::directoryUrl(const KUrl& url)
-{
-    if (!url.isValid())
-        return url;
-    KFileItem item(S_IFDIR, KFileItem::Unknown, url);
-    item.refresh();
-    return item.isDir() ? url : url.upUrl();
-}
-
-void KFileWidgetPrivate::setLocationText( const QString& text )
+void KFileWidgetPrivate::setDummyHistoryEntry( const QString& text, const QPixmap& icon,
+                                               bool usePreviousPixmapIfNull )
 {
     // setCurrentItem() will cause textChanged() being emitted,
     // so slotLocationChanged() will be called. Make sure we don't clear
     // the KDirOperator's view-selection in there
     QObject::disconnect( locationEdit, SIGNAL( editTextChanged( const QString& ) ),
-                         q, SLOT( _k_slotLocationChanged( const QString& ) ) );
-    locationEdit->setCurrentIndex( 0 );
+                        q, SLOT( _k_slotLocationChanged( const QString& ) ) );
+
+    bool dummyExists = dummyAdded;
+
+    int cursorPosition = locationEdit->lineEdit()->cursorPosition();
+
+    if ( dummyAdded ) {
+        if ( !icon.isNull() ) {
+            locationEdit->setItemIcon( 0, icon );
+            locationEdit->setItemText( 0, text );
+        } else {
+            if ( !usePreviousPixmapIfNull ) {
+                locationEdit->setItemIcon( 0, QPixmap() );
+            }
+            locationEdit->setItemText( 0, text );
+        }
+    } else {
+        if ( !text.isEmpty() ) {
+            if ( !icon.isNull() ) {
+                locationEdit->insertItem( 0, icon, text );
+            } else {
+                if ( !usePreviousPixmapIfNull ) {
+                    locationEdit->insertItem( 0, QPixmap(), text );
+                } else {
+                    locationEdit->insertItem( 0, text );
+                }
+            }
+            dummyAdded = true;
+            dummyExists = true;
+        }
+    }
+
+    if ( dummyExists && !text.isEmpty() ) {
+        locationEdit->setCurrentIndex( 0 );
+    }
+
+    locationEdit->lineEdit()->setCursorPosition( cursorPosition );
+
     QObject::connect( locationEdit, SIGNAL( editTextChanged ( const QString& ) ),
-                      q, SLOT( _k_slotLocationChanged( const QString& )) );
-    locationEdit->setEditText( text );
+                    q, SLOT( _k_slotLocationChanged( const QString& )) );
+}
+
+void KFileWidgetPrivate::removeDummyHistoryEntry()
+{
+    if ( !dummyAdded || locationEdit->lineEdit()->isModified() ) {
+        return;
+    }
+
+    // setCurrentItem() will cause textChanged() being emitted,
+    // so slotLocationChanged() will be called. Make sure we don't clear
+    // the KDirOperator's view-selection in there
+    QObject::disconnect( locationEdit, SIGNAL( editTextChanged( const QString& ) ),
+                        q, SLOT( _k_slotLocationChanged( const QString& ) ) );
+
+    locationEdit->removeItem( 0 );
+    locationEdit->setCurrentIndex( -1 );
+    dummyAdded = false;
+
+    QObject::connect( locationEdit, SIGNAL( editTextChanged ( const QString& ) ),
+                    q, SLOT( _k_slotLocationChanged( const QString& )) );
+}
+
+void KFileWidgetPrivate::setLocationText( const KUrl& url )
+{
+    if ( !url.isEmpty() ) {
+        QPixmap mimeTypeIcon = KIconLoader::global()->loadMimeTypeIcon( KMimeType::iconNameForUrl( url ), KIconLoader::Small );
+        setDummyHistoryEntry( url.fileName(), mimeTypeIcon );
+    } else {
+        removeDummyHistoryEntry();
+    }
+
+    // don't change selection when user has clicked on an item
+    if ( operationMode == KFileWidget::Saving && !locationEdit->isVisible())
+       setNonExtSelection();
+}
+
+void KFileWidgetPrivate::setLocationText( const KUrl::List& urlList )
+{
+    if ( urlList.count() > 1 ) {
+        QString urls;
+        foreach (const KUrl &url, urlList) {
+            urls += QString( "\"%1\"" ).arg( url.fileName() ) + ' ';
+        }
+        urls = urls.left( urls.size() - 1 );
+
+        setDummyHistoryEntry( urls, QPixmap(), false );
+    } else if ( urlList.count() ) {
+        const QPixmap mimeTypeIcon = KIconLoader::global()->loadMimeTypeIcon( KMimeType::iconNameForUrl( urlList[0] ),  KIconLoader::Small );
+        setDummyHistoryEntry( urlList[0].fileName(), mimeTypeIcon );
+    } else {
+        removeDummyHistoryEntry();
+    }
 
     // don't change selection when user has clicked on an item
     if ( operationMode == KFileWidget::Saving && !locationEdit->isVisible())
@@ -1067,7 +1188,7 @@ void KFileWidgetPrivate::updateLocationWhatsThis()
 void KFileWidgetPrivate::initSpeedbar()
 {
     placesView = new KFilePlacesView( q );
-    placesView->setModel(new KFilePlacesModel(placesView));
+    placesView->setModel(model);
     placesView->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
 
     placesView->setObjectName( QLatin1String( "url bar" ) );
@@ -1097,6 +1218,8 @@ void KFileWidgetPrivate::initGUI()
     placesViewSplitter->setChildrenCollapsible(false);
     boxLayout->addWidget(placesViewSplitter);
 
+    QObject::connect(placesViewSplitter, SIGNAL(splitterMoved(int,int)), q, SLOT(_k_placesViewSplitterMoved()));
+
     vbox = new QVBoxLayout();
     vbox->setMargin(0);
     QWidget *vboxWidget = new QWidget();
@@ -1108,12 +1231,6 @@ void KFileWidgetPrivate::initGUI()
 
     lafBox = new QGridLayout();
 
-    // The default minimum width of the location editor and the filter widget
-    // is so huge, that it is no possible for the user to adjust the width
-    // of the speedbar, hence it will be reduced.
-    locationEdit->setMinimumWidth(40);
-    filterWidget->setMinimumWidth(40);
-
     lafBox->setSpacing(KDialog::spacingHint());
     lafBox->addWidget(locationLabel, 0, 0, Qt::AlignVCenter);
     lafBox->addWidget(locationEdit, 0, 1, Qt::AlignVCenter);
@@ -1180,9 +1297,10 @@ void KFileWidgetPrivate::_k_urlEntered(const KUrl& url)
     }
 
     bool blocked = locationEdit->blockSignals( true );
-    locationEdit->setCurrentIndex( 0 );
-    if ( keepLocation )
-        locationEdit->setEditText( filename );
+    if ( keepLocation ) {
+        locationEdit->changeUrl( 0, KIcon( KMimeType::iconNameForUrl( filename ) ), filename );
+        locationEdit->lineEdit()->setModified( true );
+    }
 
     locationEdit->blockSignals( blocked );
 
@@ -1200,10 +1318,17 @@ void KFileWidgetPrivate::_k_urlEntered(const KUrl& url)
 
 void KFileWidgetPrivate::_k_locationAccepted( const QString& url )
 {
-    q->setSelection( url );
+    ops->setCurrentItem( url );
     q->slotOk();
 }
 
+void KFileWidgetPrivate::_k_locationActivated( const QString& url )
+{
+    // the location has been activated by selecting an entry
+    // from the combo box of the location bar
+    q->setUrl( url );
+}
+
 void KFileWidgetPrivate::_k_enterUrl( const KUrl& url )
 {
     KUrl fixedUrl( url );
@@ -1241,17 +1366,19 @@ void KFileWidget::setSelection(const QString& url)
     /* we strip the first / from the path to avoid file://usr which means
      *  / on host usr
      */
-    KFileItem i(KFileItem::Unknown, KFileItem::Unknown, u, true );
+    KIO::UDSEntry entry;
+    bool res = KIO::NetAccess::stat(u, entry, this);
+    KFileItem i(entry, u);
     //    KFileItem i(u.path());
     kDebug(kfile_area) << "KFileItem " << u.path() << " " << i.isDir() << " " << u.isLocalFile() << " " << QFile::exists( u.path() );
-    if ( i.isDir() && u.isLocalFile() && QFile::exists( u.path() ) ) {
+    if ( res && i.isDir() && u.isLocalFile() && QFile::exists( u.path() ) ) {
         // trust isDir() only if the file is
         // local (we cannot stat non-local urls) and if it exists!
         // (as KFileItem does not check if the file exists or not
         // -> the statbuffer is undefined -> isDir() is unreliable) (Simon)
         setUrl(u, true);
     }
-    else {
+    else if ( res ) {
         QString filename = u.url();
         int sep = filename.lastIndexOf('/');
         if (sep >= 0) { // there is a / in it
@@ -1265,7 +1392,7 @@ void KFileWidget::setSelection(const QString& url)
             filename = u.fileName();
             kDebug(kfile_area) << "filename " << filename;
             d->selection = filename;
-            d->setLocationText( filename );
+            d->setLocationText( u );
 
             // tell the line edit that it has been edited
             // otherwise we won't know this was set by the user
@@ -1280,20 +1407,27 @@ void KFileWidget::setSelection(const QString& url)
         d->url = d->ops->url();
         d->url.addPath(filename);
     }
+    else {
+        d->setLocationText(url);
+        d->locationEdit->lineEdit()->setModified( true );
+    }
 }
 
 void KFileWidgetPrivate::_k_slotLoadingFinished()
 {
-    if ( !selection.isNull() )
+    if ( !selection.isEmpty() )
         ops->setCurrentItem( selection );
 }
 
 void KFileWidgetPrivate::_k_fileCompletion( const QString& match )
 {
-    if ( match.isEmpty() && ops->view() )
+    if ( match.isEmpty() && ops->view() ) {
         ops->view()->clearSelection();
-    else
+    } else {
         ops->setCurrentItem( match );
+        setDummyHistoryEntry( locationEdit->currentText(), KIconLoader::global()->loadMimeTypeIcon( KMimeType::iconNameForUrl( match ), KIconLoader::Small ), !locationEdit->currentText().isEmpty() );
+        locationEdit->setCompletedText( match );
+    }
 }
 
 void KFileWidgetPrivate::_k_slotLocationChanged( const QString& text )
@@ -1303,6 +1437,14 @@ void KFileWidgetPrivate::_k_slotLocationChanged( const QString& text )
     if ( text.isEmpty() && ops->view() )
         ops->view()->clearSelection();
 
+    if ( text.isEmpty() ) {
+        removeDummyHistoryEntry();
+    } else {
+        setDummyHistoryEntry( text );
+    }
+
+    ops->setCurrentItem( text );
+
     updateFilter();
 }
 
@@ -1450,6 +1592,13 @@ KUrl KFileWidget::baseUrl() const
     return d->ops->url();
 }
 
+void KFileWidget::resizeEvent(QResizeEvent* event)
+{
+    d->updateSplitterSize();
+
+    QWidget::resizeEvent(event);
+}
+
 void KFileWidget::showEvent(QShowEvent* event)
 {
     if ( !d->hasView ) { // delayed view-creation
@@ -1459,17 +1608,6 @@ void KFileWidget::showEvent(QShowEvent* event)
     }
     d->ops->clearHistory();
 
-    QList<int> sizes = d->placesViewSplitter->sizes();
-    if (sizes.count() == 2) {
-        // restore width of speedbar
-        KConfigGroup configGroup( KGlobal::config(), ConfigGroup );
-        const int speedbarWidth = configGroup.readEntry( SpeedbarWidth, d->placesView->sizeHintForColumn(0) );
-        const int availableWidth = width();
-        sizes[0] = speedbarWidth;
-        sizes[1] = availableWidth - speedbarWidth;
-        d->placesViewSplitter->setSizes( sizes );
-    }
-
     QWidget::showEvent(event);
 }
 
@@ -1530,7 +1668,7 @@ void KFileWidgetPrivate::readConfig( const KConfigGroup &configGroup)
     urlNavigator->setUrlEditable( !configGroup.readEntry(BreadcrumbNavigation, true) );
 
     int w1 = q->minimumSize().width();
-    int w2 = toolbar->sizeHint().width() + 10;
+    int w2 = toolbar->sizeHint().width();
     if (w1 < w2)
         q->setMinimumWidth(w2);
 }
@@ -1546,7 +1684,7 @@ void KFileWidgetPrivate::writeConfig(KConfigGroup &configGroup)
     const bool showSpeedbar = placesView && !placesView->isHidden();
     configGroup.writeEntry( ShowSpeedbar, showSpeedbar );
     if (showSpeedbar) {
-        QList<int> sizes = placesViewSplitter->sizes();
+        const QList<int> sizes = placesViewSplitter->sizes();
         Q_ASSERT( sizes.count() > 0 );
         configGroup.writeEntry( SpeedbarWidth, sizes[0] );
     }
@@ -1567,8 +1705,7 @@ void KFileWidgetPrivate::readRecentFiles( KConfig *kc )
                                              DefaultRecentURLsNumber ) );
     locationEdit->setUrls( cg.readPathEntry( RecentFiles, QStringList() ),
                            KUrlComboBox::RemoveBottom );
-    locationEdit->insertItem(0, QString()); // dummy item without pixmap
-    locationEdit->setCurrentIndex( 0 );
+    locationEdit->setCurrentIndex( -1 );
 }
 
 void KFileWidgetPrivate::saveRecentFiles( KConfig *kc )
@@ -1644,6 +1781,12 @@ void KFileWidgetPrivate::_k_slotAutoSelectExtClicked()
     updateLocationEditExtension (extension /* extension hasn't changed */);
 }
 
+void KFileWidgetPrivate::_k_placesViewSplitterMoved()
+{
+    const QList<int> sizes = placesViewSplitter->sizes();
+    speedBarWidth = sizes[0];
+}
+
 static QString getExtensionFromPatternList(const QStringList &patternList)
 {
     QString ret;
@@ -1905,6 +2048,25 @@ void KFileWidgetPrivate::updateFilter()
     }
 }
 
+// Updates the splitter size. This is necessary since we call to this method when the widget is
+// shown as well as when it is resized. This is also very important, because this will be
+// contained in other widget, which can try to resize this one, and make the places view be
+// wider than what the user wanted.
+void KFileWidgetPrivate::updateSplitterSize()
+{
+    QList<int> sizes = placesViewSplitter->sizes();
+    if (sizes.count() == 2) {
+        // restore width of speedbar
+        KConfigGroup configGroup( KGlobal::config(), ConfigGroup );
+        const int speedbarWidth = speedBarWidth == -1 ? configGroup.readEntry( SpeedbarWidth, placesView->sizeHintForColumn(0) )
+                                                      : speedBarWidth;
+        const int availableWidth = q->width();
+        sizes[0] = speedbarWidth + 1; // without this pixel, our places view is reduced 1 pixel each time is shown.
+        sizes[1] = availableWidth - speedbarWidth - 1;
+        placesViewSplitter->setSizes( sizes );
+    }
+}
+
 // applies only to a file that doesn't already exist
 void KFileWidgetPrivate::appendExtension (KUrl &url)
 {
@@ -1965,20 +2127,26 @@ void KFileWidgetPrivate::appendExtension (KUrl &url)
 void KFileWidgetPrivate::addToRecentDocuments()
 {
     int m = ops->mode();
+    int atmost = KRecentDocument::maximumItems();
+    //don't add more than we need. KRecentDocument::add() is pretty slow
 
     if ( m & KFile::LocalOnly ) {
-        QStringList files = q->selectedFiles();
+        const QStringList files = q->selectedFiles();
         QStringList::ConstIterator it = files.begin();
-        for ( ; it != files.end(); ++it )
+        for ( ; it != files.end() && atmost > 0; ++it ) {
             KRecentDocument::add( *it );
+            atmost--;
+        }
     }
 
     else { // urls
         KUrl::List urls = q->selectedUrls();
         KUrl::List::ConstIterator it = urls.begin();
-        for ( ; it != urls.end(); ++it ) {
-            if ( (*it).isValid() )
+        for ( ; it != urls.end() && atmost > 0; ++it ) {
+            if ( (*it).isValid() ) {
                 KRecentDocument::add( *it );
+                atmost--;
+            }
         }
     }
 }
