@@ -34,6 +34,7 @@
 #include "css_valueimpl.h"
 #include "css_ruleimpl.h"
 #include "css_stylesheetimpl.h"
+#include "css_mediaquery.h"
 #include "cssproperties.h"
 #include "cssvalues.h"
 #include "csshelper.h"
@@ -144,9 +145,13 @@ unsigned int CSSParser::defaultNamespace()
 
 void CSSParser::runParser(int length)
 {
+    // the flex scanner sometimes give invalid reads for any
+    // smaller padding - try e.g. css/invalid-rules-005.html
     data[length-1] = 0;
     data[length-2] = 0;
-    data[length-3] = ' ';
+    data[length-3] = 0;
+    data[length-4] = 0;
+    data[length-5] = ' ';
 
     yyTok = -1;
     block_nesting = 0;
@@ -165,7 +170,9 @@ void CSSParser::parseSheet( CSSStyleSheetImpl *sheet, const DOMString &string )
 {
     styleElement = sheet;
 
-    int length = string.length() + 3;
+    int length = string.length() + 5;
+    if (data) 
+        free( data );
     data = (unsigned short *)malloc( length *sizeof( unsigned short ) );
     memcpy( data, string.unicode(), string.length()*sizeof( unsigned short) );
 
@@ -186,14 +193,14 @@ CSSRuleImpl *CSSParser::parseRule( DOM::CSSStyleSheetImpl *sheet, const DOM::DOM
     styleElement = sheet;
 
     const char khtml_rule[] = "@-khtml-rule{";
-    int length = string.length() + 4 + strlen(khtml_rule);
+    int length = string.length() + 6 + strlen(khtml_rule);
     assert( !data );
     data = (unsigned short *)malloc( length *sizeof( unsigned short ) );
     for ( unsigned int i = 0; i < strlen(khtml_rule); i++ )
         data[i] = khtml_rule[i];
     memcpy( data + strlen( khtml_rule ), string.unicode(), string.length()*sizeof( unsigned short) );
     // qDebug("parse string = '%s'", QString::fromRawData( (const QChar *)data, length ).toLatin1().constData() );
-    data[length-4] = '}';
+    data[length-6] = '}';
 
     runParser(length);
 
@@ -226,14 +233,14 @@ bool CSSParser::parseValue( DOM::CSSStyleDeclarationImpl *declaration, int _id,
     styleElement = declaration->stylesheet();
 
     const char khtml_value[] = "@-khtml-value{";
-    int length = string.length() + 4 + strlen(khtml_value);
+    int length = string.length() + 6 + strlen(khtml_value);
     assert( !data );
     data = (unsigned short *)malloc( length *sizeof( unsigned short ) );
     for ( unsigned int i = 0; i < strlen(khtml_value); i++ )
         data[i] = khtml_value[i];
     memcpy( data + strlen( khtml_value ), string.unicode(), string.length()*sizeof( unsigned short) );
-    data[length-4] = '}';
     // qDebug("parse string = '%s'", QString::fromRawData( (const QChar *)data, length ).toLatin1().constData() );
+    data[length-6] = '}';
 
     id = _id;
     important = _important;
@@ -263,13 +270,13 @@ bool CSSParser::parseDeclaration( DOM::CSSStyleDeclarationImpl *declaration, con
     styleElement = declaration->stylesheet();
 
     const char khtml_decls[] = "@-khtml-decls{";
-    int length = string.length() + 4 + strlen(khtml_decls);
+    int length = string.length() + 6 + strlen(khtml_decls);
     assert( !data );
     data = (unsigned short *)malloc( length *sizeof( unsigned short ) );
     for ( unsigned int i = 0; i < strlen(khtml_decls); i++ )
         data[i] = khtml_decls[i];
     memcpy( data + strlen( khtml_decls ), string.unicode(), string.length()*sizeof( unsigned short) );
-    data[length-4] = '}';
+    data[length-6] = '}';
 
     runParser(length);
 
@@ -286,6 +293,38 @@ bool CSSParser::parseDeclaration( DOM::CSSStyleDeclarationImpl *declaration, con
     return ok;
 }
 
+bool CSSParser::parseMediaQuery(DOM::MediaListImpl* queries, const DOM::DOMString& string)
+{
+    if (string.isEmpty() || string.isNull()) {
+        return true;
+    }
+
+    mediaQuery = 0;
+    // can't use { because tokenizer state switches from mediaquery to initial state when it sees { token.
+    // instead insert one " " (which is S in parser.y)
+    const char khtml_queries[] = "@-khtml-mediaquery ";
+    int length = string.length() + 6 + strlen(khtml_queries);
+    if (data)
+        free( data );
+    data = (unsigned short *)malloc( length *sizeof( unsigned short ) );
+    for ( unsigned int i = 0; i < strlen(khtml_queries); i++ )
+        data[i] = khtml_queries[i];
+    memcpy( data + strlen( khtml_queries ), string.unicode(), string.length()*sizeof( unsigned short) );
+    data[length-6] = '}';
+
+    runParser(length);
+
+    bool ok = false;
+    if (mediaQuery) {
+        ok = true;
+        queries->appendMediaQuery(mediaQuery);
+        mediaQuery = 0;
+    }
+
+    return ok;
+}
+
+
 void CSSParser::addProperty( int propId, CSSValueImpl *value, bool important )
 {
     CSSProperty *prop = new CSSProperty;
@@ -368,6 +407,8 @@ bool CSSParser::validUnit( Value *value, int unitflags, bool strict )
     case CSSPrimitiveValue::CSS_GRAD:
     case CSSPrimitiveValue::CSS_HZ:
     case CSSPrimitiveValue::CSS_KHZ:
+    case CSSPrimitiveValue::CSS_DPI:
+    case CSSPrimitiveValue::CSS_DPCM:
     case CSSPrimitiveValue::CSS_DIMENSION:
     default:
         break;
@@ -2074,12 +2115,6 @@ static bool parseColor(int unit, const QString &name, QRgb& rgb)
     return false;
 }
 
-//Needed by the Canvas stuff --- it needs way of using rgba(), etc., parsing as well
-bool CSSParser::parseColor(const QString &name, QRgb& rgb)
-{
-    return ::parseColor(CSSPrimitiveValue::CSS_IDENT, name, rgb);
-}
-
 CSSPrimitiveValueImpl *CSSParser::parseColor()
 {
     return parseColorFromValue(valueList->current());
@@ -2407,10 +2442,12 @@ int DOM::CSSParser::lex( void *_yylval )
     case QEMS:
         length--;
     case GRADS:
+    case DPCM:
         length--;
     case DEGS:
     case RADS:
     case KHERZ:
+    case DPI:
         length--;
     case MSECS:
     case HERZ:
