@@ -74,7 +74,7 @@ void CallStackDock::displayStack(InterpreterContext* ic)
     m_view->setRowCount(ic->callStack.count());
 
     int row = 0;
-    foreach (CallStackEntry entry, ic->callStack)
+    foreach (const CallStackEntry &entry, ic->callStack)
     {
         int displayRow = ic->callStack.count() - row - 1; //Want newest entry on top
         QTableWidgetItem *function = new QTableWidgetItem(entry.name);
@@ -99,4 +99,14 @@ void CallStackDock::slotViewItem(QTableWidgetItem* item)
     emit displayScript(entry.doc.get(), entry.lineNumber);
 }
 
+KJS::ExecState* CallStackDock::selectedFrameContext()
+{
+    QList<QTableWidgetItem*> selected = m_view->selectedItems();
+    if (selected.isEmpty())
+        return 0;
+
+    return m_activeCtx->execContexts[m_view->rowCount() - m_view->row(selected[0]) - 1];
+    
+}
+
 }
