@@ -75,8 +75,7 @@ static QString cssPropertyName( const Identifier &p, bool& hadPixelPrefix )
     // syntax for accessing properties, which camel-cases them
     // and can add prefixes to produce things like pixelFoo
     QString prop = p.qstring();
-    int i = prop.length();
-    while ( --i ) {
+    for (int i = prop.length() - 1; i >= 0; --i) {
         char c = prop[i].toLatin1();
         if ( c >= 'A' && c <= 'Z' )
             prop.insert( i, '-' );
@@ -541,9 +540,10 @@ bool DOMMediaList::getOwnPropertySlot(ExecState *exec, const Identifier& propert
 
 void DOMMediaList::put(ExecState *exec, const Identifier &propertyName, JSValue *value, int attr)
 {
-  if (propertyName == "mediaText")
-    m_impl->setMediaText(value->toString(exec).domString());
-  else
+  if (propertyName == "mediaText") {
+    DOMExceptionTranslator exception(exec);
+    m_impl->setMediaText(value->toString(exec).domString(), exception);
+  } else
     DOMObject::put(exec, propertyName, value, attr);
 }
 
@@ -556,14 +556,15 @@ JSValue *DOMMediaListProtoFunc::callAsFunction(ExecState *exec, JSObject *thisOb
 {
   KJS_CHECK_THIS( KJS::DOMMediaList, thisObj );
   DOM::MediaListImpl& mediaList = *static_cast<DOMMediaList *>(thisObj)->impl();
+  DOMExceptionTranslator exception(exec);
   switch (id) {
     case DOMMediaList::Item:
       return jsString(mediaList.item(args[0]->toInteger(exec)));
     case DOMMediaList::DeleteMedium:
-      mediaList.deleteMedium(args[0]->toString(exec).domString());
+      mediaList.deleteMedium(args[0]->toString(exec).domString(), exception);
       return jsUndefined();
     case DOMMediaList::AppendMedium:
-      mediaList.appendMedium(args[0]->toString(exec).domString());
+      mediaList.appendMedium(args[0]->toString(exec).domString(), exception);
       return jsUndefined();
     default:
       return jsUndefined();
