@@ -33,6 +33,10 @@
 #include <html/html_baseimpl.h>
 #include <html/html_documentimpl.h>
 #include <html/html_miscimpl.h>
+#include "HTMLAudioElement.h"
+#include "HTMLVideoElement.h"
+#include "JSHTMLAudioElement.h"
+#include "JSHTMLVideoElement.h"
 #include <kdebug.h>
 #include <khtml_part.h>
 #include <QtCore/QList>
@@ -50,6 +54,7 @@
 #include <khtmlpart_p.h>
 
 using namespace KJS;
+using namespace khtml;
 
 // -------------------------------------------------------------------------
 /* Source for DOMNodeConstantsTable.
@@ -326,7 +331,7 @@ JSValue* DOMNode::getValueProperty(ExecState *exec, int token) const
     // Retrieves the ordinal position of the object, in source order, as the object
     // appears in the document's all collection
     // i.e. document.all[n.sourceIndex] == n
-    DOM::DocumentImpl* doc = node.getDocument();
+    DOM::DocumentImpl* doc = node.document();
     if (doc->isHTMLDocument()) {
       HTMLCollectionImpl all(doc, HTMLCollectionImpl::DOC_ALL);
       unsigned long i = 0;
@@ -342,7 +347,7 @@ JSValue* DOMNode::getValueProperty(ExecState *exec, int token) const
     // no DOM standard, found in IE only
 
     // Make sure our layout is up to date before we allow a query on these attributes.
-    DOM::DocumentImpl* docimpl = node.getDocument();
+    DOM::DocumentImpl* docimpl = node.document();
     if (docimpl) {
       docimpl->updateLayout();
     }
@@ -381,14 +386,14 @@ JSValue* DOMNode::getValueProperty(ExecState *exec, int token) const
     case ScrollLeft:
       if (rend && rend->layer()) {
           if (rend->isRoot() && !rend->style()->hidesOverflow())
-              return jsNumber( node.getDocument()->view() ? node.getDocument()->view()->contentsX() : 0);
+              return jsNumber( node.document()->view() ? node.document()->view()->contentsX() : 0);
           return jsNumber( rend->layer()->scrollXOffset() );
       }
       return jsNumber( 0 );
     case ScrollTop:
       if (rend && rend->layer()) {
           if (rend->isRoot() && !rend->style()->hidesOverflow())
-              return jsNumber( node.getDocument()->view() ? node.getDocument()->view()->contentsY() : 0);
+              return jsNumber( node.document()->view() ? node.document()->view()->contentsY() : 0);
           return jsNumber( rend->layer()->scrollYOffset() );
       }
       return jsNumber( 0 );
@@ -496,7 +501,7 @@ void DOMNode::putValueProperty(ExecState *exec, int token, JSValue* value, int /
     break;
   default:
     // Make sure our layout is up to date
-    DOM::DocumentImpl* docimpl = node.getDocument();
+    DOM::DocumentImpl* docimpl = node.document();
     if (docimpl)
       docimpl->updateLayout();
 
@@ -511,7 +516,7 @@ void DOMNode::putValueProperty(ExecState *exec, int token, JSValue* value, int /
           if (rend->style()->hidesOverflow())
             rend->layer()->scrollToXOffset(value->toInt32(exec));
           else if (rend->isRoot()) {
-            KHTMLView* sview = node.getDocument()->view();
+            KHTMLView* sview = node.document()->view();
             if (sview)
               sview->setContentsPos(value->toInt32(exec), sview->contentsY());
           }
@@ -522,7 +527,7 @@ void DOMNode::putValueProperty(ExecState *exec, int token, JSValue* value, int /
           if (rend->style()->hidesOverflow())
             rend->layer()->scrollToYOffset(value->toInt32(exec));
           else if (rend->isRoot()) {
-            KHTMLView* sview = node.getDocument()->view();
+            KHTMLView* sview = node.document()->view();
             if (sview)
               sview->setContentsPos(sview->contentsX(), value->toInt32(exec));
           }
@@ -578,8 +583,10 @@ JSValue* DOMNodeProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, co
       return jsBoolean(node.hasAttributes());
     case DOMNode::HasChildNodes:
       return jsBoolean(node.hasChildNodes());
-    case DOMNode::CloneNode:
-      return getDOMNode(exec,node.cloneNode(args[0]->toBoolean(exec)));
+    case DOMNode::CloneNode: {
+      RefPtr<NodeImpl> clone = node.cloneNode(args[0]->toBoolean(exec));
+      return getDOMNode(exec, clone.get());
+    }
     case DOMNode::Normalize:
       node.normalize();
       return jsUndefined();
@@ -632,7 +639,7 @@ JSValue* DOMNodeProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, co
     {
       // see http://www.faqts.com/knowledge_base/view.phtml/aid/5756
       // and http://msdn.microsoft.com/workshop/author/dhtml/reference/methods/insertAdjacentHTML.asp
-      SharedPtr<DOM::RangeImpl> range = node.getDocument()->createRange();
+      SharedPtr<DOM::RangeImpl> range = node.document()->createRange();
 
       range->setStartBefore(&node, exception);
       if (exception.triggered()) return jsUndefined();
@@ -892,6 +899,12 @@ AttrImpl *toAttr(JSValue *val)
   abort              DOMDocument::Abort                        DontDelete|Function 0
   load               DOMDocument::Load                         DontDelete|Function 1
   loadXML            DOMDocument::LoadXML                      DontDelete|Function 2
+  execCommand        DOMDocument::ExecCommand                  DontDelete|Function 3
+  queryCommandEnabled DOMDocument::QueryCommandEnabled         DontDelete|Function 1
+  queryCommandIndeterm DOMDocument::QueryCommandIndeterm       DontDelete|Function 1
+  queryCommandState DOMDocument::QueryCommandState             DontDelete|Function 1
+  queryCommandSupported DOMDocument::QueryCommandSupported     DontDelete|Function 1
+  queryCommandValue DOMDocument::QueryCommandValue             DontDelete|Function 1
   getElementsByClassName DOMDocument::GetElementsByClassName   DontDelete|Function 1
 @end
 */
@@ -1110,6 +1123,31 @@ JSValue* DOMDocumentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj
   case DOMDocument::LoadXML:
     doc.loadXML(s);
     break;
+  case DOMDocument::ExecCommand: {
+    return jsBoolean(doc.execCommand(args[0]->toString(exec).domString(), args[1]->toBoolean(exec), args[2]->toString(exec).domString()));
+  }
+  case DOMDocument::QueryCommandEnabled: {
+    return jsBoolean(doc.queryCommandEnabled(args[0]->toString(exec).domString()));
+  }
+  case DOMDocument::QueryCommandIndeterm: {
+    return jsBoolean(doc.queryCommandIndeterm(args[0]->toString(exec).domString()));
+  }
+  case DOMDocument::QueryCommandState: {
+    return jsBoolean(doc.queryCommandState(args[0]->toString(exec).domString()));
+  }
+  case DOMDocument::QueryCommandSupported: {
+    return jsBoolean(doc.queryCommandSupported(args[0]->toString(exec).domString()));
+  }
+  case DOMDocument::QueryCommandValue: {
+    DOM::DOMString commandValue(doc.queryCommandValue(args[0]->toString(exec).domString()));
+    // Method returns null DOMString to signal command is unsupported.
+    // Microsoft documentation for this method says:
+    // "If not supported [for a command identifier], this method returns a Boolean set to false."
+    if (commandValue.isNull())
+      return jsBoolean(false);
+    else
+      return jsString(commandValue);
+  }
   case DOMDocument::GetElementsByClassName:
     return getDOMNodeList(exec, doc.getElementsByClassName(s));
   default:
@@ -1347,9 +1385,13 @@ JSValue* DOMDOMImplementationProtoFunc::callAsFunction(ExecState *exec, JSObject
 
       DOM::DocumentTypeImpl* docType = static_cast<DOM::DocumentTypeImpl*>(supposedDocType);
 
+      // ### no real leak but lifetime of part is not tied to document
+      KHTMLPart* newPart = new KHTMLPart(part->view(), part);
       DOM::DocumentImpl* doc = implementation.createDocument(valueToStringWithNullCheck(exec, args[0]),
                                                              valueToStringWithNullCheck(exec, args[1]),
-                                                             docType, exception);
+                                                             docType,
+                                                             newPart->view(),
+                                                             exception);
       if (!doc)
         return jsNull();
       KUrl url = static_cast<DocumentImpl*>(part->document().handle())->URL();
@@ -1622,13 +1664,23 @@ bool KJS::checkNodeSecurity(ExecState *exec, const DOM::NodeImpl* n)
   // Check to see if the currently executing interpreter is allowed to access the specified node
   if (!n)
     return true;
-  KHTMLView *view = n->getDocument()->view();
+  KHTMLView *view = n->document()->view();
   Window* win = view && view->part() ? Window::retrieveWindow(view->part()) : 0L;
   if ( !win || !win->isSafeScript(exec) )
     return false;
   return true;
 }
 
+// adopted from binding/JSHTMLElementFactory.cpp
+#define CREATE_WRAPPER_FUNCTION(name) \
+static DOMObject* create##name##Wrapper(ExecState* exec, DOM::NodeImpl* n) \
+{ \
+    return new JSHTML##name##Element(exec, static_cast<HTML##name##Element*>(n)); \
+}
+
+CREATE_WRAPPER_FUNCTION(Audio)
+CREATE_WRAPPER_FUNCTION(Video)
+
 JSValue* KJS::getDOMNode(ExecState *exec, DOM::NodeImpl* n)
 {
   DOMObject *ret = 0;
@@ -1640,10 +1692,20 @@ JSValue* KJS::getDOMNode(ExecState *exec, DOM::NodeImpl* n)
 
   switch (n->nodeType()) {
     case DOM::Node::ELEMENT_NODE:
-      if (static_cast<DOM::ElementImpl*>(n)->isHTMLElement())
-        ret = new HTMLElement(exec, static_cast<DOM::HTMLElementImpl*>(n));
-      else
-        ret = new DOMElement(exec, static_cast<DOM::ElementImpl*>(n));
+      switch (n->id()) {
+      case ID_AUDIO:
+	ret = createAudioWrapper(exec, n);
+	break;
+      case ID_VIDEO:
+	ret = createVideoWrapper(exec, n);
+	break;
+      default:
+	if (static_cast<DOM::ElementImpl*>(n)->isHTMLElement())
+	  ret = new HTMLElement(exec, static_cast<DOM::HTMLElementImpl*>(n));
+	else
+	  ret = new DOMElement(exec, static_cast<DOM::ElementImpl*>(n));
+	break;
+      }
       break;
     case DOM::Node::ATTRIBUTE_NODE:
       ret = new DOMAttr(exec, static_cast<DOM::AttrImpl*>(n));
