@@ -148,12 +148,12 @@ void HTMLCanvasElementImpl::attach()
     assert(!m_render);
     assert(parentNode());
 
-    RenderStyle* _style = getDocument()->styleSelector()->styleForElement(this);
+    RenderStyle* _style = document()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && parentNode()->renderer()->childAllowed() &&
         _style->display() != NONE)
     {
-        m_render = new (getDocument()->renderArena()) RenderCanvasImage(this);
+        m_render = new (document()->renderArena()) RenderCanvasImage(this);
         m_render->setStyle(_style);
         parentNode()->renderer()->addChild(m_render, nextRenderer());
     }
@@ -1055,17 +1055,25 @@ inline bool CanvasContext2DImpl::needsShadow() const
     return activeState().shadowColor.alpha() > 0;
 }
 
-QRectF CanvasContext2DImpl::clipForRepeat(QPainter *p, PathPaintOp op) const
+QPainterPath CanvasContext2DImpl::clipForPatternRepeat(QPainter *p, PathPaintOp op) const
 {
     const CanvasStyleBaseImpl *style = op == DrawFill ?
                 activeState().fillStyle.get() : activeState().strokeStyle.get();
 
     if (style->type() != CanvasStyleBaseImpl::Pattern)
-        return QRectF();
+        return QPainterPath();
 
     const CanvasPatternImpl *pattern = static_cast<const CanvasPatternImpl*>(style);
-    QRectF fillBounds = activeState().transform.inverted().mapRect(QRectF(QPointF(), canvasImage->size()));
-    return pattern->clipForRepeat(p->brushOrigin(), fillBounds);
+    const QTransform &ctm = activeState().transform;
+    const QRectF fillBounds = ctm.inverted().mapRect(QRectF(QPointF(), canvasImage->size()));
+    const QRectF clipRect = pattern->clipForRepeat(p->brushOrigin(), fillBounds);
+
+    if (clipRect.isEmpty())
+        return QPainterPath();
+
+    QPainterPath path;
+    path.addRect(clipRect);
+    return path * ctm;
 }
 
 void CanvasContext2DImpl::drawPath(QPainter *p, const QPainterPath &path, const PathPaintOp op) const
@@ -1108,12 +1116,10 @@ void CanvasContext2DImpl::drawPath(QPainter *p, const QPainterPath &path, const
         drawPathWithShadow(p, fillPath, op);
     else
     {
-        QRectF repeatClip = clipForRepeat(p, op);
-        if (!repeatClip.isEmpty()) {
-            QPainterPath clipPath;
-            clipPath.addRect(repeatClip);
-            p->setClipPath(clipPath * state.transform, Qt::IntersectClip);
-        }
+        const QPainterPath repeatClip = clipForPatternRepeat(p, op);
+        if (!repeatClip.isEmpty())
+            p->setClipPath(repeatClip, Qt::IntersectClip);
+
         p->drawPath(fillPath);
     }
     p->restore();
@@ -1129,18 +1135,14 @@ void CanvasContext2DImpl::drawPathWithShadow(QPainter *p, const QPainterPath &pa
     if (radius > 7)
         radius = qMin(7 + std::pow(float(radius - 7.0), float(.7)), float(127.0));
 
-    qreal offset = radius * 2;
+    const qreal offset = radius * 2;
+    const QPainterPath repeatClip = (flags & NotUsingCanvasPattern) ?
+              QPainterPath() : clipForPatternRepeat(p, op);
 
-    bool honorRepeat = !(flags & NotUsingCanvasPattern);
-    QRectF repeatClip = clipForRepeat(p, op);
-    QPainterPath clipPath;
     QRect shapeBounds;
-
-    if (honorRepeat && !repeatClip.isEmpty()) {
-        clipPath.addRect(repeatClip);
-        clipPath = clipPath * state.transform;
-        shapeBounds = path.intersected(clipPath).controlPointRect().toAlignedRect();
-    } else
+    if (!repeatClip.isEmpty())
+        shapeBounds = path.intersected(repeatClip).controlPointRect().toAlignedRect();
+    else
         shapeBounds = path.controlPointRect().toAlignedRect();
 
     QRect clipRect;
@@ -1150,11 +1152,11 @@ void CanvasContext2DImpl::drawPathWithShadow(QPainter *p, const QPainterPath &pa
     } else
         clipRect = QRect(QPoint(), canvasImage->size());
 
-    QRect shadowRect = shapeBounds.translated(shadowOffsetX(), shadowOffsetY())
+    const QRect shadowRect = shapeBounds.translated(shadowOffsetX(), shadowOffsetY())
                         .adjusted(-offset, -offset, offset, offset) &
                         clipRect.adjusted(-offset, -offset, offset, offset);
 
-    QRect shapeRect = QRect(shapeBounds & clipRect) | shadowRect
+    const QRect shapeRect = QRect(shapeBounds & clipRect) | shadowRect
                         .translated(-shadowOffsetX(), -shadowOffsetY()) & shapeBounds;
 
     if (!shapeRect.isValid())
@@ -1173,8 +1175,8 @@ void CanvasContext2DImpl::drawPathWithShadow(QPainter *p, const QPainterPath &pa
     painter.setBrush(p->brush());
     painter.setPen(Qt::NoPen);
     painter.translate(-shapeRect.x(), -shapeRect.y());
-    if (honorRepeat && !repeatClip.isEmpty())
-        painter.setClipPath(clipPath);
+    if (!repeatClip.isEmpty())
+        painter.setClipPath(repeatClip);
     painter.drawPath(path);
     painter.end();
 
@@ -1284,9 +1286,8 @@ void CanvasContext2DImpl::arcTo(float x1, float y1, float x2, float y2, float ra
     float tDist = std::cos(theta / 2.0) * h;
 
     // As theta approaches 0, the distance to the two tangent points approach infinity.
-    // If we exceeded the limit, draw a long line toward the first tangent point.
-    // This matches CoreGraphics and Postscript behavior, but violates the HTML5 spec,
-    // which says we should do nothing in this case.
+    // If we exceeded the data type limit, draw a long line toward the first tangent point.
+    // This matches CoreGraphics and Postscript behavior.
     if (KJS::isInf(h) || KJS::isInf(tDist)) {
         QPointF point(line1.p2().x() + std::cos(angle1) * 1e10,
                       line1.p2().y() + std::sin(angle1) * 1e10);
@@ -1319,15 +1320,6 @@ void CanvasContext2DImpl::arc(float x, float y, float radius, float startAngle,
         return;
     }
 
-    QPainterPath arcPath;
-    bool wasEmpty = isPathEmpty();
-
-    if (wasEmpty) {
-        QPointF initialPoint(x + std::cos(startAngle) * radius,
-                             y + std::sin(startAngle) * radius);
-        arcPath.moveTo(initialPoint);
-    }
-
     const QRectF rect(x - radius, y - radius, radius * 2, radius * 2);
     float sweepLength = -degrees(endAngle - startAngle);
     startAngle = -degrees(startAngle);
@@ -1335,16 +1327,18 @@ void CanvasContext2DImpl::arc(float x, float y, float radius, float startAngle,
     if (counterClockWise && (sweepLength < 0 || sweepLength > 360))
     {
         sweepLength = 360 + std::fmod(sweepLength, float(360.0));
-        if (qFuzzyCompare(sweepLength, 0))
+        if (qFuzzyCompare(sweepLength + 1, 1))
             sweepLength = 360;
     }
     else if (!counterClockWise && (sweepLength > 0 || sweepLength < -360))
     {
         sweepLength = -(360 - std::fmod(sweepLength, float(360.0)));
-        if (qFuzzyCompare(sweepLength, 0))
+        if (qFuzzyCompare(sweepLength + 1, 1))
             sweepLength = 360;
     }
 
+    QPainterPath arcPath;
+    arcPath.arcMoveTo(rect, startAngle);
     arcPath.arcTo(rect, startAngle, sweepLength);
 
     // When drawing the arc, Safari will loop around the circle several times if
@@ -1368,29 +1362,23 @@ void CanvasContext2DImpl::arc(float x, float y, float radius, float startAngle,
         arcPath.arcTo(rect, startAngle, sweepLength);
     }
 
-    arcPath = arcPath * activeState().transform;
-
-    // Add the transformed arc to the path.
-    // Unfortunately we can't use QPainterPath::addPath(), since it adds the
-    // path as a closed sub path, and that's not what we want.
-    for (int i = 0; i < arcPath.elementCount();)
+    // Add the transformed arc to the path
+    if (isPathEmpty())
+        path.addPath(arcPath * activeState().transform);
+    else
     {
-        QPainterPath::Element e = arcPath.elementAt(i++);
-        switch (e.type)
-        {
-        case QPainterPath::MoveToElement:
-            if (i != 1 || wasEmpty)
-                path.moveTo(e);
-            break;
+        if (path.elementAt(path.elementCount() - 1).type != QPainterPath::MoveToElement)
+            path.connectPath(arcPath * activeState().transform);
+        else {
+            // ### This is needed to work around buggy behavior in QPainterPath::connectPath()
+            //     when the last element in the path being added to is a MoveToElement.
+            arcPath = arcPath * activeState().transform;
+            path.lineTo(arcPath.elementAt(0));
 
-        case QPainterPath::LineToElement:
-            path.lineTo(e); break;
-
-        default: { // QPainterPath::CurveToElement
-            QPointF c2 = arcPath.elementAt(i++);
-            QPointF p  = arcPath.elementAt(i++);
-            path.cubicTo(e, c2, p);
-        }
+            if (arcPath.elementCount() > 1)
+                for (int i = 1; i < arcPath.elementCount(); i += 3)
+                    path.cubicTo(arcPath.elementAt(i), arcPath.elementAt(i+1),
+                                 arcPath.elementAt(i+2));
         }
     }
 }
@@ -1463,7 +1451,7 @@ void CanvasContext2DImpl::drawImage(ElementImpl* image, float dx, float dy, floa
         return;
     }
 
-    if (qFuzzyCompare(dw, 0) || qFuzzyCompare(dh, 0))
+    if (qFuzzyCompare(dw + 1, 1) || qFuzzyCompare(dh + 1, 1))
         return;
 
     QPainter* p = acquirePainter();
@@ -1491,8 +1479,8 @@ void CanvasContext2DImpl::drawImage(ElementImpl* image,
         return;
     }
 
-    if (qFuzzyCompare(sw, 0) || qFuzzyCompare(sh, 0) ||
-        qFuzzyCompare(dw, 0) || qFuzzyCompare(dh, 0))
+    if (qFuzzyCompare(sw + 1, 1) || qFuzzyCompare(sh + 1, 1) ||
+        qFuzzyCompare(dw + 1, 1) || qFuzzyCompare(dh + 1, 1))
         return;
 
     QPainter* p = acquirePainter();
