@@ -51,7 +51,6 @@
 #include <kdebug.h>
 #include <kmimetype.h>
 #include <kmessagebox.h>
-#include <kapplication.h>
 #include <krandom.h>
 #include <klocale.h>
 #ifndef KHTML_NO_WALLET
@@ -91,8 +90,8 @@ HTMLFormElementImpl::HTMLFormElementImpl(DocumentImpl *doc, bool implicit)
 
 HTMLFormElementImpl::~HTMLFormElementImpl()
 {
-    if (getDocument() && getDocument()->view() && getDocument()->view()->part()) {
-        getDocument()->view()->part()->dequeueWallet(this);
+    if (document() && document()->view() && document()->view()->part()) {
+        document()->view()->part()->dequeueWallet(this);
     }
     QListIterator<HTMLGenericFormElementImpl*> it(formElements);
     while (it.hasNext())
@@ -240,7 +239,7 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
     for(unsigned int i=0; i < strLength; ++i) if(str[i].toLatin1() == ',') str[i] = space;
     const QStringList charsets = str.split(' ');
     QTextCodec* codec = 0;
-    KHTMLView *view = getDocument()->view();
+    KHTMLView *view = document()->view();
     {
         QStringList::ConstIterator it = charsets.begin();
         const QStringList::ConstIterator itEnd = charsets.end();
@@ -306,7 +305,7 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
                 else
                 {
                     QByteArray hstr("--");
-                    hstr += m_boundary.toLatin1().constData();
+                    hstr += m_boundary.toLatin1();
                     hstr += "\r\n";
                     hstr += "Content-Disposition: form-data; name=\"";
                     hstr += (*it).data();
@@ -425,7 +424,7 @@ void HTMLFormElementImpl::setEnctype( const DOMString& type )
 
 static QString calculateAutoFillKey(const HTMLFormElementImpl& e)
 {
-    KUrl k(e.getDocument()->URL());
+    KUrl k(e.document()->URL());
     k.setRef(QString());
     k.setQuery(QString());
     // ensure that we have the user / password inside the url
@@ -448,7 +447,7 @@ void HTMLFormElementImpl::doAutoFill()
         return;
 
     // assert(view())
-    getDocument()->view()->part()->openWallet(this);
+    document()->view()->part()->openWallet(this);
 #endif // KHTML_NO_WALLET
 }
 
@@ -473,7 +472,7 @@ void HTMLFormElementImpl::walletOpened(KWallet::Wallet *w) {
                     current->inputType() == HTMLInputElementImpl::TEXT) &&
                     !current->readOnly() &&
                     map.contains(current->name().string())) {
-                getDocument()->setFocusNode(current);
+                document()->setFocusNode(current);
                 current->setValue(map[current->name().string()]);
             }
         }
@@ -522,12 +521,12 @@ void HTMLFormElementImpl::submitFromKeyboard()
 void HTMLFormElementImpl::gatherWalletData()
 {
 #ifndef KHTML_NO_WALLET
-    KHTMLView* const view = getDocument()->view();
+    KHTMLView* const view = document()->view();
     // check if we have any password input's
     m_walletMap.clear();
     m_havePassword = false;
     m_haveTextarea = false;
-    const KUrl formUrl(getDocument()->URL());
+    const KUrl formUrl(document()->URL());
     if (view && !view->nonPasswordStorableSite(formUrl.host())) {
         for (QListIterator<HTMLGenericFormElementImpl*> it(formElements); it.hasNext();) {
             HTMLGenericFormElementImpl* const cur = it.next();
@@ -552,7 +551,7 @@ void HTMLFormElementImpl::gatherWalletData()
 
 bool HTMLFormElementImpl::prepareSubmit()
 {
-    KHTMLView* const view = getDocument()->view();
+    KHTMLView* const view = document()->view();
     if(m_insubmit || !view || !view->part() || view->part()->onlyLocalReferences())
         return m_insubmit;
 
@@ -586,9 +585,9 @@ void HTMLFormElementImpl::submit(  )
 #endif
 
     bool ok;
-    KHTMLView* const view = getDocument()->view();
+    KHTMLView* const view = document()->view();
     const QByteArray form_data = formData(ok);
-    const KUrl formUrl(getDocument()->URL());
+    const KUrl formUrl(document()->URL());
 
     if (ok && view) {
         if (m_walletMap.isEmpty()) {
@@ -685,7 +684,7 @@ void HTMLFormElementImpl::submit(  )
 
 void HTMLFormElementImpl::reset(  )
 {
-    KHTMLView* const view = getDocument()->view();
+    KHTMLView* const view = document()->view();
     if(m_inreset || !view || !view->part()) return;
 
     m_inreset = true;
@@ -735,16 +734,16 @@ void HTMLFormElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ONSUBMIT:
         setHTMLEventListener(EventImpl::SUBMIT_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onsubmit", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onsubmit", this));
         break;
     case ATTR_ONRESET:
         setHTMLEventListener(EventImpl::RESET_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onreset", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onreset", this));
         break;
     case ATTR_NAME:
         if (inDocument() && m_name != attr->value()) {
-            getDocument()->underDocNamedCache().remove(m_name.string(),        this);
-            getDocument()->underDocNamedCache().add   (attr->value().string(), this);
+            document()->underDocNamedCache().remove(m_name.string(),        this);
+            document()->underDocNamedCache().add   (attr->value().string(), this);
         }
         m_name = attr->value();
         //Fallthrough intentional
@@ -755,25 +754,25 @@ void HTMLFormElementImpl::parseAttribute(AttributeImpl *attr)
 
 void HTMLFormElementImpl::removedFromDocument()
 {
-    getDocument()->underDocNamedCache().remove(m_name.string(), this);
+    document()->underDocNamedCache().remove(m_name.string(), this);
     HTMLElementImpl::removedFromDocument();
 }
 
 void HTMLFormElementImpl::insertedIntoDocument()
 {
-    getDocument()->underDocNamedCache().add(m_name.string(), this);
+    document()->underDocNamedCache().add(m_name.string(), this);
     HTMLElementImpl::insertedIntoDocument();
 }
 
 void HTMLFormElementImpl::removeId(const QString& id)
 {
-    getDocument()->underDocNamedCache().remove(id, this);
+    document()->underDocNamedCache().remove(id, this);
     HTMLElementImpl::removeId(id);
 }
 
 void HTMLFormElementImpl::addId   (const QString& id)
 {
-    getDocument()->underDocNamedCache().add(id, this);
+    document()->underDocNamedCache().add(id, this);
     HTMLElementImpl::addId(id);
 }
 
@@ -936,7 +935,7 @@ DOMString HTMLGenericFormElementImpl::name() const
     if (m_name) return m_name;
 
 // ###
-//     DOMString n = getDocument()->htmlMode() != DocumentImpl::XHtml ?
+//     DOMString n = document()->htmlMode() != DocumentImpl::XHtml ?
 //                   getAttribute(ATTR_NAME) : getAttribute(ATTR_ID);
     const DOMString n = getAttribute(ATTR_NAME);
     if (n.isNull())
@@ -970,7 +969,7 @@ void HTMLGenericFormElementImpl::setDisabled( bool _disabled )
     if ( m_disabled != _disabled ) {
         m_disabled = _disabled;
         // Trigger dynamic restyles
-        getDocument()->dynamicDomRestyler().restyleDepedent(this, OtherStateDependency);
+        document()->dynamicDomRestyler().restyleDepedent(this, OtherStateDependency);
         // We need to update rendering under all circumstances
         if (!changed() && m_render) {
             m_render->updateFromElement();
@@ -1031,7 +1030,7 @@ void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
     if (evt->target()==this && !m_disabled)
     {
         // Report focus in/out changes to the browser extension (editable widgets only)
-        KHTMLView* const view = getDocument()->view();
+        KHTMLView* const view = document()->view();
         if (view && evt->id() == EventImpl::DOMFOCUSIN_EVENT && isEditable() && m_render && m_render->isWidget()) {
             KHTMLPartBrowserExtension *ext = static_cast<KHTMLPartBrowserExtension *>(view->part()->browserExtension());
             QWidget* const widget = static_cast<RenderWidget*>(m_render)->widget();
@@ -1124,18 +1123,18 @@ DOMString HTMLButtonElementImpl::type() const
         case BUTTON:
             return "button";
     }
-    return DOMString();
+    return "";
 }
 
 void HTMLButtonElementImpl::blur()
 {
-    if(getDocument()->focusNode() == this)
-        getDocument()->setFocusNode(0);
+    if(document()->focusNode() == this)
+        document()->setFocusNode(0);
 }
 
 void HTMLButtonElementImpl::focus()
 {
-    getDocument()->setFocusNode(this);
+    document()->setFocusNode(this);
 }
 
 void HTMLButtonElementImpl::parseAttribute(AttributeImpl *attr)
@@ -1238,10 +1237,10 @@ void HTMLFieldSetElementImpl::attach()
     assert(!m_render);
     assert(parentNode());
 
-    RenderStyle* const _style = getDocument()->styleSelector()->styleForElement(this);
+    RenderStyle* const _style = document()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new (getDocument()->renderArena()) RenderFieldset(this);
+        m_render = new (document()->renderArena()) RenderFieldset(this);
         m_render->setStyle(_style);
     }
     HTMLGenericFormElementImpl::attach();
@@ -1282,7 +1281,7 @@ HTMLInputElementImpl::HTMLInputElementImpl(DocumentImpl *doc, HTMLFormElementImp
 
 HTMLInputElementImpl::~HTMLInputElementImpl()
 {
-    if (getDocument()) getDocument()->deregisterMaintainsState(this);
+    if (document()) document()->deregisterMaintainsState(this);
 }
 
 NodeImpl::Id HTMLInputElementImpl::id() const
@@ -1372,8 +1371,8 @@ QString HTMLInputElementImpl::state( )
     case RADIO:
         return QLatin1String(checked() ? "on" : "off");
     case TEXT:
-        if (autoComplete() && value() != getAttribute(ATTR_VALUE) && getDocument()->view())
-            getDocument()->view()->addFormCompletionItem(name().string(), value().string());
+        if (autoComplete() && value() != getAttribute(ATTR_VALUE) && document()->view())
+            document()->view()->addFormCompletionItem(name().string(), value().string());
         /* nobreak */
     default:
         return value().string() + (m_unsubmittedFormChange ? 'M' : '.');
@@ -1480,11 +1479,11 @@ void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ONSELECT:
         setHTMLEventListener(EventImpl::SELECT_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string(), "onselect", this));
+            document()->createHTMLEventListener(attr->value().string(), "onselect", this));
         break;
     case ATTR_ONCHANGE:
         setHTMLEventListener(EventImpl::CHANGE_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string(), "onchange", this));
+            document()->createHTMLEventListener(attr->value().string(), "onchange", this));
         break;
     default:
         HTMLGenericFormElementImpl::parseAttribute(attr);
@@ -1530,8 +1529,8 @@ void HTMLInputElementImpl::attach()
 
     switch( m_type ) {
     case PASSWORD:
-        if (getDocument()->isHTMLDocument())
-            static_cast<HTMLDocumentImpl*>(getDocument())->setAutoFill();
+        if (document()->isHTMLDocument())
+            static_cast<HTMLDocumentImpl*>(document())->setAutoFill();
         break;
     case HIDDEN:
     case IMAGE:
@@ -1543,21 +1542,21 @@ void HTMLInputElementImpl::attach()
         break;
     };
 
-    RenderStyle* const _style = getDocument()->styleSelector()->styleForElement(this);
+    RenderStyle* const _style = document()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
         switch(m_type)
         {
         case TEXT:
         case PASSWORD:
-        case ISINDEX:      m_render = new (getDocument()->renderArena()) RenderLineEdit(this);   break;
-        case CHECKBOX:  m_render = new (getDocument()->renderArena()) RenderCheckBox(this); break;
-        case RADIO:        m_render = new (getDocument()->renderArena()) RenderRadioButton(this); break;
-        case SUBMIT:      m_render = new (getDocument()->renderArena()) RenderSubmitButton(this); break;
-        case IMAGE:       m_render =  new (getDocument()->renderArena()) RenderImageButton(this); break;
-        case RESET:      m_render = new (getDocument()->renderArena()) RenderResetButton(this);   break;
-        case FILE:         m_render =  new (getDocument()->renderArena()) RenderFileButton(this);    break;
-        case BUTTON:  m_render = new (getDocument()->renderArena()) RenderPushButton(this);
+        case ISINDEX:      m_render = new (document()->renderArena()) RenderLineEdit(this);   break;
+        case CHECKBOX:  m_render = new (document()->renderArena()) RenderCheckBox(this); break;
+        case RADIO:        m_render = new (document()->renderArena()) RenderRadioButton(this); break;
+        case SUBMIT:      m_render = new (document()->renderArena()) RenderSubmitButton(this); break;
+        case IMAGE:       m_render =  new (document()->renderArena()) RenderImageButton(this); break;
+        case RESET:      m_render = new (document()->renderArena()) RenderResetButton(this);   break;
+        case FILE:         m_render =  new (document()->renderArena()) RenderFileButton(this);    break;
+        case BUTTON:  m_render = new (document()->renderArena()) RenderPushButton(this);
         case HIDDEN:   break;
         }
     }
@@ -1740,7 +1739,7 @@ void HTMLInputElementImpl::setChecked(bool _checked)
 //     setIndeterminate(false);
 
     // Trigger dynamic restyles
-    getDocument()->dynamicDomRestyler().restyleDepedent(this, OtherStateDependency);
+    document()->dynamicDomRestyler().restyleDepedent(this, OtherStateDependency);
     // We need to update rendering under all circumstances
     if (!changed() && m_render) {
         m_render->updateFromElement();
@@ -1756,7 +1755,7 @@ void HTMLInputElementImpl::setIndeterminate(bool _indeterminate)
     m_indeterminate = _indeterminate;
 
     // Trigger dynamic restyles
-//     getDocument()->dynamicDomRestyler().restyleDepedent(this, OtherStateDependency);
+//     document()->dynamicDomRestyler().restyleDepedent(this, OtherStateDependency);
     // We need to update rendering under all circumstances
     if (!changed() && m_render) {
         m_render->updateFromElement();
@@ -1802,13 +1801,13 @@ void HTMLInputElementImpl::setValue(DOMString val)
 
 void HTMLInputElementImpl::blur()
 {
-    if(getDocument()->focusNode() == this)
-	getDocument()->setFocusNode(0);
+    if(document()->focusNode() == this)
+	document()->setFocusNode(0);
 }
 
 void HTMLInputElementImpl::focus()
 {
-    getDocument()->setFocusNode(this);
+    document()->setFocusNode(this);
 }
 
 void HTMLInputElementImpl::defaultEventHandler(EventImpl *evt)
@@ -1829,7 +1828,7 @@ void HTMLInputElementImpl::defaultEventHandler(EventImpl *evt)
 		m_render->absolutePosition(offsetX,offsetY);
 		xPos = me->clientX()-offsetX;
 		yPos = me->clientY()-offsetY;
-                KHTMLView* v = getDocument()->view();
+                KHTMLView* v = document()->view();
                 if ( v ) {
                     xPos += v->contentsX();
                     yPos += v->contentsY();
@@ -1955,7 +1954,7 @@ NodeImpl* HTMLLabelElementImpl::getFormElement()
 	    const DOMString formElementId = getAttribute(ATTR_FOR);
 	    NodeImpl *newNode=0L;
     	    if (!formElementId.isEmpty())
-	        newNode=getDocument()->getElementById(formElementId);
+	        newNode=document()->getElementById(formElementId);
     	    if (!newNode){
     		const uint children=childNodeCount();
     		if (children>1)
@@ -1987,7 +1986,7 @@ void HTMLLabelElementImpl::defaultEventHandler(EventImpl *evt)
 	if (act) {
 	    NodeImpl* const formNode=getFormElement();
 	    if (formNode && evt->target() != formNode) {
-		getDocument()->setFocusNode(formNode);
+		document()->setFocusNode(formNode);
 		if (formNode->id()==ID_INPUT)
 		    static_cast<DOM::HTMLInputElementImpl*>(formNode)->click();
 		evt->setDefaultHandled();
@@ -2018,10 +2017,10 @@ void HTMLLegendElementImpl::attach()
     assert(!attached());
     assert(!m_render);
     assert(parentNode());
-    RenderStyle* const _style = getDocument()->styleSelector()->styleForElement(this);
+    RenderStyle* const _style = document()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new (getDocument()->renderArena()) RenderLegend(this);
+        m_render = new (document()->renderArena()) RenderLegend(this);
         m_render->setStyle(_style);
     }
     HTMLGenericFormElementImpl::attach();
@@ -2054,7 +2053,7 @@ HTMLSelectElementImpl::HTMLSelectElementImpl(DocumentImpl *doc, HTMLFormElementI
 
 HTMLSelectElementImpl::~HTMLSelectElementImpl()
 {
-    if (getDocument()) getDocument()->deregisterMaintainsState(this);
+    if (document()) document()->deregisterMaintainsState(this);
 }
 
 NodeImpl::Id HTMLSelectElementImpl::id() const
@@ -2076,7 +2075,7 @@ long HTMLSelectElementImpl::selectedIndex() const
 {
     // return the number of the first option selected
     uint o = 0;
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
     const unsigned int itemsSize = items.size();
     for (unsigned int i = 0; i < itemsSize; ++i) {
         if (items[i]->id() == ID_OPTION) {
@@ -2092,7 +2091,7 @@ long HTMLSelectElementImpl::selectedIndex() const
 void HTMLSelectElementImpl::setSelectedIndex( long  index )
 {
     // deselect all other options and select only the new one
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
     int listIndex;
     const int itemsSize = int(items.size());
     for (listIndex = 0; listIndex < itemsSize; ++listIndex) {
@@ -2141,7 +2140,7 @@ void HTMLSelectElementImpl::remove( long index )
     int exceptioncode = 0;
     const int listIndex = optionToListIndex(index);
 
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
     if(listIndex < 0 || index >= int(items.size()))
         return; // ### what should we do ? remove the last item?
 
@@ -2166,13 +2165,13 @@ void HTMLSelectElementImpl::remove( long index )
 
 void HTMLSelectElementImpl::blur()
 {
-    if(getDocument()->focusNode() == this)
-	getDocument()->setFocusNode(0);
+    if(document()->focusNode() == this)
+	document()->setFocusNode(0);
 }
 
 void HTMLSelectElementImpl::focus()
 {
-    getDocument()->setFocusNode(this);
+    document()->setFocusNode(this);
 }
 
 DOMString HTMLInputElementImpl::valueWithDefault() const
@@ -2218,7 +2217,7 @@ DOMString HTMLInputElementImpl::valueWithDefault() const
 DOMString HTMLSelectElementImpl::value( ) const
 {
     uint i;
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
     const uint itemsSize = items.size();
     for (i = 0; i < itemsSize; ++i) {
         if ( items[i]->id() == ID_OPTION
@@ -2232,7 +2231,7 @@ void HTMLSelectElementImpl::setValue(DOMStringImpl* value)
 {
     // find the option with value() matching the given parameter
     // and make it the current selection.
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
     for (int i = 0; i < items.size(); i++)
         if (items[i]->id() == ID_OPTION && static_cast<HTMLOptionElementImpl*>(items[i])->value() == value) {
             static_cast<HTMLOptionElementImpl*>(items[i])->setSelected(true);
@@ -2243,7 +2242,7 @@ void HTMLSelectElementImpl::setValue(DOMStringImpl* value)
 QString HTMLSelectElementImpl::state( )
 {
     QString state;
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
 
     const int l = items.count();
 
@@ -2265,7 +2264,7 @@ void HTMLSelectElementImpl::restoreState(const QString &_state)
         state[0] = 'X';
     }
 
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
 
     const int l = items.count();
     for(int i = 0; i < l; ++i) {
@@ -2335,7 +2334,7 @@ void HTMLSelectElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ONCHANGE:
         setHTMLEventListener(EventImpl::CHANGE_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string(), "onchange", this));
+            document()->createHTMLEventListener(attr->value().string(), "onchange", this));
         break;
     default:
         HTMLGenericFormElementImpl::parseAttribute(attr);
@@ -2348,10 +2347,10 @@ void HTMLSelectElementImpl::attach()
     assert(parentNode());
     assert(!renderer());
 
-    RenderStyle* const _style = getDocument()->styleSelector()->styleForElement(this);
+    RenderStyle* const _style = document()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new (getDocument()->renderArena()) RenderSelect(this);
+        m_render = new (document()->renderArena()) RenderSelect(this);
         m_render->setStyle(_style);
     }
 
@@ -2363,7 +2362,7 @@ bool HTMLSelectElementImpl::encoding(const QTextCodec* codec, khtml::encodingLis
 {
     bool successful = false;
     const QByteArray enc_name = fixUpfromUnicode(codec, name().string());
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
 
     uint i;
     const uint itemsSize = items.size();
@@ -2397,7 +2396,7 @@ bool HTMLSelectElementImpl::encoding(const QTextCodec* codec, khtml::encodingLis
 
 int HTMLSelectElementImpl::optionToListIndex(int optionIndex) const
 {
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
     const int itemsSize = int(items.size());
     if (optionIndex < 0 || optionIndex >= itemsSize)
         return -1;
@@ -2421,7 +2420,7 @@ int HTMLSelectElementImpl::optionToListIndex(int optionIndex) const
 
 int HTMLSelectElementImpl::listToOptionIndex(int listIndex) const
 {
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
     if (listIndex < 0 || listIndex >= int(items.size()) ||
         items[listIndex]->id() != ID_OPTION)
         return -1;
@@ -2487,7 +2486,7 @@ void HTMLSelectElementImpl::setRecalcListItems()
 
 void HTMLSelectElementImpl::reset()
 {
-    QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
     uint i;
     const uint itemsSize = items.size();
     bool anySelected = false;
@@ -2518,7 +2517,7 @@ void HTMLSelectElementImpl::notifyOptionSelected(HTMLOptionElementImpl *selected
 {
     if (selected && !m_multiple) {
         // deselect all other options
-        QVector<HTMLGenericFormElementImpl*> items = listItems();
+        const QVector<HTMLGenericFormElementImpl*> items = listItems();
         uint i;
 	const uint itemsSize = items.size();
         for (i = 0; i < itemsSize; ++i) {
@@ -2625,7 +2624,7 @@ long HTMLOptionElementImpl::index() const
 {
     // Let's do this dynamically. Might be a bit slow, but we're sure
     // we won't forget to update a member variable in some cases...
-    QVector<HTMLGenericFormElementImpl*> items = getSelect()->listItems();
+    const QVector<HTMLGenericFormElementImpl*> items = getSelect()->listItems();
     const int l = items.count();
     int optionIndex = 0;
     for(int i = 0; i < l; ++i) {
@@ -2724,7 +2723,7 @@ HTMLTextAreaElementImpl::HTMLTextAreaElementImpl(DocumentImpl *doc, HTMLFormElem
 
 HTMLTextAreaElementImpl::~HTMLTextAreaElementImpl()
 {
-    if (getDocument()) getDocument()->deregisterMaintainsState(this);
+    if (document()) document()->deregisterMaintainsState(this);
 }
 
 NodeImpl::Id HTMLTextAreaElementImpl::id() const
@@ -2798,11 +2797,11 @@ void HTMLTextAreaElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ONSELECT:
         setHTMLEventListener(EventImpl::SELECT_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onselect", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onselect", this));
         break;
     case ATTR_ONCHANGE:
         setHTMLEventListener(EventImpl::CHANGE_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onchange", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onchange", this));
         break;
     default:
         HTMLGenericFormElementImpl::parseAttribute(attr);
@@ -2815,10 +2814,10 @@ void HTMLTextAreaElementImpl::attach()
     assert(!m_render);
     assert(parentNode());
 
-    RenderStyle* const _style = getDocument()->styleSelector()->styleForElement(this);
+    RenderStyle* const _style = document()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new (getDocument()->renderArena()) RenderTextArea(this);
+        m_render = new (document()->renderArena()) RenderTextArea(this);
         m_render->setStyle(_style);
     }
 
@@ -2943,19 +2942,19 @@ void HTMLTextAreaElementImpl::setDefaultValue(DOMString _defaultValue)
     while (it.hasNext()) {
         removeChild(it.next(), exceptioncode);
     }
-    insertBefore(getDocument()->createTextNode(_defaultValue.implementation()),firstChild(), exceptioncode);
+    insertBefore(document()->createTextNode(_defaultValue.implementation()),firstChild(), exceptioncode);
     setValue(_defaultValue);
 }
 
 void HTMLTextAreaElementImpl::blur()
 {
-    if(getDocument()->focusNode() == this)
-	getDocument()->setFocusNode(0);
+    if(document()->focusNode() == this)
+	document()->setFocusNode(0);
 }
 
 void HTMLTextAreaElementImpl::focus()
 {
-    getDocument()->setFocusNode(this);
+    document()->setFocusNode(this);
 }
 
 bool HTMLTextAreaElementImpl::isEditable()
