@@ -92,7 +92,8 @@ public:
     virtual void setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &charset, const DOM::DOMString &mimetype);
     virtual void error(int err, const QString &text);
     bool isLoading() const;
-    bool sheetLoaded();
+    virtual bool checkAddPendingSheet();
+    virtual bool checkRemovePendingSheet();
 
     bool isAlternate() const { return m_alternate; }
     bool isCSSStyleSheet() const { return m_isCSSSheet; }
@@ -200,7 +201,8 @@ public:
     virtual void childrenChanged();
 
     bool isLoading() const;
-    bool sheetLoaded();
+    virtual bool checkAddPendingSheet();
+    virtual bool checkRemovePendingSheet();
 
 protected:
     CSSStyleSheetImpl *m_sheet;
