@@ -44,6 +44,9 @@
 #include <html/html_miscimpl.h>
 #include <html/html_tableimpl.h>
 #include <html/html_objectimpl.h>
+#include <html/HTMLAudioElement.h>
+#include <html/HTMLVideoElement.h>
+#include <html/HTMLSourceElement.h>
 #include <xml/dom_textimpl.h>
 #include <xml/dom_nodeimpl.h>
 #include <misc/htmlhashes.h>
@@ -60,6 +63,12 @@
 #include <kdebug.h>
 #include <klocale.h>
 
+// Turn off inlining to avoid warning with newer gcc.
+#undef __inline
+#define __inline
+#include "doctypes.cpp"
+#undef __inline
+
 using namespace DOM;
 using namespace khtml;
 
@@ -304,6 +313,76 @@ void KHTMLParser::parseToken(Token *t)
     }
 }
 
+void KHTMLParser::parseDoctypeToken(DoctypeToken* t)
+{
+    // Ignore any doctype after the first. TODO It should be also ignored when processing DocumentFragment
+    if (current != document || document->doctype())
+        return;
+
+    DocumentTypeImpl* doctype = new DocumentTypeImpl(document->implementation(), document, t->name, t->publicID, t->systemID);
+    if (!t->internalSubset.isEmpty())
+        doctype->setInternalSubset(t->internalSubset);
+    document->addChild(doctype);
+
+    // Determine parse mode here
+    // This code more or less mimics Mozilla's implementation.
+    //
+    // There are three possible parse modes:
+    // COMPAT - quirks mode emulates WinIE
+    // and NS4.  CSS parsing is also relaxed in this mode, e.g., unit types can
+    // be omitted from numbers.
+    // ALMOST STRICT - This mode is identical to strict mode
+    // except for its treatment of line-height in the inline box model.  For
+    // now (until the inline box model is re-written), this mode is identical
+    // to STANDARDS mode.
+    // STRICT - no quirks apply.  Web pages will obey the specifications to
+    // the letter.
+
+    if (!document->isHTMLDocument()) // FIXME Could document be non-HTML?
+        return;
+    DOM::HTMLDocumentImpl* htmldoc = static_cast<DOM::HTMLDocumentImpl*> (document);
+    if (t->name.toLower() == "html") {
+        document->setDocType(doctype);
+        if (!t->internalSubset.isEmpty() || t->publicID.isEmpty()) {
+            // Internal subsets always denote full standards, as does
+            // a doctype without a public ID.
+            htmldoc->changeModes(DOM::DocumentImpl::Strict, DOM::DocumentImpl::Html4);
+        } else {
+            // We have to check a list of public IDs to see what we
+            // should do.
+            QString lowerPubID = t->publicID.toLower();
+            QByteArray pubIDStr = lowerPubID.toLocal8Bit();
+
+            // Look up the entry in our gperf-generated table.
+            const PubIDInfo* doctypeEntry = findDoctypeEntry(pubIDStr.constData(), t->publicID.length());
+            if (!doctypeEntry) {
+                // The DOCTYPE is not in the list.  Assume strict mode.
+                // ### Doesn't make any sense, but it's what Mozilla does.
+                htmldoc->changeModes(DOM::DocumentImpl::Strict, DOM::DocumentImpl::Html4);
+            } else {
+                switch ((!t->systemID.isEmpty()) ?
+                            doctypeEntry->mode_if_sysid :
+                            doctypeEntry->mode_if_no_sysid) {
+                    case PubIDInfo::eQuirks3:
+                        htmldoc->changeModes(DOM::DocumentImpl::Compat, DOM::DocumentImpl::Html3);
+                        break;
+                    case PubIDInfo::eQuirks:
+                        htmldoc->changeModes(DOM::DocumentImpl::Compat, DOM::DocumentImpl::Html4);
+                        break;
+                    case PubIDInfo::eAlmostStandards:
+                        htmldoc->changeModes(DOM::DocumentImpl::Transitional, DOM::DocumentImpl::Html4);
+                        break;
+                    default:
+                        assert(!"Unknown parse mode");
+                }
+            }
+        }
+    } else {
+        // Malformed doctype implies quirks mode.
+        htmldoc->changeModes(DOM::DocumentImpl::Compat, DOM::DocumentImpl::Html3);
+    }
+}
+
 static bool isTableRelatedTag(int id)
 {
     return (id == ID_TR || id == ID_TD || id == ID_TABLE || id == ID_TBODY || id == ID_TFOOT || id == ID_THEAD ||
@@ -1062,6 +1141,17 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         break;
     }
 
+// media
+    case ID_AUDIO:
+        n = new HTMLAudioElement(document);
+        break;
+    case ID_VIDEO:
+        n = new HTMLVideoElement(document);
+        break;
+    case ID_SOURCE:
+        n = new HTMLSourceElement(document);
+        break;
+
 // tables
     case ID_TABLE:
         n = new HTMLTableElementImpl(document);
@@ -1318,7 +1408,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
 
     NodeImpl* residualElem = prev->node;
     NodeImpl* blockElem = prevMaxElem ? prevMaxElem->node : current;
-    NodeImpl* parentElem = elem->node;
+    RefPtr<NodeImpl> parentElem = elem->node;
 
     // Check to see if the reparenting that is going to occur is allowed according to the DOM.
     // FIXME: We should either always allow it or perform an additional fixup instead of
@@ -1354,8 +1444,8 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
         // in the block stack.
         // This will also affect how we ultimately reparent the block, since we want it to end up
         // under the reopened residual tags (e.g., the <i> in the above example.)
-        NodeImpl* prevNode = 0;
-        NodeImpl* currNode = 0;
+        RefPtr<NodeImpl> prevNode = 0;
+        RefPtr<NodeImpl> currNode = 0;
         currElem = maxElem;
         while (currElem->node != residualElem) {
             if (isResidualStyleTag(currElem->node->id())) {
@@ -1365,11 +1455,11 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
                 removeForbidden(currElem->id, forbiddenTag);
 
                 // Change the stack element's node to point to the clone.
-                currElem->setNode(currNode);
+                currElem->setNode(currNode.get());
 
                 // Attach the previous node as a child of this new node.
                 if (prevNode)
-                    currNode->appendChild(prevNode, exceptionCode);
+                    currNode->appendChild(prevNode.get(), exceptionCode);
                 else // The new parent for the block element is going to be the innermost clone.
                     parentElem = currNode;
 
@@ -1381,7 +1471,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
 
         // Now append the chain of new residual style elements if one exists.
         if (prevNode)
-            elem->node->appendChild(prevNode, exceptionCode);
+            elem->node->appendChild(prevNode.get(), exceptionCode);
     }
 
     // We need to make a clone of |residualElem| and place it just inside |blockElem|.
@@ -1397,7 +1487,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
 
     if (!advancedResidual) {
     // Step 2: Clone |residualElem|.
-    NodeImpl* newNode = residualElem->cloneNode(false); // Shallow clone. We don't pick up the same kids.
+    RefPtr<NodeImpl> newNode = residualElem->cloneNode(false); // Shallow clone. We don't pick up the same kids.
 
     // Step 3: Place |blockElem|'s children under |newNode|.  Remove all of the children of |blockElem|
     // before we've put |newElem| into the document.  That way we'll only do one attachment of all
@@ -1421,7 +1511,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
 
     // Step 4: Place |newNode| under |blockElem|.  |blockElem| is still out of the document, so no
     // attachment can occur yet.
-    blockElem->appendChild(newNode, exceptionCode);
+    blockElem->appendChild(newNode.get(), exceptionCode);
     }
 
     // Step 5: Reparent |blockElem|.  Now the full attachment of the fixed up tree takes place.
@@ -1477,15 +1567,15 @@ void KHTMLParser::reopenResidualStyleTags(HTMLStackElem* elem, DOM::NodeImpl* ma
     // Loop for each tag that needs to be reopened.
     while (elem) {
         // Create a shallow clone of the DOM node for this element.
-        NodeImpl* newNode = elem->node->cloneNode(false);
+        RefPtr<NodeImpl> newNode = elem->node->cloneNode(false);
 
         // Append the new node. In the malformed table case, we need to insert before the table,
         // which will be the last child.
         int exceptionCode = 0;
         if (malformedTableParent)
-            malformedTableParent->insertBefore(newNode, malformedTableParent->lastChild(), exceptionCode);
+            malformedTableParent->insertBefore(newNode.get(), malformedTableParent->lastChild(), exceptionCode);
         else
-            current->appendChild(newNode, exceptionCode);
+            current->appendChild(newNode.get(), exceptionCode);
         // FIXME: Is it really OK to ignore the exceptions here?
 
         // Now push a new stack element for this node we just created.
@@ -1501,7 +1591,7 @@ void KHTMLParser::reopenResidualStyleTags(HTMLStackElem* elem, DOM::NodeImpl* ma
         malformedTableParent = 0;
 
         // Update |current| manually to point to the new node.
-        setCurrent(newNode);
+        setCurrent(newNode.get());
 
         // Advance to the next tag that needs to be reopened.
         HTMLStackElem* next = elem->next;
