@@ -51,6 +51,7 @@ namespace DOM
   class HTMLDocument;
   class HTMLDocumentImpl;
   class DocumentImpl;
+  class Document;
   class HTMLTitleElementImpl;
   class HTMLFrameElementImpl;
   class HTMLIFrameElementImpl;
@@ -63,7 +64,11 @@ namespace DOM
   class HTMLEventListener;
   class EventListener;
   class HTMLPartContainerElementImpl;
-  class HTMLObjectBaseElementImpl;  
+  class HTMLObjectBaseElementImpl;
+  class Position;
+  class Selection;
+  class Range;
+  class Editor;
 }
 
 namespace KJS
@@ -84,9 +89,13 @@ namespace khtml
   class DrawContentsEvent;
   class CachedObject;
   class RenderWidget;
+  class RenderBlock;
   class CSSStyleSelector;
   class HTMLTokenizer;
   class XMLTokenizer;
+  struct EditorContext;
+  class EditCommandImpl;
+  class KHTMLPartAccessor;
 }
 
 namespace KJS {
@@ -212,11 +221,16 @@ class KHTML_EXPORT KHTMLPart : public KParts::ReadOnlyPart
   friend class KHTMLPartBrowserExtension;
   friend class DOM::DocumentImpl;
   friend class DOM::HTMLDocumentImpl;
+  friend class DOM::Selection;
+  friend class DOM::Editor;
   friend class KHTMLPartBrowserHostExtension;
   friend class khtml::HTMLTokenizer;
   friend class khtml::XMLTokenizer;
   friend class khtml::RenderWidget;
+  friend class khtml::RenderBlock;
   friend class khtml::CSSStyleSelector;
+  friend class khtml::EditCommandImpl;
+  friend class khtml::KHTMLPartAccessor;
   friend class KHTMLPartIface;
   friend class KHTMLPartFunction;
   friend class KHTMLPopupGUIClient;
@@ -244,7 +258,7 @@ public:
    *
    * KHTML basically consists of two objects: The KHTMLPart itself,
    * holding the document data (DOM document), and the KHTMLView,
-   * derived from QScrollView, in which the document content is
+   * derived from QScrollArea, in which the document content is
    * rendered in. You can specify two different parent objects for a
    * KHTMLPart, one parent for the KHTMLPart document and one parent
    * for the KHTMLView. If the second @p parent argument is 0L, then
@@ -583,7 +597,12 @@ public:
    * You may call
    * this function many times in sequence. But remember: The fewer calls
    * you make, the faster the widget will be.
+   *
+   * For historic and backward compatibility reasons, this method will force
+   * the use of strict mode for the document, unless setAlwaysHonourDoctype()
+   * has been called previously.
    */
+   // FIXME KDE5: always honour doctype, remove setAlwaysHonourDoctype()
   virtual void write( const QString &str );
 
   /**
@@ -804,6 +823,12 @@ public:
    */
   bool hasSelection() const;
 
+  /**
+   * Returns the instance of the attached html editor interface.
+   *
+   */
+  DOM::Editor *editor() const;
+
   /**
    * Marks all text in the document as selected.
    */
@@ -910,7 +935,7 @@ public:
 
   /**
    * Returns whether a frame with the specified name is exists or not.
-   * In contrary to the findFrame method this one also returns @p true
+   * In contrast to the findFrame method this one also returns @p true
    * if the frame is defined but no displaying component has been
    * found/loaded, yet.
    */
@@ -978,24 +1003,43 @@ public:
    */
   void preloadScript( const QString &url, const QString &script );
 
+  /**
+   * Returns whether the given point is inside the current selection.
+   *
+   * The coordinates are content-coordinates.
+   */
+   bool isPointInsideSelection(int x, int y);
+
   /**
    * @internal
    */
   bool restored() const;
 
-  // ### KDE4 remove me
+  /**
+   * Sets whether the document's Doctype should always be used
+   * to determine the parsing mode for the document.
+   *
+   * Without this, parsing will be forced to
+   * strict mode when using the write( const QString &str )
+   * method for backward compatibility reasons.
+   *
+   */
+   // ### KDE5 remove - fix write( const QString &str ) instead
+  void setAlwaysHonourDoctype( bool b = true );
+
+  // ### KDE5 remove me
   enum FormNotification { NoNotification = 0, Before, Only, Unused=255 };
   /**
    * Determine if signal should be emitted before, instead or never when a
    * submitForm() happens.
-   * ### KDE4 remove me
+   * ### KDE5 remove me
    */
   void setFormNotification(FormNotification fn);
 
   /**
    * Determine if signal should be emitted before, instead or never when a
    * submitForm() happens.
-   * ### KDE4 remove me
+   * ### KDE5 remove me
    */
   FormNotification formNotification() const;
 
@@ -1173,6 +1217,11 @@ protected:
    */
   virtual bool doCloseStream();
 
+  /**
+   * @internal
+   */
+  virtual void timerEvent(QTimerEvent *);
+
 public Q_SLOTS:
 
   /**
@@ -1363,7 +1412,7 @@ private Q_SLOTS:
   void slotDecZoom();
   void slotIncZoomFast();
   void slotDecZoomFast();
-  
+
   void slotIncFontSize();
   void slotDecFontSize();
   void slotIncFontSizeFast();
@@ -1539,9 +1588,13 @@ private:
   void setStatusBarText( const QString& text, StatusBarPriority p);
 
   bool restoreURL( const KUrl &url );
+  void clearCaretRectIfNeeded();
+  void setFocusNodeIfNeeded(const DOM::Selection &);
+  void selectionLayoutChanged();
+  void notifySelectionChanged(bool closeTyping=true);
   void resetFromScript();
   void emitSelectionChanged();
-  void onFirstData(const QString& firstData);
+  void onFirstData();
   // Returns whether callingHtmlPart may access this part
   bool checkFrameAccess(KHTMLPart *callingHtmlPart);
   bool openUrlInFrame(const KUrl &url, const KParts::OpenUrlArguments& arguments, const KParts::BrowserArguments &browserArguments);
@@ -1632,7 +1685,6 @@ private:
 
   bool initFindNode( bool selection, bool reverse, bool fromCursor );
 
-  void extendSelection( DOM::NodeImpl* node, int offset, DOM::Node& selectionNode, long& selectionOffset, bool right, bool paragraph );
   /** extends the current selection to the given content-coordinates @p x, @p y
    * @param x content x-coordinate
    * @param y content y-coordinate
@@ -1642,7 +1694,7 @@ private:
    *	caller has to ensure that the node has a renderer.
    * @internal
    */
-  void extendSelectionTo(int x, int y, int absX, int absY, const DOM::Node &innerNode);
+  void extendSelectionTo(int x, int y, const DOM::Node &innerNode);
   /** checks whether a selection is extended.
    * @return @p true if a selection is extended by the mouse.
    */
@@ -1666,12 +1718,82 @@ private:
 
   void decFontSize(const int stepping[], int count);
 
-  void emitCaretPositionChanged(const DOM::Node &node, long offset);
+  void emitCaretPositionChanged(const DOM::Position &pos);
 
   void setDebugScript( bool enable );
 
   void runAdFilter();
 
+  khtml::EditorContext *editorContext() const;
+
+  /**
+   * initialises the caret if it hasn't been used yet.
+   * @internal
+   */
+  void initCaret();
+
+  /**
+   * Returns the selected part of the HTML.
+   */
+  const DOM::Selection &caret() const;
+
+  /**
+   * Returns the drag caret of the HTML.
+   */
+  const DOM::Selection &dragCaret() const;
+
+  /**
+   * Sets the current caret to the given selection.
+   */
+  void setCaret(const DOM::Selection &, bool closeTyping=true);
+
+  /**
+   * Sets the current drag caret.
+   */
+  void setDragCaret(const DOM::Selection &);
+
+  /**
+   * Clears the current selection.
+   */
+  void clearSelection();
+
+  /**
+   * Invalidates the current selection.
+   */
+  void invalidateSelection();
+
+  /**
+   * Controls the visibility of the selection.
+   */
+  void setSelectionVisible(bool flag=true);
+
+  /**
+   * Paints the caret.
+   */
+  void paintCaret(QPainter *p, const QRect &rect) const;
+
+  /**
+   * Paints the drag caret.
+   */
+  void paintDragCaret(QPainter *p, const QRect &rect) const;
+
+  /**
+   * Returns selectedText without any leading or trailing whitespace,
+   * and with non-breaking-spaces turned into normal spaces.
+   *
+   * Note that hasSelection can return true and yet simplifiedSelectedText can be empty,
+   * e.g. when selecting a single space.
+   */
+  QString simplifiedSelectedText() const;
+
+  bool handleMouseMoveEventDrag(khtml::MouseMoveEvent *event);
+  bool handleMouseMoveEventOver(khtml::MouseMoveEvent *event);
+  void handleMouseMoveEventSelection(khtml::MouseMoveEvent *event);
+
+  void handleMousePressEventSingleClick(khtml::MousePressEvent *event);
+  void handleMousePressEventDoubleClick(khtml::MouseDoubleClickEvent *event);
+  void handleMousePressEventTripleClick(khtml::MouseDoubleClickEvent *event);
+
   KHTMLPartPrivate *d;
   friend class KHTMLPartPrivate;
 };
