@@ -40,6 +40,8 @@
 #undef LOADER_DEBUG
 //#define LOADER_DEBUG
 
+//#define PRELOAD_DEBUG
+
 #include "loader.h"
 #include "seed.h"
 #include <imload/image.h>
@@ -59,7 +61,6 @@
 #include <QtGui/QWidget>
 #include <QtCore/QDebug>
 
-#include <kapplication.h>
 #include <kauthorized.h>
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
@@ -137,6 +138,14 @@ void CachedObject::setRequest(Request *_request)
 
 void CachedObject::ref(CachedObjectClient *c)
 {
+    if (m_preloadResult == PreloadNotReferenced) {
+        if (isLoaded())
+            m_preloadResult = PreloadReferencedWhileComplete;
+        else if (m_prospectiveRequest)
+            m_preloadResult = PreloadReferencedWhileLoading;
+        else
+            m_preloadResult = PreloadReferenced;
+    }
     // unfortunately we can be ref'ed multiple times from the
     // same object,  because it uses e.g. the same foreground
     // and the same background picture. so deal with it.
@@ -414,6 +423,7 @@ CachedImage::CachedImage(DocLoader* dl, const DOMString &url, KIO::CacheControl
     imgSource = 0;
     setAccept( acceptHeader );
     m_showAnimations = dl->showAnimations();
+    m_loading = true;
 
     if ( KHTMLGlobal::defaultHTMLSettings()->isAdFiltered( url.string() ) ) {
         m_wasBlocked = true;
@@ -675,6 +685,7 @@ void CachedImage::imageDone(khtmlImLoad::Image* /*img*/)
     kDebug(6060)<<"Image is done:" << this;
 #endif
     m_status = Persistent;
+    m_loading = false;
     doNotifyFinished();
 }
 
@@ -945,6 +956,7 @@ void CachedImage::finish()
 	const QPixmap &pm = pixmap();
 	do_notify( pm, pm.rect() );
     }*/
+    m_loading = false;
     QSize s = pixmap_size();
     setSize( s.width() * s.height() * 2);
 }
@@ -1035,6 +1047,7 @@ DocLoader::DocLoader(KHTMLPart* part, DocumentImpl* doc)
 
 DocLoader::~DocLoader()
 {
+    clearPreloads();
     Cache::loader()->cancelRequests( this );
     Cache::docloader->removeAll( this );
 }
@@ -1070,7 +1083,7 @@ bool DocLoader::needReload(CachedObject *existing, const QString& fullURL)
     {
        if (!m_reloadedURLs.contains(fullURL))
        {
-          if (existing && existing->isExpired())
+          if (existing && existing->isExpired() && !existing->isPreloaded())
           {
              Cache::removeCacheEntry(existing);
              m_reloadedURLs.append(fullURL);
@@ -1082,22 +1095,91 @@ bool DocLoader::needReload(CachedObject *existing, const QString& fullURL)
     {
        if (!m_reloadedURLs.contains(fullURL))
        {
-          if (existing)
+          if (existing && !existing->isPreloaded())
           {
              Cache::removeCacheEntry(existing);
           }
-          m_reloadedURLs.append(fullURL);
-          reload = true;
+          if (!existing || !existing->isPreloaded()) {
+              m_reloadedURLs.append(fullURL);
+              reload = true;
+          }
        }
     }
     return reload;
 }
 
+void DocLoader::registerPreload(CachedObject* resource)
+{
+    if (!resource || resource->isLoaded() || m_preloads.contains(resource))
+        return;
+    resource->increasePreloadCount();
+    m_preloads.insert(resource);
+    resource->setProspectiveRequest();
+#ifdef PRELOAD_DEBUG
+    fprintf(stderr, "PRELOADING %s\n", resource->url().string().toLatin1().data());
+#endif
+}
+ 
+void DocLoader::clearPreloads()
+{
+    printPreloadStats();
+    QSet<CachedObject*>::iterator end = m_preloads.end();
+    for (QSet<CachedObject*>::iterator it = m_preloads.begin(); it != end; ++it) {
+        CachedObject* res = *it;
+        res->decreasePreloadCount();
+        if (res->preloadResult() == CachedObject::PreloadNotReferenced || res->hadError())
+            Cache::removeCacheEntry(res);
+    }
+    m_preloads.clear();
+}
+
+void DocLoader::printPreloadStats()
+{
+#ifdef PRELOAD_DEBUG
+    unsigned scripts = 0;
+    unsigned scriptMisses = 0;
+    unsigned stylesheets = 0;
+    unsigned stylesheetMisses = 0;
+    unsigned images = 0;
+    unsigned imageMisses = 0;
+    QSet<CachedObject*>::iterator end = m_preloads.end();
+    for (QSet<CachedObject*>::iterator it = m_preloads.begin(); it != end; ++it) {
+        CachedObject* res = *it;
+        if (res->preloadResult() == CachedObject::PreloadNotReferenced)
+            fprintf(stderr,"!! UNREFERENCED PRELOAD %s\n", res->url().string().toLatin1().data());
+        else if (res->preloadResult() == CachedObject::PreloadReferencedWhileComplete)
+            fprintf(stderr,"HIT COMPLETE PRELOAD %s\n", res->url().string().toLatin1().data());
+        else if (res->preloadResult() == CachedObject::PreloadReferencedWhileLoading)
+            fprintf(stderr,"HIT LOADING PRELOAD %s\n", res->url().string().toLatin1().data());
+        
+        if (res->type() == CachedObject::Script) {
+            scripts++;
+            if (res->preloadResult() < CachedObject::PreloadReferencedWhileLoading)
+                scriptMisses++;
+        } else if (res->type() == CachedObject::CSSStyleSheet) {
+            stylesheets++;
+            if (res->preloadResult() < CachedObject::PreloadReferencedWhileLoading)
+                stylesheetMisses++;
+        } else {
+            images++;
+            if (res->preloadResult() < CachedObject::PreloadReferencedWhileLoading)
+                imageMisses++;
+        }
+    }
+    if (scripts)
+        fprintf(stderr, "SCRIPTS: %d (%d hits, hit rate %d%%)\n", scripts, scripts - scriptMisses, (scripts - scriptMisses) * 100 / scripts);
+    if (stylesheets)
+        fprintf(stderr, "STYLESHEETS: %d (%d hits, hit rate %d%%)\n", stylesheets, stylesheets - stylesheetMisses, (stylesheets - stylesheetMisses) * 100 / stylesheets);
+    if (images)
+        fprintf(stderr, "IMAGES:  %d (%d hits, hit rate %d%%)\n", images, images - imageMisses, (images - imageMisses) * 100 / images);
+#endif
+}
+
 #define DOCLOADER_SECCHECK(doRedirectCheck) \
     KUrl fullURL (m_doc->completeURL( url.string() )); \
     if ( !fullURL.isValid() || \
          ( m_part && m_part->onlyLocalReferences() && fullURL.protocol() != "file" && fullURL.protocol() != "data") || \
-         doRedirectCheck && ( kapp && m_doc && !KAuthorized::authorizeUrlAction("redirect", m_doc->URL(), fullURL))) \
+         doRedirectCheck && (m_doc && !KAuthorized::authorizeUrlAction("redirect", m_doc->URL(), fullURL))) \
          return 0L;
 
 CachedImage *DocLoader::requestImage( const DOM::DOMString &url)
