@@ -29,8 +29,8 @@
 
 #include <QtGui/qdrawutil.h>
 #include <QtGui/QPainter>
+#include <QtGui/QApplication>
 
-#include <kapplication.h>
 #include <kdebug.h>
 #include <kglobalsettings.h>
 
@@ -67,7 +67,7 @@ RenderImage::RenderImage(NodeImpl *_element)
     m_selectionState = SelectionNone;
     berrorPic = false;
 
-    const KHTMLSettings *settings = _element->getDocument()->view()->part()->settings();
+    const KHTMLSettings *settings = _element->document()->view()->part()->settings();
     bUnfinishedImageFrame = settings->unfinishedImageFrame();
 
     setIntrinsicWidth( 0 );
@@ -235,7 +235,7 @@ void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty)
                     (hasSufficientContrast(Qt::white, bg) ? Qt::white : Qt::black);
 	paintInfo.p->setPen(QPen(fg, 1));
 	paintInfo.p->setBrush( Qt::NoBrush );
-	paintInfo.p->drawRect(_tx, _ty, m_width, m_height);
+	paintInfo.p->drawRect(_tx, _ty, m_width - 1, m_height - 1);
         if (!(m_width <= 5 || m_height <= 5)) {
             if (!loadingIcon) {
                 loadingIcon = new QPixmap();
@@ -256,7 +256,7 @@ void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty)
             if ( !berrorPic ) {
                 //qDebug("qDrawShadePanel %d/%d/%d/%d", _tx + leftBorder, _ty + topBorder, cWidth, cHeight);
                 qDrawShadePanel( paintInfo.p, _tx + leftBorder + leftPad, _ty + topBorder + topPad, cWidth, cHeight,
-                                 KApplication::palette(), true, 1 );
+                                 QApplication::palette(), true, 1 );
             }
 
             QPixmap pix = *Cache::brokenPixmap;
@@ -364,8 +364,8 @@ bool RenderImage::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty,
 
         HTMLImageElementImpl* i = element()->id() == ID_IMG ? static_cast<HTMLImageElementImpl*>(element()) : 0;
         HTMLMapElementImpl* map;
-        if (i && i->getDocument()->isHTMLDocument() &&
-            (map = static_cast<HTMLDocumentImpl*>(i->getDocument())->getMap(i->imageMap()))) {
+        if (i && i->document()->isHTMLDocument() &&
+            (map = static_cast<HTMLDocumentImpl*>(i->document())->getMap(i->imageMap()))) {
             // we're a client side image map
             inside = map->mapMouseEvent(_x - tx, _y - ty, contentWidth(), contentHeight(), info);
             info.setInnerNonSharedNode(element());
@@ -410,7 +410,7 @@ void RenderImage::updateFromElement()
 
     if (!u.isEmpty() &&
         ( !m_cachedImage || m_cachedImage->url() != u ) ) {
-        CachedImage *new_image = element()->getDocument()->docLoader()->
+        CachedImage *new_image = element()->document()->docLoader()->
                                  requestImage(khtml::parseURL(u));
 
         if(new_image && new_image != m_cachedImage)
