@@ -48,6 +48,7 @@
 #include "xml/dom2_eventsimpl.h"
 #include "khtml_part.h"
 #include "xml/dom_docimpl.h"
+#include "xml/dom_position.h"
 #include "misc/helper.h"
 #include "misc/paintbuffer.h"
 #include "css/cssvalues.h"
@@ -134,6 +135,52 @@ FindSelectionResult RenderReplaced::checkSelectionPoint(int _x, int _y, int _tx,
     return SelectionPointInside;
 }
 
+long RenderReplaced::caretMinOffset() const 
+{
+    return 0;
+}
+
+// Returns 1 since a replaced element can have the caret positioned 
+// at its beginning (0), or at its end (1).
+long RenderReplaced::caretMaxOffset() const 
+{
+    return 1;
+}
+
+unsigned long RenderReplaced::caretMaxRenderedOffset() const
+{
+    return 1;
+}
+
+Position RenderReplaced::positionForCoordinates(int _x, int _y)
+{
+    InlineBox *box = placeHolderBox();
+    if (!box)
+        return Position(element(), 0);
+  
+    RootInlineBox *root = box->root();
+  
+    int absx, absy;
+    containingBlock()->absolutePosition(absx, absy);
+  
+    int top = absy + root->topOverflow();
+    int bottom = root->nextRootBox() ? absy + root->nextRootBox()->topOverflow() : absy + root->bottomOverflow();
+  
+    if (_y < top)
+        return Position(element(), caretMinOffset()); // coordinates are above
+        
+    if (_y >= bottom)
+        return Position(element(), caretMaxOffset()); // coordinates are below
+        
+    if (element()) {
+        if (_x <= absx + xPos() + (width() / 2))
+            return Position(element(), 0);
+        return Position(element(), 1);
+    }
+  
+    return RenderBox::positionForCoordinates(_x, _y);
+}
+
 // -----------------------------------------------------------------------------
 
 RenderWidget::RenderWidget(DOM::NodeImpl* node)
@@ -143,7 +190,7 @@ RenderWidget::RenderWidget(DOM::NodeImpl* node)
     m_underMouse = 0;
     // a widget doesn't support being anonymous
     assert(!isAnonymous());
-    m_view  = node->getDocument()->view();
+    m_view  = node->document()->view();
     m_arena.reset(renderArena());
     m_resizePending = false;
     m_discardResizes = false;
@@ -355,7 +402,7 @@ void RenderWidget::layout( )
 
 void RenderWidget::updateFromElement()
 {
-    if (m_widget) {
+    if (m_widget && !qobject_cast<KHTMLView*>(m_widget)) {
         // Color:
         QColor color = style()->color();
         QColor backgroundColor = style()->backgroundColor();
@@ -584,14 +631,6 @@ static void setInPaintEventFlag(QWidget* w, bool b = true, bool recurse=true)
       }
 }
 
-static bool qtVersionDetected = false;
-static bool qt43;
-
-static void detectQtVersion() {
-    qtVersionDetected = true;
-    qt43 = QString::fromLatin1(qVersion()).startsWith("4.3");
-}
-
 static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int ty, bool buffered = false)
 {
     if (r.isNull() || r.isEmpty() )
@@ -636,16 +675,15 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
     //
     // Testcase: paintEvent(...) { QPainter p(this); aChildWidget->render( aPixmapTarget, ...); }
     //
-    if (!qt43 || !buffered)
-        p->end();
+    p->end();
 
     setInPaintEventFlag( widget, false );
 
-    widget->render( d, (buffered ? QPoint(0,0) : thePoint) + (qt43 ? QPoint(0,0) : r.topLeft()), r);
+    widget->render( d, (buffered ? QPoint(0,0) : thePoint) + r.topLeft(), r);
 
     setInPaintEventFlag( widget );
 
-    if (!qt43 || !buffered) {
+//    if (!buffered) {
         p->begin(x);
         p->setWorldTransform(t);
         p->setWindow(w);
@@ -658,7 +696,7 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
             p->setOpacity(op);
         p->setPen(pen);
         p->setBrush(brush);
-    }
+//    } else {
     if (buffered) {
         // transfer results
         QPoint off(r.x(), r.y());
@@ -676,9 +714,7 @@ void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty)
     //    can't use QWidget::render to directly paint widgets on the view anymore.
     //    Results are unreliable for subrects, leaving blank squares. (cf. kde #158607)
     //
-    if (!qtVersionDetected)
-        detectQtVersion();
-    bool buffered = !qt43 || p->combinedMatrix().m22() != 1.0 || (p->device()->devType() == QInternal::Printer);
+    bool buffered = true; // p->combinedMatrix().m22() != 1.0 || (p->device()->devType() == QInternal::Printer);
 
     QRect rr = pI.r;
     rr.translate(-tx, -ty);
@@ -1056,7 +1092,7 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
 	break;
     }
     case EventImpl::MOUSEOUT_EVENT: {
-        QWidget* target = m_underMouse ? m_underMouse : m_widget;
+        QWidget* target = m_underMouse ? (QWidget*) m_underMouse : m_widget;
 	QEvent moe( QEvent::Leave );
 	QApplication::sendEvent(target, &moe);
 //        qDebug() << "received MOUSEOUT, forwarding to" << target ;
