@@ -80,13 +80,19 @@
 #include <html/html_miscimpl.h>
 #include <html/html_tableimpl.h>
 #include <html/html_objectimpl.h>
+#include <html/HTMLAudioElement.h>
+#include <html/HTMLVideoElement.h>
+#include <html/HTMLSourceElement.h>
+#include <editing/htmlediting.h>
+#include <editing/jsediting.h>
 
-#include <kapplication.h>
 #include <kio/job.h>
 
 #include <stdlib.h>
 #include <limits.h>
 
+template class QStack<DOM::NodeImpl*>;
+
 using namespace DOM;
 using namespace khtml;
 
@@ -159,7 +165,9 @@ DOMImplementationImpl* DOMImplementationImpl::getInterface(const DOMString& /*fe
 }
 
 DocumentImpl *DOMImplementationImpl::createDocument( const DOMString &namespaceURI, const DOMString &qualifiedName,
-                                                     DocumentTypeImpl* dtype, int &exceptioncode )
+                                                     DocumentTypeImpl* dtype,
+                                                     KHTMLView* v,
+                                                     int &exceptioncode )
 {
     exceptioncode = 0;
 
@@ -169,21 +177,20 @@ DocumentImpl *DOMImplementationImpl::createDocument( const DOMString &namespaceU
 
     // WRONG_DOCUMENT_ERR: Raised if doctype has already been used with a different document or was
     // created from a different implementation.
-    if (dtype && (dtype->getDocument() || dtype->implementation() != this)) {
+    if (dtype && (dtype->document() || dtype->implementation() != this)) {
         exceptioncode = DOMException::WRONG_DOCUMENT_ERR;
         return 0;
     }
 
-    // ### this is completely broken.. without a view it will not work (Dirk)
-    DocumentImpl *doc = new DocumentImpl(this, 0);
+    // ### view can be 0 which can cause problems
+    DocumentImpl* doc;
+    if (namespaceURI == XHTML_NAMESPACE)
+        doc = new HTMLDocumentImpl(this, v);
+    else
+        doc = new DocumentImpl(this, v);
+
     if (dtype) {
         doc->setDocType(dtype);
-    } else {
-        // ### this should be created during parsing a <!DOCTYPE>
-        doc->setDocType(new DocumentTypeImpl(this, doc,
-                                             DOMString() /* qualifiedName */,
-                                             DOMString() /* publicId */,
-                                             DOMString() /* systemId */));
     }
 
     // the document must be created empty if all parameters are null
@@ -207,29 +214,21 @@ CSSStyleSheetImpl *DOMImplementationImpl::createCSSStyleSheet(DOMStringImpl* /*t
 	// generate an exception.
 	CSSStyleSheetImpl *parent = 0L;
 	CSSStyleSheetImpl *sheet = new CSSStyleSheetImpl(parent, DOMString());
-	sheet->setMedia(new MediaListImpl(sheet, media));
+	sheet->setMedia(new MediaListImpl(sheet, media, true /*fallbackToDescriptor*/));
 	return sheet;
 }
 
 DocumentImpl *DOMImplementationImpl::createDocument( KHTMLView *v )
 {
     DocumentImpl* doc = new DocumentImpl(this, v);
-    // ### this should be created during parsing a <!DOCTYPE>
-    doc->setDocType(new DocumentTypeImpl(doc->implementation(), doc,
-                                         DOMString() /* qualifiedName */,
-                                         DOMString() /* publicId */,
-                                         DOMString() /* systemId */));
+
     return doc;
 }
 
 HTMLDocumentImpl *DOMImplementationImpl::createHTMLDocument( KHTMLView *v )
 {
     HTMLDocumentImpl* doc = new HTMLDocumentImpl(this, v);
-    // ### this should be created during parsing a <!DOCTYPE>
-    doc->setDocType(new DocumentTypeImpl(doc->implementation(), doc,
-                                         DOMString() /* qualifiedName */,
-                                         DOMString() /* publicId */,
-                                         DOMString() /* systemId */));
+
     return doc;
 }
 
@@ -333,7 +332,7 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     : NodeBaseImpl( 0 ), m_domtree_version(0), m_counterDict(),
       m_imageLoadEventTimer(0)
 {
-    m_document.resetSkippingRef(this); //Make getDocument return us..
+    m_document.resetSkippingRef(this); //Make document return us..
     m_selfOnlyRefCount = 0;
 
     m_paintDevice = 0;
@@ -395,6 +394,7 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_loadingXMLDoc = 0;
     m_documentElement = 0;
     m_cssTarget = 0;
+    m_jsEditor = 0;
     m_dynamicDomRestyler = new khtml::DynamicDomRestyler();
     m_stateRestorePos = 0;
 }
@@ -480,6 +480,7 @@ DocumentImpl::~DocumentImpl()
     delete m_attrMap;
     delete m_namespaceMap;
     delete m_dynamicDomRestyler;
+    delete m_jsEditor;
     m_defaultView->deref();
     m_styleSheets->deref();
     if (m_addedStyleSheets)
@@ -549,7 +550,7 @@ ElementImpl *DocumentImpl::createElement( const DOMString &name, int* pException
     if ( pExceptioncode && *pExceptioncode )
         return 0;
 
-    XMLElementImpl* e = new XMLElementImpl( getDocument(), id );
+    XMLElementImpl* e = new XMLElementImpl( document(), id );
     e->setHTMLCompat( htmlMode() != XHtml ); // Not a real HTML element, but inside an html-compat doc all tags are uppercase.
     return e;
 }
@@ -560,7 +561,7 @@ AttrImpl *DocumentImpl::createAttribute( const DOMString &tagName, int* pExcepti
                   false /* allocate */, isHTMLDocument(), pExceptioncode);
     if ( pExceptioncode && *pExceptioncode )
         return 0;
-    AttrImpl* attr = new AttrImpl( 0, getDocument(), id, DOMString("").implementation());
+    AttrImpl* attr = new AttrImpl( 0, document(), id, DOMString("").implementation());
     attr->setHTMLCompat( htmlMode() != XHtml );
     return attr;
 }
@@ -590,6 +591,11 @@ EntityReferenceImpl *DocumentImpl::createEntityReference ( const DOMString &name
     return new EntityReferenceImpl(docPtr(), name.implementation());
 }
 
+EditingTextImpl *DocumentImpl::createEditingTextNode(const DOMString &text)
+{
+    return new EditingTextImpl(docPtr(), text);
+}
+
 NodeImpl *DocumentImpl::importNode(NodeImpl *importedNode, bool deep, int &exceptioncode)
 {
     NodeImpl *result = 0;
@@ -707,7 +713,7 @@ ElementImpl *DocumentImpl::createElementNS( const DOMString &_namespaceURI, cons
     if (!e) {
         Id id = getId(NodeImpl::ElementId, _namespaceURI.implementation(), prefix.implementation(),
                       localName.implementation(), false, false /*HTML already looked up*/);
-        e = new XMLElementImpl( getDocument(), id, prefix.implementation() );
+        e = new XMLElementImpl( document(), id, prefix.implementation() );
     }
 
     return e;
@@ -726,7 +732,7 @@ AttrImpl *DocumentImpl::createAttributeNS( const DOMString &_namespaceURI,
     splitPrefixLocalName(_qualifiedName.implementation(), prefix, localName, colonPos);
     Id id = getId(NodeImpl::AttributeId, _namespaceURI.implementation(), prefix.implementation(),
                   localName.implementation(), false, true /*lookupHTML*/);
-    AttrImpl* attr = new AttrImpl(0, getDocument(), id, DOMString("").implementation(),
+    AttrImpl* attr = new AttrImpl(0, document(), id, DOMString("").implementation(),
                          prefix.implementation());
     attr->setHTMLCompat( _namespaceURI.isNull() && htmlMode() != XHtml );
     return attr;
@@ -1003,6 +1009,17 @@ ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
         n = new HTMLScriptElementImpl(docPtr());
         break;
 
+// media
+    case ID_AUDIO:
+        n = new HTMLAudioElement(docPtr());
+        break;
+    case ID_VIDEO:
+        n = new HTMLVideoElement(docPtr());
+        break;
+    case ID_SOURCE:
+        n = new HTMLSourceElement(docPtr());
+        break;
+
 // tables
     case ID_TABLE:
         n = new HTMLTableElementImpl(docPtr());
@@ -1075,6 +1092,9 @@ ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
     case ID_NOBR:
     case ID_BDO:
     case ID_NOFRAMES:
+    case ID_NOSCRIPT:
+    case ID_NOEMBED:
+    case ID_NOLAYER:
         n = new HTMLGenericElementImpl(docPtr(), id);
         break;
 
@@ -1287,6 +1307,9 @@ void DocumentImpl::updateDocumentsRendering()
 
 void DocumentImpl::updateLayout()
 {
+    if (ElementImpl* oe = ownerElement())
+        oe->document()->updateLayout();
+
     bool oldIgnore = m_ignorePendingStylesheets;
 
     if (!haveStylesheetsLoaded()) {
@@ -1370,6 +1393,23 @@ void DocumentImpl::clearSelection()
         static_cast<RenderCanvas*>(m_render)->clearSelection();
 }
 
+void DocumentImpl::updateSelection()
+{
+    if (!m_render)
+        return;
+
+    RenderCanvas *canvas = static_cast<RenderCanvas*>(m_render);
+    Selection s = part()->caret();
+    if (s.isEmpty() || s.state() == Selection::CARET) {
+        canvas->clearSelection();
+    }
+    else {
+        RenderObject *startRenderer = s.start().node() ? s.start().node()->renderer() : 0;
+        RenderObject *endRenderer = s.end().node() ? s.end().node()->renderer() : 0;
+        static_cast<RenderCanvas*>(m_render)->setSelection(startRenderer, s.start().offset(), endRenderer, s.end().offset());
+    }
+}
+
 khtml::Tokenizer *DocumentImpl::createTokenizer()
 {
     return new khtml::XMLTokenizer(docPtr(),m_view);
@@ -1494,7 +1534,7 @@ CSSStyleSheetImpl* DocumentImpl::elementSheet()
     return m_elemSheet;
 }
 
-void DocumentImpl::determineParseMode( const QString &/*str*/ )
+void DocumentImpl::determineParseMode()
 {
     // For XML documents, use strict parse mode
     pMode = Strict;
@@ -1714,7 +1754,7 @@ ElementImpl* DocumentImpl::findAccessKeyElement(QChar c)
 
 int DocumentImpl::nodeAbsIndex(NodeImpl *node)
 {
-    assert(node->getDocument() == this);
+    assert(node->document() == this);
 
     int absIndex = 0;
     for (NodeImpl *n = node; n && n != this; n = n->traversePreviousNode())
@@ -1735,7 +1775,7 @@ void DocumentImpl::processHttpEquiv(const DOMString &equiv, const DOMString &con
 {
     assert(!equiv.isNull() && !content.isNull());
 
-    KHTMLView *v = getDocument()->view();
+    KHTMLView *v = document()->view();
 
     if(strcasecmp(equiv, "refresh") == 0 && v && v->part()->metaRefreshEnabled())
     {
@@ -1765,9 +1805,9 @@ void DocumentImpl::processHttpEquiv(const DOMString &equiv, const DOMString &con
                   (str[str.length()-1] == ';' || str[str.length()-1] == ','))
                 str.resize(str.length()-1);
             str = parseURL( DOMString(str) ).string();
-            QString newURL = getDocument()->completeURL( str );
+            QString newURL = document()->completeURL( str );
             if ( ok )
-                v->part()->scheduleRedirection(delay, getDocument()->completeURL( str ),  delay < 2 || newURL == URL().url());
+                v->part()->scheduleRedirection(delay, document()->completeURL( str ),  delay < 2 || newURL == URL().url());
         }
     }
     else if(strcasecmp(equiv, "expires") == 0)
@@ -1876,7 +1916,7 @@ bool DocumentImpl::childTypeAllowed( unsigned short type )
     return false;
 }
 
-NodeImpl *DocumentImpl::cloneNode ( bool deep )
+WTF::PassRefPtr<NodeImpl> DocumentImpl::cloneNode ( bool deep )
 {
 #if 0
     NodeImpl *dtn = m_doctype->cloneNode(deep);
@@ -1884,15 +1924,16 @@ NodeImpl *DocumentImpl::cloneNode ( bool deep )
 #endif
 
     int exceptioncode;
-    DocumentImpl *clone = m_implementation->createDocument("",
+    WTF::RefPtr<NodeImpl> clone = m_implementation->createDocument("",
 							   "",
-							   0, exceptioncode);
+                                                           0, 0,
+                                                           exceptioncode);
     assert( exceptioncode == 0 );
 
     // ### attributes, styles, ...
 
     if (deep)
-	cloneChildNodes(clone);
+	cloneChildNodes(clone.get());
 
     return clone;
 }
@@ -2360,7 +2401,7 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
     if( !m_render ) return;
 
     // Make sure newFocusNode is actually in this document
-    if (newFocusNode && (newFocusNode->getDocument() != this))
+    if (newFocusNode && (newFocusNode->document() != this))
         return;
 
     if (m_focusNode != newFocusNode) {
@@ -2466,7 +2507,7 @@ bool DocumentImpl::isURLAllowed(const QString& url) const
         return false;
 
     // do we allow this suburl ?
-    if ( !kapp || (newURL.protocol() != "javascript" && !KAuthorized::authorizeUrlAction("redirect", thisPart->url(), newURL)) )
+    if (newURL.protocol() != "javascript" && !KAuthorized::authorizeUrlAction("redirect", thisPart->url(), newURL))
         return false;
 
     // We allow one level of self-reference because some sites depend on that.
@@ -2855,6 +2896,46 @@ void DOM::DocumentImpl::releaseCachedNodeListInfo(NodeListImpl::Cache* entry)
     entry->deref();
 }
 
+// ----------------------------------------------------------------------------
+// Support for Javascript execCommand, and related methods
+
+JSEditor *DocumentImpl::jsEditor()
+{
+    if (!m_jsEditor)
+        m_jsEditor = new JSEditor(this);
+    return m_jsEditor;
+}
+
+bool DocumentImpl::execCommand(const DOMString &command, bool userInterface, const DOMString &value)
+{
+    return jsEditor()->execCommand(jsEditor()->commandImp(command), userInterface, value);
+}
+
+bool DocumentImpl::queryCommandEnabled(const DOMString &command)
+{
+    return jsEditor()->queryCommandEnabled(jsEditor()->commandImp(command));
+}
+
+bool DocumentImpl::queryCommandIndeterm(const DOMString &command)
+{
+    return jsEditor()->queryCommandIndeterm(jsEditor()->commandImp(command));
+}
+
+bool DocumentImpl::queryCommandState(const DOMString &command)
+{
+    return jsEditor()->queryCommandState(jsEditor()->commandImp(command));
+}
+
+bool DocumentImpl::queryCommandSupported(const DOMString &command)
+{
+    return jsEditor()->queryCommandSupported(jsEditor()->commandImp(command));
+}
+
+DOMString DocumentImpl::queryCommandValue(const DOMString &command)
+{
+    return jsEditor()->queryCommandValue(jsEditor()->commandImp(command));
+}
+
 // ----------------------------------------------------------------------------
 
 DocumentFragmentImpl::DocumentFragmentImpl(DocumentImpl *doc) : NodeBaseImpl(doc)
@@ -2906,11 +2987,11 @@ DOMString DocumentFragmentImpl::toString() const
     return result;
 }
 
-NodeImpl *DocumentFragmentImpl::cloneNode ( bool deep )
+WTF::PassRefPtr<NodeImpl> DocumentFragmentImpl::cloneNode ( bool deep )
 {
-    DocumentFragmentImpl *clone = new DocumentFragmentImpl( docPtr() );
+    WTF::RefPtr<DocumentFragmentImpl> clone = new DocumentFragmentImpl( docPtr() );
     if (deep)
-        cloneChildNodes(clone);
+        cloneChildNodes(clone.get());
     return clone;
 }
 
@@ -2943,7 +3024,7 @@ DocumentTypeImpl::~DocumentTypeImpl()
 
 DOMString DocumentTypeImpl::toString() const
 {
-    DOMString result = "<!DOCTYPE";
+    DOMString result = "<!DOCTYPE ";
     result += m_qualifiedName;
     if (!m_publicId.isEmpty()) {
 	result += " PUBLIC \"";
@@ -2984,7 +3065,7 @@ bool DocumentTypeImpl::childTypeAllowed( unsigned short /*type*/ )
     return false;
 }
 
-NodeImpl *DocumentTypeImpl::cloneNode ( bool /*deep*/ )
+WTF::PassRefPtr<NodeImpl> DocumentTypeImpl::cloneNode ( bool /*deep*/ )
 {
     DocumentTypeImpl *clone = new DocumentTypeImpl(implementation(),
 						   0,
