@@ -47,6 +47,7 @@
 #include <css/cssstyleselector.h>
 #include <css/cssvalues.h>
 #include <css/cssproperties.h>
+#include <khtml_part.h>
 
 #include <QtCore/QTextIStream>
 #include <QTextDocument>
@@ -124,14 +125,14 @@ DOMString AttrImpl::namespaceURI() const
 {
     if (m_htmlCompat)
         return DOMString();
-    return getDocument()->getName(NamespaceId, m_attrId >> 16);
+    return document()->getName(NamespaceId, m_attrId >> 16);
 }
 
 DOMString AttrImpl::localName() const
 {
     if (m_htmlCompat)
        return DOMString();
-    return getDocument()->getName(AttributeId, m_attrId);
+    return document()->getName(AttributeId, m_attrId);
 }
 
 DOMString AttrImpl::nodeValue() const
@@ -141,7 +142,7 @@ DOMString AttrImpl::nodeValue() const
 
 DOMString AttrImpl::name() const
 {
-    DOMString n = getDocument()->getName(AttributeId, m_attrId);
+    DOMString n = document()->getName(AttributeId, m_attrId);
 
     // compat mode always return attribute names in lowercase.
     // that's not formally in the specification, but common
@@ -161,8 +162,10 @@ void AttrImpl::createTextChild()
     if (m_value->length() > 0) {
 	TextImpl* textNode = ownerDocument()->createTextNode(m_value);
 	int exceptioncode;
-	appendChild(textNode, exceptioncode);
-	assert(exceptioncode == 0);
+
+	// We want to use addChild and not appendChild here to avoid triggering
+	// mutation events. childrenChanged() will still be called.
+	addChild(textNode);
     }
 }
 
@@ -222,7 +225,7 @@ void AttrImpl::setNodeValue( const DOMString &v, int &exceptioncode )
     setValue(v, exceptioncode);
 }
 
-NodeImpl *AttrImpl::cloneNode ( bool /*deep*/)
+WTF::PassRefPtr<NodeImpl> AttrImpl::cloneNode ( bool /*deep*/)
 {
      AttrImpl* attr = new AttrImpl(0, docPtr(), m_attrId, m_value, m_prefix);
      attr->setHTMLCompat(m_htmlCompat);
@@ -366,6 +369,16 @@ ElementImpl::~ElementImpl()
         m_prefix->deref();
 }
 
+void ElementImpl::removeAttribute( NodeImpl::Id id, int &exceptioncode )
+{
+    if (namedAttrMap) {
+        namedAttrMap->removeNamedItem(id, false, 0, exceptioncode);
+        if (exceptioncode == DOMException::NOT_FOUND_ERR) {
+            exceptioncode = 0;
+        }
+    }
+}
+
 unsigned short ElementImpl::nodeType() const
 {
     return Node::ELEMENT_NODE;
@@ -419,7 +432,7 @@ void ElementImpl::setAttributeNS( const DOMString &namespaceURI, const DOMString
         return;
     DOMString prefix, localName;
     splitPrefixLocalName(qualifiedName.implementation(), prefix, localName, colonPos);
-    NodeImpl::Id id = getDocument()->getId(AttributeId, namespaceURI.implementation(),
+    NodeImpl::Id id = document()->getId(AttributeId, namespaceURI.implementation(),
                             prefix.implementation(), localName.implementation(), false, true /*lookupHTML*/);
     attributes()->setValue(id, value.implementation(), 0, prefix.implementation(),
                            true /*nsAware*/, !namespaceURI.isNull() /*hasNS*/);
@@ -461,15 +474,15 @@ void ElementImpl::setAttributeMap( NamedAttrMapImpl* list )
     }
 }
 
-NodeImpl *ElementImpl::cloneNode(bool deep)
+WTF::PassRefPtr<NodeImpl> ElementImpl::cloneNode(bool deep)
 {
-    ElementImpl *clone;
+    WTF::RefPtr<ElementImpl> clone; // Make sure to guard...
     if ( !localName().isNull() )
-        clone = getDocument()->createElementNS( namespaceURI(), nodeName() );
+        clone = document()->createElementNS( namespaceURI(), nodeName() );
     else
-        clone = getDocument()->createElement( nodeName() );
+        clone = document()->createElement( nodeName() );
     if (!clone) return 0;
-    finishCloneNode( clone, deep );
+    finishCloneNode( clone.get(), deep );
     return clone;
 }
 
@@ -504,18 +517,18 @@ bool ElementImpl::hasAttributes() const
     return namedAttrMap && namedAttrMap->length() > 0;
 }
 
-bool ElementImpl::hasAttribute( const DOMString& name )
+bool ElementImpl::hasAttribute( const DOMString& name ) const
 {
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
     if (!id) return false;
     if (!namedAttrMap) return false;
     return namedAttrMap->getValue(id, false, name.implementation()) != 0;
 }
 
 bool ElementImpl::hasAttributeNS( const DOMString &namespaceURI,
-                              const DOMString &localName )
+                                  const DOMString &localName ) const
 {
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId,namespaceURI.implementation(),
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId,namespaceURI.implementation(),
 						 0/*prefix*/, localName.implementation(), true, true);
     if (!id) return false;
     if (!namedAttrMap) return false;
@@ -524,7 +537,7 @@ bool ElementImpl::hasAttributeNS( const DOMString &namespaceURI,
 
 DOMString ElementImpl::getAttribute( const DOMString &name )
 {
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId,name.implementation(),true,true);
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId,name.implementation(),true,true);
     if (!id) return DOMString();
 
     return getAttribute(id, false, name);
@@ -532,7 +545,7 @@ DOMString ElementImpl::getAttribute( const DOMString &name )
 
 void ElementImpl::setAttribute( const DOMString &name, const DOMString &value, int& exceptioncode )
 {
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId, name.implementation(), false /* allocate */,
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId, name.implementation(), false /* allocate */,
                                                  true, &exceptioncode);
 
     setAttribute(id, value, name, exceptioncode);
@@ -540,7 +553,7 @@ void ElementImpl::setAttribute( const DOMString &name, const DOMString &value, i
 
 void ElementImpl::removeAttribute( const DOMString &name, int& exceptioncode )
 {
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
     if (!id) return;
 
     attributes(false)->removeNamedItem(id, false, name.implementation(), exceptioncode);
@@ -551,7 +564,7 @@ void ElementImpl::removeAttribute( const DOMString &name, int& exceptioncode )
 
 AttrImpl* ElementImpl::getAttributeNode( const DOMString &name )
 {
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
     if (!id) return 0;
     if (!namedAttrMap) return 0;
 
@@ -601,7 +614,7 @@ DOMString ElementImpl::getAttributeNS( const DOMString &namespaceURI,
       return DOMString();
     }
 
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId, namespaceURI.implementation(), 0/*prefix*/, localName.implementation(), true, true);
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId, namespaceURI.implementation(), 0/*prefix*/, localName.implementation(), true, true);
     return getAttribute(id, true);
 }
 
@@ -609,7 +622,7 @@ void ElementImpl::removeAttributeNS( const DOMString &namespaceURI,
                                  const DOMString &localName,
                                  int& exceptioncode)
 {
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId, namespaceURI.implementation(), 0/*prefix*/, localName.implementation(), false, true);
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId, namespaceURI.implementation(), 0/*prefix*/, localName.implementation(), false, true);
     attributes(false)->removeNamedItem(id, true, 0, exceptioncode);
 }
 
@@ -622,7 +635,7 @@ AttrImpl* ElementImpl::getAttributeNodeNS( const DOMString &namespaceURI,
       return 0;
     }
 
-    NodeImpl::Id id = getDocument()->getId(NodeImpl::AttributeId, namespaceURI.implementation(),
+    NodeImpl::Id id = document()->getId(NodeImpl::AttributeId, namespaceURI.implementation(),
 						0/*prefix*/, localName.implementation(), true, true);
     if (!attributes(true)) return 0;
 
@@ -652,7 +665,7 @@ DOMString ElementImpl::namespaceURI() const
 {
    if (m_htmlCompat)
         return DOMString();
-   return getDocument()->getName(NamespaceId, id() >> 16);
+   return document()->getName(NamespaceId, id() >> 16);
 }
 
 DOMString ElementImpl::prefix() const
@@ -708,12 +721,12 @@ NamedAttrMapImpl* ElementImpl::defaultMap() const
 
 RenderStyle *ElementImpl::styleForRenderer(RenderObject * /*parentRenderer*/)
 {
-    return getDocument()->styleSelector()->styleForElement(this);
+    return document()->styleSelector()->styleForElement(this);
 }
 
 RenderObject *ElementImpl::createRenderer(RenderArena *arena, RenderStyle *style)
 {
-    if (getDocument()->documentElement() == this && style->display() == NONE) {
+    if (document()->documentElement() == this && style->display() == NONE) {
         // Ignore display: none on root elements.  Force a display of block in that case.
         RenderBlock* result = new (arena) RenderBlock(this);
         if (result) result->setStyle(style);
@@ -746,7 +759,7 @@ void ElementImpl::close()
 
 void ElementImpl::detach()
 {
-    getDocument()->dynamicDomRestyler().resetDependencies(this);
+    document()->dynamicDomRestyler().resetDependencies(this);
 
     NodeBaseImpl::detach();
 }
@@ -755,39 +768,39 @@ void ElementImpl::structureChanged()
 {
     NodeBaseImpl::structureChanged();
 
-    if (!getDocument()->renderer())
+    if (!document()->renderer())
         return; // the document is about to be destroyed
 
-    getDocument()->dynamicDomRestyler().restyleDepedent(this, StructuralDependency);
+    document()->dynamicDomRestyler().restyleDepedent(this, StructuralDependency);
     // In theory BackwardsStructurualDependencies are indifferent to prepend,
     // but it's too rare to optimize.
-    getDocument()->dynamicDomRestyler().restyleDepedent(this, BackwardsStructuralDependency);
+    document()->dynamicDomRestyler().restyleDepedent(this, BackwardsStructuralDependency);
 }
 
 void ElementImpl::backwardsStructureChanged()
 {
     NodeBaseImpl::backwardsStructureChanged();
 
-    if (!getDocument()->renderer())
+    if (!document()->renderer())
         return; // the document is about to be destroyed
 
     // Most selectors are not affected by append. Fire the few that are.
-    getDocument()->dynamicDomRestyler().restyleDepedent(this, BackwardsStructuralDependency);
+    document()->dynamicDomRestyler().restyleDepedent(this, BackwardsStructuralDependency);
 }
 
 void ElementImpl::attributeChanged(NodeImpl::Id id)
 {
-    if (!getDocument()->renderer())
+    if (!document()->renderer())
         return; // the document is about to be destroyed
 
 #if 0 // one-one dependencies for attributes disabled
-    getDocument()->dynamicDomRestyler().restyleDepedent(this, AttributeDependency);
+    document()->dynamicDomRestyler().restyleDepedent(this, AttributeDependency);
 #endif
-    if (getDocument()->dynamicDomRestyler().checkDependency(id, PersonalDependency))
+    if (document()->dynamicDomRestyler().checkDependency(id, PersonalDependency))
         setChanged(true);
-    if (getDocument()->dynamicDomRestyler().checkDependency(id, AncestorDependency))
+    if (document()->dynamicDomRestyler().checkDependency(id, AncestorDependency))
         setChangedAscendentAttribute(true);
-    if (getDocument()->dynamicDomRestyler().checkDependency(id, PredecessorDependency) && parent())
+    if (document()->dynamicDomRestyler().checkDependency(id, PredecessorDependency) && parent())
         // Any element that dependt on a predecessors attribute, also depend structurally on parent
         parent()->structureChanged();
 }
@@ -813,7 +826,7 @@ void ElementImpl::recalcStyle( StyleChange change )
     qDebug("recalcStyle(%d: %s, changed: %d)[%p: %s]", change, debug, changed(), this, tagName().string().toLatin1().constData());
 #endif
     if ( hasParentRenderer && (change >= Inherit || changed() || (change==NoInherit && affectedByNoInherit())) ) {
-        RenderStyle *newStyle = getDocument()->styleSelector()->styleForElement(this);
+        RenderStyle *newStyle = document()->styleSelector()->styleForElement(this);
         newStyle->ref();
         StyleChange ch = diff( _style, newStyle );
         if (ch == Detach) {
@@ -864,7 +877,43 @@ bool ElementImpl::isFocusable() const
     // Only make editable elements selectable if its parent element
     // is not editable. FIXME: this is not 100% right as non-editable elements
     // within editable elements are focusable too.
-    return contentEditable() && !(parentNode() && parentNode()->contentEditable());
+    return isContentEditable() && !(parentNode() && parentNode()->isContentEditable());
+}
+
+bool ElementImpl::isContentEditable() const
+{
+    if (document()->part() && document()->part()->isEditable())
+        return true;
+
+    // document()->updateRendering();
+
+    if (!renderer()) {
+        if (parentNode())
+            return parentNode()->isContentEditable();
+        else
+            return false;
+    }
+
+    return renderer()->style()->userInput() == UI_ENABLED;
+}
+
+void ElementImpl::setContentEditable(bool enabled) {
+    // FIXME: the approach is flawed, better use an enum instead of bool
+    int value;
+    if (enabled)
+        value = CSS_VAL_ENABLED;
+    else {
+        // Intelligently use "none" or "disabled", depending on the type of
+        // element
+        // FIXME: intelligence not impl'd yet
+        value = CSS_VAL_NONE;
+
+        // FIXME: reset caret if it is in this node or a child
+    }/*end if*/
+    // FIXME: use addCSSProperty when I get permission to move it here
+//    kDebug(6000) << "CSS_PROP__KHTML_USER_INPUT: "<< value << endl;
+    getInlineStyleDecls()->setProperty(CSS_PROP__KHTML_USER_INPUT, value, false);
+    setChanged();
 }
 
 // DOM Section 1.1.1
@@ -880,7 +929,7 @@ bool ElementImpl::childAllowed( NodeImpl *newChild )
     if (isXMLElementNode() || newChild->isXMLElementNode())
 	return true;
     else
-	return checkChild(id(), newChild->id(), getDocument()->inStrictMode());
+	return checkChild(id(), newChild->id(), document()->inStrictMode());
 }
 
 bool ElementImpl::childTypeAllowed( unsigned short type )
@@ -914,7 +963,7 @@ void ElementImpl::createNonCSSDecl()
     CSSStyleDeclarationImpl *ncd = new CSSStyleDeclarationImpl(0);
     m_style.combinedDecls->nonCSSDecls = ncd;
     ncd->ref();
-    ncd->setParent(getDocument()->elementSheet());
+    ncd->setParent(document()->elementSheet());
     ncd->setNode(this);
     ncd->setStrictParsing( false );
     m_hasCombinedStyle = true;
@@ -932,7 +981,7 @@ void ElementImpl::createInlineDecl( )
 
     CSSStyleDeclarationImpl *dcl = new CSSStyleDeclarationImpl(0);
     dcl->ref();
-    dcl->setParent(getDocument()->elementSheet());
+    dcl->setParent(document()->elementSheet());
     dcl->setNode(this);
     dcl->setStrictParsing( false );
     if (m_hasCombinedStyle)
@@ -944,21 +993,21 @@ void ElementImpl::createInlineDecl( )
 void ElementImpl::dispatchAttrRemovalEvent(NodeImpl::Id /*id*/, DOMStringImpl * /*value*/)
 {
     // ### enable this stuff again
-    if (!getDocument()->hasListenerType(DocumentImpl::DOMATTRMODIFIED_LISTENER))
+    if (!document()->hasListenerType(DocumentImpl::DOMATTRMODIFIED_LISTENER))
 	return;
     //int exceptioncode = 0;
     //dispatchEvent(new MutationEventImpl(EventImpl::DOMATTRMODIFIED_EVENT,true,false,attr,attr->value(),
-    //attr->value(), getDocument()->attrName(attr->id()),MutationEvent::REMOVAL),exceptioncode);
+    //attr->value(), document()->attrName(attr->id()),MutationEvent::REMOVAL),exceptioncode);
 }
 
 void ElementImpl::dispatchAttrAdditionEvent(NodeImpl::Id /*id*/, DOMStringImpl * /*value*/)
 {
     // ### enable this stuff again
-    if (!getDocument()->hasListenerType(DocumentImpl::DOMATTRMODIFIED_LISTENER))
+    if (!document()->hasListenerType(DocumentImpl::DOMATTRMODIFIED_LISTENER))
 	return;
    //int exceptioncode = 0;
    //dispatchEvent(new MutationEventImpl(EventImpl::DOMATTRMODIFIED_EVENT,true,false,attr,attr->value(),
-   //attr->value(),getDocument()->attrName(attr->id()),MutationEvent::ADDITION),exceptioncode);
+   //attr->value(),document()->attrName(attr->id()),MutationEvent::ADDITION),exceptioncode);
 }
 
 void ElementImpl::updateId(DOMStringImpl* oldId, DOMStringImpl* newId)
@@ -975,12 +1024,12 @@ void ElementImpl::updateId(DOMStringImpl* oldId, DOMStringImpl* newId)
 
 void ElementImpl::removeId(const QString& id)
 {
-  getDocument()->getElementByIdCache().remove(id, this);
+  document()->getElementByIdCache().remove(id, this);
 }
 
 void ElementImpl::addId(const QString& id)
 {
-  getDocument()->getElementByIdCache().add(id, this);
+  document()->getElementByIdCache().add(id, this);
 }
 
 void ElementImpl::insertedIntoDocument()
@@ -1022,21 +1071,21 @@ DOMString ElementImpl::openTagStartToString(bool expandurls) const
 	    if (attr) {
 		result += attr->toString();
 	    } else {
-		result += getDocument()->getName( NodeImpl::AttributeId, attribute->id());
+		result += document()->getName( NodeImpl::AttributeId, attribute->id());
 		if (!attribute->value().isNull()) {
 		    result += "=\"";
 		    // FIXME: substitute entities for any instances of " or '
 		    // Expand out all urls, i.e. the src and href attributes
 		    if(expandurls && ( attribute->id() == ATTR_SRC || attribute->id() == ATTR_HREF))
-			if(getDocument()) {
+			if(document()) {
                             //We need to sanitize the urls - strip out the passwords.
 			    //FIXME:   are src=  and href=  the only places that might have a password and need to be sanitized?
-			    KUrl safeURL(getDocument()->completeURL(attribute->value().string()));
+			    KUrl safeURL(document()->completeURL(attribute->value().string()));
 			    safeURL.setPass(QString());
 			    result += Qt::escape(safeURL.prettyUrl());
 			}
 		        else {
-		 	    kWarning() << "getDocument() returned false";
+		 	    kWarning() << "document() returned false";
 			    result += attribute->value();
 			}
 		    else
@@ -1098,37 +1147,6 @@ DOMString ElementImpl::toString() const
     return result;
 }
 
-bool ElementImpl::contentEditable() const {
-#if 0
-    DOM::CSSPrimitiveValueImpl *val = static_cast<DOM::CSSPrimitiveValueImpl *>
-    		(const_cast<ElementImpl *>(this)->getInlineStyleDecls()
-		->getPropertyCSSValue(CSS_PROP__KONQ_USER_INPUT));
-//    kDebug() << "val" << val;
-    return val ? val->getIdent() == CSS_VAL_ENABLED : false;
-#endif
-    return NodeImpl::contentEditable();
-}
-
-void ElementImpl::setContentEditable(bool enabled) {
-    // FIXME: the approach is flawed, better use an enum instead of bool
-    int value;
-    if (enabled)
-        value = CSS_VAL_ENABLED;
-    else {
-        // Intelligently use "none" or "disabled", depending on the type of
-        // element
-	// FIXME: intelligence not impl'd yet
-	value = CSS_VAL_NONE;
-
-        // FIXME: reset caret if it is in this node or a child
-    }/*end if*/
-    // FIXME: use addCSSProperty when I get permission to move it here
-//    kDebug(6000) << "CSS_PROP__KHTML_USER_INPUT: "<< value;
-    getInlineStyleDecls()->setProperty(CSS_PROP__KHTML_USER_INPUT, value, false);
-    setChanged();
-
-}
-
 // -------------------------------------------------------------------------
 
 XMLElementImpl::XMLElementImpl(DocumentImpl *doc, NodeImpl::Id id)
@@ -1157,12 +1175,12 @@ DOMString XMLElementImpl::localName() const
 {
     if ( m_htmlCompat )
        return DOMString(); // was created with non-namespace-aware createElement()
-    return getDocument()->getName(ElementId, m_id);
+    return document()->getName(ElementId, m_id);
 }
 
 DOMString XMLElementImpl::tagName() const
 {
-    DOMString tn = getDocument()->getName(ElementId, id());
+    DOMString tn = document()->getName(ElementId, id());
     if (m_htmlCompat)
         tn = tn.upper();
 
@@ -1172,10 +1190,10 @@ DOMString XMLElementImpl::tagName() const
     return tn;
 }
 
-NodeImpl *XMLElementImpl::cloneNode ( bool deep )
+WTF::PassRefPtr<NodeImpl> XMLElementImpl::cloneNode ( bool deep )
 {
-    XMLElementImpl *clone = new XMLElementImpl(docPtr(), id(), m_prefix);
-    finishCloneNode( clone, deep );
+    WTF::RefPtr<ElementImpl> clone = new XMLElementImpl(docPtr(), id(), m_prefix);
+    finishCloneNode( clone.get(), deep );
     return clone;
 }
 
@@ -1270,7 +1288,7 @@ Node NamedAttrMapImpl::setNamedItem ( NodeImpl* arg, bool nsAware, DOMStringImpl
     }
 
     // WRONG_DOCUMENT_ERR: Raised if arg was created from a different document than the one that created this map.
-    if (arg->getDocument() != m_element->getDocument()) {
+    if (arg->document() != m_element->document()) {
         exceptioncode = DOMException::WRONG_DOCUMENT_ERR;
         return 0;
     }
@@ -1432,7 +1450,7 @@ void NamedAttrMapImpl::setValue(NodeImpl::Id id, DOMStringImpl *value, DOMString
 	m_attrs[m_attrCount-1].m_attrId = 0; /* "has implementation" flag */
 	m_attrs[m_attrCount-1].m_data.attr->ref();
         m_attrs[m_attrCount-1].m_data.attr->setHTMLCompat( !hasNS &&
-                                                 m_element->getDocument()->htmlMode() != DocumentImpl::XHtml );
+                                                 m_element->document()->htmlMode() != DocumentImpl::XHtml );
     }
     if (m_element) {
 	if (id == ATTR_ID)
@@ -1471,7 +1489,7 @@ NodeImpl::Id NamedAttrMapImpl::mapId(DOMStringImpl* namespaceURI,
     if (!m_element)
 	return 0;
 
-    return m_element->getDocument()->getId(NodeImpl::AttributeId, namespaceURI, 0, localName, readonly,
+    return m_element->document()->getId(NodeImpl::AttributeId, namespaceURI, 0, localName, readonly,
                                            true /*lookupHTML*/);
 }
 
@@ -1493,7 +1511,8 @@ void NamedAttrMapImpl::copyAttributes(NamedAttrMapImpl *other)
 	    m_attrs[i].m_data.value->ref();
 	}
 	else {
-	    m_attrs[i].m_data.attr = static_cast<AttrImpl*>(other->m_attrs[i].m_data.attr->cloneNode(true));
+	    WTF::RefPtr<NodeImpl> clonedAttr = other->m_attrs[i].m_data.attr->cloneNode(true);
+	    m_attrs[i].m_data.attr = static_cast<AttrImpl*>(clonedAttr.get());
 	    m_attrs[i].m_data.attr->ref();
 	    m_attrs[i].m_data.attr->setElement(m_element);
 	}
