@@ -213,7 +213,7 @@ void CharacterDataImpl::dispatchModifiedEvent(DOMStringImpl *prevValue)
        // ### to fully support dynamic changes to :contains selector
        // backwardsStructureChanged should be called for all changes
     }
-    if (!getDocument()->hasListenerType(DocumentImpl::DOMCHARACTERDATAMODIFIED_LISTENER))
+    if (!document()->hasListenerType(DocumentImpl::DOMCHARACTERDATAMODIFIED_LISTENER))
         return;
 
     DOMStringImpl *newValue = str->copy();
@@ -245,33 +245,29 @@ void CharacterDataImpl::checkCharDataOperation( const unsigned long offset, int
     }
 }
 
-long CharacterDataImpl::minOffset() const
-{
-  RenderText *r = static_cast<RenderText *>(renderer());
-  if (!r || !r->isText()) return 0;
-
-  // take :first-letter into consideration
-#ifdef __GNUC__
-#warning FIXME
-#endif
-#if 0
-  if (r->forcedMinOffset()) {
-    RenderFlow *firstLetter = static_cast<RenderFlow *>(r->previousSibling());
-    if (firstLetter && firstLetter->isFlow() && firstLetter->isFirstLetter()) {
-      RenderText *letterText = static_cast<RenderText *>(firstLetter->firstChild());
-      return letterText->minOffset();
-    }
-  }
-#endif
-
-  return r->minOffset();
-}
-
 long CharacterDataImpl::maxOffset() const
 {
   RenderText *r = static_cast<RenderText *>(renderer());
-  if (!r || !r->isText()) return (long)length();
-  return r->maxOffset();
+  if (!r || !r->isText()) return 0;
+    return (long)length();
+}
+
+long CharacterDataImpl::caretMinOffset() const
+{
+    RenderText *r = static_cast<RenderText *>(renderer());
+    return r && r->isText() ? r->caretMinOffset() : 0;
+}
+
+long CharacterDataImpl::caretMaxOffset() const
+{
+    RenderText *r = static_cast<RenderText *>(renderer());
+    return r && r->isText() ? r->caretMaxOffset() : (long)length();
+}
+
+unsigned long CharacterDataImpl::caretMaxRenderedOffset() const
+{
+    RenderText *r = static_cast<RenderText *>(renderer());
+    return r ? r->caretMaxRenderedOffset() : length();
 }
 
 // ---------------------------------------------------------------------------
@@ -286,9 +282,9 @@ unsigned short CommentImpl::nodeType() const
     return Node::COMMENT_NODE;
 }
 
-NodeImpl *CommentImpl::cloneNode(bool /*deep*/)
+WTF::PassRefPtr<NodeImpl> CommentImpl::cloneNode(bool /*deep*/)
 {
-    return getDocument()->createComment( str );
+    return document()->createComment( str );
 }
 
 NodeImpl::Id CommentImpl::id() const
@@ -452,9 +448,9 @@ unsigned short TextImpl::nodeType() const
     return Node::TEXT_NODE;
 }
 
-NodeImpl *TextImpl::cloneNode(bool /*deep*/)
+WTF::PassRefPtr<NodeImpl> TextImpl::cloneNode(bool /*deep*/)
 {
-    return getDocument()->createTextNode(str);
+    return document()->createTextNode(str);
 }
 
 bool TextImpl::rendererIsNeeded(RenderStyle *style)
@@ -502,7 +498,7 @@ bool TextImpl::rendererIsNeeded(RenderStyle *style)
     return true;
 }
 
-RenderObject *TextImpl::createRenderer(RenderArena *arena, RenderStyle */*style*/)
+RenderObject *TextImpl::createRenderer(RenderArena *arena, RenderStyle * /*style*/)
 {
     return new (arena) RenderText(this, str);
 }
@@ -586,9 +582,9 @@ unsigned short CDATASectionImpl::nodeType() const
     return Node::CDATA_SECTION_NODE;
 }
 
-NodeImpl *CDATASectionImpl::cloneNode(bool /*deep*/)
+WTF::PassRefPtr<NodeImpl> CDATASectionImpl::cloneNode(bool /*deep*/)
 {
-    return getDocument()->createCDATASection(str);
+    return document()->createCDATASection(str);
 }
 
 // DOM Section 1.1.1
@@ -608,5 +604,26 @@ DOMString CDATASectionImpl::toString() const
     return DOMString("<![CDATA[") + nodeValue() + "]]>";
 }
 
+// ---------------------------------------------------------------------------
+ 
+EditingTextImpl::EditingTextImpl(DocumentImpl *impl, const DOMString &text)
+    : TextImpl(impl, text.implementation())
+{
+}
 
+EditingTextImpl::EditingTextImpl(DocumentImpl *impl)
+    : TextImpl(impl)
+{
+}
+
+EditingTextImpl::~EditingTextImpl()
+{
+}
+
+bool EditingTextImpl::rendererIsNeeded(RenderStyle */*style*/)
+{
+    return true;
+}
+
+  
 
