@@ -100,7 +100,7 @@ unsigned short EntityImpl::nodeType() const
     return Node::ENTITY_NODE;
 }
 
-NodeImpl *EntityImpl::cloneNode ( bool /*deep*/)
+WTF::PassRefPtr<NodeImpl> EntityImpl::cloneNode ( bool /*deep*/)
 {
     // Spec says cloning Document nodes is "implementation dependent"
     // so we do not support it...
@@ -185,7 +185,7 @@ unsigned short EntityReferenceImpl::nodeType() const
     return Node::ENTITY_REFERENCE_NODE;
 }
 
-NodeImpl *EntityReferenceImpl::cloneNode ( bool deep )
+WTF::PassRefPtr<NodeImpl> EntityReferenceImpl::cloneNode ( bool deep )
 {
     EntityReferenceImpl *clone = new EntityReferenceImpl(docPtr(),m_entityName);
     // ### make sure children are readonly
@@ -273,7 +273,7 @@ unsigned short NotationImpl::nodeType() const
     return Node::NOTATION_NODE;
 }
 
-NodeImpl *NotationImpl::cloneNode ( bool /*deep*/)
+WTF::PassRefPtr<NodeImpl> NotationImpl::cloneNode ( bool /*deep*/)
 {
     // Spec says cloning Document nodes is "implementation dependent"
     // so we do not support it...
@@ -366,7 +366,7 @@ void ProcessingInstructionImpl::setNodeValue( const DOMString &_nodeValue, int &
     setData(_nodeValue, exceptioncode);
 }
 
-NodeImpl *ProcessingInstructionImpl::cloneNode ( bool /*deep*/)
+WTF::PassRefPtr<NodeImpl> ProcessingInstructionImpl::cloneNode ( bool /*deep*/)
 {
     // ### copy m_localHref
     return new ProcessingInstructionImpl(docPtr(),m_target,m_data);
@@ -415,9 +415,9 @@ void ProcessingInstructionImpl::checkStyleSheet()
                 // ### some validation on the URL?
                 // ### FIXME charset
                 if (m_cachedSheet) m_cachedSheet->deref(this);
-                m_cachedSheet = getDocument()->docLoader()->requestStyleSheet(getDocument()->completeURL(href.string()), QString());
+                m_cachedSheet = document()->docLoader()->requestStyleSheet(document()->completeURL(href.string()), QString());
                 if (m_cachedSheet) {
-                    getDocument()->addPendingSheet(); //before ref, because during the ref it might load!
+                    document()->addPendingSheet(); //before ref, because during the ref it might load!
                     m_cachedSheet->ref( this );
                 }
             }
@@ -435,7 +435,7 @@ void ProcessingInstructionImpl::setStyleSheet(const DOM::DOMString &url, const D
 {
     if (m_sheet)
 	m_sheet->deref();
-    m_sheet = new CSSStyleSheetImpl(getDocument(), url);
+    m_sheet = new CSSStyleSheetImpl(document(), url);
     m_sheet->ref();
     m_sheet->setCharset(charset);
     m_sheet->parseString(khtml::isAcceptableCSSMimetype(mimetype) ? sheet : "");
@@ -443,7 +443,7 @@ void ProcessingInstructionImpl::setStyleSheet(const DOM::DOMString &url, const D
 	m_cachedSheet->deref(this);
     m_cachedSheet = 0;
 
-    getDocument()->styleSheetLoaded();
+    document()->styleSheetLoaded();
 }
 
 void ProcessingInstructionImpl::setStyleSheet(CSSStyleSheetImpl* sheet)
