@@ -49,14 +49,17 @@
 #include <QtCore/QTextStream>
 #include <QtCore/QRegExp>
 #include <QtGui/QFont>
+#include <QDir>
 #include <kcomponentdata.h>
 #include <kstandarddirs.h>
+#include <kconfiggroup.h>
 #include <kglobal.h>
 #include <kconfig.h>
 #include <klibloader.h>
 #include <kapplication.h>
 #include <klocale.h>
 #include <kdebug.h>
+#include <kde_file.h>
 
 #ifdef Q_OS_LINUX
 #include <sys/prctl.h>
@@ -65,6 +68,10 @@
 #endif
 #endif
 
+#ifdef Q_WS_MACX
+#include <kkernel_mac.h>
+#endif
+
 #include <kdeversion.h>
 
 #include "klauncher_cmds.h"
@@ -116,6 +123,7 @@ static struct {
   int initpipe[2];
   int wrapper; /* socket for wrapper communication */
   int wrapper_old; /* old socket for wrapper communication */
+  int accepted_fd; /* socket accepted and that must be closed in the child process */
   char result;
   int exit_status;
   pid_t fork;
@@ -132,6 +140,15 @@ static struct {
   bool suicide;
 } d;
 
+struct child
+{
+  pid_t pid;
+  int sock; /* fd to write message when child is dead*/
+  struct child *next;
+};
+
+static struct child *children;
+
 #ifdef Q_WS_X11
 extern "C" {
 int kdeinit_xio_errhandler( Display * );
@@ -146,6 +163,19 @@ extern "C" KParts::Plugin* _kinit_init_kparts() { return new KParts::Plugin(); }
 #include <kio/authinfo.h>
 extern "C" KIO::AuthInfo* _kioslave_init_kio() { return new KIO::AuthInfo(); }
 
+/*
+ * Clean up the file descriptor table by closing all file descriptors
+ * that are still open.
+ *
+ * This function is called very early in the main() function, so that
+ * we don't leak anything that was leaked to us.
+ */
+static void cleanup_fds()
+{
+    for (int fd = 3; fd < FD_SETSIZE; ++fd)
+       close(fd);
+}
+
 /*
  * Close fd's which are only useful for the parent process.
  * Restore default signal handlers.
@@ -181,15 +211,20 @@ static void close_fds()
       close(d.launcher[0]);
       d.launcher_pid = 0;
    }
-   if (d.wrapper)
+   if (d.wrapper != -1)
    {
       close(d.wrapper);
-      d.wrapper = 0;
+      d.wrapper = -1;
    }
-   if (d.wrapper_old)
+   if (d.wrapper_old != -1)
    {
       close(d.wrapper_old);
-      d.wrapper_old = 0;
+      d.wrapper_old = -1;
+   }
+   if (d.accepted_fd != -1)
+   {
+      close(d.accepted_fd);
+      d.accepted_fd = -1;
    }
 #ifdef Q_WS_X11
    if (X11fd >= 0)
@@ -204,10 +239,49 @@ static void close_fds()
    }
 #endif
 
-   signal(SIGCHLD, SIG_DFL);
-   signal(SIGPIPE, SIG_DFL);
+   KDE_signal(SIGCHLD, SIG_DFL);
+   KDE_signal(SIGPIPE, SIG_DFL);
 }
 
+/* Notify wrapper program that the child it started has finished. */
+static void child_died(pid_t exit_pid, int exit_status)
+{
+   struct child *child = children;
+   struct child *prev = NULL;
+
+   while (child)
+   {
+      if (child->pid == exit_pid)
+      {
+         /* Send a message with the return value of the child on the control socket */
+         klauncher_header request_header;
+         long request_data[2];
+         request_header.cmd = LAUNCHER_DIED;
+         request_header.arg_length = sizeof(long) * 2;
+         request_data[0] = exit_pid;
+         request_data[1] = exit_status;
+         write(child->sock, &request_header, sizeof(request_header));
+         write(child->sock, request_data, request_header.arg_length);
+         close(child->sock);
+
+         if (prev)
+         {
+            prev->next = child->next;
+         }
+         else
+         {
+            child = NULL;
+         }
+         free(child);
+         return;
+      }
+
+      prev = child;
+      child = child->next;
+   }
+}
+
+
 static void exitWithErrorMsg(const QString &errorMsg)
 {
    fprintf( stderr, "%s\n", errorMsg.toLocal8Bit().data() );
@@ -225,21 +299,21 @@ static void setup_tty( const char* tty )
 {
     if( tty == NULL || *tty == '\0' )
         return;
-    int fd = open( tty, O_WRONLY );
+    int fd = KDE_open( tty, O_WRONLY );
     if( fd < 0 )
     {
-        perror( "kdeinit4: couldn't open() tty" );
+        perror( "kdeinit4: could not open() tty" );
         return;
     }
     if( dup2( fd, STDOUT_FILENO ) < 0 )
     {
-        perror( "kdeinit4: couldn't dup2() tty" );
+        perror( "kdeinit4: could not dup2() tty" );
         close( fd );
         return;
     }
     if( dup2( fd, STDERR_FILENO ) < 0 )
     {
-        perror( "kdeinit4: couldn't dup2() tty" );
+        perror( "kdeinit4: could not dup2() tty" );
         close( fd );
         return;
     }
@@ -335,9 +409,9 @@ QByteArray execpath_avoid_loops( const QByteArray& exec, int envc, const char* e
              paths = QString(path).split( QRegExp( "[:\b]" ));
      }
      else
-         paths = QString( getenv("PATH") ).split( QRegExp( "[:\b]" ), QString::KeepEmptyParts );
+         paths = QString::fromLocal8Bit( qgetenv("PATH") ).split( QRegExp( "[:\b]" ), QString::KeepEmptyParts );
      QByteArray execpath = QFile::encodeName(
-         s_instance->dirs()->findExe( exec, paths.join( QString( ":" ))));
+         s_instance->dirs()->findExe( exec, paths.join( QLatin1String( ":" ))));
      if( avoid_loops && !execpath.isEmpty())
      {
          int pos = execpath.lastIndexOf( '/' );
@@ -399,6 +473,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
      else
         execpath = exec;
   }
+  fprintf(stderr,"kdeinit4: preparing to launch %s\n", execpath.constData());
   if (!args)
   {
     argc = 1;
@@ -410,8 +485,6 @@ static pid_t launch(int argc, const char *_name, const char *args,
      d.result = 3;
      d.errorMsg = i18n("Unable to start new process.\n"
                        "The system may have reached the maximum number of open files possible or the maximum number of open files that you are allowed to use has been reached.").toUtf8();
-     close(d.fd[0]);
-     close(d.fd[1]);
      d.fork = 0;
      return d.fork;
   }
@@ -456,6 +529,20 @@ static pid_t launch(int argc, const char *_name, const char *args,
 
      if (cwd && *cwd)
         chdir(cwd);
+     else {
+         // Can't use KGlobalSettings::documentPath() here, we don't have a main component set,
+         // and we don't want to set one; e.g. kioslaves will do that.
+         // So we have to duplicate this stuff from KGlobalSettings.
+         KConfigGroup g(s_instance->config(), "Paths");
+         const QString documentPath = g.readPathEntry("Documents",
+#ifdef Q_WS_WIN
+                                                       getWin32ShellFoldersPath("Personal")
+#else
+                                                       QDir::homePath());
+#endif
+         const QByteArray docPath = QFile::encodeName(documentPath);
+         chdir(docPath.constData());
+     }
 
      if( reset_env ) // KWRAPPER/SHELL
      {
@@ -465,7 +552,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
               environ[tmp_env_count];
               tmp_env_count++)
              unset_envs.append( environ[ tmp_env_count ] );
-         foreach(QByteArray tmp, unset_envs)
+         foreach(const QByteArray &tmp, unset_envs)
          {
              int pos = tmp.indexOf( '=' );
              if( pos >= 0 )
@@ -529,7 +616,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
      }
 
 
-     if ( getenv("KDE_IS_PRELINKED") && !execpath.isEmpty() && !launcher)
+     if ( !qgetenv("KDE_IS_PRELINKED").isEmpty() && !execpath.isEmpty() && !launcher)
          libpath.truncate(0);
 
      QLibrary l(libpath);
@@ -720,20 +807,20 @@ static void init_signals()
 
   if (pipe(d.deadpipe) != 0)
   {
-     perror("kdeinit4: Aborting. Can't create pipe: ");
+     perror("kdeinit4: Aborting. Can not create pipe: ");
      exit(255);
   }
 
   options = fcntl(d.deadpipe[0], F_GETFL);
   if (options == -1)
   {
-     perror("kdeinit4: Aborting. Can't make pipe non-blocking: ");
+     perror("kdeinit4: Aborting. Can not make pipe non-blocking: ");
      exit(255);
   }
 
   if (fcntl(d.deadpipe[0], F_SETFL, options | O_NONBLOCK) == -1)
   {
-     perror("kdeinit4: Aborting. Can't make pipe non-blocking: ");
+     perror("kdeinit4: Aborting. Can not make pipe non-blocking: ");
      exit(255);
   }
 
@@ -770,9 +857,9 @@ static void init_kdeinit_socket()
   //struct sockaddr_un sa_old;
   kde_socklen_t socklen;
   long options;
-  const char *home_dir = getenv("HOME");
+  const QByteArray home_dir = qgetenv("HOME");
   int max_tries = 10;
-  if (!home_dir || !home_dir[0])
+  if (home_dir.isEmpty())
   {
      fprintf(stderr, "kdeinit4: Aborting. $HOME not set!");
      exit(255);
@@ -781,7 +868,7 @@ static void init_kdeinit_socket()
 
   {
      QByteArray path = home_dir;
-     QByteArray readOnly = getenv("KDE_HOME_READONLY");
+     QByteArray readOnly = qgetenv("KDE_HOME_READONLY");
      if (access(path.data(), R_OK|W_OK))
      {
        if (errno == ENOENT)
@@ -796,7 +883,7 @@ static void init_kdeinit_socket()
        }
      }
 #if 0 // obsolete in kde4. Should we check writing to another file instead?
-     path = getenv("ICEAUTHORITY");
+     path = qgetenv("ICEAUTHORITY");
      if (path.isEmpty())
      {
         path = home_dir;
@@ -860,14 +947,21 @@ static void init_kdeinit_socket()
   options = fcntl(d.wrapper, F_GETFL);
   if (options == -1)
   {
-     perror("kdeinit4: Aborting. Can't make socket non-blocking: ");
+     perror("kdeinit4: Aborting. Can not make socket non-blocking: ");
      close(d.wrapper);
      exit(255);
   }
 
   if (fcntl(d.wrapper, F_SETFL, options | O_NONBLOCK) == -1)
   {
-     perror("kdeinit4: Aborting. Can't make socket non-blocking: ");
+     perror("kdeinit4: Aborting. Can not make socket non-blocking: ");
+     close(d.wrapper);
+     exit(255);
+  }
+
+  if (fcntl(d.wrapper, F_SETFD, FD_CLOEXEC) == -1)
+  {
+     perror("kdeinit4: Aborting. Can not make socket close-on-execute: ");
      close(d.wrapper);
      exit(255);
   }
@@ -894,7 +988,7 @@ static void init_kdeinit_socket()
   /** set permissions **/
   if (chmod(sock_file, 0600) != 0)
   {
-     perror("kdeinit4: Aborting. Can't set permissions on socket: ");
+     perror("kdeinit4: Aborting. Can not set permissions on socket: ");
      fprintf(stderr, "Wrong permissions of socket '%s'\n", sock_file);
      unlink(sock_file);
      close(d.wrapper);
@@ -923,7 +1017,7 @@ static void init_kdeinit_socket()
   {
      // perror("kdeinit4: Aborting. Can't make socket non-blocking: ");
      close(d.wrapper_old);
-     d.wrapper_old = 0;
+     d.wrapper_old = -1;
      return;
   }
 
@@ -931,7 +1025,15 @@ static void init_kdeinit_socket()
   {
      // perror("kdeinit4: Aborting. Can't make socket non-blocking: ");
      close(d.wrapper_old);
-     d.wrapper_old = 0;
+     d.wrapper_old = -1;
+     return;
+  }
+
+  if (fcntl(d.wrapper, F_SETFD, FD_CLOEXEC) == -1)
+  {
+     //perror("kdeinit4: Aborting. Can't make socket close-on-execute: ");
+     close(d.wrapper);
+     d.wrapper_old = -1;
      return;
   }
 
@@ -948,7 +1050,7 @@ static void init_kdeinit_socket()
 	      // perror("kdeinit4: Aborting. bind() failed: ");
 	      fprintf(stderr, "Could not bind to socket '%s'\n", sock_file_old);
 	      close(d.wrapper_old);
-	      d.wrapper_old = 0;
+	      d.wrapper_old = -1;
 	      return;
 	  }
 	  max_tries--;
@@ -962,7 +1064,7 @@ static void init_kdeinit_socket()
      fprintf(stderr, "Wrong permissions of socket '%s'\n", sock_file);
      unlink(sock_file_old);
      close(d.wrapper_old);
-     d.wrapper_old = 0;
+     d.wrapper_old = -1;
      return;
   }
 
@@ -971,7 +1073,7 @@ static void init_kdeinit_socket()
      // perror("kdeinit4: Aborting. listen() failed: ");
      unlink(sock_file_old);
      close(d.wrapper_old);
-     d.wrapper_old = 0;
+     d.wrapper_old = -1;
   }
 #endif
 }
@@ -1040,6 +1142,10 @@ static void handle_launcher_request(int sock = -1)
        sock = d.launcher[0];
        launcher = true;
    }
+   else
+   {
+       d.accepted_fd = sock;
+   }
 
    klauncher_header request_header;
    char *request_data = 0L;
@@ -1162,8 +1268,8 @@ static void handle_launcher_request(int sock = -1)
      }
 
       // support for the old a bit broken way of setting DISPLAY for multihead
-      QByteArray olddisplay = getenv(DISPLAY);
-      QByteArray kdedisplay = getenv("KDE_DISPLAY");
+      QByteArray olddisplay = qgetenv(DISPLAY);
+      QByteArray kdedisplay = qgetenv("KDE_DISPLAY");
       bool reset_display = (! olddisplay.isEmpty() &&
                             ! kdedisplay.isEmpty() &&
                             olddisplay != kdedisplay);
@@ -1187,6 +1293,20 @@ static void handle_launcher_request(int sock = -1)
          response_data = pid;
          write(sock, &response_header, sizeof(response_header));
          write(sock, &response_data, response_header.arg_length);
+
+         /* add new child to list */
+         struct child *child = (struct child *) malloc(sizeof(struct child));
+         child->pid = pid;
+         child->sock = dup(sock);
+         child->next = children;
+         children = child;
+
+         /* check child hasn't terminated already */
+         if (kill(pid, 0))
+         {
+            children = child->next;
+            free(child);
+         }
       }
       else
       {
@@ -1275,6 +1395,7 @@ static void handle_requests(pid_t waitForPid)
       fd_set e_set;
       int result;
       pid_t exit_pid;
+      int exit_status;
       char c;
 
       /* Flush the pipe of death */
@@ -1282,7 +1403,7 @@ static void handle_requests(pid_t waitForPid)
 
       /* Handle dying children */
       do {
-        exit_pid = waitpid(-1, 0, WNOHANG);
+        exit_pid = waitpid(-1, &exit_status, WNOHANG);
         if (exit_pid > 0)
         {
 #ifndef NDEBUG
@@ -1291,22 +1412,18 @@ static void handle_requests(pid_t waitForPid)
            if (waitForPid && (exit_pid == waitForPid))
               return;
 
-           if (d.launcher_pid)
-           {
-           // TODO send process died message
-              klauncher_header request_header;
-              long request_data[2];
-              request_header.cmd = LAUNCHER_DIED;
-              request_header.arg_length = sizeof(long) * 2;
-              request_data[0] = exit_pid;
-              request_data[1] = 0; /* not implemented yet */
-              write(d.launcher[0], &request_header, sizeof(request_header));
-              write(d.launcher[0], request_data, request_header.arg_length);
-           }
+           if( WIFEXITED( exit_status )) // fix process return value
+               exit_status = WEXITSTATUS(exit_status);
+           else if( WIFSIGNALED( exit_status ))
+               exit_status = 128 + WTERMSIG( exit_status );
+           child_died(exit_pid, exit_status);
         }
       }
       while( exit_pid > 0);
 
+      /* Set up the next loop */
+      d.accepted_fd = -1;
+
       FD_ZERO(&rd_set);
       FD_ZERO(&wr_set);
       FD_ZERO(&e_set);
@@ -1316,7 +1433,7 @@ static void handle_requests(pid_t waitForPid)
          FD_SET(d.launcher[0], &rd_set);
       }
       FD_SET(d.wrapper, &rd_set);
-      if (d.wrapper_old)
+      if (d.wrapper_old != -1)
       {
          FD_SET(d.wrapper_old, &rd_set);
       }
@@ -1335,28 +1452,18 @@ static void handle_requests(pid_t waitForPid)
          int sock = accept(d.wrapper, (struct sockaddr *)&client, &sClient);
          if (sock >= 0)
          {
-            if (fork() == 0)
-            {
-                close_fds();
-                handle_launcher_request(sock);
-                exit(255); /* Terminate process. */
-            }
+            handle_launcher_request(sock);
             close(sock);
          }
       }
-      if ((result > 0) && (FD_ISSET(d.wrapper_old, &rd_set)))
+      if ((result > 0) && d.wrapper_old != -1 && (FD_ISSET(d.wrapper_old, &rd_set)))
       {
          struct sockaddr_un client;
          kde_socklen_t sClient = sizeof(client);
          int sock = accept(d.wrapper_old, (struct sockaddr *)&client, &sClient);
          if (sock >= 0)
          {
-            if (fork() == 0)
-            {
-                close_fds();
-                handle_launcher_request(sock);
-                exit(255); /* Terminate process. */
-            }
+            handle_launcher_request(sock);
             close(sock);
          }
       }
@@ -1389,13 +1496,13 @@ static void handle_requests(pid_t waitForPid)
 static void kdeinit_library_path()
 {
    QStringList ltdl_library_path =
-     QFile::decodeName(getenv("LTDL_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
+     QFile::decodeName(qgetenv("LTDL_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
 #ifdef Q_OS_DARWIN
    QStringList ld_library_path =
-     QFile::decodeName(getenv("DYLD_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
+     QFile::decodeName(qgetenv("DYLD_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
 #else
    QStringList ld_library_path =
-     QFile::decodeName(getenv("LD_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
+     QFile::decodeName(qgetenv("LD_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
 #endif
 
    QByteArray extra_path;
@@ -1433,7 +1540,7 @@ static void kdeinit_library_path()
 //   if (!extra_path.isEmpty())
 //      lt_dlsetsearchpath(extra_path.data());
 
-   QByteArray display = getenv(DISPLAY);
+   QByteArray display = qgetenv(DISPLAY);
    if (display.isEmpty())
    {
 #if defined(Q_WS_X11) || defined(Q_WS_QWS)
@@ -1460,7 +1567,7 @@ static void kdeinit_library_path()
    strcpy(sock_file_old, socketName.data());
 #endif
 
-   display.replace(":","_");
+   display.replace(':','_');
    // WARNING, if you change the socket name, adjust kwrapper too
    QByteArray socketName = QFile::encodeName(KStandardDirs::locateLocal("socket", QString("kdeinit4_%1").arg(QLatin1String(display)), *s_instance));
    if (socketName.length() >= MAX_SOCK_FILE)
@@ -1506,7 +1613,7 @@ int kdeinit_xio_errhandler( Display *disp )
     qWarning( "kdeinit4: sending SIGHUP to children." );
 
     /* this should remove all children we started */
-    signal(SIGHUP, SIG_IGN);
+    KDE_signal(SIGHUP, SIG_IGN);
     kill(0, SIGHUP);
 
     sleep(2);
@@ -1515,7 +1622,7 @@ int kdeinit_xio_errhandler( Display *disp )
     qWarning( "kdeinit4: sending SIGTERM to children." );
 
     /* and if they don't listen to us, this should work */
-    signal(SIGTERM, SIG_IGN);
+    KDE_signal(SIGTERM, SIG_IGN);
     kill(0, SIGTERM);
 
     if ( disp )
@@ -1571,7 +1678,7 @@ static int initXconnection()
     (void) setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (char *) &on, (int) sizeof(on));
     return fd;
   } else
-    fprintf(stderr, "kdeinit4: Can't connect to the X Server.\n" \
+    fprintf(stderr, "kdeinit4: Can not connect to the X Server.\n" \
      "kdeinit4: Might not terminate at end of session.\n");
 
   return -1;
@@ -1619,6 +1726,7 @@ int main(int argc, char **argv, char **envp)
 {
    int i;
    pid_t pid;
+   bool do_fork = true;
    int launch_klauncher = 1;
    int launch_kded = 1;
    int keep_running = 1;
@@ -1633,6 +1741,13 @@ int main(int argc, char **argv, char **envp)
          launch_klauncher = 0;
       if (strcmp(safe_argv[i], "--no-kded") == 0)
          launch_kded = 0;
+#ifdef Q_WS_MACX
+      // make it nofork to match KUniqueApplication, technically command-line incompatible
+      if (strcmp(safe_argv[i], "--nofork") == 0)
+#else
+      if (strcmp(safe_argv[i], "--no-fork") == 0)
+#endif
+         do_fork = false;
       if (strcmp(safe_argv[i], "--suicide") == 0)
          d.suicide = true;
       if (strcmp(safe_argv[i], "--exit") == 0)
@@ -1641,6 +1756,11 @@ int main(int argc, char **argv, char **envp)
       {
         printf("Usage: kdeinit4 [options]\n");
      // printf("    --no-dcop         Do not start dcopserver\n");
+#ifdef Q_WS_MACX
+        printf("    --nofork          Do not fork\n");
+#else
+        printf("    --no-fork         Do not fork\n");
+#endif
      // printf("    --no-klauncher    Do not start klauncher\n");
         printf("    --no-kded         Do not start kded\n");
         printf("    --suicide         Terminate when no KDE applications are left running\n");
@@ -1649,27 +1769,41 @@ int main(int argc, char **argv, char **envp)
       }
    }
 
-   pipe(d.initpipe);
+   cleanup_fds();
 
-   // Fork here and let parent process exit.
-   // Parent process may only exit after all required services have been
-   // launched. (dcopserver/klauncher and services which start with '+')
-   signal( SIGCHLD, secondary_child_handler);
-   if (fork() > 0) // Go into background
-   {
-      close(d.initpipe[1]);
-      d.initpipe[1] = -1;
-      // wait till init is complete
-      char c;
-      while( read(d.initpipe[0], &c, 1) < 0)
-          ;
-      // then exit;
+   if (do_fork) {
+#ifdef Q_WS_MACX
+      mac_fork_and_reexec_self();
+#else
+      pipe(d.initpipe);
+
+      // Fork here and let parent process exit.
+      // Parent process may only exit after all required services have been
+      // launched. (dcopserver/klauncher and services which start with '+')
+      KDE_signal( SIGCHLD, secondary_child_handler);
+      if (fork() > 0) // Go into background
+      {
+         close(d.initpipe[1]);
+         d.initpipe[1] = -1;
+         // wait till init is complete
+         char c;
+         while( read(d.initpipe[0], &c, 1) < 0)
+            ;
+         // then exit;
+         close(d.initpipe[0]);
+         d.initpipe[0] = -1;
+         return 0;
+      }
       close(d.initpipe[0]);
       d.initpipe[0] = -1;
-      return 0;
+#endif
    }
-   close(d.initpipe[0]);
-   d.initpipe[0] = -1;
+
+   /** We start a few processes (including klauncher and kded) before
+    * we close our pipe, so make sure it doesn't leak */
+   if (d.initpipe[1] != -1)
+      fcntl(d.initpipe[1], F_SETFD, FD_CLOEXEC);
+
    d.my_pid = getpid();
 
    /** Make process group leader (for shutting down children later) **/
@@ -1694,10 +1828,12 @@ int main(int argc, char **argv, char **envp)
    d.maxname = strlen(argv[0]);
    d.launcher_pid = 0;
    d.kded_pid = 0;
-   d.wrapper = 0;
-   d.wrapper_old = 0;
+   d.wrapper = -1;
+   d.wrapper_old = -1;
+   d.accepted_fd = -1;
    d.debug_wait = false;
    d.launcher_ok = false;
+   children = NULL;
    init_signals();
 #ifdef Q_WS_X11
    setupX();
@@ -1712,7 +1848,7 @@ int main(int argc, char **argv, char **envp)
       init_kdeinit_socket();
    }
 #ifdef Q_WS_X11
-   if (!d.suicide && !getenv("KDE_IS_PRELINKED"))
+   if (!d.suicide && qgetenv("KDE_IS_PRELINKED").isEmpty())
    {
        QString konq = KStandardDirs::locate("lib", "libkonq.so.5", *s_instance);
        // can't use KLibLoader here as it would unload the library
@@ -1796,10 +1932,13 @@ int main(int argc, char **argv, char **envp)
    if (!keep_running)
       return 0;
 
-   char c = 0;
-   write(d.initpipe[1], &c, 1); // Kdeinit is started.
-   close(d.initpipe[1]);
-   d.initpipe[1] = -1;
+   if (d.initpipe[1] != -1)
+   {
+      char c = 0;
+      write(d.initpipe[1], &c, 1); // Kdeinit is started.
+      close(d.initpipe[1]);
+      d.initpipe[1] = -1;
+   }
 
    handle_requests(0);
 
