@@ -30,13 +30,16 @@
 #include <signal.h>
 #include <sys/time.h>
 
-#include <QtCore/QFile>
-#ifdef Q_WS_WIN
-#include <QtCore/QProcess>
+#ifdef Q_WS_X11
+#include <kstartupinfo.h>
+#include <X11/Xlib.h>
 #endif
 
+#include <QtCore/QFile>
+
 #include <kconfig.h>
 #include <kdebug.h>
+#include <kde_file.h>
 #include <klibloader.h>
 #include <klocale.h>
 #include <kprotocolmanager.h>
@@ -50,13 +53,6 @@
 #include <kio/connection.h>
 #include <kio/slaveinterface.h>
 
-#ifdef Q_WS_X11
-#include <kstartupinfo.h>
-#include <X11/Xlib.h>
-#endif
-
-#include <QtCore/QCoreApplication>
-
 // Dispose slaves after being idle for SLAVE_MAX_IDLE seconds
 #define SLAVE_MAX_IDLE	30
 
@@ -206,15 +202,15 @@ KLauncher::KLauncher(int _kdeinitSocket)
    lastRequest = 0;
    bProcessingQueue = false;
 
-   mSlaveDebug = getenv("KDE_SLAVE_DEBUG_WAIT");
+   mSlaveDebug = QString::fromLocal8Bit(qgetenv("KDE_SLAVE_DEBUG_WAIT"));
    if (!mSlaveDebug.isEmpty())
    {
       qWarning("Klauncher running in slave-debug mode for slaves of protocol '%s'", qPrintable(mSlaveDebug));
    }
-   mSlaveValgrind = getenv("KDE_SLAVE_VALGRIND");
+   mSlaveValgrind = QString::fromLocal8Bit(qgetenv("KDE_SLAVE_VALGRIND"));
    if (!mSlaveValgrind.isEmpty())
    {
-      mSlaveValgrindSkin = getenv("KDE_SLAVE_VALGRIND_SKIN");
+      mSlaveValgrindSkin = QString::fromLocal8Bit(qgetenv("KDE_SLAVE_VALGRIND_SKIN"));
       qWarning("Klauncher running slaves through valgrind for slaves of protocol '%s'", qPrintable(mSlaveValgrind));
    }
 #ifdef Q_WS_WIN
@@ -256,7 +252,7 @@ KLauncher::destruct()
 void KLauncher::setLaunchEnv(const QString &name, const QString &value)
 {
 #ifdef Q_WS_WIN
-    
+
 #else
    klauncher_header request_header;
    QByteArray requestData;
@@ -320,8 +316,8 @@ KLauncher::slotKDEInitData(int)
    if (result == -1)
    {
       kDebug(7016) << "Exiting on read_socket errno:" << errno;
-      ::signal( SIGHUP, SIG_IGN);
-      ::signal( SIGTERM, SIG_IGN);
+      KDE_signal( SIGHUP, SIG_IGN);
+      KDE_signal( SIGTERM, SIG_IGN);
       destruct(); // Exit!
    }
    requestData.resize(request_header.arg_length);
@@ -329,12 +325,12 @@ KLauncher::slotKDEInitData(int)
                         request_header.arg_length);
 
    processRequestReturn(request_header.cmd,requestData);
-       
+
 }
 #endif
-   
+
 void KLauncher::processRequestReturn(int status, const QByteArray &requestData)
-{   
+{
    if (status == LAUNCHER_DIED)
    {
      long *request_data;
@@ -557,31 +553,30 @@ KLauncher::requestStart(KLaunchRequest *request)
 #ifdef Q_WS_WIN
    requestList.append( request );
    lastRequest = request;
-   
-   QProcess *process  = new QProcess;
-   process->setProcessChannelMode(QProcess::MergedChannels);
+
+   KProcess *process  = new KProcess;
+   process->setOutputChannelMode(KProcess::MergedChannels);
    connect(process ,SIGNAL(readyReadStandardOutput()),this, SLOT(slotGotOutput()) );
-   processList << process;
-   
+   connect(process ,SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(slotFinished(int, QProcess::ExitStatus)) );
+   request->process = process;
+
 // process.setEnvironment(envlist);
    QStringList args;
-   foreach (QString arg, request->arg_list)
+   foreach (const QString &arg, request->arg_list)
       args << arg;
 
-   process->start(request->name,args);
-		
-   _PROCESS_INFORMATION* _pid = process->pid();
-    int pid = _pid ? _pid->dwProcessId : 0;
+   process->setProgram(request->name,args);
+   process->start();
 
-   if (pid)
+   if (!process->waitForStarted())
    {
-      request->pid = pid;
-      QByteArray data((char *)&pid, sizeof(int));
-      processRequestReturn(LAUNCHER_OK,data);
+       processRequestReturn(LAUNCHER_ERROR,"");
    }
-   else 
+   else
    {
-      processRequestReturn(LAUNCHER_ERROR,"");
+       request->pid = process->pid();
+       QByteArray data((char *)&request->pid, sizeof(int));
+       processRequestReturn(LAUNCHER_OK,data);
    }
    return;
 
@@ -595,10 +590,10 @@ KLauncher::requestStart(KLaunchRequest *request)
    appendLong(requestData, request->arg_list.count() + 1);
    requestData.append(request->name.toLocal8Bit());
    requestData.append('\0');
-   foreach (QString arg, request->arg_list)
+   foreach (const QString &arg, request->arg_list)
        requestData.append(arg.toLocal8Bit()).append('\0');
    appendLong(requestData, request->envs.count());
-   foreach (QString env, request->envs)
+   foreach (const QString &env, request->envs)
        requestData.append(env.toLocal8Bit()).append('\0');
    appendLong(requestData, 0); // avoid_loops, always false here
 #ifdef Q_WS_X11
@@ -652,6 +647,7 @@ void KLauncher::exec_blind(const QString &name, const QStringList &arg_list, con
 }
 
 
+// KDE5: remove
 bool
 KLauncher::start_service_by_name(const QString &serviceName, const QStringList &urls,
     const QStringList &envs, const QString& startup_id, bool blind, const QDBusMessage &msg)
@@ -675,7 +671,7 @@ KLauncher::start_service_by_desktop_path(const QString &serviceName, const QStri
 {
    KService::Ptr service;
    // Find service
-   if (serviceName.startsWith(QLatin1Char('/')))
+   if (QFileInfo(serviceName).isAbsolute() )
    {
       // Full path
       service = new KService(serviceName);
@@ -804,7 +800,7 @@ KLauncher::send_service_startup_info( KLaunchRequest *request, KService::Ptr ser
     KStartupInfoId id;
     id.initId( startup_id.toLatin1() );
     QString dpy_str;
-    foreach (QString env, envs) {
+    foreach (const QString &env, envs) {
         if (env.startsWith(QLatin1String("DISPLAY=")))
             dpy_str = env.mid(8);
     }
@@ -852,7 +848,7 @@ KLauncher::cancel_service_startup_info( KLaunchRequest* request, const QString&
     if( !startup_id.isEmpty() && startup_id != "0" )
     {
         QString dpy_str;
-        foreach (QString env, envs) {
+        foreach (const QString &env, envs) {
             if (env.startsWith(QLatin1String("DISPLAY=")))
                 dpy_str = env.mid(8);
         }
@@ -876,7 +872,8 @@ KLauncher::cancel_service_startup_info( KLaunchRequest* request, const QString&
 
 bool
 KLauncher::kdeinit_exec(const QString &app, const QStringList &args,
-   const QStringList &envs, const QString &startup_id, bool wait, const QDBusMessage &msg)
+                        const QString& workdir, const QStringList &envs,
+                        const QString &startup_id, bool wait, const QDBusMessage &msg)
 {
    KLaunchRequest *request = new KLaunchRequest;
    request->autoStart = false;
@@ -900,6 +897,7 @@ KLauncher::kdeinit_exec(const QString &app, const QStringList &args,
    request->startup_id = startup_id;
 #endif
    request->envs = envs;
+   request->cwd = workdir;
    if( !app.endsWith("kbuildsycoca4") ) // avoid stupid loop
    {
        // Find service, if any - strip path if needed
@@ -1177,15 +1175,36 @@ void KLauncher::reparseConfiguration()
       slave->reparseConfiguration();
 }
 
+#ifdef Q_WS_WIN
 void
 KLauncher::slotGotOutput()
 {
-#ifdef Q_WS_WIN
-  foreach (QProcess *p, processList) {
-    QByteArray _stdout = p->readAllStandardOutput();
-    kDebug(7016) << _stdout.data();
-  }
-#endif
+  KProcess *p = static_cast<KProcess *>(sender());
+  QByteArray _stdout = p->readAllStandardOutput();
+  kDebug(7016) << _stdout.data();
 }
 
+void
+KLauncher::slotFinished(int exitCode, QProcess::ExitStatus exitStatus )
+{
+    KProcess *p = static_cast<KProcess *>(sender());
+    kDebug(7016) << "process finished exitcode=" << exitCode << "exitStatus=" << exitStatus;
+
+    foreach (KLaunchRequest *request, requestList)
+    {
+        if (request->process == p)
+        {
+            kDebug() << "found process setting to done";
+            if (exitCode == 0  && exitStatus == QProcess::NormalExit)
+                request->status = KLaunchRequest::Done;
+            else
+                request->status = KLaunchRequest::Error;
+            requestDone(request);
+            request->process = 0;
+        }
+    }
+    delete p;
+}
+#endif
+
 #include "klauncher.moc"
