@@ -256,7 +256,7 @@ public:
     /**
      * Returns the mimetype for the desired output format.
      *
-     * This is only valid if setFilterMimeType() has been called
+     * This is only valid if setMimeFilter() has been called
      * previously.
      *
      * @see setFilterMimeType()
@@ -691,6 +691,11 @@ public:
      */
     static void setStartDir( const KUrl& directory );
 
+#ifdef Q_WS_WIN
+public Q_SLOTS:
+    int exec();
+#endif
+
 Q_SIGNALS:
     /**
       * Emitted when the user selects a file. It is only emitted in single-
@@ -734,6 +739,11 @@ protected:
      */
     virtual void keyPressEvent( QKeyEvent *e );
 
+    /**
+     * Reimplemented for saving the dialog geometry.
+     */
+    virtual void hideEvent( QHideEvent *event );
+
 protected Q_SLOTS:
     virtual void slotOk();
     virtual void accept();
