@@ -16,6 +16,7 @@
 #include <kio/kio_export.h>
 
 #include <kbuttongroup.h>
+#include <kcombobox.h>
 #include <klistwidgetsearchline.h>
 #include <kapplication.h>
 #include <klocale.h>
@@ -27,7 +28,6 @@
 
 #include <QtGui/QLayout>
 #include <QtGui/QLabel>
-#include <QtGui/QComboBox>
 #include <QtCore/QTimer>
 #include <QtGui/QRadioButton>
 #include <QtCore/QFileInfo>
@@ -234,7 +234,7 @@ class KIconDialog::KIconDialogPrivate
     KIconLoader::Context mContext;
 
     QStringList mFileList;
-    QComboBox *mpCombo;
+    KComboBox *mpCombo;
     QPushButton *mpBrowseBut;
     QRadioButton *mpSystemIcons, *mpOtherIcons;
     QProgressBar *mpProgress;
@@ -305,7 +305,7 @@ void KIconDialog::KIconDialogPrivate::init()
     mpSystemIcons = new QRadioButton(i18n("S&ystem icons:"), bgroup);
     connect(mpSystemIcons, SIGNAL(clicked()), q, SLOT(_k_slotSystemIconClicked()));
     grid->addWidget(mpSystemIcons, 1, 0);
-    mpCombo = new QComboBox(bgroup);
+    mpCombo = new KComboBox(bgroup);
     connect(mpCombo, SIGNAL(activated(int)), q, SLOT(_k_slotContext(int)));
     grid->addWidget(mpCombo, 1, 1);
     mpOtherIcons = new QRadioButton(i18n("O&ther icons:"), bgroup);
@@ -644,6 +644,7 @@ class KIconButton::KIconButtonPrivate
     KIconButton *q;
 
     int iconSize;
+    int buttonIconSize;
     bool m_bStrictIconSize;
 
     bool mbUser;
@@ -677,6 +678,7 @@ KIconButton::KIconButtonPrivate::KIconButtonPrivate(KIconButton *qq, KIconLoader
 {
     m_bStrictIconSize = false;
     iconSize = 0; // let KIconLoader choose the default
+    buttonIconSize = -1; //When buttonIconSize is -1, iconSize will be used for the button
 
     mGroup = KIconLoader::Desktop;
     mContext = KIconLoader::Application;
@@ -709,7 +711,10 @@ bool KIconButton::strictIconSize() const
 
 void KIconButton::setIconSize( int size )
 {
-    QPushButton::setIconSize(QSize(size, size));
+    if (d->buttonIconSize == -1) {
+        QPushButton::setIconSize(QSize(size, size));
+    }
+
     d->iconSize = size;
 }
 
@@ -718,6 +723,17 @@ int KIconButton::iconSize() const
     return d->iconSize;
 }
 
+void KIconButton::setButtonIconSize( int size )
+{
+    QPushButton::setIconSize(QSize(size, size));
+    d->buttonIconSize = size;
+}
+
+int KIconButton::buttonIconSize() const
+{
+    return QPushButton::iconSize().height();
+}
+
 void KIconButton::setIconType(KIconLoader::Group group, KIconLoader::Context context, bool user)
 {
     d->mGroup = group;
