@@ -110,7 +110,7 @@ public:
         , m_processedFiles(0)
         , m_processedDirs(0)
         , m_srcList(src)
-        , m_currentStatSrc(m_srcList.begin())
+        , m_currentStatSrc(m_srcList.constBegin())
         , m_bCurrentOperationIsLink(false)
         , m_bSingleFileCopy(false)
         , m_bOnlyRenames(mode==CopyJob::Move)
@@ -151,7 +151,7 @@ public:
     QList<CopyInfo> dirs;
     KUrl::List dirsToRemove;
     KUrl::List m_srcList;
-    KUrl::List::Iterator m_currentStatSrc;
+    KUrl::List::const_iterator m_currentStatSrc;
     bool m_bCurrentSrcIsDir;
     bool m_bCurrentOperationIsLink;
     bool m_bSingleFileCopy;
@@ -268,7 +268,7 @@ KIO_EXPORT bool kio_resolve_local_urls = true;
 void CopyJobPrivate::slotResultStating( KJob *job )
 {
     Q_Q(CopyJob);
-    //kDebug(7007) << "CopyJobPrivate::slotResultStating";
+    //kDebug(7007);
     // Was there an error while stating the src ?
     if (job->error() && destinationState != DEST_NOT_STATED )
     {
@@ -315,7 +315,7 @@ void CopyJobPrivate::slotResultStating( KJob *job )
         else {
             // Treat symlinks to dirs as dirs here, so no test on isLink
             destinationState = isDir ? DEST_IS_DIR : DEST_IS_FILE;
-            //kDebug(7007) << "CopyJobPrivate::slotResultStating dest is dir:" << bDir;
+            //kDebug(7007) << "dest is dir:" << bDir;
         }
         const bool isGlobalDest = m_dest == m_globalDest;
         if ( isGlobalDest )
@@ -374,7 +374,7 @@ void CopyJobPrivate::slotResultStating( KJob *job )
          && !entry.isLink()
          && m_mode != CopyJob::Link ) // No recursion in Link mode either.
     {
-        //kDebug(7007) << " Source is a directory ";
+        //kDebug(7007) << "Source is a directory";
 
         m_bCurrentSrcIsDir = true; // used by slotEntries
         if ( destinationState == DEST_IS_DIR ) // (case 1)
@@ -412,7 +412,7 @@ void CopyJobPrivate::slotResultStating( KJob *job )
     }
     else
     {
-        //kDebug(7007) << " Source is a file (or a symlink), or we are linking -> no recursive listing ";
+        //kDebug(7007) << "Source is a file (or a symlink), or we are linking -> no recursive listing";
         statNextSrc();
     }
 }
@@ -431,6 +431,9 @@ void CopyJobPrivate::slotReport()
         return;
     // If showProgressInfo was set, progressId() is > 0.
     switch (state) {
+        case STATE_RENAMING:
+            q->setTotalAmount(KJob::Files, m_srcList.count());
+            // fall-through intended
         case STATE_COPYING_FILES:
             q->setProcessedAmount( KJob::Files, m_processedFiles );
             if (m_bURLDirty)
@@ -470,7 +473,14 @@ void CopyJobPrivate::slotReport()
             if (m_bURLDirty)
             {
                 m_bURLDirty = false;
-                emitCopying( q, m_currentSrcURL, m_currentDestURL );
+                if (m_mode==CopyJob::Move)
+                {
+                    emitMoving( q, m_currentSrcURL, m_currentDestURL );
+                }
+                else
+                {
+                    emitCopying( q, m_currentSrcURL, m_currentDestURL );
+                }
             }
             q->setTotalAmount(KJob::Bytes, m_totalSize);
             q->setTotalAmount(KJob::Files, files.count());
@@ -485,8 +495,8 @@ void CopyJobPrivate::slotReport()
 void CopyJobPrivate::slotEntries(KIO::Job* job, const UDSEntryList& list)
 {
     //Q_Q(CopyJob);
-    UDSEntryList::ConstIterator it = list.begin();
-    UDSEntryList::ConstIterator end = list.end();
+    UDSEntryList::ConstIterator it = list.constBegin();
+    UDSEntryList::ConstIterator end = list.constEnd();
     for (; it != end; ++it) {
         const UDSEntry& entry = *it;
         struct CopyInfo info;
@@ -518,7 +528,7 @@ void CopyJobPrivate::slotEntries(KIO::Job* job, const UDSEntryList& list)
                     url.addPath( displayName );
                 }
             }
-            //kDebug(7007) << "displayName=" << displayName << " url=" << url;
+            //kDebug(7007) << "displayName=" << displayName << "url=" << url;
             if (!localPath.isEmpty() && kio_resolve_local_urls) {
                 url = KUrl();
                 url.setPath(localPath);
@@ -526,7 +536,7 @@ void CopyJobPrivate::slotEntries(KIO::Job* job, const UDSEntryList& list)
 
             info.uSource = url;
             info.uDest = m_currentDest;
-            //kDebug(7007) << " uSource=" << info.uSource << " uDest(1)=" << info.uDest;
+            //kDebug(7007) << "uSource=" << info.uSource << "uDest(1)=" << info.uDest;
             // Append filename or dirname to destination URL, if allowed
             if ( destinationState == DEST_IS_DIR &&
                  // "copy/move as <foo>" means 'foo' is the dest for the base srcurl
@@ -544,7 +554,7 @@ void CopyJobPrivate::slotEntries(KIO::Job* job, const UDSEntryList& list)
                     for ( int n = 0; n < numberOfSlashes + 1; ++n ) {
                         pos = path.lastIndexOf( '/', pos - 1 );
                         if ( pos == -1 ) { // error
-                            kWarning(7007) << "kioslave bug: not enough slashes in UDS_URL " << path << " - looking for " << numberOfSlashes << " slashes";
+                            kWarning(7007) << "kioslave bug: not enough slashes in UDS_URL" << path << "- looking for" << numberOfSlashes << "slashes";
                             break;
                         }
                     }
@@ -566,7 +576,7 @@ void CopyJobPrivate::slotEntries(KIO::Job* job, const UDSEntryList& list)
                 info.uDest.addPath( destFileName );
             }
             //kDebug(7007) << " uDest(2)=" << info.uDest;
-            //kDebug(7007) << " " << info.uSource << " -> " << info.uDest;
+            //kDebug(7007) << " " << info.uSource << "->" << info.uDest;
             if ( info.linkDest.isEmpty() && isDir && m_mode != CopyJob::Link ) // Dir
             {
                 dirs.append( info ); // Directories
@@ -604,7 +614,7 @@ void CopyJobPrivate::statNextSrc()
 void CopyJobPrivate::statCurrentSrc()
 {
     Q_Q(CopyJob);
-    if ( m_currentStatSrc != m_srcList.end() )
+    if ( m_currentStatSrc != m_srcList.constEnd() )
     {
         m_currentSrcURL = (*m_currentStatSrc);
         m_bURLDirty = true;
@@ -717,6 +727,7 @@ void CopyJobPrivate::startRenameJob( const KUrl& slave_url )
     // Append filename or dirname to destination URL, if allowed
     if ( destinationState == DEST_IS_DIR && !m_asMethod )
         dest.addPath( m_currentSrcURL.fileName() );
+    m_currentDestURL = dest;
     kDebug(7007) << "This seems to be a suitable case for trying to rename before stat+[list+]copy+del";
     state = STATE_RENAMING;
 
@@ -757,7 +768,7 @@ void CopyJobPrivate::skip( const KUrl & sourceUrl )
 {
     // If this is one if toplevel sources,
     // remove it from d->m_srcList, for a correct FilesRemoved() signal
-    //kDebug(7007) << "CopyJob::skip: looking for " << sourceUrl;
+    //kDebug(7007) << "looking for " << sourceUrl;
     m_srcList.removeAll( sourceUrl );
     dirsToRemove.removeAll( sourceUrl );
 }
@@ -766,19 +777,19 @@ bool CopyJobPrivate::shouldOverwrite( const QString& path ) const
 {
     if ( m_bOverwriteAll )
         return true;
-    QStringList::ConstIterator sit = m_overwriteList.begin();
-    for( ; sit != m_overwriteList.end(); ++sit )
-        if ( path.startsWith( *sit ) )
+    Q_FOREACH(const QString& overwritePath, m_overwriteList) {
+        if ( path.startsWith(overwritePath) )
             return true;
+    }
     return false;
 }
 
 bool CopyJobPrivate::shouldSkip( const QString& path ) const
 {
-    QStringList::ConstIterator sit = m_skipList.begin();
-    for( ; sit != m_skipList.end(); ++sit )
-        if ( path.startsWith( *sit ) )
+    Q_FOREACH(const QString& skipPath, m_skipList) {
+        if ( path.startsWith(skipPath) )
             return true;
+    }
     return false;
 }
 
@@ -920,9 +931,9 @@ void CopyJobPrivate::slotResultConflictCreatingDirs( KJob * job )
                 if ( path.startsWith( oldPath ) ) {
                     QString n = path;
                     n.replace( 0, oldPath.length(), newPath );
-                    kDebug(7007) << "dirs list: " << (*renamedirit).uSource.path()
-                                  << " was going to be " << path
-                                  << ", changed into " << n << endl;
+                    kDebug(7007) << "dirs list:" << (*renamedirit).uSource.path()
+                                  << "was going to be" << path
+                                  << ", changed into" << n;
                     (*renamedirit).uDest.setPath( n );
                 }
             }
@@ -934,9 +945,9 @@ void CopyJobPrivate::slotResultConflictCreatingDirs( KJob * job )
                 if ( path.startsWith( oldPath ) ) {
                     QString n = path;
                     n.replace( 0, oldPath.length(), newPath );
-                    kDebug(7007) << "files list: " << (*renamefileit).uSource.path()
-                                  << " was going to be " << path
-                                  << ", changed into " << n << endl;
+                    kDebug(7007) << "files list:" << (*renamefileit).uSource.path()
+                                  << "was going to be" << path
+                                  << ", changed into" << n;
                     (*renamefileit).uDest.setPath( n );
                 }
             }
@@ -1095,9 +1106,12 @@ void CopyJobPrivate::slotResultCopyingFiles( KJob * job )
             //required for the undo feature
             emit q->copyingLinkDone( q, (*it).uSource, target, (*it).uDest );
         }
-        else
+        else {
             //required for the undo feature
             emit q->copyingDone( q, (*it).uSource, (*it).uDest, (*it).mtime, false, false );
+            if (m_mode == CopyJob::Move)
+                org::kde::KDirNotify::emitFileMoved( (*it).uSource.url(), (*it).uDest.url() );
+        }
         // remove from list, to move on to next file
         files.erase( it );
     }
@@ -1107,7 +1121,7 @@ void CopyJobPrivate::slotResultCopyingFiles( KJob * job )
     m_processedSize += m_fileProcessedSize;
     m_fileProcessedSize = 0;
 
-    //kDebug(7007) << files.count() << " files remaining";
+    //kDebug(7007) << files.count() << "files remaining";
 
     // Merge metadata from subjob
     KIO::Job* kiojob = dynamic_cast<KIO::Job*>(job);
@@ -1239,7 +1253,6 @@ void CopyJobPrivate::slotResultConflictCopyingFiles( KJob * job )
             assert( 0 );
     }
     state = STATE_COPYING_FILES;
-    //emit processedAmount( this, KJob::Files, m_processedFiles );
     copyNextFile();
 }
 
@@ -1256,7 +1269,7 @@ KIO::Job* CopyJobPrivate::linkNextFile( const KUrl& uSource, const KUrl& uDest,
         // This is the case of creating a real symlink
         KIO::SimpleJob *newJob = KIO::symlink( uSource.path(), uDest, flags|HideProgressInfo /*no GUI*/ );
         Scheduler::scheduleJob(newJob);
-        //kDebug(7007) << "CopyJob::copyNextFile : Linking target=" << uSource.path() << " link=" << uDest;
+        //kDebug(7007) << "Linking target=" << uSource.path() << "link=" << uDest;
         //emit linking( this, uSource.path(), uDest );
         m_bCurrentOperationIsLink = true;
         m_currentSrcURL=uSource;
@@ -1266,12 +1279,12 @@ KIO::Job* CopyJobPrivate::linkNextFile( const KUrl& uSource, const KUrl& uDest,
         return newJob;
     } else {
         Q_Q(CopyJob);
-        //kDebug(7007) << "CopyJob::copyNextFile : Linking URL=" << uSource << " link=" << uDest;
+        //kDebug(7007) << "Linking URL=" << uSource << "link=" << uDest;
         if ( uDest.isLocalFile() ) {
             // if the source is a devices url, handle it a littlebit special
 
             QString path = uDest.path();
-            //kDebug(7007) << "CopyJob::copyNextFile path=" << path;
+            //kDebug(7007) << "path=" << path;
             QFile f( path );
             if ( f.open( QIODevice::ReadWrite ) )
             {
@@ -1303,7 +1316,7 @@ KIO::Job* CopyJobPrivate::linkNextFile( const KUrl& uSource, const KUrl& uDest,
             }
             else
             {
-                kDebug(7007) << "CopyJob::copyNextFile ERR_CANNOT_OPEN_FOR_WRITING";
+                kDebug(7007) << "ERR_CANNOT_OPEN_FOR_WRITING";
                 q->setError( ERR_CANNOT_OPEN_FOR_WRITING );
                 q->setErrorText( uDest.path() );
                 q->emitResult();
@@ -1323,7 +1336,7 @@ void CopyJobPrivate::copyNextFile()
 {
     Q_Q(CopyJob);
     bool bCopyFile = false;
-    //kDebug(7007) << "CopyJob::copyNextFile()";
+    //kDebug(7007);
     // Take the first file in the list
     QList<CopyInfo>::Iterator it = files.begin();
     // Is this URL on the skip list ?
@@ -1370,7 +1383,7 @@ void CopyJobPrivate::copyNextFile()
             KIO::SimpleJob *newJob = KIO::symlink( (*it).linkDest, uDest, flags | HideProgressInfo /*no GUI*/ );
             Scheduler::scheduleJob(newJob);
             newjob = newJob;
-            //kDebug(7007) << "CopyJob::copyNextFile : Linking target=" << (*it).linkDest << " link=" << uDest;
+            //kDebug(7007) << "Linking target=" << (*it).linkDest << "link=" << uDest;
             m_currentSrcURL = KUrl( (*it).linkDest );
             m_currentDestURL = uDest;
             m_bURLDirty = true;
@@ -1384,7 +1397,7 @@ void CopyJobPrivate::copyNextFile()
             KIO::FileCopyJob * moveJob = KIO::file_move( uSource, uDest, (*it).permissions, flags | HideProgressInfo/*no GUI*/ );
             moveJob->setSourceSize( (*it).size );
             newjob = moveJob;
-            //kDebug(7007) << "CopyJob::copyNextFile : Moving " << uSource << " to " << uDest;
+            //kDebug(7007) << "Moving" << uSource << "to" << uDest;
             //emit moving( this, uSource, uDest );
             m_currentSrcURL=uSource;
             m_currentDestURL=uDest;
@@ -1408,7 +1421,7 @@ void CopyJobPrivate::copyNextFile()
                 copyJob->setModificationTime( dt );
             }
             newjob = copyJob;
-            //kDebug(7007) << "CopyJob::copyNextFile : Copying " << uSource << " to " << uDest;
+            //kDebug(7007) << "Copying" << uSource << "to" << uDest;
             m_currentSrcURL=uSource;
             m_currentDestURL=uDest;
             m_bURLDirty = true;
@@ -1445,7 +1458,7 @@ void CopyJobPrivate::deleteNextDir()
     {
         // This step is done, move on
         state = STATE_SETTING_DIR_ATTRIBUTES;
-        m_directoriesCopiedIterator = m_directoriesCopied.begin();
+        m_directoriesCopiedIterator = m_directoriesCopied.constBegin();
         setNextDirAttribute();
     }
 }
@@ -1453,11 +1466,11 @@ void CopyJobPrivate::deleteNextDir()
 void CopyJobPrivate::setNextDirAttribute()
 {
     Q_Q(CopyJob);
-    while (m_directoriesCopiedIterator != m_directoriesCopied.end() &&
+    while (m_directoriesCopiedIterator != m_directoriesCopied.constEnd() &&
            (*m_directoriesCopiedIterator).mtime == -1) {
         ++m_directoriesCopiedIterator;
     }
-    if ( m_directoriesCopiedIterator != m_directoriesCopied.end() ) {
+    if ( m_directoriesCopiedIterator != m_directoriesCopied.constEnd() ) {
         const KUrl url = (*m_directoriesCopiedIterator).uDest;
         const time_t mtime = (*m_directoriesCopiedIterator).mtime;
         const QDateTime dt = QDateTime::fromTime_t(mtime);
@@ -1472,8 +1485,8 @@ void CopyJobPrivate::setNextDirAttribute()
         // TODO: can be removed now. Or reintroduced as a fast path for local files
         // if launching even more jobs as done above is a performance problem.
         //
-        QLinkedList<CopyInfo>::Iterator it = m_directoriesCopied.begin();
-        for ( ; it != m_directoriesCopied.end() ; ++it ) {
+        QLinkedList<CopyInfo>::const_iterator it = m_directoriesCopied.constBegin();
+        for ( ; it != m_directoriesCopied.constEnd() ; ++it ) {
             const KUrl& url = (*it).uDest;
             if ( url.isLocalFile() && (*it).mtime != (time_t)-1 ) {
                 const QByteArray path = QFile::encodeName( url.path() );
@@ -1517,12 +1530,13 @@ void CopyJobPrivate::setNextDirAttribute()
 void CopyJobPrivate::slotProcessedSize( KJob*, qulonglong data_size )
 {
   Q_Q(CopyJob);
-  //kDebug(7007) << "CopyJobPrivate::slotProcessedSize " << data_size;
+  //kDebug(7007) << data_size;
   m_fileProcessedSize = data_size;
   q->setProcessedAmount(KJob::Bytes, m_processedSize + m_fileProcessedSize);
 
   if ( m_processedSize + m_fileProcessedSize > m_totalSize )
   {
+    // Example: download any attachment from bugs.kde.org
     m_totalSize = m_processedSize + m_fileProcessedSize;
     //kDebug(7007) << "Adjusting m_totalSize to " << m_totalSize;
     q->setTotalAmount(KJob::Bytes, m_totalSize); // safety
@@ -1610,10 +1624,10 @@ void CopyJobPrivate::slotResultRenaming( KJob* job )
             tmpFile.setAutoRemove(false);
             tmpFile.open();
             QByteArray _tmp( QFile::encodeName(tmpFile.fileName()) );
-            kDebug(7007) << "CopyJobPrivate::slotResult KTemporaryFile using " << _tmp << " as intermediary";
-            if ( ::rename( _src, _tmp ) == 0 )
+            kDebug(7007) << "KTemporaryFile using" << _tmp << "as intermediary";
+            if ( KDE_rename( _src, _tmp ) == 0 )
             {
-                if ( !QFile::exists( _dest ) && ::rename( _tmp, _dest ) == 0 )
+                if ( !QFile::exists( _dest ) && KDE_rename( _tmp, _dest ) == 0 )
                 {
                     kDebug(7007) << "Success.";
                     err = 0;
@@ -1621,8 +1635,8 @@ void CopyJobPrivate::slotResultRenaming( KJob* job )
                 else
                 {
                     // Revert back to original name!
-                    if ( ::rename( _tmp, _src ) != 0 ) {
-                        kError(7007) << "Couldn't rename " << tmpFile.fileName() << " back to " << _src << " !" << endl;
+                    if ( KDE_rename( _tmp, _src ) != 0 ) {
+                        kError(7007) << "Couldn't rename" << tmpFile.fileName() << "back to" << _src << '!';
                         // Severe error, abort
                         q->Job::slotResult( job ); // will set the error and emit result(this)
                         return;
@@ -1749,13 +1763,13 @@ void CopyJobPrivate::slotResultRenaming( KJob* job )
                 }
             }
         } else if ( err != KIO::ERR_UNSUPPORTED_ACTION ) {
-            kDebug(7007) << "Couldn't rename " << m_currentSrcURL << " to " << dest << ", aborting";
+            kDebug(7007) << "Couldn't rename" << m_currentSrcURL << "to" << dest << ", aborting";
             q->setError( err );
             q->setErrorText( errText );
             q->emitResult();
             return;
         }
-        kDebug(7007) << "Couldn't rename " << m_currentSrcURL << " to " << dest << ", reverting to normal way, starting with stat";
+        kDebug(7007) << "Couldn't rename" << m_currentSrcURL << "to" << dest << ", reverting to normal way, starting with stat";
         //kDebug(7007) << "KIO::stat on " << m_currentSrcURL;
         KIO::Job* job = KIO::stat( m_currentSrcURL, StatJob::SourceSide, 2, KIO::HideProgressInfo );
         state = STATE_STATING;
@@ -1765,6 +1779,7 @@ void CopyJobPrivate::slotResultRenaming( KJob* job )
     else
     {
         //kDebug(7007) << "Renaming succeeded, move on";
+        ++m_processedFiles;
         emit q->copyingDone( q, *m_currentStatSrc, dest, -1 /*mtime unknown, and not needed*/, true, true );
         statNextSrc();
     }
@@ -1773,7 +1788,7 @@ void CopyJobPrivate::slotResultRenaming( KJob* job )
 void CopyJob::slotResult( KJob *job )
 {
     Q_D(CopyJob);
-    //kDebug(7007) << "CopyJobPrivate::slotResult() d->state=" << (int) d->state;
+    //kDebug(7007) << "d->state=" << (int) d->state;
     // In each case, what we have to do is :
     // 1 - check for errors and treat them
     // 2 - removeSubjob(job);
@@ -1789,7 +1804,7 @@ void CopyJob::slotResult( KJob *job )
             break;
         }
         case STATE_LISTING: // recursive listing finished
-            //kDebug(7007) << "totalSize: " << (unsigned int) d->m_totalSize << " files: " << d->files.count() << " d->dirs: " << d->dirs.count();
+            //kDebug(7007) << "totalSize:" << (unsigned int) d->m_totalSize << "files:" << d->files.count() << "d->dirs:" << d->dirs.count();
             // Was there an error ?
             if (job->error())
             {
@@ -1830,9 +1845,14 @@ void KIO::CopyJob::setDefaultPermissions( bool b )
     d_func()->m_defaultPermissions = b;
 }
 
+KIO::CopyJob::CopyMode KIO::CopyJob::operationMode() const
+{
+    return d_func()->m_mode;
+}
+
 CopyJob *KIO::copy(const KUrl& src, const KUrl& dest, JobFlags flags)
 {
-    //kDebug(7007) << "KIO::copy src=" << src << " dest=" << dest;
+    //kDebug(7007) << "src=" << src << "dest=" << dest;
     KUrl::List srcList;
     srcList.append( src );
     return CopyJobPrivate::newJob(srcList, dest, CopyJob::Copy, false, flags);
@@ -1840,7 +1860,7 @@ CopyJob *KIO::copy(const KUrl& src, const KUrl& dest, JobFlags flags)
 
 CopyJob *KIO::copyAs(const KUrl& src, const KUrl& dest, JobFlags flags)
 {
-    //kDebug(7007) << "KIO::copyAs src=" << src << " dest=" << dest;
+    //kDebug(7007) << "src=" << src << "dest=" << dest;
     KUrl::List srcList;
     srcList.append( src );
     return CopyJobPrivate::newJob(srcList, dest, CopyJob::Copy, true, flags);
@@ -1848,13 +1868,13 @@ CopyJob *KIO::copyAs(const KUrl& src, const KUrl& dest, JobFlags flags)
 
 CopyJob *KIO::copy( const KUrl::List& src, const KUrl& dest, JobFlags flags )
 {
-    //kDebug(7007) << src << " " << dest;
+    //kDebug(7007) << src << dest;
     return CopyJobPrivate::newJob(src, dest, CopyJob::Copy, false, flags);
 }
 
 CopyJob *KIO::move(const KUrl& src, const KUrl& dest, JobFlags flags)
 {
-    //kDebug(7007) << src << " " << dest;
+    //kDebug(7007) << src << dest;
     KUrl::List srcList;
     srcList.append( src );
     return CopyJobPrivate::newJob(srcList, dest, CopyJob::Move, false, flags);
@@ -1862,7 +1882,7 @@ CopyJob *KIO::move(const KUrl& src, const KUrl& dest, JobFlags flags)
 
 CopyJob *KIO::moveAs(const KUrl& src, const KUrl& dest, JobFlags flags)
 {
-    //kDebug(7007) << src << " " << dest;
+    //kDebug(7007) << src << dest;
     KUrl::List srcList;
     srcList.append( src );
     return CopyJobPrivate::newJob(srcList, dest, CopyJob::Move, true, flags);
@@ -1870,7 +1890,7 @@ CopyJob *KIO::moveAs(const KUrl& src, const KUrl& dest, JobFlags flags)
 
 CopyJob *KIO::move( const KUrl::List& src, const KUrl& dest, JobFlags flags)
 {
-    //kDebug(7007) << src << " " << dest;
+    //kDebug(7007) << src << dest;
     return CopyJobPrivate::newJob(src, dest, CopyJob::Move, false, flags);
 }
 
