@@ -55,7 +55,7 @@ K_GLOBAL_STATIC(KDirListerCache, kDirListerCache)
 KDirListerCache::KDirListerCache()
     : itemsCached( 10 ) // keep the last 10 directories around
 {
-    //kDebug(7004) << "+KDirListerCache";
+    //kDebug(7004);
 
   connect( &pendingUpdateTimer, SIGNAL(timeout()), this, SLOT(processPendingUpdates()) );
   pendingUpdateTimer.setSingleShot( true );
@@ -73,14 +73,14 @@ KDirListerCache::KDirListerCache()
   connect(kdirnotify, SIGNAL(FilesChanged(QStringList)), SLOT(slotFilesChanged(QStringList)));
   connect(kdirnotify, SIGNAL(FilesRemoved(QStringList)), SLOT(slotFilesRemoved(QStringList)));
 
-  // The use of QUrl::url() in ~DirItem (sendSignal) crashes if the static for QRegExpEngine got deleted already,
+  // The use of KUrl::url() in ~DirItem (sendSignal) crashes if the static for QRegExpEngine got deleted already,
   // so we need to destroy the KDirListerCache before that.
   qAddPostRoutine(kDirListerCache.destroy);
 }
 
 KDirListerCache::~KDirListerCache()
 {
-    kDebug(7004) << "-KDirListerCache";
+    kDebug(7004);
 
     qDeleteAll(itemsInUse);
     itemsInUse.clear();
@@ -101,16 +101,17 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
   KUrl _url(_u);
   _url.cleanPath(); // kill consecutive slashes
   _url.adjustPath(KUrl::RemoveTrailingSlash);
-  QString urlStr = _url.url();
+  const QString urlStr = _url.url();
 
-  if ( !validUrl( lister, _url ) )
-    return false;
+  if (!validUrl(lister, _url)) {
+        kDebug(7004) << lister << "url=" << _url << "not a valid url";
+        return false;
+  }
 
 #ifdef DEBUG_CACHE
   printDebug();
 #endif
-  kDebug(7004) << lister << "url=" << _url
-                << "keep=" << _keep << "reload=" << _reload << endl;
+  //kDebug(7004) << lister << "url=" << _url << "keep=" << _keep << "reload=" << _reload;
 
   if ( !_keep )
   {
@@ -139,90 +140,54 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
       lister->d->rootFileItem = KFileItem();
   }
 
-  lister->d->lstDirs.append( _url );
+    lister->d->complete = false;
 
-  if ( lister->d->url.isEmpty() || !_keep ) // set toplevel URL only if not set yet
-    lister->d->url = _url;
+    lister->d->lstDirs.append(_url);
 
-  DirItem *itemU = itemsInUse.value(urlStr);
-  DirItem *itemC;
+    if (lister->d->url.isEmpty() || !_keep) // set toplevel URL only if not set yet
+        lister->d->url = _url;
 
-  DirectoryData& dirData = directoryData[urlStr]; // find or insert
+    DirItem *itemU = itemsInUse.value(urlStr);
 
-  if ( dirData.listersCurrentlyListing.isEmpty() )
-  {
-    // if there is an update running for _url already we get into
-    // the following case - it will just be restarted by updateDirectory().
+    DirectoryData& dirData = directoryData[urlStr]; // find or insert
 
-    if ( itemU )
-    {
-      kDebug(7004) << "Entry already in use:" << _url;
+    if (dirData.listersCurrentlyListing.isEmpty()) {
+        // if there is an update running for _url already we get into
+        // the following case - it will just be restarted by updateDirectory().
 
-      bool oldState = lister->d->complete;
-      lister->d->complete = false;
+        dirData.listersCurrentlyListing.append(lister);
 
-      emit lister->started( _url );
+        DirItem *itemFromCache;
+        if (itemU || (!_reload && (itemFromCache = itemsCached.take(urlStr)) ) ) {
+            if (itemU) {
+                kDebug(7004) << "Entry already in use:" << _url;
+            } else {
+                kDebug(7004) << "Entry in cache:" << _url;
+                itemFromCache->decAutoUpdate();
+                itemsInUse.insert(urlStr, itemFromCache);
+                itemU = itemFromCache;
+            }
 
-      if ( lister->d->rootFileItem.isNull() && lister->d->url == _url )
-        lister->d->rootFileItem = itemU->rootItem;
+            emit lister->started(_url);
 
-      lister->d->addNewItems( itemU->lstItems );
-      lister->d->emitItems();
+            // List items from the cache in a delayed manner, just like things would happen
+            // if we were not using the cache.
+            KDirLister::Private::CachedItemsJob* cachedItemsJob =
+                new KDirLister::Private::CachedItemsJob(lister, itemU->lstItems, itemU->rootItem, _url, _reload, true /*emit completed*/);
+            cachedItemsJob->start();
+            lister->d->m_cachedItemsJob = cachedItemsJob;
 
-      lister->d->complete = oldState;
+        } else {
+            // dir not in cache or _reload is true
+            if (_reload) {
+                kDebug(7004) << "Reloading directory:" << _url;
+                itemsCached.remove(urlStr);
+            } else {
+                kDebug(7004) << "Listing directory:" << _url;
+            }
 
-      emit lister->completed( _url );
-      if ( lister->d->complete )
-        emit lister->completed();
-
-      // _url is already in use, so there is already an entry in listersCurrentlyHolding
-      assert( !dirData.listersCurrentlyHolding.isEmpty() );
-      dirData.listersCurrentlyHolding.append( lister );
-
-      if ( _reload || !itemU->complete )
-        updateDirectory( _url );
-    }
-    else if ( !_reload && (itemC = itemsCached.take(urlStr)) )
-    {
-      kDebug(7004) << "Entry in cache:" << _url;
-
-      itemC->decAutoUpdate();
-      itemsInUse.insert( urlStr, itemC );
-      itemU = itemC;
-
-      bool oldState = lister->d->complete;
-      lister->d->complete = false;
-
-      emit lister->started( _url );
-
-      if ( lister->d->rootFileItem.isNull() && lister->d->url == _url )
-        lister->d->rootFileItem = itemC->rootItem;
-
-      lister->d->addNewItems( itemC->lstItems );
-      lister->d->emitItems();
-
-      lister->d->complete = oldState;
-
-      emit lister->completed( _url );
-      if ( lister->d->complete )
-        emit lister->completed();
-
-      Q_ASSERT( dirData.listersCurrentlyHolding.isEmpty() );
-      dirData.listersCurrentlyHolding.append( lister );
-
-      if ( !itemC->complete )
-        updateDirectory( _url );
-    }
-    else  // dir not in cache or _reload is true
-    {
-      kDebug(7004) << "Entry not in cache or reloaded:" << _url;
-
-      dirData.listersCurrentlyListing.append( lister );
-
-      if (_reload)
-          itemsCached.remove( urlStr );
-      itemU = new DirItem( _url );
-      itemsInUse.insert( urlStr, itemU );
+            itemU = new DirItem(_url);
+            itemsInUse.insert(urlStr, itemU);
 
 //        // we have a limit of MAX_JOBS_PER_LISTER concurrently running jobs
 //        if ( lister->d->numJobs() >= MAX_JOBS_PER_LISTER )
@@ -230,60 +195,99 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
 //          pendingUpdates.insert( _url );
 //        }
 //        else
-//        {
+            {
+                KIO::ListJob* job = KIO::listDir(_url, KIO::HideProgressInfo);
+                jobs.insert(job, KIO::UDSEntryList());
 
-      if ( lister->d->url == _url )
-        lister->d->rootFileItem = KFileItem();
+                lister->d->jobStarted(job);
+                lister->d->connectJob(job);
 
-      KIO::ListJob* job = KIO::listDir( _url, KIO::HideProgressInfo );
-      jobs.insert( job, KIO::UDSEntryList() );
+                if (lister->d->window)
+                    job->ui()->setWindow(lister->d->window);
 
-      lister->d->jobStarted( job );
-      lister->d->connectJob( job );
+                connect(job, SIGNAL(entries(KIO::Job *, KIO::UDSEntryList)),
+                        this, SLOT(slotEntries(KIO::Job *, KIO::UDSEntryList)));
+                connect(job, SIGNAL(result(KJob *)),
+                        this, SLOT(slotResult(KJob *)));
+                connect(job, SIGNAL(redirection(KIO::Job *,KUrl)),
+                        this, SLOT(slotRedirection(KIO::Job *,KUrl)));
 
-      if ( lister->d->window )
-        job->ui()->setWindow( lister->d->window );
+                emit lister->started(_url);
+            }
+        }
+    } else {
 
-      connect( job, SIGNAL( entries( KIO::Job *, const KIO::UDSEntryList & ) ),
-               this, SLOT( slotEntries( KIO::Job *, const KIO::UDSEntryList & ) ) );
-      connect( job, SIGNAL( result( KJob * ) ),
-               this, SLOT( slotResult( KJob * ) ) );
-      connect( job, SIGNAL( redirection( KIO::Job *, const KUrl & ) ),
-               this, SLOT( slotRedirection( KIO::Job *, const KUrl & ) ) );
+        kDebug(7004) << "Entry currently being listed:" << _url;
 
-      emit lister->started( _url );
+        emit lister->started( _url );
 
-//        }
+        dirData.listersCurrentlyListing.append( lister );
+
+        KIO::ListJob *job = jobForUrl( urlStr );
+        // job will be 0 if we were listing from cache rather than listing from a kio job.
+        if( job ) {
+            lister->d->jobStarted( job );
+            lister->d->connectJob( job );
+        }
+        Q_ASSERT( itemU );
+
+        // List existing items in a delayed manner, just like things would happen
+        // if we were not using the cache.
+        KDirLister::Private::CachedItemsJob* cachedItemsJob =
+            new KDirLister::Private::CachedItemsJob(lister, itemU->lstItems, itemU->rootItem, _url, _reload, false /*do not emit completed*/);
+        cachedItemsJob->start();
+        lister->d->m_cachedItemsJob = cachedItemsJob;
     }
-  }
-  else
-  {
-    kDebug(7004) << "Entry currently being listed:" << _url;
 
-    emit lister->started( _url );
+    // automatic updating of directories
+    if (lister->d->autoUpdate)
+        itemU->incAutoUpdate();
 
-    dirData.listersCurrentlyListing.append( lister );
+    return true;
+}
 
-    KIO::ListJob *job = jobForUrl( urlStr );
-    Q_ASSERT( job );
+void KDirLister::Private::CachedItemsJob::done()
+{
+    kDirListerCache->emitItemsFromCache(m_lister, m_items, m_rootItem, m_url, m_reload, m_emitCompleted);
+    emitResult();
+}
 
-    lister->d->jobStarted( job );
-    lister->d->connectJob( job );
+void KDirListerCache::emitItemsFromCache(KDirLister* lister, const KFileItemList& items, const KFileItem& rootItem, const KUrl& _url, bool _reload, bool _emitCompleted)
+{
+    lister->d->m_cachedItemsJob = 0;
 
-    Q_ASSERT( itemU );
+    const QString urlStr = _url.url();
+    DirItem *itemU = kDirListerCache->itemsInUse.value(urlStr);
+    Q_ASSERT(itemU); // hey we're listing that dir, so this can't be 0, right?
 
-    if ( lister->d->rootFileItem.isNull() && lister->d->url == _url )
-      lister->d->rootFileItem = itemU->rootItem;
+    KDirLister::Private* kdl = lister->d;
 
-    lister->d->addNewItems( itemU->lstItems );
-    lister->d->emitItems();
-  }
+    const bool oldComplete = kdl->complete;
+    kdl->complete = false;
 
-  // automatic updating of directories
-  if ( lister->d->autoUpdate )
-    itemU->incAutoUpdate();
+    if ( kdl->rootFileItem.isNull() && kdl->url == _url )
+        kdl->rootFileItem = rootItem;
 
-  return true;
+    kdl->addNewItems( items );
+    kdl->emitItems();
+
+    // _emitCompleted is usually true, but is false for the special case where
+    // listDir() was called while another directory listing for this dir was happening,
+    // so we "joined" it.
+    // but then we have to check if completed was emitted already, by the running job...
+    if (_emitCompleted && !oldComplete) {
+
+        DirectoryData& dirData = directoryData[urlStr];
+        dirData.listersCurrentlyHolding.append( lister );
+        dirData.listersCurrentlyListing.removeAll( lister );
+
+        kdl->complete = true;
+        emit lister->completed( _url );
+        emit lister->completed();
+
+        if ( _reload || !itemU->complete )
+            updateDirectory( _url );
+    }
 }
 
 bool KDirListerCache::validUrl( const KDirLister *lister, const KUrl& url ) const
@@ -292,7 +296,7 @@ bool KDirListerCache::validUrl( const KDirLister *lister, const KUrl& url ) cons
   {
     if ( lister->d->autoErrorHandling )
     {
-      QString tmp = i18n("Malformed URL\n%1",  url.prettyUrl() );
+      QString tmp = i18n("Malformed URL\n%1", url.prettyUrl() );
       KMessageBox::error( lister->d->errorParent, tmp );
     }
     return false;
@@ -302,7 +306,7 @@ bool KDirListerCache::validUrl( const KDirLister *lister, const KUrl& url ) cons
   {
     if ( lister->d->autoErrorHandling )
     {
-      QString tmp = i18n("URL cannot be listed\n%1",  url.prettyUrl() );
+      QString tmp = i18n("URL cannot be listed\n%1", url.prettyUrl() );
       KMessageBox::error( lister->d->errorParent, tmp );
     }
     return false;
@@ -333,6 +337,12 @@ void KDirListerCache::stop( KDirLister *lister )
         }
     }
 
+    if (lister->d->m_cachedItemsJob) {
+        delete lister->d->m_cachedItemsJob;
+        lister->d->m_cachedItemsJob = 0;
+        stopped = true;
+    }
+
     if ( stopped ) {
         emit lister->canceled();
         lister->d->complete = true;
@@ -348,6 +358,11 @@ void KDirListerCache::stop( KDirLister *lister, const KUrl& _u )
     url.adjustPath( KUrl::RemoveTrailingSlash );
     const QString urlStr = url.url();
 
+    if (lister->d->m_cachedItemsJob && lister->d->m_cachedItemsJob->url() == url) {
+        delete lister->d->m_cachedItemsJob;
+        lister->d->m_cachedItemsJob = 0;
+    }
+
     // TODO: consider to stop all the "child jobs" of url as well
     kDebug(7004) << lister << " url=" << url;
 
@@ -444,7 +459,8 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KUrl& _url, bool not
     const QString urlStr = url.url();
 
     DirectoryDataHash::iterator dit = directoryData.find(urlStr);
-    Q_ASSERT(dit != directoryData.end());
+    if (dit == directoryData.end())
+        return;
     DirectoryData& dirData = *dit;
     dirData.listersCurrentlyHolding.removeAll(lister);
 
@@ -504,7 +520,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KUrl& _url, bool not
             if ( isManuallyMounted || containsManuallyMounted )
             {
                 kDebug(7004) << "Not adding a watch on " << item->url << " because it " <<
-                    ( isManuallyMounted ? "is manually mounted" : "contains a manually mounted subdir" ) << endl;
+                    ( isManuallyMounted ? "is manually mounted" : "contains a manually mounted subdir" );
                 item->complete = false; // set to "dirty"
             }
             else
@@ -742,7 +758,7 @@ void KDirListerCache::slotFilesChanged( const QStringList &fileList ) // from KD
     KUrl url( *it );
     if ( url.isLocalFile() )
     {
-      kDebug(7004) << "KDirListerCache::slotFilesChanged " << url;
+      kDebug(7004) << url;
       KFileItem *fileitem = findByUrl( 0, url );
       if ( fileitem )
       {
@@ -1618,6 +1634,7 @@ void KDirListerCache::deleteDir( const KUrl& dirUrl )
         // items cached now
         int count = itemsInUse.remove( deletedUrlStr );
         Q_ASSERT( count == 0 );
+        Q_UNUSED( count ); //keep gcc "unused variable" complaining quiet when in release mode
     }
 
     // remove the children from the cache
@@ -1653,7 +1670,7 @@ void KDirListerCache::printDebug()
                      << " rootItem: " << ( !itu.value()->rootItem.isNull() ? itu.value()->rootItem.url() : KUrl() )
                      << " autoUpdates refcount: " << itu.value()->autoUpdates
                      << " complete: " << itu.value()->complete
-                     << QString(" with %1 items.").arg(itu.value()->lstItems.count()) << endl;
+                     << QString(" with %1 items.").arg(itu.value()->lstItems.count());
     }
 
     kDebug(7004) << "Directory data: ";
@@ -1680,9 +1697,9 @@ void KDirListerCache::printDebug()
     const QList<QString> cachedDirs = itemsCached.keys();
     foreach(const QString& cachedDir, cachedDirs) {
         DirItem* dirItem = itemsCached.object(cachedDir);
-        kDebug(7004) << "   " << cachedDir << "  rootItem: "
-                     << ( !dirItem->rootItem.isNull() ? dirItem->rootItem.url().prettyUrl() : QString("NULL") )
-                     << QString(" with %1 items.").arg(dirItem->lstItems.count()) << endl;
+        kDebug(7004) << "   " << cachedDir << "rootItem:"
+                     << (!dirItem->rootItem.isNull() ? dirItem->rootItem.url().prettyUrl() : QString("NULL") )
+                     << "with" << dirItem->lstItems.count() << "items.";
     }
 }
 #endif
@@ -2044,7 +2061,10 @@ bool KDirLister::doMimeFilter( const QString& mime, const QStringList& filters )
   if ( filters.isEmpty() )
     return true;
 
-  KMimeType::Ptr mimeptr = KMimeType::mimeType(mime);
+  const KMimeType::Ptr mimeptr = KMimeType::mimeType(mime);
+  if ( !mimeptr )
+    return false;
+
   //kDebug(7004) << "doMimeFilter: investigating: "<<mimeptr->name();
   QStringList::const_iterator it = filters.begin();
   for ( ; it != filters.end(); ++it )
@@ -2052,7 +2072,6 @@ bool KDirLister::doMimeFilter( const QString& mime, const QStringList& filters )
       return true;
     //else   kDebug(7004) << "doMimeFilter: compared without result to  "<<*it;
 
-
   return false;
 }
 
@@ -2109,7 +2128,6 @@ void KDirLister::Private::addNewItems( const KFileItemList& items )
   // TODO: make this faster - test if we have a filter at all first
   // DF: was this profiled? The matchesFoo() functions should be fast, w/o filters...
   // Of course if there is no filter and we can do a range-insertion instead of a loop, that might be good.
-  // But that's for Qt4, not possible with QPtrList.
   KFileItemList::const_iterator kit = items.begin();
   const KFileItemList::const_iterator kend = items.end();
   for ( ; kit != kend; ++kit )
