@@ -24,6 +24,8 @@
 #include <kicon.h>
 #include <klocale.h>
 #include <kglobal.h>
+#include <kio/copyjob.h>
+#include <kio/jobuidelegate.h>
 #include <kurl.h>
 #include <kdebug.h>
 #include <QMimeData>
@@ -311,8 +313,8 @@ void KDirModelPrivate::_k_slotNewItems(const KFileItemList& items)
     const int newItemsCount = items.count();
     const int newRowCount = dirNode->m_childNodes.count() + newItemsCount;
 #ifndef NDEBUG // debugIndex only defined in debug mode
-    kDebug(7008) << items.count() << " in " << dir
-             << " index=" << debugIndex(index) << " newRowCount=" << newRowCount << endl;
+    kDebug(7008) << items.count() << "in" << dir
+             << "index=" << debugIndex(index) << "newRowCount=" << newRowCount;
 #endif
     q->beginInsertRows( index, newRowCount - newItemsCount, newRowCount - 1 ); // parent, first, last
 
@@ -453,6 +455,12 @@ QVariant KDirModel::data( const QModelIndex & index, int role ) const
                 return item.mimeComment();
             }
             break;
+        case Qt::EditRole:
+            switch (index.column()) {
+            case Name:
+                return item.text();
+            }
+            break;
         case Qt::DecorationRole:
             if (index.column() == Name) {
                 if (!node->preview().isNull()) {
@@ -522,8 +530,21 @@ void KDirModel::sort( int column, Qt::SortOrder order )
 bool KDirModel::setData( const QModelIndex & index, const QVariant & value, int role )
 {
     switch (role) {
-    case Qt::DisplayRole:
-        // TODO handle renaming here?
+    case Qt::EditRole:
+        if (index.column() == Name && value.type() == QVariant::String) {
+            Q_ASSERT(index.isValid());
+            KDirModelNode* node = static_cast<KDirModelNode*>(index.internalPointer());
+            const KFileItem& item = node->item();
+            const QString newName = value.toString();
+            if (newName.isEmpty() || newName == item.text())
+                return true;
+            KUrl newurl(item.url());
+            newurl.setPath(newurl.directory(KUrl::AppendTrailingSlash) + newName);
+            KIO::Job * job = KIO::moveAs(item.url(), newurl, newurl.isLocalFile() ? KIO::HideProgressInfo : KIO::DefaultFlags);
+            job->ui()->setAutoErrorHandlingEnabled(true);
+            // TODO undo handling
+            return true;
+        }
         break;
     case Qt::DecorationRole:
         if (index.column() == Name) {
@@ -618,7 +639,7 @@ QModelIndex KDirModel::indexForUrl(const KUrl& url) const
 {
     const QPair<int, KDirModelNode*> result = d->nodeForUrl(url); // O(n*m) (m is the depth from the root)
     if (!result.second) {
-        kWarning(7007) << url << "not found";
+        kDebug(7007) << url << "not found";
         return QModelIndex();
     }
     return d->indexForNode(result.second, result.first); // O(1)
@@ -706,7 +727,7 @@ Qt::ItemFlags KDirModel::flags( const QModelIndex & index ) const
                     }
                 }
             }
-	}
+        }
     }
 
     return f;
@@ -743,9 +764,8 @@ void KDirModel::fetchMore( const QModelIndex & parent )
         return;
     dirNode->setPopulated( true );
 
-    //const KUrl url = parentItem.url();
-    //kDebug(7008) << "listing" << url;
-    d->m_dirLister->openUrl(parentItem.url(), KDirLister::Keep | KDirLister::Reload);
+    const KUrl parentUrl = parentItem.url();
+    d->m_dirLister->openUrl(parentUrl, KDirLister::Keep);
 }
 
 bool KDirModel::dropMimeData( const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent )
@@ -779,8 +799,13 @@ void KDirModel::expandToUrl(const KUrl& url)
 
     d->m_urlsBeingFetched[result.second].append(url);
 
-    if (result.second == d->m_rootNode) // the root is fetched by default, so it must be currently being fetched
+    if (result.second == d->m_rootNode) {
+        kDebug(7008) << "Remembering to emit expand after listing the root url";
+        // the root is fetched by default, so it must be currently being fetched
         return;
+    }
+
+    kDebug(7008) << "Remembering to emit expand after listing" << result.second->item().url();
 
     // start a new fetch to look for the next level down the URL
     const QModelIndex parentIndex = d->indexForNode(result.second, result.first);
