@@ -43,6 +43,7 @@
 #include <kcrash.h>
 #include <kconfig.h>
 #include <kconfiggroup.h>
+#include <kde_file.h>
 #include <kdesu/client.h>
 #include <klocale.h>
 
@@ -102,13 +103,14 @@ public:
     KIO::filesize_t sentListEntries;
     KRemoteEncoding *remotefile;
     time_t timeout;
+    enum { Idle, InsideMethod, FinishedCalled, ErrorCalled } m_state;
     QByteArray timeoutData;
- 
+
     // Reconstructs configGroup from configData and mIncomingMetaData
     void rebuildConfig()
     {
         configGroup->deleteGroup(KConfigGroup::WriteConfigFlags());
-        
+
         // mIncomingMetaData cascades over config, so we write config first,
         // to let it be overwritten
         MetaData::ConstIterator end = configData.constEnd();
@@ -119,6 +121,19 @@ public:
         for (MetaData::ConstIterator it = q->mIncomingMetaData.constBegin(); it != end; ++it)
             configGroup->writeEntry(it.key(), it->toUtf8(), KConfigGroup::WriteConfigFlags());
     }
+
+    void verifyState(const char* cmdName)
+    {
+        if ((m_state != FinishedCalled) && (m_state != ErrorCalled)){
+            kWarning(7019) << cmdName << "did not call finished() or error()! Please fix the KIO slave.";
+        }
+    }
+    void verifyErrorFinishedNotCalled(const char* cmdName)
+    {
+        if (m_state == FinishedCalled || m_state == ErrorCalled) {
+            kWarning(7019) << cmdName << "called finished() or error(), but it's not supposed to! Please fix the KIO slave.";
+        }
+    }
 };
 
 }
@@ -134,7 +149,7 @@ static const char *s_protocol;
 extern "C" {
 static void genericsig_handler(int sigNumber)
 {
-   signal(sigNumber,SIG_IGN);
+   KDE_signal(sigNumber,SIG_IGN);
    //WABA: Don't do anything that requires malloc, we can deadlock on it since
    //a SIGTERM signal can come in while we are in malloc/free.
    //kDebug()<<"kioslave : exiting due to signal "<<sigNumber;
@@ -142,7 +157,7 @@ static void genericsig_handler(int sigNumber)
    //in lengthy operations in the various slaves
    if (globalSlave!=0)
       globalSlave->setKillFlag();
-   signal(SIGALRM,SIG_DFL);
+   KDE_signal(SIGALRM,SIG_DFL);
    alarm(5);  //generate an alarm signal in 5 seconds, in this time the slave has to exit
 }
 }
@@ -163,26 +178,26 @@ SlaveBase::SlaveBase( const QByteArray &protocol,
     if (!getenv("KDE_DEBUG"))
     {
         KCrash::setCrashHandler( sigsegv_handler );
-        signal(SIGILL,&sigsegv_handler);
-        signal(SIGTRAP,&sigsegv_handler);
-        signal(SIGABRT,&sigsegv_handler);
-        signal(SIGBUS,&sigsegv_handler);
-        signal(SIGALRM,&sigsegv_handler);
-        signal(SIGFPE,&sigsegv_handler);
+        KDE_signal(SIGILL,&sigsegv_handler);
+        KDE_signal(SIGTRAP,&sigsegv_handler);
+        KDE_signal(SIGABRT,&sigsegv_handler);
+        KDE_signal(SIGBUS,&sigsegv_handler);
+        KDE_signal(SIGALRM,&sigsegv_handler);
+        KDE_signal(SIGFPE,&sigsegv_handler);
 #ifdef SIGPOLL
-        signal(SIGPOLL, &sigsegv_handler);
+        KDE_signal(SIGPOLL, &sigsegv_handler);
 #endif
 #ifdef SIGSYS
-        signal(SIGSYS, &sigsegv_handler);
+        KDE_signal(SIGSYS, &sigsegv_handler);
 #endif
 #ifdef SIGVTALRM
-        signal(SIGVTALRM, &sigsegv_handler);
+        KDE_signal(SIGVTALRM, &sigsegv_handler);
 #endif
 #ifdef SIGXCPU
-        signal(SIGXCPU, &sigsegv_handler);
+        KDE_signal(SIGXCPU, &sigsegv_handler);
 #endif
 #ifdef SIGXFSZ
-        signal(SIGXFSZ, &sigsegv_handler);
+        KDE_signal(SIGXFSZ, &sigsegv_handler);
 #endif
     }
 
@@ -192,9 +207,9 @@ SlaveBase::SlaveBase( const QByteArray &protocol,
     act.sa_flags = 0;
     sigaction( SIGPIPE, &act, 0 );
 
-    signal(SIGINT,&genericsig_handler);
-    signal(SIGQUIT,&genericsig_handler);
-    signal(SIGTERM,&genericsig_handler);
+    KDE_signal(SIGINT,&genericsig_handler);
+    KDE_signal(SIGQUIT,&genericsig_handler);
+    KDE_signal(SIGTERM,&genericsig_handler);
 #endif
 
     globalSlave=this;
@@ -270,7 +285,7 @@ void SlaveBase::dispatchLoop()
             ret = d->appConnection.isConnected() ? 0 : -1;
         }
 
-        if (ret == -1) { // some error occurred, perhaps no more applicationi
+        if (ret == -1) { // some error occurred, perhaps no more application
             // When the app exits, should the slave be put back in the pool ?
             if (!d->exit_loop && d->isConnectedToApp && !d->poolSocket.isEmpty()) {
                 disconnectSlave();
@@ -296,8 +311,8 @@ void SlaveBase::connectSlave(const QString &address)
 
     if (!d->appConnection.inited())
     {
-        kDebug(7019) << "SlaveBase: failed to connect to " << address << endl
-		      << "Reason: " << d->appConnection.errorString() << endl;
+        kDebug(7019) << "SlaveBase: failed to connect to" << address << endl
+		      << "Reason:" << d->appConnection.errorString();
         exit();
         return;
     }
@@ -345,13 +360,17 @@ KConfigGroup *SlaveBase::config()
 
 void SlaveBase::sendMetaData()
 {
-   if (mOutgoingMetaData.isEmpty())
-      return;
+    sendAndKeepMetaData();
+    mOutgoingMetaData.clear();
+}
 
-   KIO_DATA << mOutgoingMetaData;
+void SlaveBase::sendAndKeepMetaData()
+{
+    if (!mOutgoingMetaData.isEmpty()) {
+        KIO_DATA << mOutgoingMetaData;
 
-   send( INF_META_DATA, data );
-   mOutgoingMetaData.clear(); // Clear
+        send(INF_META_DATA, data);
+    }
 }
 
 KRemoteEncoding *SlaveBase::remoteEncoding()
@@ -386,6 +405,15 @@ void SlaveBase::opened()
 
 void SlaveBase::error( int _errid, const QString &_text )
 {
+    if (d->m_state == d->ErrorCalled) {
+        kWarning(7019) << "error() called twice! Please fix the KIO slave.";
+        return;
+    } else if (d->m_state == d->FinishedCalled) {
+        kWarning(7019) << "error() called after finished()! Please fix the KIO slave.";
+        return;
+    }
+
+    d->m_state = d->ErrorCalled;
     mIncomingMetaData.clear(); // Clear meta data
     d->rebuildConfig();
     mOutgoingMetaData.clear();
@@ -406,6 +434,15 @@ void SlaveBase::connected()
 
 void SlaveBase::finished()
 {
+    if (d->m_state == d->FinishedCalled) {
+        kWarning(7019) << "finished() called twice! Please fix the KIO slave.";
+        return;
+    } else if (d->m_state == d->ErrorCalled) {
+        kWarning(7019) << "finished() called after error()! Please fix the KIO slave.";
+        return;
+    }
+
+    d->m_state = d->FinishedCalled;
     mIncomingMetaData.clear(); // Clear meta data
     d->rebuildConfig();
     sendMetaData();
@@ -508,7 +545,7 @@ void SlaveBase::position( KIO::filesize_t _pos )
 
 void SlaveBase::processedPercent( float /* percent */ )
 {
-  kDebug(7019) << "SlaveBase::processedPercent: STUB";
+  kDebug(7019) << "STUB";
 }
 
 
@@ -543,7 +580,7 @@ static bool isSubCommand(int cmd)
 
 void SlaveBase::mimeType( const QString &_type)
 {
-  // kDebug(7019) << "(" << getpid() << ") SlaveBase::mimeType '" << _type << "'";
+  // kDebug(7019) << "(" << getpid() << ")" << _type;
   int cmd;
   do
   {
@@ -686,10 +723,10 @@ void SlaveBase::listEntries( const UDSEntryList& list )
 static void sigsegv_handler(int sig)
 {
 #ifdef Q_OS_UNIX
-    signal(sig,SIG_DFL); // Next one kills
+    KDE_signal(sig,SIG_DFL); // Next one kills
 
     //Kill us if we deadlock
-    signal(SIGALRM,SIG_DFL);
+    KDE_signal(SIGALRM,SIG_DFL);
     alarm(5);  //generate an alarm signal in 5 seconds, in this time the slave has to exit
 
     // Debug and printf should be avoided because they might
@@ -787,7 +824,7 @@ bool SlaveBase::openPasswordDialog( AuthInfo& info, const QString &errorMsg )
     const long windowId = metaData("window-id").toLong();
     const unsigned long userTimestamp = metaData("user-timestamp").toULong();
 
-    kDebug(7019) << "SlaveBase::openPasswordDialog window-id=" << windowId;
+    kDebug(7019) << "window-id=" << windowId;
 
     QDBusInterface kps( "org.kde.kded", "/modules/kpasswdserver", "org.kde.KPasswdServer" );
 
@@ -823,8 +860,7 @@ bool SlaveBase::openPasswordDialog( AuthInfo& info, const QString &errorMsg )
 
     info = authResult;
 
-    kDebug(7019) << "SlaveBase::openPasswordDialog: username=" << info.username;
-    kDebug(7019) << "SlaveBase::openPasswordDialog: password=[hidden]";
+    kDebug(7019) << "username=" << info.username << "password=[hidden]";
 
     return true;
 }
@@ -855,7 +891,7 @@ int SlaveBase::messageBox( const QString &text, MessageBoxType type, const QStri
 
 bool SlaveBase::canResume( KIO::filesize_t offset )
 {
-    kDebug(7019) << "SlaveBase::canResume offset=" << KIO::number(offset);
+    kDebug(7019) << "offset=" << KIO::number(offset);
     d->needSendCanResume = false;
     KIO_DATA << KIO_FILESIZE_T(offset);
     send( MSG_RESUME, data );
@@ -864,7 +900,7 @@ bool SlaveBase::canResume( KIO::filesize_t offset )
         int cmd;
         if ( waitForAnswer( CMD_RESUMEANSWER, CMD_NONE, data, &cmd ) != -1 )
         {
-            kDebug(7019) << "SlaveBase::canResume returning " << (cmd == CMD_RESUMEANSWER);
+            kDebug(7019) << "returning" << (cmd == CMD_RESUMEANSWER);
             return cmd == CMD_RESUMEANSWER;
         } else
             return false;
@@ -884,7 +920,7 @@ int SlaveBase::waitForAnswer( int expected1, int expected2, QByteArray & data, i
             result = d->appConnection.read( &cmd, data );
         }
         if (result == -1) {
-            kDebug(7019) << "SlaveBase::waitForAnswer has read error.";
+            kDebug(7019) << "read error.";
             return -1;
         }
 
@@ -899,7 +935,7 @@ int SlaveBase::waitForAnswer( int expected1, int expected2, QByteArray & data, i
         }
         else
         {
-            kWarning() << "Got cmd " << cmd << " while waiting for an answer!";
+            kWarning(7019) << "Got cmd " << cmd << " while waiting for an answer!";
         }
     }
 }
@@ -939,20 +975,25 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
         QString host, user;
         quint16 port;
         stream >> host >> port >> user >> passwd;
+        d->m_state = d->InsideMethod;
         setHost( host, port, user, passwd );
-    }
-    break;
-    case CMD_CONNECT:
+        d->verifyErrorFinishedNotCalled("setHost()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_CONNECT: {
         openConnection( );
-        break;
-    case CMD_DISCONNECT:
+    } break;
+    case CMD_DISCONNECT: {
         closeConnection( );
-        break;
-    case CMD_SLAVE_STATUS:
+    } break;
+    case CMD_SLAVE_STATUS: {
+        d->m_state = d->InsideMethod;
         slave_status();
-        break;
-    case CMD_SLAVE_CONNECT:
-    {
+        // TODO verify that the slave has called slaveStatus()?
+        d->verifyErrorFinishedNotCalled("slave_status()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_SLAVE_CONNECT: {
         d->onHold = false;
         QString app_socket;
         QDataStream stream( data );
@@ -962,8 +1003,7 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
         d->isConnectedToApp = true;
         connectSlave(app_socket);
     } break;
-    case CMD_SLAVE_HOLD:
-    {
+    case CMD_SLAVE_HOLD: {
         KUrl url;
         QDataStream stream( data );
         stream >> url;
@@ -974,9 +1014,12 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
         // Do not close connection!
         connectSlave(d->poolSocket);
     } break;
-    case CMD_REPARSECONFIGURATION:
+    case CMD_REPARSECONFIGURATION: {
+        d->m_state = d->InsideMethod;
         reparseConfiguration();
-        break;
+        d->verifyErrorFinishedNotCalled("reparseConfiguration()");
+        d->m_state = d->Idle;
+    } break;
     case CMD_CONFIG: {
         stream >> d->configData;
         d->rebuildConfig();
@@ -985,21 +1028,22 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
 #endif
         delete d->remotefile;
         d->remotefile = 0;
-        break;
-    }
-    case CMD_GET:
-    {
-        stream >> url;
-        get( url );
     } break;
-    case CMD_OPEN:
-    {
+    case CMD_GET: {
+        stream >> url;
+        d->m_state = d->InsideMethod;
+        get( url );
+        d->verifyState("get()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_OPEN: {
         stream >> url >> i;
         QIODevice::OpenMode mode = QFlag(i);
-        open(url, mode);
+        d->m_state = d->InsideMethod;
+        open(url, mode); //krazy:exclude=syscalls
+        d->m_state = d->Idle;
     } break;
-    case CMD_PUT:
-    {
+    case CMD_PUT: {
         int permissions;
         qint8 iOverwrite, iResume;
         stream >> url >> iOverwrite >> iResume >> permissions;
@@ -1012,98 +1056,135 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
         //   (the resume bool is currently unused)
         d->needSendCanResume = true   /* !resume */;
 
+        d->m_state = d->InsideMethod;
         put( url, permissions, flags);
+        d->verifyState("put()");
+        d->m_state = d->Idle;
     } break;
-    case CMD_STAT:
+    case CMD_STAT: {
         stream >> url;
-        stat( url );
-        break;
-    case CMD_MIMETYPE:
+        d->m_state = d->InsideMethod;
+        stat( url ); //krazy:exclude=syscalls
+        d->verifyState("stat()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_MIMETYPE: {
         stream >> url;
+        d->m_state = d->InsideMethod;
         mimetype( url );
-        break;
-    case CMD_LISTDIR:
+        d->verifyState("mimetype()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_LISTDIR: {
         stream >> url;
+        d->m_state = d->InsideMethod;
         listDir( url );
-        break;
-    case CMD_MKDIR:
+        d->verifyState("listDir()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_MKDIR: {
         stream >> url >> i;
-        mkdir( url, i );
-        break;
-    case CMD_RENAME:
-    {
+        d->m_state = d->InsideMethod;
+        mkdir( url, i ); //krazy:exclude=syscalls
+        d->verifyState("mkdir()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_RENAME: {
         qint8 iOverwrite;
         KUrl url2;
         stream >> url >> url2 >> iOverwrite;
         JobFlags flags;
         if ( iOverwrite != 0 ) flags |= Overwrite;
-        rename( url, url2, flags );
+        d->m_state = d->InsideMethod;
+        rename( url, url2, flags ); //krazy:exclude=syscalls
+        d->verifyState("rename()");
+        d->m_state = d->Idle;
     } break;
-    case CMD_SYMLINK:
-    {
+    case CMD_SYMLINK: {
         qint8 iOverwrite;
         QString target;
         stream >> target >> url >> iOverwrite;
         JobFlags flags;
         if ( iOverwrite != 0 ) flags |= Overwrite;
+        d->m_state = d->InsideMethod;
         symlink( target, url, flags );
+        d->verifyState("symlink()");
+        d->m_state = d->Idle;
     } break;
-    case CMD_COPY:
-    {
+    case CMD_COPY: {
         int permissions;
         qint8 iOverwrite;
         KUrl url2;
         stream >> url >> url2 >> permissions >> iOverwrite;
         JobFlags flags;
         if ( iOverwrite != 0 ) flags |= Overwrite;
+        d->m_state = d->InsideMethod;
         copy( url, url2, permissions, flags );
+        d->verifyState("copy()");
+        d->m_state = d->Idle;
     } break;
-    case CMD_DEL:
-    {
+    case CMD_DEL: {
         qint8 isFile;
         stream >> url >> isFile;
+        d->m_state = d->InsideMethod;
         del( url, isFile != 0);
+        d->verifyState("del()");
+        d->m_state = d->Idle;
     } break;
-    case CMD_CHMOD:
+    case CMD_CHMOD: {
         stream >> url >> i;
+        d->m_state = d->InsideMethod;
         chmod( url, i);
-        break;
-    case CMD_CHOWN:
-    {
+        d->verifyState("chmod()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_CHOWN: {
         QString owner, group;
         stream >> url >> owner >> group;
+        d->m_state = d->InsideMethod;
         chown(url, owner, group);
-        break;
-    }
-    case CMD_SETMODIFICATIONTIME:
-    {
+        d->verifyState("chown()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_SETMODIFICATIONTIME: {
         QDateTime dt;
         stream >> url >> dt;
+        d->m_state = d->InsideMethod;
         setModificationTime(url, dt);
+        d->verifyState("setModificationTime()");
+        d->m_state = d->Idle;
     } break;
-    case CMD_SPECIAL:
+    case CMD_SPECIAL: {
+        d->m_state = d->InsideMethod;
         special( data );
-        break;
+        d->verifyState("special()");
+        d->m_state = d->Idle;
+    } break;
     case CMD_META_DATA: {
         //kDebug(7019) << "(" << getpid() << ") Incoming meta-data...";
         stream >> mIncomingMetaData;
         d->rebuildConfig();
-        break;
-    }
-    case CMD_SUBURL:
+    } break;
+    case CMD_SUBURL: {
         stream >> url;
+        d->m_state = d->InsideMethod;
         setSubUrl(url);
-        break;
-    case CMD_NONE:
-        fprintf(stderr, "Got unexpected CMD_NONE!\n");
-        break;
-    case CMD_MULTI_GET:
+        d->verifyErrorFinishedNotCalled("setSubUrl()");
+        d->m_state = d->Idle;
+    } break;
+    case CMD_NONE: {
+        kWarning(7019) << "Got unexpected CMD_NONE!";
+    } break;
+    case CMD_MULTI_GET: {
+        d->m_state = d->InsideMethod;
         multiGet( data );
-        break;
-    default:
+        d->verifyState("multiGet()");
+        d->m_state = d->Idle;
+    } break;
+    default: {
         // Some command we don't understand.
         // Just ignore it, it may come from some future version of KDE.
-        break;
+    } break;
     }
 }
 
@@ -1113,7 +1194,7 @@ bool SlaveBase::checkCachedAuthentication( AuthInfo& info )
     long windowId = metaData("window-id").toLong();
     unsigned long userTimestamp = metaData("user-timestamp").toULong();
 
-    kDebug(7019) << "SlaveBase::checkCachedAuthInfo window = " << windowId << " url = " << info.url.url();
+    kDebug(7019) << "window =" << windowId << "url =" << info.url;
 
     QDBusInterface kps( "org.kde.kded", "/modules/kpasswdserver", "org.kde.KPasswdServer" );
 
