@@ -91,7 +91,8 @@ public:
 
     /**
      * Call to signal an error.
-     * This also finishes the job, no need to call finished.
+     * This also finishes the job, so you must not call
+     * finished() after calling this.
      *
      * If the Error code is KIO::ERR_SLAVE_DEFINED then the
      * _text should contain the complete translated text of
@@ -115,7 +116,8 @@ public:
 
     /**
      * Call to signal successful completion of any command
-     * (besides openConnection and closeConnection)
+     * besides openConnection and closeConnection. Do not
+     * call this after calling error().
      */
     void finished();
 
@@ -439,6 +441,12 @@ public:
      * Rename @p oldname into @p newname.
      * If the slave returns an error ERR_UNSUPPORTED_ACTION, the job will
      * ask for copy + del instead.
+     *
+     * Important: the slave must implement the logic "if the destination already
+     * exists, error ERR_DIR_ALREADY_EXIST or ERR_FILE_ALREADY_EXIST".
+     * For performance reasons no stat is done in the destination before hand,
+     * the slave must do it.
+     *
      * @param src where to move the file from
      * @param dest where to move the file to
      * @param flags: We support Overwrite here
@@ -591,7 +599,7 @@ public:
     virtual void dispatchOpenCommand( int command, const QByteArray &data );
 
     /**
-     * Read data send by the job, after a dataReq
+     * Read data sent by the job, after a dataReq
      *
      * @param buffer buffer where data is stored
      * @return 0 on end of data,
@@ -750,9 +758,25 @@ public:
 
     /**
      * Internal function to transmit meta data to the application.
+     * m_outgoingMetaData will be cleared; this means that if the slave is for
+     * example put on hold and picked up by a different KIO::Job later the new
+     * job will not see the metadata sent before.
+     * See kio/DESIGN.krun for an overview of the state
+     * progression of a job/slave.
+     * @warning calling this method may seriously interfere with the operation
+     * of KIO which relies on the presence of some metadata at some points in time.
+     * You should not use it if you are not familiar with KIO and not before
+     * the slave is connected to the last job before returning to idle state.
      */
     void sendMetaData();
 
+    /**
+     * Internal function to transmit meta data to the application.
+     * Like sendMetaData() but m_outgoingMetaData will not be cleared.
+     * This method is mainly useful in code that runs before the slave is connected
+     * to its final job.
+     */
+    void sendAndKeepMetaData();
 
     /** If your ioslave was killed by a signal, wasKilled() returns true.
      Check it regularly in lengthy functions (e.g. in get();) and return
