@@ -83,6 +83,8 @@ static QString realSystemPath()
 }
 #endif
 
+Q_DECLARE_METATYPE(KIO::Job*)
+
 void JobTest::initTestCase()
 {
     s_referenceTimeStamp = QDateTime::currentDateTime().addSecs( -30 ); // 30 seconds ago
@@ -109,6 +111,10 @@ void JobTest::initTestCase()
         }
     }
 #endif
+
+    qRegisterMetaType<KIO::Job*>("KIO::Job*");
+    qRegisterMetaType<KUrl>("KUrl");
+    qRegisterMetaType<time_t>("time_t");
 }
 
 static void delDir(const QString& pathOrUrl) {
@@ -242,6 +248,7 @@ void JobTest::copyLocalFile( const QString& src, const QString& dest )
     // cleanup and retry with KIO::copy()
     QFile::remove( dest );
     job = KIO::copy(u, d, KIO::HideProgressInfo );
+    QSignalSpy spyCopyingDone(job, SIGNAL(copyingDone(KIO::Job*,const KUrl&,const KUrl&,time_t,bool,bool)));
     job->setUiDelegate(0);
     ok = KIO::NetAccess::synchronousRun(job, 0);
     QVERIFY( ok );
@@ -259,6 +266,7 @@ void JobTest::copyLocalFile( const QString& src, const QString& dest )
         QCOMPARE( srcInfo.lastModified(), destInfo.lastModified() );
 #endif
     }
+    QCOMPARE(spyCopyingDone.count(), 1);
 }
 
 void JobTest::copyLocalDirectory( const QString& src, const QString& _dest, int flags )
@@ -1061,4 +1069,102 @@ void JobTest::slotMimetype(KIO::Job* job, const QString& type)
     m_mimetype = type;
 }
 
+void JobTest::deleteFile()
+{
+    const QString dest = otherTmpDir() + "fileFromHome_copied";
+    QVERIFY(QFile::exists(dest));
+    KIO::Job* job = KIO::del(KUrl(dest), KIO::HideProgressInfo);
+    job->setUiDelegate(0);
+    bool ok = KIO::NetAccess::synchronousRun(job, 0);
+    QVERIFY(ok);
+    QVERIFY(!QFile::exists(dest));
+}
+
+void JobTest::deleteDirectory()
+{
+    const QString dest = otherTmpDir() + "dirFromHome_copied";
+    QVERIFY(QFile::exists(dest));
+    KIO::Job* job = KIO::del(KUrl(dest), KIO::HideProgressInfo);
+    job->setUiDelegate(0);
+    bool ok = KIO::NetAccess::synchronousRun(job, 0);
+    QVERIFY(ok);
+    QVERIFY(!QFile::exists(dest));
+}
+
+void JobTest::deleteManyFiles()
+{
+    const int numFiles = 100;
+    const QString baseDir = homeTmpDir();
+    for (int i = 0; i < numFiles; ++i) {
+        // create empty file
+        QFile f(baseDir + QString::number(i));
+        QVERIFY(f.open(QIODevice::WriteOnly));
+    }
+    for (int i = 0; i < numFiles; ++i) {
+        // delete each file independently. lots of jobs. this stress-tests kio scheduling.
+        const QString file = baseDir + QString::number(i);
+        QVERIFY(QFile::exists(file));
+        //kDebug() << file;
+        KIO::Job* job = KIO::del(KUrl(file), KIO::HideProgressInfo);
+        job->setUiDelegate(0);
+        bool ok = KIO::NetAccess::synchronousRun(job, 0);
+        QVERIFY(ok);
+        QVERIFY(!QFile::exists(file));
+    }
+}
+
+void JobTest::stat()
+{
+#if 1
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile( filePath );
+    KIO::StatJob* job = KIO::stat(filePath, KIO::HideProgressInfo);
+    QVERIFY(job);
+    bool ok = KIO::NetAccess::synchronousRun(job, 0);
+    QVERIFY(ok);
+    // TODO set setSide, setDetails
+    const KIO::UDSEntry& entry = job->statResult();
+    QVERIFY(!entry.isDir());
+    QVERIFY(!entry.isLink());
+    QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QString("fileFromHome"));
+#else
+    // Testing stat over HTTP
+    KIO::StatJob* job = KIO::stat(KUrl("http://www.kde.org"), KIO::HideProgressInfo);
+    QVERIFY(job);
+    bool ok = KIO::NetAccess::synchronousRun(job, 0);
+    QVERIFY(ok);
+    // TODO set setSide, setDetails
+    const KIO::UDSEntry& entry = job->statResult();
+    QVERIFY(!entry.isDir());
+    QVERIFY(!entry.isLink());
+    QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QString());
+#endif
+}
+
+void JobTest::mimeType()
+{
+#if 1
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile( filePath );
+    KIO::MimetypeJob* job = KIO::mimetype(filePath, KIO::HideProgressInfo);
+    QVERIFY(job);
+    QSignalSpy spyMimeType(job, SIGNAL(mimetype(KIO::Job*, QString)));
+    bool ok = KIO::NetAccess::synchronousRun(job, 0);
+    QVERIFY(ok);
+    QCOMPARE(spyMimeType.count(), 1);
+    QCOMPARE(spyMimeType[0][0], QVariant::fromValue(static_cast<KIO::Job*>(job)));
+    QCOMPARE(spyMimeType[0][1].toString(), QString("application/octet-stream"));
+#else
+    // Testing mimetype over HTTP
+    KIO::MimetypeJob* job = KIO::mimetype(KUrl("http://www.kde.org"), KIO::HideProgressInfo);
+    QVERIFY(job);
+    QSignalSpy spyMimeType(job, SIGNAL(mimetype(KIO::Job*, QString)));
+    bool ok = KIO::NetAccess::synchronousRun(job, 0);
+    QVERIFY(ok);
+    QCOMPARE(spyMimeType.count(), 1);
+    QCOMPARE(spyMimeType[0][0], QVariant::fromValue(static_cast<KIO::Job*>(job)));
+    QCOMPARE(spyMimeType[0][1].toString(), QString("text/html"));
+#endif
+}
+
 #include "jobtest.moc"
