@@ -31,6 +31,8 @@ QTEST_KDEMAIN( KDirListerTest, NoGUI )
 
 void KDirListerTest::initTestCase()
 {
+    m_exitCount = 1;
+
     s_referenceTimeStamp = QDateTime::currentDateTime().addSecs( -120 ); // 2 minutes ago
 
     // Create test data:
@@ -77,6 +79,7 @@ void KDirListerTest::testOpenUrl()
     QCOMPARE(spyClear.count(), 1);
     QCOMPARE(spyClearKUrl.count(), 0);
     QCOMPARE(m_items.count(), 0);
+    QVERIFY(!m_dirLister.isFinished());
 
     // then wait for completed
     qDebug("waiting for completed");
@@ -90,38 +93,20 @@ void KDirListerTest::testOpenUrl()
     QCOMPARE(spyClear.count(), 1);
     QCOMPARE(spyClearKUrl.count(), 0);
     QCOMPARE(m_items.count(), 4);
+    QVERIFY(m_dirLister.isFinished());
     disconnect(&m_dirLister, 0, this, 0);
 }
 
 // This test assumes testOpenUrl was run before. So m_dirLister is holding the items already.
 void KDirListerTest::testOpenUrlFromCache()
 {
-    m_items.clear();
-    const QString path = m_tempDir.name();
-    QSignalSpy spyStarted(&m_dirLister, SIGNAL(started(KUrl)));
-    QSignalSpy spyClear(&m_dirLister, SIGNAL(clear()));
-    QSignalSpy spyClearKUrl(&m_dirLister, SIGNAL(clear(KUrl)));
-    QSignalSpy spyCompleted(&m_dirLister, SIGNAL(completed()));
-    QSignalSpy spyCompletedKUrl(&m_dirLister, SIGNAL(completed(KUrl)));
-    QSignalSpy spyCanceled(&m_dirLister, SIGNAL(canceled()));
-    QSignalSpy spyCanceledKUrl(&m_dirLister, SIGNAL(canceled(KUrl)));
-    connect(&m_dirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
-    // The call to openUrl itself, emits started, the items, and completed, since it's all in the cache
-    // ### Maybe we should get rid of this behavior difference...
-    m_dirLister.openUrl(KUrl(path), KDirLister::NoFlags); // TODO turn those bools into an enum
-
-    QCOMPARE(spyStarted.count(), 1);
-    QCOMPARE(spyCompleted.count(), 1);
-    QCOMPARE(spyCompletedKUrl.count(), 1);
-    QCOMPARE(spyCanceled.count(), 0);
-    QCOMPARE(spyCanceledKUrl.count(), 0);
-    QCOMPARE(spyClear.count(), 1);
-    QCOMPARE(spyClearKUrl.count(), 0);
-    QCOMPARE(m_items.count(), 4);
+    // Do the same again, it should behave the same, even with the items in the cache
+    testOpenUrl();
 
     // Get into the case where another dirlister is holding the items
     {
         m_items.clear();
+        const QString path = m_tempDir.name();
         KDirLister secondDirLister;
         QSignalSpy spyStarted(&secondDirLister, SIGNAL(started(KUrl)));
         QSignalSpy spyClear(&secondDirLister, SIGNAL(clear()));
@@ -131,7 +116,21 @@ void KDirListerTest::testOpenUrlFromCache()
         QSignalSpy spyCanceled(&secondDirLister, SIGNAL(canceled()));
         QSignalSpy spyCanceledKUrl(&secondDirLister, SIGNAL(canceled(KUrl)));
         connect(&secondDirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
-        secondDirLister.openUrl(KUrl(path), KDirLister::NoFlags); // TODO turn those bools into an enum
+        secondDirLister.openUrl(KUrl(path), KDirLister::NoFlags);
+        QCOMPARE(spyStarted.count(), 1);
+        QCOMPARE(spyCompleted.count(), 0);
+        QCOMPARE(spyCompletedKUrl.count(), 0);
+        QCOMPARE(spyCanceled.count(), 0);
+        QCOMPARE(spyCanceledKUrl.count(), 0);
+        QCOMPARE(spyClear.count(), 1);
+        QCOMPARE(spyClearKUrl.count(), 0);
+        QCOMPARE(m_items.count(), 0);
+        QVERIFY(!secondDirLister.isFinished());
+
+        // then wait for completed
+        qDebug("waiting for completed");
+        connect(&secondDirLister, SIGNAL(completed()), this, SLOT(exitLoop()));
+        enterLoop();
         QCOMPARE(spyStarted.count(), 1);
         QCOMPARE(spyCompleted.count(), 1);
         QCOMPARE(spyCompletedKUrl.count(), 1);
@@ -140,6 +139,7 @@ void KDirListerTest::testOpenUrlFromCache()
         QCOMPARE(spyClear.count(), 1);
         QCOMPARE(spyClearKUrl.count(), 0);
         QCOMPARE(m_items.count(), 4);
+        QVERIFY(secondDirLister.isFinished());
     }
 
     disconnect(&m_dirLister, 0, this, 0);
@@ -164,7 +164,7 @@ void KDirListerTest::testNewItems()
     kDebug() << "Creating new file";
     QFile file(path+"toplevelfile_new");
     QVERIFY(file.open(QIODevice::WriteOnly));
-    file.write("foo");
+    file.write(QByteArray("foo"));
     file.close();
 
     int numTries = 0;
@@ -208,7 +208,7 @@ void KDirListerTest::testRefreshItems()
 
     QFile file(fileName);
     QVERIFY(file.open(QIODevice::Append));
-    file.write("<html>");
+    file.write(QByteArray("<html>"));
     file.close();
     QCOMPARE(QFileInfo(fileName).size(), 11LL /*Hello world*/ + 6 /*<html>*/);
 
@@ -267,6 +267,10 @@ void KDirListerTest::testDeleteItem()
     m_items.clear();
     connect(&m_dirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
     m_dirLister.openUrl(KUrl(path), KDirLister::NoFlags);
+    QVERIFY(!m_dirLister.isFinished());
+    connect(&m_dirLister, SIGNAL(completed()), this, SLOT(exitLoop()));
+    enterLoop();
+    QVERIFY(m_dirLister.isFinished());
     QCOMPARE(m_items.count(), 4);
 
     disconnect(&m_dirLister, 0, this, 0);
@@ -309,6 +313,92 @@ void KDirListerTest::testRenameItem()
     m_refreshedItems.clear();
 }
 
+void KDirListerTest::testConcurrentListing()
+{
+    m_items.clear();
+    m_items2.clear();
+
+    KDirLister dirLister2;
+
+    const QString path = m_tempDir.name();
+
+    // spy for m_dirLister signals
+    QSignalSpy spyStarted1(&m_dirLister, SIGNAL(started(KUrl)));
+    QSignalSpy spyClear1(&m_dirLister, SIGNAL(clear()));
+    QSignalSpy spyClearKUrl1(&m_dirLister, SIGNAL(clear(KUrl)));
+    QSignalSpy spyCompleted1(&m_dirLister, SIGNAL(completed()));
+    QSignalSpy spyCompletedKUrl1(&m_dirLister, SIGNAL(completed(KUrl)));
+    QSignalSpy spyCanceled1(&m_dirLister, SIGNAL(canceled()));
+    QSignalSpy spyCanceledKUrl1(&m_dirLister, SIGNAL(canceled(KUrl)));
+    connect(&m_dirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
+
+    // spy for dirLister2 signals
+    QSignalSpy spyStarted2(&dirLister2, SIGNAL(started(KUrl)));
+    QSignalSpy spyClear2(&dirLister2, SIGNAL(clear()));
+    QSignalSpy spyClearKUrl2(&dirLister2, SIGNAL(clear(KUrl)));
+    QSignalSpy spyCompleted2(&dirLister2, SIGNAL(completed()));
+    QSignalSpy spyCompletedKUrl2(&dirLister2, SIGNAL(completed(KUrl)));
+    QSignalSpy spyCanceled2(&dirLister2, SIGNAL(canceled()));
+    QSignalSpy spyCanceledKUrl2(&dirLister2, SIGNAL(canceled(KUrl)));
+    connect(&dirLister2, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems2(KFileItemList)));
+
+    // Before dirLister2 has time to emit the items, let's make m_dirLister move to another dir.
+    // This reproduces the use case "clicking on a folder in dolphin iconview, and dirlister2
+    // is the one used by the "folder panel". m_dirLister is going to list the subdir,
+    // while dirLister2 wants to list the folder that m_dirLister has just left.
+    dirLister2.stop(); // like dolphin does, noop.
+    dirLister2.openUrl(KUrl(path), KDirLister::NoFlags);
+    m_dirLister.openUrl(KUrl(path+"subdir"), KDirLister::NoFlags);
+
+    QCOMPARE(spyStarted1.count(), 1);
+    QCOMPARE(spyCompleted1.count(), 0);
+    QCOMPARE(spyCompletedKUrl1.count(), 0);
+    QCOMPARE(spyCanceled1.count(), 0);
+    QCOMPARE(spyCanceledKUrl1.count(), 0);
+    QCOMPARE(spyClear1.count(), 1);
+    QCOMPARE(spyClearKUrl1.count(), 0);
+    QCOMPARE(m_items.count(), 0);
+
+    QCOMPARE(spyStarted2.count(), 1);
+    QCOMPARE(spyCompleted2.count(), 0);
+    QCOMPARE(spyCompletedKUrl2.count(), 0);
+    QCOMPARE(spyCanceled2.count(), 0);
+    QCOMPARE(spyCanceledKUrl2.count(), 0);
+    QCOMPARE(spyClear2.count(), 1);
+    QCOMPARE(spyClearKUrl2.count(), 0);
+    QCOMPARE(m_items2.count(), 0);
+    QVERIFY(!m_dirLister.isFinished());
+    QVERIFY(!dirLister2.isFinished());
+
+    // then wait for completed
+    qDebug("waiting for completed");
+    connect(&m_dirLister, SIGNAL(completed()), this, SLOT(exitLoop()));
+    connect(&dirLister2, SIGNAL(completed()), this, SLOT(exitLoop()));
+    enterLoop(2);
+
+    QCOMPARE(spyStarted1.count(), 1);
+    QCOMPARE(spyCompleted1.count(), 1);
+    QCOMPARE(spyCompletedKUrl1.count(), 1);
+    QCOMPARE(spyCanceled1.count(), 0);
+    QCOMPARE(spyCanceledKUrl1.count(), 0);
+    QCOMPARE(spyClear1.count(), 1);
+    QCOMPARE(spyClearKUrl1.count(), 0);
+    QCOMPARE(m_items.count(), 3);
+
+    QCOMPARE(spyStarted2.count(), 1);
+    QCOMPARE(spyCompleted2.count(), 1);
+    QCOMPARE(spyCompletedKUrl2.count(), 1);
+    QCOMPARE(spyCanceled2.count(), 0);
+    QCOMPARE(spyCanceledKUrl2.count(), 0);
+    QCOMPARE(spyClear2.count(), 1);
+    QCOMPARE(spyClearKUrl2.count(), 0);
+    QCOMPARE(m_items2.count(), 4);
+    QVERIFY(m_dirLister.isFinished());
+
+    disconnect(&m_dirLister, 0, this, 0);
+    disconnect(&dirLister2, 0, this, 0);
+}
+
 void KDirListerTest::testOpenAndStop()
 {
     m_items.clear();
@@ -332,19 +422,24 @@ void KDirListerTest::testOpenAndStop()
     QCOMPARE(spyClear.count(), 1);
     QCOMPARE(spyClearKUrl.count(), 0);
     QCOMPARE(m_items.count(), 0); // we had time to stop before the job even started
+    QVERIFY(m_dirLister.isFinished());
     disconnect(&m_dirLister, 0, this, 0);
 }
 
-void KDirListerTest::enterLoop()
+void KDirListerTest::enterLoop(int exitCount)
 {
     qDebug("enterLoop");
+    m_exitCount = exitCount;
     m_eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
 }
 
 void KDirListerTest::exitLoop()
 {
     qDebug("exitLoop");
-    m_eventLoop.quit();
+    --m_exitCount;
+    if (m_exitCount <= 0) {
+        m_eventLoop.quit();
+    }
 }
 
 void KDirListerTest::slotNewItems(const KFileItemList& lst)
@@ -352,6 +447,11 @@ void KDirListerTest::slotNewItems(const KFileItemList& lst)
     m_items += lst;
 }
 
+void KDirListerTest::slotNewItems2(const KFileItemList& lst)
+{
+    m_items2 += lst;
+}
+
 void KDirListerTest::slotRefreshItems(const QList<QPair<KFileItem, KFileItem> > & lst)
 {
     m_refreshedItems += lst;
