@@ -36,26 +36,29 @@ private Q_SLOTS:
     void testRefreshItems();
     void testDeleteItem();
     void testRenameItem();
+    void testConcurrentListing();
     void testOpenAndStop();
     // TODO test filtering
-    // TODO test concurrent listings from two KDirListers
 
 protected Q_SLOTS: // 'more private than private slots' - i.e. not seen by qtestlib
     void exitLoop();
     void slotNewItems(const KFileItemList&);
+    void slotNewItems2(const KFileItemList&);
     void slotRefreshItems(const QList<QPair<KFileItem, KFileItem> >&);
 
 Q_SIGNALS:
     void refreshItemsReceived();
 
 private:
-    void enterLoop();
+    void enterLoop(int exitCount = 1);
 
 private:
+    int m_exitCount;
     QEventLoop m_eventLoop;
     KTempDir m_tempDir;
     KDirLister m_dirLister;
     KFileItemList m_items;
+    KFileItemList m_items2;
     QList<QPair<KFileItem, KFileItem> > m_refreshedItems;
 };
 
