@@ -36,38 +36,62 @@
 
 QTEST_KDEMAIN( KDirModelTest, NoGUI )
 
+#ifndef Q_WS_WIN
+#define SPECIALCHARS "specialchars%:"
+#else
+#define SPECIALCHARS "specialchars%"
+#endif
+
 void KDirModelTest::initTestCase()
 {
     s_referenceTimeStamp = QDateTime::currentDateTime().addSecs( -30 ); // 30 seconds ago
+    m_tempDir = 0;
+    m_topLevelFileNames << "toplevelfile_1"
+                        << "toplevelfile_2"
+                        << "toplevelfile_3"
+                        << SPECIALCHARS
+                        ;
+    recreateTestData();
 
+    fillModel( false );
+}
+
+void KDirModelTest::recreateTestData()
+{
+    delete m_tempDir;
+    m_tempDir = new KTempDir;
     // Create test data:
     /*
      * PATH/toplevelfile_1
      * PATH/toplevelfile_2
      * PATH/toplevelfile_3
-     * PATH/file%2fslash
+     * PATH/specialchars%:
      * PATH/subdir
      * PATH/subdir/testfile
      * PATH/subdir/subsubdir
      * PATH/subdir/subsubdir/testfile
      */
-    const QString path = m_tempDir.name();
-    m_topLevelFileNames << "toplevelfile_1"
-                        << "toplevelfile_2"
-                        << "toplevelfile_3"
-                        << "file%2fslash";
-    foreach(QString f, m_topLevelFileNames) {
+    const QString path = m_tempDir->name();
+    foreach(const QString &f, m_topLevelFileNames) {
         createTestFile(path+f);
     }
     createTestDirectory(path+"subdir");
     createTestDirectory(path+"subdir/subsubdir", NoSymlink);
 
-    fillModel( false );
+    m_dirIndex = QModelIndex();
+    m_fileIndex = QModelIndex();
+    m_secondFileIndex = QModelIndex();
+}
+
+void KDirModelTest::cleanupTestCase()
+{
+    delete m_tempDir;
+    m_tempDir = 0;
 }
 
 void KDirModelTest::fillModel( bool reload )
 {
-    const QString path = m_tempDir.name();
+    const QString path = m_tempDir->name();
     KDirLister* dirLister = m_dirModel.dirLister();
     dirLister->openUrl(KUrl(path), reload ? KDirLister::Reload : KDirLister::NoFlags);
     connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
@@ -87,13 +111,13 @@ void KDirModelTest::fillModel( bool reload )
             m_fileIndex = idx;
         else if (item.url().fileName() == "toplevelfile_2")
             m_secondFileIndex = idx;
-        else if (item.url().fileName().endsWith("slash"))
-            m_slashFileIndex = idx;
+        else if (item.url().fileName().startsWith("special"))
+            m_specialFileIndex = idx;
     }
     QVERIFY(m_dirIndex.isValid());
     QVERIFY(m_fileIndex.isValid());
     QVERIFY(m_secondFileIndex.isValid());
-    QVERIFY(m_slashFileIndex.isValid());
+    QVERIFY(m_specialFileIndex.isValid());
 
     // Now list subdir/
     QVERIFY(m_dirModel.canFetchMore(m_dirIndex));
@@ -118,6 +142,7 @@ void KDirModelTest::fillModel( bool reload )
 
     // Index of ... well, subdir/subsubdir/testfile
     m_fileInSubdirIndex = m_dirModel.index(0, 0, subdirIndex);
+    disconnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
 }
 
 void KDirModelTest::enterLoop()
@@ -179,8 +204,8 @@ void KDirModelTest::testNames()
     QString fileName = m_dirModel.data(m_fileIndex, Qt::DisplayRole).toString();
     QCOMPARE(fileName, QString("toplevelfile_1"));
 
-    QString slashFileName = m_dirModel.data(m_slashFileIndex, Qt::DisplayRole).toString();
-    QCOMPARE(slashFileName, QString("file/slash"));
+    QString specialFileName = m_dirModel.data(m_specialFileIndex, Qt::DisplayRole).toString();
+    QCOMPARE(specialFileName, QString(SPECIALCHARS));
 
     QString dirName = m_dirModel.data(m_dirIndex, Qt::DisplayRole).toString();
     QCOMPARE(dirName, QString("subdir"));
@@ -203,25 +228,25 @@ void KDirModelTest::testItemForIndex()
     QVERIFY(!fileItem.isNull());
     QCOMPARE(fileItem.name(), QString("toplevelfile_1"));
     QVERIFY(!fileItem.isDir());
-    QCOMPARE(fileItem.url().path(), m_tempDir.name() + "toplevelfile_1");
+    QCOMPARE(fileItem.url().path(), m_tempDir->name() + "toplevelfile_1");
 
     KFileItem dirItem = m_dirModel.itemForIndex(m_dirIndex);
     QVERIFY(!dirItem.isNull());
     QCOMPARE(dirItem.name(), QString("subdir"));
     QVERIFY(dirItem.isDir());
-    QCOMPARE(dirItem.url().path(), m_tempDir.name() + "subdir");
+    QCOMPARE(dirItem.url().path(), m_tempDir->name() + "subdir");
 
     KFileItem fileInDirItem = m_dirModel.itemForIndex(m_fileInDirIndex);
     QVERIFY(!fileInDirItem.isNull());
     QCOMPARE(fileInDirItem.name(), QString("testfile"));
     QVERIFY(!fileInDirItem.isDir());
-    QCOMPARE(fileInDirItem.url().path(), m_tempDir.name() + "subdir/testfile");
+    QCOMPARE(fileInDirItem.url().path(), m_tempDir->name() + "subdir/testfile");
 
     KFileItem fileInSubdirItem = m_dirModel.itemForIndex(m_fileInSubdirIndex);
     QVERIFY(!fileInSubdirItem.isNull());
     QCOMPARE(fileInSubdirItem.name(), QString("testfile"));
     QVERIFY(!fileInSubdirItem.isDir());
-    QCOMPARE(fileInSubdirItem.url().path(), m_tempDir.name() + "subdir/subsubdir/testfile");
+    QCOMPARE(fileInSubdirItem.url().path(), m_tempDir->name() + "subdir/subsubdir/testfile");
 }
 
 void KDirModelTest::testIndexForItem()
@@ -298,7 +323,7 @@ Q_DECLARE_METATYPE(QModelIndex) // needed for .value<QModelIndex>()
 
 void KDirModelTest::testModifyFile()
 {
-    const QString file = m_tempDir.name() + "toplevelfile_2";
+    const QString file = m_tempDir->name() + "toplevelfile_2";
     const KUrl url(file);
 
     qRegisterMetaType<QModelIndex>("QModelIndex"); // beats me why Qt doesn't do that
@@ -311,8 +336,8 @@ void KDirModelTest::testModifyFile()
 
     // In stat mode, kdirwatch doesn't notice file changes; we need to trigger it
     // by creating a file.
-    //createTestFile(m_tempDir.name() + "toplevelfile_5");
-    KDirWatch::self()->setDirty(m_tempDir.name());
+    //createTestFile(m_tempDir->name() + "toplevelfile_5");
+    KDirWatch::self()->setDirty(m_tempDir->name());
 
     // Wait for KDirWatch to notify the change (especially when using Stat)
     enterLoop();
@@ -330,9 +355,9 @@ void KDirModelTest::testModifyFile()
 
 void KDirModelTest::testRenameFile()
 {
-    const QString file = m_tempDir.name() + "toplevelfile_2";
+    const QString file = m_tempDir->name() + "toplevelfile_2";
     const KUrl url(file);
-    const QString newFile = m_tempDir.name() + "toplevelfile_2_renamed";
+    const QString newFile = m_tempDir->name() + "toplevelfile_2_renamed";
     const KUrl newUrl(newFile);
 
     qRegisterMetaType<QModelIndex>("QModelIndex"); // beats me why Qt doesn't do that
@@ -362,56 +387,97 @@ void KDirModelTest::testRenameFile()
 
 void KDirModelTest::testExpandToUrl_data()
 {
-    QTest::addColumn<QString>("expandToPath");
+    QTest::addColumn<bool>("newdir"); // whether to re-create a new KTempDir completely, to avoid cached fileitems
+    QTest::addColumn<QString>("expandToPath"); // relative path
     QTest::addColumn<QStringList>("expectedExpandSignals");
 
-    const QString path = m_tempDir.name();
     QTest::newRow("the root, nothing to do")
-        << path << QStringList();
-    QTest::newRow("already known child, nothing to do")
-        << path+"subdir" << QStringList();
-    const QString subsubdir = path+"subdir/subsubdir";
-    QStringList sigs; sigs << subsubdir;
-    // must list subdir and then expand is emitted
+        << false << QString() << QStringList();
+    // When KDirLister was sync, subdir would already be known by the time we call expand
+    // But now that listing is always async, we get a reliable result: expand(subdir) emitted.
+    QTest::newRow("subdir")
+        << false << "subdir" << (QStringList()<<"subdir");
+
+    const QString subsubdir = "subdir/subsubdir";
+    // Must list root, emit expand for subdir, list subdir, emit expand for subsubdir.
     QTest::newRow("subdir/subsubdir")
-        << subsubdir << sigs;
-    // must list subdir, emit expand for subsubdir, and then list subsubdir
-    QTest::newRow("subdir/subsubdir/testfile")
-        << subsubdir + "/testfile" << sigs;
-    // TODO: we need an async test too (to emit expand twice)
+        << false << subsubdir << (QStringList()<<"subdir"<<subsubdir);
+
+    // Must list root, emit expand for subdir, list subdir, emit expand for subsubdir, list subsubdir.
+    QTest::newRow("subdir/subsubdir/testfile sync")
+        << false << subsubdir + "/testfile" << (QStringList()<<"subdir"<<subsubdir);
+
+    // Do a cold-cache test too, but nowadays it doesn't change anything anymore,
+    // apart from testing different code paths inside KDirLister.
+    QTest::newRow("subdir/subsubdir/testfile with reload")
+        << true << subsubdir + "/testfile" << (QStringList()<<"subdir"<<subsubdir);
 }
 
 void KDirModelTest::testExpandToUrl()
 {
-    const QString path = m_tempDir.name();
-    QFETCH(QString, expandToPath);
+    QFETCH(bool, newdir);
+    QFETCH(QString, expandToPath); // relative
     QFETCH(QStringList, expectedExpandSignals);
 
+    if (newdir) {
+        recreateTestData();
+        // WARNING! m_dirIndex, m_fileIndex, m_secondFileIndex etc. are not valid anymore after this point!
+
+    }
+
+    const QString path = m_tempDir->name();
     KDirModel dirModelForExpand;
     KDirLister* dirListerForExpand = dirModelForExpand.dirLister();
-    dirListerForExpand->openUrl(KUrl(path), KDirLister::NoFlags); // it gets them from the cache, so this is sync
+    dirListerForExpand->openUrl(KUrl(path), KDirLister::NoFlags); // async
     connect(&dirModelForExpand, SIGNAL(expand(QModelIndex)),
             this, SLOT(slotExpand(QModelIndex)));
     connect(&dirModelForExpand, SIGNAL(rowsInserted(QModelIndex,int,int)),
             this, SLOT(slotRowsInserted(QModelIndex,int,int)));
     m_rowsInsertedEmitted = false;
-    m_nextExpectedExpandPath = expectedExpandSignals.isEmpty() ? QString() : expectedExpandSignals.first();
+    m_expectedExpandSignals = expectedExpandSignals;
+    m_dirModelForExpand = &dirModelForExpand;
+    m_nextExpectedExpandSignals = 0;
     QSignalSpy spyExpand(&dirModelForExpand, SIGNAL(expand(QModelIndex)));
-    dirModelForExpand.expandToUrl(KUrl(expandToPath));
+    dirModelForExpand.expandToUrl(KUrl(path + expandToPath));
+    QCOMPARE(spyExpand.count(), 0); // nothing emitted yet
     if (expectedExpandSignals.isEmpty()) {
+        QTest::qWait(20); // to make sure we process queued connection calls, otherwise spyExpand.count() is always 0 even if there's a bug...
         QCOMPARE(spyExpand.count(), 0);
     } else {
         enterLoop();
-        QCOMPARE(spyExpand.count(), 1);
+        QCOMPARE(spyExpand.count(), expectedExpandSignals.count());
         QVERIFY(m_rowsInsertedEmitted);
     }
+    m_dirModelForExpand = 0;
+}
+
+void KDirModelTest::slotExpand(const QModelIndex& index)
+{
+    QVERIFY(index.isValid());
+    const QString path = m_tempDir->name();
+    KFileItem item = m_dirModelForExpand->itemForIndex(index);
+    QVERIFY(!item.isNull());
+    QCOMPARE(item.url().path(), path + m_expectedExpandSignals[m_nextExpectedExpandSignals++]);
+    // if rowsInserted wasn't emitted yet, then any proxy model would be unable to do anything with index at this point
+    QVERIFY(m_rowsInsertedEmitted);
+    if (m_nextExpectedExpandSignals == m_expectedExpandSignals.count())
+        m_eventLoop.quit(); // done
+}
+
+void KDirModelTest::slotRowsInserted(const QModelIndex&, int, int)
+{
+    m_rowsInsertedEmitted = true;
 }
 
 // Must be done last because it changes the other indexes
 void KDirModelTest::testDeleteFile()
 {
+    // if the previous test called recreateTestData, we need to refill the model
+    if (!m_fileIndex.isValid())
+        fillModel(false);
+
     const int oldTopLevelRowCount = m_dirModel.rowCount();
-    const QString file = m_tempDir.name() + "toplevelfile_1";
+    const QString file = m_tempDir->name() + "toplevelfile_1";
     const KUrl url(file);
 
     qRegisterMetaType<QModelIndex>("QModelIndex"); // beats me why Qt doesn't do that
@@ -431,18 +497,3 @@ void KDirModelTest::testDeleteFile()
     disconnect( &m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
                 &m_eventLoop, SLOT(quit()) );
 }
-
-void KDirModelTest::slotExpand(const QModelIndex& index)
-{
-    QVERIFY(index.isValid());
-    KFileItem item = m_dirModel.itemForIndex(index);
-    QVERIFY(!item.isNull());
-    QCOMPARE(m_nextExpectedExpandPath, item.url().path());
-    // if rowsInserted wasn't emitted yet, then any proxy model would be unable to do anything with index at this point
-    QVERIFY(m_rowsInsertedEmitted);
-}
-
-void KDirModelTest::slotRowsInserted(const QModelIndex&, int, int)
-{
-    m_rowsInsertedEmitted = true;
-}
