@@ -349,48 +349,47 @@ void KioslaveTest::printUDSEntry( const KIO::UDSEntry & entry )
     // It's rather rare to iterate that way, usually you'd use numberValue/stringValue directly.
     // This is just to print out all that we got
 
-    /*
-    KIO::UDSEntry::ConstIterator it = entry.begin();
-    for( ; it != entry.end(); it++ ) {
-        switch ( it.key() ) {
+    const QList<uint> keys = entry.listFields();
+    QList<uint>::const_iterator it = keys.begin();
+    for( ; it != keys.end(); it++ ) {
+        switch ( *it ) {
             case KIO::UDSEntry::UDS_FILE_TYPE:
-                kDebug() << "File Type : " << (mode_t)(it.value().toNumber());
-                if ( S_ISDIR( (mode_t)(it.value().toNumber()) ) )
                 {
-                    kDebug() << "is a dir";
+                    mode_t mode = (mode_t)entry.numberValue(*it);
+                    kDebug() << "File Type : " << mode;
+                    if ( S_ISDIR( mode ) )
+                    {
+                        kDebug() << "is a dir";
+                    }
                 }
                 break;
             case KIO::UDSEntry::UDS_ACCESS:
-                kDebug() << "Access permissions : " << (mode_t)(it.value().toNumber());
+                kDebug() << "Access permissions : " << (mode_t)( entry.numberValue(*it) ) ;
                 break;
             case KIO::UDSEntry::UDS_USER:
-                kDebug() << "User : " << (it.value().toString().toAscii().constData() );
+                kDebug() << "User : " << ( entry.stringValue(*it) );
                 break;
             case KIO::UDSEntry::UDS_GROUP:
-                kDebug() << "Group : " << (it.value().toString().toAscii().constData() );
+                kDebug() << "Group : " << ( entry.stringValue(*it) );
                 break;
             case KIO::UDSEntry::UDS_NAME:
-                kDebug() << "Name : " << (it.value().toString().toAscii().constData() );
+                kDebug() << "Name : " << ( entry.stringValue(*it) );
                 //m_strText = decodeFileName( it.value().toString() );
                 break;
             case KIO::UDSEntry::UDS_URL:
-                kDebug() << "URL : " << (it.value().toString().toAscii().constData() );
+                kDebug() << "URL : " << ( entry.stringValue(*it) );
                 break;
             case KIO::UDSEntry::UDS_MIME_TYPE:
-                kDebug() << "MimeType : " << (it.value().toString().toAscii().constData() );
+                kDebug() << "MimeType : " << ( entry.stringValue(*it) );
                 break;
             case KIO::UDSEntry::UDS_LINK_DEST:
-                kDebug() << "LinkDest : " << (it.value().toString().toAscii().constData() );
+                kDebug() << "LinkDest : " << ( entry.stringValue(*it) );
                 break;
             case KIO::UDSEntry::UDS_SIZE:
-                kDebug() << "Size: " << KIO::convertSize(it.value().toNumber());
+                kDebug() << "Size: " << KIO::convertSize(entry.numberValue(*it));
                 break;
         }
     }
-    */
-    kDebug() << "this test has been turned off temporarily";
-
-    // TODO: turn it on again
 }
 
 void KioslaveTest::slotEntries(KIO::Job* job, const KIO::UDSEntryList& list) {
@@ -470,13 +469,14 @@ int main(int argc, char **argv) {
 
   KCmdLineOptions options;
   options.add("s");
-  options.add("src <src>", ki18n("Source URL"), "");
+  options.add("src <src>", ki18n("Source URL"), QByteArray());
   options.add("d");
-  options.add("dest <dest>", ki18n("Destination URL"), "");
+  options.add("dest <dest>", ki18n("Destination URL"), QByteArray());
   options.add("o");
-  options.add("operation <operation>", ki18n("Operation (list,listrecursive,stat,get,put,copy,move,del,mkdir)"), "copy");
+  options.add("operation <operation>", ki18n("Operation (list,listrecursive,stat,get,put,copy,move,del,mkdir)"), QByteArray("copy"));
   options.add("p");
-  options.add("progress <progress>", ki18n("Progress Type (none,default,status)"), "default");
+  options.add("progress <progress>", ki18n("Progress Type (none,default,status)"),
+          QByteArray("default"));
 
   const char version[] = "v0.0.0 0000";   // :-)
   KLocalizedString description = ki18n("Test for kioslaves");
@@ -514,7 +514,7 @@ int main(int argc, char **argv) {
     op = KioslaveTest::Delete;
   } else if ( tmps == "mkdir") {
     op = KioslaveTest::Mkdir;
-  } else KCmdLineArgs::usage("unknown operation");
+  } else KCmdLineArgs::usage(QByteArray("unknown operation"));
 
   tmps = args->getOption("progress");
   if ( tmps == "none") {
@@ -523,7 +523,7 @@ int main(int argc, char **argv) {
     pr = KioslaveTest::ProgressDefault;
   } else if ( tmps == "status") {
     pr = KioslaveTest::ProgressStatus;
-  } else KCmdLineArgs::usage("unknown progress mode");
+  } else KCmdLineArgs::usage(QByteArray("unknown progress mode"));
 
   args->clear(); // Free up memory
 
