@@ -470,17 +470,19 @@ void FileProtocol::open(const KUrl &url, QIODevice::OpenMode mode)
 
     int flags = 0;
     if (mode & QIODevice::ReadOnly) {
-        flags |= O_RDONLY;
-    }
-    if (mode & QIODevice::WriteOnly) {
-        flags |= O_WRONLY | O_CREAT;
-    }
-    if (mode & QIODevice::Append) {
-        flags |= O_WRONLY | O_APPEND;
-    } else if (mode & QIODevice::WriteOnly) {
-        if (!(mode & QIODevice::ReadOnly) || mode & QIODevice::Truncate) {
-            flags |= O_TRUNC;
+        if (mode & QIODevice::WriteOnly) {
+            flags = O_RDWR | O_CREAT;
+        } else {
+            flags = O_RDONLY;
         }
+    } else if (mode & QIODevice::WriteOnly) {
+        flags = O_WRONLY | O_CREAT;
+    }
+
+    if (mode & QIODevice::Append) {
+        flags |= O_APPEND;
+    } else if (mode & QIODevice::Truncate) {
+        flags |= O_TRUNC;
     }
 
     int fd = KDE_open( openPath.data(), flags);
@@ -490,8 +492,12 @@ void FileProtocol::open(const KUrl &url, QIODevice::OpenMode mode)
     }
     // Determine the mimetype of the file to be retrieved, and emit it.
     // This is mandatory in all slaves (for KRun/BrowserRun to work).
-    KMimeType::Ptr mt = KMimeType::findByUrl( url, buff.st_mode, true /* local URL */ );
-    emit mimeType( mt->name() );
+    // If we're not opening the file ReadOnly or ReadWrite, don't attempt to
+    // read the file and send the mimetype.
+    if (mode & QIODevice::ReadOnly){
+        KMimeType::Ptr mt = KMimeType::findByUrl( url, buff.st_mode, true /* local URL */ );
+        emit mimeType( mt->name() );
+   }
 
     totalSize( buff.st_size );
     position( 0 );
@@ -525,6 +531,7 @@ void FileProtocol::read(KIO::filesize_t bytes)
             }
             break;
         }
+        if (bytes <= 0) break;
     }
 }
 
@@ -764,7 +771,7 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
                           _dest_orig.data(),
                           MOVEFILE_REPLACE_EXISTING|MOVEFILE_COPY_ALLOWED ) == 0 )
 #else
-        if ( ::rename( _dest.data(), _dest_orig.data() ) )
+        if ( KDE_rename( _dest.data(), _dest_orig.data() ) )
 #endif
         {
             kWarning(7101) << " Couldn't rename " << _dest << " to " << _dest_orig;
@@ -922,7 +929,7 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 #ifdef USE_SENDFILE
        if (use_sendfile) {
             off_t sf = processed_size;
-            n = ::sendfile( dest_fd, src_fd, &sf, MAX_IPC_SIZE );
+            n = KDE_sendfile( dest_fd, src_fd, &sf, MAX_IPC_SIZE );
             processed_size = sf;
             if ( n == -1 && ( errno == EINVAL || errno == ENOSYS ) ) { //not all filesystems support sendfile()
                 kDebug(7101) << "sendfile() not supported, falling back ";
@@ -1039,8 +1046,9 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 void FileProtocol::rename( const KUrl &src, const KUrl &dest,
                            KIO::JobFlags _flags )
 {
-    QByteArray _src( QFile::encodeName(src.toLocalFile()));
-    QByteArray _dest( QFile::encodeName(dest.toLocalFile()));
+    char off_t_should_be_64_bits[sizeof(off_t) >= 8 ? 1 : -1]; (void) off_t_should_be_64_bits;
+    QByteArray _src(QFile::encodeName(src.toLocalFile()));
+    QByteArray _dest(QFile::encodeName(dest.toLocalFile()));
     KDE_struct_stat buff_src;
     if ( KDE_lstat( _src.data(), &buff_src ) == -1 ) {
         if ( errno == EACCES )
@@ -1073,7 +1081,7 @@ void FileProtocol::rename( const KUrl &src, const KUrl &dest,
         }
     }
 
-    if ( ::rename( _src.data(), _dest.data()))
+    if ( KDE_rename( _src.data(), _dest.data()))
     {
         if (( errno == EACCES ) || (errno == EPERM)) {
             error( KIO::ERR_ACCESS_DENIED, _dest );
@@ -1550,7 +1558,7 @@ void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, co
     bool fstype_empty = !_fstype || !*_fstype;
     QByteArray fstype = KShell::quoteArg(_fstype).toLatin1(); // good guess
     QByteArray readonly = _ro ? "-r" : "";
-    QString epath = QLatin1String(getenv("PATH"));
+    QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if(!epath.isEmpty())
         path += QLatin1String(":") + epath;
@@ -1660,7 +1668,7 @@ void FileProtocol::unmount( const QString& _point )
 			<< _point.toLocal8Bit();
 
 		if( (mnttab = KDE_fopen( MNTTAB, "r" )) == NULL ) {
-			err = "couldn't open mnttab";
+			err = "could not open mnttab";
 			kDebug(7101) << "VOLMGT: " << err;
 			error( KIO::ERR_COULD_NOT_UNMOUNT, err );
 			return;
@@ -1728,7 +1736,7 @@ void FileProtocol::unmount( const QString& _point )
 		return;
 	}
 #else
-    QString epath = getenv("PATH");
+    QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if (!epath.isEmpty())
        path += ':' + epath;
@@ -1757,7 +1765,7 @@ void FileProtocol::unmount( const QString& _point )
 
 bool FileProtocol::pmount(const QString &dev)
 {
-    QString epath = getenv("PATH");
+    QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if (!epath.isEmpty())
         path += ':' + epath;
@@ -1782,7 +1790,7 @@ bool FileProtocol::pumount(const QString &point)
     QString dev = mp->realDeviceName();
     if (dev.isEmpty()) return false;
 
-    QString epath = getenv("PATH");
+    QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if (!epath.isEmpty())
         path += ':' + epath;
