@@ -24,6 +24,7 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <config.h>
+
 #include "nodes.h"
 #include "function.h"
 #include "scriptfunction.h"
@@ -99,7 +100,7 @@ void SourceStream::flush()
 {
   if (bufUsed)
   {
-    str.append(reinterpret_cast<const UChar*>(buffer), bufUsed);
+    str.append(UString(reinterpret_cast<const UChar*>(buffer), bufUsed));
     bufUsed = 0;
   }
 }
@@ -178,7 +179,7 @@ SourceStream& SourceStream::append(const UTF16* src, int srcLen)
   if (kBufSize - bufUsed < srcLen)
     flush();
   if (kBufSize - bufUsed < srcLen)
-    str.append(reinterpret_cast<const UChar*>(src), srcLen);
+    str.append(UString(reinterpret_cast<const UChar*>(src), srcLen));
   else
   {
     UTF16* dst = &buffer[bufUsed];
@@ -209,7 +210,7 @@ SourceStream& SourceStream::append(const UString& inStr, bool quote)
       case '\t':    c = 't';    break;
       case '\\':                break;
       default:
-        if (c >= 128 || !isprint(c)) {
+        if (c >= 128 || !isprint(c)) { // ### FIXME: use Unicode tables
           char hexValue[8];
           int len = sprintf(hexValue, (c < 256) ? "\\x%02X" : "\\u%04X", c);
           UTF16* dst = &buffer[bufUsed];
@@ -230,7 +231,7 @@ SourceStream& SourceStream::append(const UString& inStr, bool quote)
   return *this;
 }
 
-UString DeclaredFunctionImp::toSource() const
+UString FunctionImp::toSource() const
 {
   SourceStream str;
   str << "function ";
@@ -303,7 +304,7 @@ void ArrayNode::streamTo(SourceStream &s) const
   s << '[' << element;
   for (int i = 0; i < elision; i++)
     s << ',';
-  // Parser consumes one elision comma if there's array elements 
+  // Parser consumes one elision comma if there's array elements
   // present in the expression.
   if (opt && element)
     s << ',';
@@ -348,10 +349,7 @@ void PropertyNode::streamTo(SourceStream &s) const
 
 void PropertyNameNode::streamTo(SourceStream &s) const
 {
-  if (str.isNull())
-    s << UString::from(numeric);
-  else
-    s.append(str.ustring(), kQuote);
+  s.append(str.ustring(), kQuote);
 }
 
 void BracketAccessorNode::streamTo(SourceStream &s) const
@@ -391,40 +389,11 @@ void FunctionCallReferenceNode::streamTo(SourceStream &s) const
   s << expr << args;
 }
 
-void FunctionCallBracketNode::streamTo(SourceStream &s) const
-{
-  s << base << '[' << subscript << ']' << args;
-}
-
-void FunctionCallParenBracketNode::streamTo(SourceStream &s) const
-{
-  s << '(' << base << '[' << subscript << "])" << args;
-}
-
-void FunctionCallDotNode::streamTo(SourceStream &s) const
-{
-  s << base << '.' << ident << args;
-}
-
-void FunctionCallParenDotNode::streamTo(SourceStream &s) const
-{
-  s << '(' << base << '.' << ident << ")" << args;
-}
-
 void PostfixNode::streamTo(SourceStream &s) const
 {
   s << m_loc << m_oper;
 }
 
-void PostfixErrorNode::streamTo(SourceStream& s) const
-{
-  s << m_expr;
-  if (m_oper == OpPlusPlus)
-    s << "++";
-  else
-    s << "--";
-}
-
 void DeleteReferenceNode::streamTo(SourceStream &s) const
 {
   s << "delete " << loc;
@@ -445,7 +414,7 @@ void TypeOfValueNode::streamTo(SourceStream &s) const
   s << "typeof " << m_expr;
 }
 
-void TypeOfReferenceNode::streamTo(SourceStream &s) const
+void TypeOfVarNode::streamTo(SourceStream &s) const
 {
   s << "typeof " << loc;
 }
@@ -455,15 +424,6 @@ void PrefixNode::streamTo(SourceStream &s) const
   s << m_oper << m_loc;
 }
 
-void PrefixErrorNode::streamTo(SourceStream& s) const 
-{ 
-  if (m_oper == OpPlusPlus) 
-    s << "++"; 
-  else 
-    s << "--"; 
-  s << m_expr; 
-}
-
 void UnaryPlusNode::streamTo(SourceStream &s) const
 {
   s << "+ " << expr;
@@ -624,29 +584,6 @@ void AssignNode::streamTo(SourceStream &s) const
 //  s.append(m_ident, opStr, m_right);
 }
 
-void AssignBracketNode::streamTo(SourceStream &s) const
-{
-  s << m_base << '[' << m_subscript << ']';
-  streamAssignmentOperatorTo(s, m_oper);
-  s << m_right;
-}
-
-void AssignDotNode::streamTo(SourceStream &s) const
-{
-  s << m_base << '.' << m_ident;
-  streamAssignmentOperatorTo(s, m_oper);
-  s << m_right;
-}
-
-
-
-void AssignErrorNode::streamTo(SourceStream& s) const 
-{ 
-    s << m_left; 
-    streamAssignmentOperatorTo(s, m_oper); 
-    s << m_right; 
-}
-
 void CommaNode::streamTo(SourceStream &s) const
 {
   s.append(expr1, ", ", expr2);
@@ -674,11 +611,6 @@ void VarStatementNode::streamTo(SourceStream &s) const
   s << SourceStream::Endl << next << ';';
 }
 
-void StaticVarStatementNode::streamTo(SourceStream &s) const
-{
-  s << originalStatement;
-}
-
 void BlockNode::streamTo(SourceStream &s) const
 {
   s << SourceStream::Endl << '{' << SourceStream::Indent
@@ -847,11 +779,6 @@ void SourceElementsNode::streamTo(SourceStream &s) const
     s << n->node;
 }
 
-void ErrorNode::streamTo(SourceStream& s) const
-{
-  s << kid;
-}
-
 void PackageNameNode::streamTo(SourceStream &s) const
 {
     if (names)
