@@ -40,7 +40,7 @@
 // If we improve our stack usage, we can bump this number.
 #define KJS_MAX_STACK 100
 #else
-#define KJS_MAX_STACK 1000
+#define KJS_MAX_STACK 900
 #endif
 
 #define JAVASCRIPT_CALL_TRACING 0
@@ -96,7 +96,8 @@ JSValue *JSObject::call(ExecState *exec, JSObject *thisObj, const List &args)
   }
 #endif
 
-  JSValue *ret = callAsFunction(exec,thisObj,args); 
+  JSValue *ret = callAsFunction(exec,thisObj,args);
+  assert(ret);
 
 #if KJS_MAX_STACK > 0
   --depth;
@@ -126,16 +127,16 @@ void JSObject::mark()
   markStackDepth++;
   for (int i = 0; i < markStackDepth; i++)
     putchar('-');
-  
+
   printf("%s (%p)\n", className().UTF8String().c_str(), this);
 #endif
-  
+
   JSValue *proto = _proto;
   if (!proto->marked())
     proto->mark();
 
   _prop.mark();
-  
+
 #if JAVASCRIPT_MARK_TRACING
   markStackDepth--;
 #endif
@@ -163,7 +164,7 @@ JSValue *JSObject::get(ExecState *exec, const Identifier &propertyName) const
 
   if (const_cast<JSObject *>(this)->getPropertySlot(exec, propertyName, slot))
     return slot.getValue(exec, const_cast<JSObject *>(this), propertyName);
-    
+
   return jsUndefined();
 }
 
@@ -172,25 +173,25 @@ JSValue *JSObject::get(ExecState *exec, unsigned propertyName) const
   PropertySlot slot;
   if (const_cast<JSObject *>(this)->getPropertySlot(exec, propertyName, slot))
     return slot.getValue(exec, const_cast<JSObject *>(this), propertyName);
-    
+
   return jsUndefined();
 }
 
 bool JSObject::getPropertySlot(ExecState *exec, unsigned propertyName, PropertySlot& slot)
 {
   JSObject *imp = this;
-  
+
   while (true) {
     if (imp->getOwnPropertySlot(exec, propertyName, slot))
       return true;
-    
+
     JSValue *proto = imp->_proto;
     if (!proto->isObject())
       break;
-    
+
     imp = static_cast<JSObject *>(proto);
   }
-  
+
   return false;
 }
 
@@ -199,6 +200,29 @@ bool JSObject::getOwnPropertySlot(ExecState *exec, unsigned propertyName, Proper
   return getOwnPropertySlot(exec, Identifier::from(propertyName), slot);
 }
 
+// Ideally, we would like to inline this, since it's ultra-hot, but with the large VM
+// loop, it seems like the code side gets the g++-4.3.x inliner in the paranoid mode, so not only
+// does it not inline this, but it also doesn't inline setValueSlot() and hasGetterSetterProperties() (!!!).
+bool JSObject::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot)
+{
+    if (JSValue **location = getDirectLocation(propertyName)) {
+        if (_prop.hasGetterSetterProperties() && location[0]->type() == GetterSetterType)
+            fillGetterPropertySlot(slot, location);
+        else
+            slot.setValueSlot(this, location);
+        return true;
+    }
+
+    // non-standard Netscape extension
+    if (propertyName == exec->propertyNames().underscoreProto) {
+        slot.setValueSlot(this, &_proto);
+        return true;
+    }
+
+    return false;
+}
+
+
 static void throwSetterError(ExecState *exec)
 {
   throwError(exec, TypeError, "setting a property that has only a getter");
@@ -219,21 +243,26 @@ void JSObject::put(ExecState *exec, const Identifier &propertyName, JSValue *val
       }
       proto = proto->prototype() ? proto->prototype()->getObject() : 0;
     }
-    
+
     setPrototype(value);
     return;
   }
 
-  /* TODO: check for write permissions directly w/o this call */
-  /* Doesn't look very easy with the PropertyMap API - David */
   // putValue() is used for JS assignemnts. It passes no attribute.
   // Assume that a C++ implementation knows what it is doing
-  // and let it override the canPut() check.
-  if ((attr == None || attr == DontDelete) && !canPut(exec,propertyName)) {
+  // and don't spend time doing a read-only check for it.
+  bool checkRO = (attr == None || attr == DontDelete);
+
+  if (checkRO) {
+    // Check for static properties that are ReadOnly; the property map will check the dynamic properties.
+    // We don't have to worry about setters being read-only as they can't be added with such an attribute.
+    const HashEntry* entry = findPropertyHashEntry(propertyName);
+    if (entry && entry->attr & ReadOnly) {
 #ifdef KJS_VERBOSE
-    fprintf( stderr, "WARNING: canPut %s said NO\n", propertyName.ascii() );
+      fprintf( stderr, "WARNING: static property %s is ReadOnly\n", propertyName.ascii() );
 #endif
-    return;
+      return;
+    }
   }
 
   // Check if there are any setters or getters in the prototype chain
@@ -244,13 +273,13 @@ void JSObject::put(ExecState *exec, const Identifier &propertyName, JSValue *val
       hasGettersOrSetters = true;
       break;
     }
-      
+
     if (!obj->_proto->isObject())
       break;
-      
+
     obj = static_cast<JSObject *>(obj->_proto);
   }
-  
+
   if (hasGettersOrSetters) {
     obj = this;
     while (true) {
@@ -258,32 +287,32 @@ void JSObject::put(ExecState *exec, const Identifier &propertyName, JSValue *val
       if (JSValue *gs = obj->_prop.get(propertyName, attributes)) {
         if (attributes & GetterSetter) {
           JSObject *setterFunc = static_cast<GetterSetterImp *>(gs)->getSetter();
-        
+
           if (!setterFunc) {
             throwSetterError(exec);
             return;
           }
-            
+
           List args;
           args.append(value);
-        
+
           setterFunc->call(exec, this, args);
           return;
         } else {
-          // If there's an existing property on the object or one of its 
+          // If there's an existing property on the object or one of its
           // prototype it should be replaced, so we just break here.
           break;
         }
       }
-     
+
       if (!obj->_proto->isObject())
         break;
-        
+
       obj = static_cast<JSObject *>(obj->_proto);
     }
   }
-  
-  _prop.put(propertyName,value,attr);
+
+  _prop.put(propertyName,value,attr,checkRO);
 }
 
 void JSObject::put(ExecState *exec, unsigned propertyName,
@@ -296,7 +325,7 @@ void JSObject::put(ExecState *exec, unsigned propertyName,
 bool JSObject::canPut(ExecState *, const Identifier &propertyName) const
 {
   unsigned attributes;
-    
+
   // Don't look in the prototype here. We can always put an override
   // in the object, even if the prototype has a ReadOnly property.
 
@@ -328,7 +357,7 @@ bool JSObject::deleteProperty(ExecState * /*exec*/, const Identifier &propertyNa
     if ((attributes & DontDelete))
       return false;
     _prop.remove(propertyName);
-    if (attributes & GetterSetter) 
+    if (attributes & GetterSetter)
         _prop.setHasGetterSetterProperties(_prop.containsGettersOrSetters());
     return true;
   }
@@ -409,14 +438,14 @@ void JSObject::defineGetter(ExecState*, const Identifier& propertyName, JSObject
 {
     JSValue *o = getDirect(propertyName);
     GetterSetterImp *gs;
-    
+
     if (o && o->type() == GetterSetterType) {
         gs = static_cast<GetterSetterImp *>(o);
     } else {
         gs = new GetterSetterImp;
         putDirect(propertyName, gs, GetterSetter);
     }
-    
+
     _prop.setHasGetterSetterProperties(true);
     gs->setGetter(getterFunc);
 }
@@ -425,14 +454,14 @@ void JSObject::defineSetter(ExecState*, const Identifier& propertyName, JSObject
 {
     JSValue *o = getDirect(propertyName);
     GetterSetterImp *gs;
-    
+
     if (o && o->type() == GetterSetterType) {
         gs = static_cast<GetterSetterImp *>(o);
     } else {
         gs = new GetterSetterImp;
         putDirect(propertyName, gs, GetterSetter);
     }
-    
+
     _prop.setHasGetterSetterProperties(true);
     gs->setSetter(setterFunc);
 }
@@ -476,10 +505,10 @@ bool JSObject::hasInstance(ExecState* exec, JSValue* value)
         throwError(exec, TypeError, "instanceof called on an object with an invalid prototype property.");
         return false;
     }
-    
+
     if (!value->isObject())
         return false;
-    
+
     JSObject* o = static_cast<JSObject*>(value);
     while ((o = o->prototype()->getObject())) {
         if (o == proto)
@@ -491,7 +520,7 @@ bool JSObject::hasInstance(ExecState* exec, JSValue* value)
 bool JSObject::propertyIsEnumerable(ExecState*, const Identifier& propertyName) const
 {
   unsigned attributes;
- 
+
   if (!getPropertyAttributes(propertyName, attributes))
     return false;
   else
@@ -502,14 +531,14 @@ bool JSObject::getPropertyAttributes(const Identifier& propertyName, unsigned& a
 {
   if (_prop.get(propertyName, attributes))
     return true;
-    
+
   // Look in the static hashtable of properties
   const HashEntry* e = findPropertyHashEntry(propertyName);
   if (e) {
     attributes = e->attr;
     return true;
   }
-    
+
   return false;
 }
 
@@ -572,7 +601,7 @@ void JSObject::removeDirect(const Identifier &propertyName)
 
 void JSObject::putDirectFunction(InternalFunctionImp* func, int attr)
 {
-    putDirect(func->functionName(), func, attr); 
+    putDirect(func->functionName(), func, attr);
 }
 
 void JSObject::fillGetterPropertySlot(PropertySlot& slot, JSValue **location)
@@ -648,7 +677,7 @@ JSObject *Error::create(ExecState *exec, ErrorType errtype, const UString &messa
 
   if(!sourceURL.isNull())
     err->put(exec, "sourceURL", jsString(sourceURL));
- 
+
   return err;
 
 /*
