@@ -89,6 +89,8 @@ public:
     bool getTruncatedInt32(int32_t&) const;
     bool getTruncatedUInt32(uint32_t&) const;
 
+    JSValue* getByIndex(ExecState* exec, unsigned propertyName) const;
+
     // Basic conversions.
     JSValue* toPrimitive(ExecState* exec, JSType preferredType = UnspecifiedType) const;
     bool getPrimitiveNumber(ExecState* exec, double& number, JSValue*& value);
@@ -185,13 +187,17 @@ public:
 KJS_EXPORT JSValue *jsNumberCell(double);
 
 KJS_EXPORT JSCell* jsString(); // returns empty string
-KJS_EXPORT JSCell* jsString(const UString& s); // returns empty string if passed null string
-KJS_EXPORT JSCell* jsString(const char* s); // returns empty string if passed 0
+KJS_EXPORT JSCell* jsString(const UString&); // returns empty string if passed null string
+KJS_EXPORT JSCell* jsString(const char* = ""); // returns empty string if passed 0
 KJS_EXPORT JSCell* jsString(const char* s, int len);
-KJS_EXPORT JSCell* jsString(ExecState* exec, const JSValue* value);
 
-extern const double NaN;
-extern const double Inf;
+// should be used for strings that are owned by an object that will
+// likely outlive the JSValue this makes, such as the parse tree or a
+// DOM object that contains a UString
+JSCell *jsOwnedString(const UString&);
+
+KJS_EXPORT extern const double NaN;
+KJS_EXPORT extern const double Inf;
 
 inline JSValue* jsUndefined()
 {
