@@ -1,6 +1,7 @@
 /*
     This file is part of KNewStuff2.
     Copyright (c) 2006, 2007 Josef Spillner <spillner@kde.org>
+    Copyright (c) 2008 Jeremy Whiting <jeremy@scitools.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -66,11 +67,12 @@ QString KTranslatable::representation() const
 {
     if (m_strings.isEmpty()) return QString();
 
-    QStringList langs = KGlobal::locale()->languageList();
-    for (QStringList::Iterator it = langs.begin(); it != langs.end(); ++it)
+    const QStringList langs = KGlobal::locale()->languageList();
+    for (QStringList::ConstIterator it = langs.begin(); it != langs.end(); ++it)
         if (m_strings.contains(*it)) return m_strings[*it];
 
     if (m_strings.contains(QString())) return m_strings[QString()];
+    // NOTE: this could be the source of crashes I've seen occasionally
     else return *(m_strings.begin());
 }
 
@@ -78,8 +80,8 @@ QString KTranslatable::language() const
 {
     if (m_strings.isEmpty()) return QString();
 
-    QStringList langs = KGlobal::locale()->languageList();
-    for (QStringList::Iterator it = langs.begin(); it != langs.end(); ++it)
+    const QStringList langs = KGlobal::locale()->languageList();
+    for (QStringList::ConstIterator it = langs.begin(); it != langs.end(); ++it)
         if (m_strings.contains(*it)) return (*it);
 
     if (m_strings.contains(QString())) return QString();
@@ -95,28 +97,12 @@ QString KTranslatable::translated(const QString& lang) const
 
 QStringList KTranslatable::languages() const
 {
-    QStringList strlist;
-
-    QMapIterator<QString, QString> it(m_strings);
-    while (it.hasNext()) {
-        it.next();
-        strlist << it.key();
-    }
-
-    return strlist;
+    return m_strings.keys();
 }
 
 QStringList KTranslatable::strings() const
 {
-    QStringList strlist;
-
-    QMapIterator<QString, QString> it(m_strings);
-    while (it.hasNext()) {
-        it.next();
-        strlist << it.value();
-    }
-
-    return strlist;
+    return m_strings.values();
 }
 
 QMap<QString, QString> KTranslatable::stringmap() const
@@ -126,13 +112,11 @@ QMap<QString, QString> KTranslatable::stringmap() const
 
 bool KTranslatable::isTranslated() const
 {
-    if (m_strings.count() > 1) return true;
-    return false;
+    return m_strings.count() > 1;
 }
 
 bool KTranslatable::isEmpty() const
 {
-    if (m_strings.count() > 0) return false;
-    return true;
+    return m_strings.isEmpty();
 }
 
