@@ -23,7 +23,7 @@
 
 #include <kdialog.h>
 
-class QTextBrowser;
+class KTextBrowser;
 
 /**
  * KNewStuff comments window.
@@ -37,14 +37,14 @@ class QTextBrowser;
  */
 class KDXSComments : public KDialog
 {
-Q_OBJECT
+    Q_OBJECT
 public:
-	KDXSComments(QWidget *parent);
-	void addComment(const QString& username, const QString& comment);
+    KDXSComments(QWidget *parent);
+    void addComment(const QString& username, const QString& comment);
 private Q_SLOTS:
-	void slotUrl(const QUrl& url);
+    void slotUrl(const QUrl& url);
 private:
-	QTextBrowser *m_log;
+    KTextBrowser *m_log;
 };
 
 #endif
