@@ -23,8 +23,8 @@
 
 #include <kdialog.h>
 
-class QLineEdit;
-class QTextEdit;
+class KLineEdit;
+class KTextEdit;
 class KUrlRequester;
 
 /**
@@ -39,13 +39,13 @@ class KUrlRequester;
  */
 class KDXSTranslation : public KDialog
 {
-Q_OBJECT
+    Q_OBJECT
 public:
-	KDXSTranslation(QWidget *parent);
+    KDXSTranslation(QWidget *parent);
 private:
-	QLineEdit *m_name;
-	QTextEdit *m_description;
-	KUrlRequester *m_payload;
+    KLineEdit *m_name;
+    KTextEdit *m_description;
+    KUrlRequester *m_payload;
 };
 
 #endif
