@@ -25,12 +25,12 @@
 #include <kconfiggroup.h>
 #include <kglobal.h>
 #include <kstandarddirs.h>
-#include <QtGui/QItemDelegate>
+#include <QtGui/QStyledItemDelegate>
 #include <QtGui/QPainter>
 
 //BEGIN KNotifyEventListDelegate
 
-class KNotifyEventList::KNotifyEventListDelegate : public QItemDelegate
+class KNotifyEventList::KNotifyEventListDelegate : public QStyledItemDelegate
 {
 	public:
 		KNotifyEventListDelegate(QObject *parent = 0);
@@ -40,7 +40,7 @@ class KNotifyEventList::KNotifyEventListDelegate : public QItemDelegate
 };
 
 KNotifyEventList::KNotifyEventListDelegate::KNotifyEventListDelegate(QObject *parent)
-	: QItemDelegate(parent)
+	: QStyledItemDelegate(parent)
 {
 }
 
@@ -48,12 +48,12 @@ void KNotifyEventList::KNotifyEventListDelegate::paint( QPainter* painter,
 		 const QStyleOptionViewItem& option, const QModelIndex& index ) const
 {
 	if (index.column() != 2)
-		return QItemDelegate::paint(painter, option, index);
+		return QStyledItemDelegate::paint(painter, option, index);
 
 	QVariant displayData = index.data(Qt::UserRole);
 	QString prstring=displayData.toString();
 
-	QItemDelegate::paint(painter, option, index);
+	QStyledItemDelegate::paint(painter, option, index);
 
 // 	kDebug(300) << prstring;
 
@@ -71,7 +71,7 @@ void KNotifyEventList::KNotifyEventListDelegate::paint( QPainter* painter,
 
 	int mc_x=0;
 	QSize iconsSize = option.decorationSize;
-	foreach(KIcon icon, iconList)
+	foreach(const KIcon &icon, iconList)
 	{
 		icon.paint(painter, rect.left() + mc_x + 4, rect.top() + (rect.height() - iconsSize.height()) / 2, iconsSize.width(), iconsSize.height());
 		mc_x += iconsSize.width() + 4;
@@ -109,7 +109,6 @@ void KNotifyEventList::fill( const QString & appname , const QString & context_n
 	config = new KConfig(appname + ".notifyrc" , KConfig::NoGlobals);
         config->addConfigSources(KGlobal::dirs()->findAllResources("data",
                                  appname + '/' + appname + ".notifyrc"));
-        config->reparseConfiguration();
 
 	QStringList conflist = config->groupList();
 	QRegExp rx("^Event/([^/]*)$");
