@@ -25,6 +25,8 @@
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
 
+#include <klocale.h>
+#include <kicon.h>
 #include <kmimetype.h>
 
 using namespace Kross;
@@ -98,6 +100,7 @@ namespace Kross {
 
 Action::Action(QObject* parent, const QString& name, const QDir& packagepath)
     : QAction(parent)
+    , QScriptable()
     , ChildrenInterface()
     , ErrorInterface()
     , d( new Private() )
@@ -159,7 +162,9 @@ void Action::fromDomElement(const QDomElement& element)
 
     setText( element.attribute("text") );
     setDescription( element.attribute("comment") );
+    setEnabled( true );
     setInterpreter( element.attribute("interpreter") );
+    setEnabled( QVariant(element.attribute("enabled","true")).toBool() && isEnabled() );
 
     QString icon = element.attribute("icon");
     if( icon.isEmpty() && ! d->scriptfile.isNull() )
@@ -184,9 +189,6 @@ void Action::fromDomElement(const QDomElement& element)
             }
         }
     }
-
-    bool enabled = QVariant( element.attribute("enabled","true") ).toBool();
-    setEnabled(enabled);
 }
 
 QDomElement Action::toDomElement() const
@@ -219,6 +221,11 @@ QDomElement Action::toDomElement() const
     return e;
 }
 
+Kross::Script* Action::script() const
+{
+    return d->script;
+}
+
 QString Action::name() const
 {
     return objectName();
@@ -323,11 +330,26 @@ QString Action::currentPath() const
     return d->currentpath;
 }
 
-QMap<QString, QVariant>& Action::options() const
+QVariantMap Action::options() const
 {
     return d->options;
 }
 
+void Action::addQObject(QObject* obj, const QString &name)
+{
+    this->addObject(obj, name);
+}
+
+QObject* Action::qobject(const QString &name) const
+{
+    return this->object(name);
+}
+
+QStringList Action::qobjectNames() const
+{
+    return this->objects().keys();
+}
+
 QVariant Action::option(const QString& name, const QVariant& defaultvalue)
 {
     if(d->options.contains(name))
@@ -373,11 +395,11 @@ bool Action::initialize()
     if( ! d->scriptfile.isNull() ) {
         QFile f( d->scriptfile );
         if( ! f.exists() ) {
-            setError(i18n("There exists no such scriptfile \"%1\"", d->scriptfile));
+            setError(i18n("Scriptfile \"%1\" does not exist.", d->scriptfile));
             return false;
         }
         if( d->interpretername.isNull() ) {
-            setError(i18n("Failed to determinate interpreter for scriptfile \"%1\"", d->scriptfile));
+            setError(i18n("Failed to determine interpreter for scriptfile \"%1\"", d->scriptfile));
             return false;
         }
         if( ! f.open(QIODevice::ReadOnly) ) {
@@ -392,7 +414,7 @@ bool Action::initialize()
     if( ! interpreter ) {
         InterpreterInfo* info = Manager::self().interpreterInfo(d->interpretername);
         if( info )
-            setError(i18n("Failed to load interpreter \"%1\": %2", d->interpretername, info->errorMessage()));
+            setError(i18n("Failed to load interpreter \"%1\"", d->interpretername));
         else
             setError(i18n("No such interpreter \"%1\"", d->interpretername));
         return false;
@@ -452,7 +474,9 @@ void Action::slotTriggered()
                 krossdebug( QString("Action::slotTriggered() after exec, errorMessage=%2").arg(errorMessage()) );
             #endif
             setError(d->script);
+            //emit finished(this);
             finalize();
+            //return;
         }
     }
 
