@@ -208,7 +208,7 @@ QVariant ActionCollectionModel::data(const QModelIndex& index, int role) const
                                 return item->action->icon();
                     } break;
                     case Qt::DisplayRole:
-                        return item->action->text().replace("&","");
+                        return item->action->text().remove("&");
                     case Qt::ToolTipRole: // fall through
                     case Qt::WhatsThisRole: {
                         if( d->mode & ToolTips ) {
@@ -340,7 +340,7 @@ QMimeData* ActionCollectionModel::mimeData(const QModelIndexList& indexes) const
     QByteArray encodedData;
 
     QDataStream stream(&encodedData, QIODevice::WriteOnly);
-    foreach(QModelIndex index, indexes) {
+    foreach(const QModelIndex &index, indexes) {
         //if( ! index.isValid() ) continue;
         //QString text = data(index, Qt::DisplayRole).toString();
         QString path = fullPath(index);
