@@ -26,7 +26,6 @@
 #include <klocale.h>
 #include <kservicegroup.h>
 #include <kdebug.h>
-#include <kicon.h>
 #include <kservicetypetrader.h>
 #include <kconfig.h>
 #include <kstandarddirs.h>
@@ -184,7 +183,7 @@ bool DialogPrivate::isPluginForKCMEnabled(const KCModuleInfo *moduleinfo, KPlugi
 	bool enabled = true;
     //kDebug(700) << "check whether the '" << moduleinfo->moduleName() << "' KCM should be shown";
 	// for all parent components
-	QStringList parentComponents = moduleinfo->service()->property(
+	const QStringList parentComponents = moduleinfo->service()->property(
 			"X-KDE-ParentComponents" ).toStringList();
 	for( QStringList::ConstIterator pcit = parentComponents.begin();
 			pcit != parentComponents.end(); ++pcit )
@@ -222,12 +221,19 @@ void DialogPrivate::parseGroupFile( const QString & filename )
         }
         KConfigGroup conf(&file, group);
 
-        KVBox *page = new KVBox();
+        QWidget * page = new QWidget( q );
+
         const QString iconName = conf.readEntry("Icon");
-        QCheckBox *checkBox = new QCheckBox(i18n("component enabled:"), page);
+        QCheckBox *checkBox = new QCheckBox(i18n("Enable component"), page);
         QLabel *iconLabel = new QLabel(page);
         QLabel *comment = new QLabel(conf.readEntry("Comment"), page);
         comment->setTextFormat(Qt::RichText);
+        QVBoxLayout * layout = new QVBoxLayout(page);
+        layout->addWidget(checkBox);
+        layout->addWidget(iconLabel);
+        layout->addWidget(comment);
+        layout->addStretch();
+        page->setLayout(layout);
 
         KPageWidgetItem *item = new KPageWidgetItem(page, conf.readEntry("Name"));
         item->setIcon(KIcon(iconName));
@@ -283,7 +289,7 @@ void DialogPrivate::createDialogFromServices()
 	// read .setdlg files
 	QString setdlgpath = KStandardDirs::locate( "appdata",
                                                     KGlobal::mainComponent().componentName() + ".setdlg" );
-	QStringList setdlgaddon = KGlobal::dirs()->findAllResources( "appdata",
+	const QStringList setdlgaddon = KGlobal::dirs()->findAllResources( "appdata",
 			"ksettingsdialog/*.setdlg" );
     if (!setdlgpath.isNull()) {
         parseGroupFile(setdlgpath);
