@@ -20,18 +20,23 @@
 
 #include "resourcedata.h"
 #include "resourcemanager.h"
-#include "generated/resource.h"
+#include "resource.h"
 #include "tools.h"
+#include "resourcefiltermodel.h"
+#include "nie.h"
 
 #include <Soprano/Statement>
 #include <Soprano/StatementIterator>
 #include <Soprano/Model>
 #include <Soprano/Vocabulary/RDFS>
 #include <Soprano/Vocabulary/RDF>
+#include <Soprano/Vocabulary/Xesam>
+#include <Soprano/Vocabulary/NAO>
 
 #include "ontology/class.h"
 
 #include <QtCore/QFile>
+#include <QtCore/QDateTime>
 
 #include <kdebug.h>
 #include <kurl.h>
@@ -39,8 +44,15 @@
 using namespace Soprano;
 
 
+static inline uint qHash( const QUrl& url )
+{
+    return qHash( url.toString() );
+}
+
+
 typedef QHash<QString, Nepomuk::ResourceData*> ResourceDataHash;
 
+// FIXME: make these things thread-safe. Even better: put them into the ResourceManager and make that thread-safe!
 Q_GLOBAL_STATIC( ResourceDataHash, initializedData )
 Q_GLOBAL_STATIC( ResourceDataHash, kickoffData )
 
@@ -65,13 +77,15 @@ static Nepomuk::Variant nodeToVariant( const Soprano::Node& node )
 Nepomuk::ResourceData::ResourceData( const QUrl& uri, const QString& uriOrId, const QUrl& type )
     : m_kickoffUriOrId( uriOrId ),
       m_uri( uri ),
-      m_type( type ),
+      m_mainType( type ),
       m_ref(0),
       m_proxyData(0),
       m_cacheDirty(true)
 {
-    if( m_type.isEmpty() && !( uriOrId.isEmpty() && uri.isEmpty() ) )
-        m_type = Soprano::Vocabulary::RDFS::Resource();
+    if( m_mainType.isEmpty() )
+        m_mainType = Soprano::Vocabulary::RDFS::Resource();
+
+    m_types << m_mainType;
 
     // TODO: handle the caching in a decent Cache class and not this ugly.
     if ( s_dataCnt >= 1000 ) {
@@ -92,8 +106,6 @@ Nepomuk::ResourceData::ResourceData( const QUrl& uri, const QString& uriOrId, co
 Nepomuk::ResourceData::~ResourceData()
 {
     --s_dataCnt;
-    if( m_proxyData )
-        m_proxyData->deref();
 }
 
 
@@ -105,72 +117,67 @@ QString Nepomuk::ResourceData::kickoffUriOrId() const
 }
 
 
-QString Nepomuk::ResourceData::uri() const
+QUrl Nepomuk::ResourceData::uri() const
 {
     if( m_proxyData )
         return m_proxyData->uri();
-    return m_uri.toString();
+    return m_uri;
 }
 
 
-QUrl Nepomuk::ResourceData::type() const
+QUrl Nepomuk::ResourceData::type()
 {
     if( m_proxyData )
         return m_proxyData->type();
-    return m_type;
+    load();
+    return m_mainType;
+}
+
+
+QList<QUrl> Nepomuk::ResourceData::allTypes()
+{
+    if( m_proxyData )
+        return m_proxyData->allTypes();
+    load();
+    return m_types;
 }
 
 
 void Nepomuk::ResourceData::deleteData()
 {
-    if( m_proxyData )
-        if( m_proxyData->deref() )
-            m_proxyData->deleteData();
-
-    m_proxyData = 0;
-
-    if( !m_uri.isEmpty() )
-        initializedData()->remove( m_uri.toString() );
-    if( !m_kickoffUriOrId.isEmpty() )
-        kickoffData()->remove( m_kickoffUriOrId );
+    if( m_proxyData ) {
+        m_proxyData->deref();
+        m_proxyData = 0;
+    }
+    else {
+        if( !m_uri.isEmpty() )
+            initializedData()->remove( m_uri.toString() );
+        if( !m_kickoffUriOrId.isEmpty() )
+            kickoffData()->remove( m_kickoffUriOrId );
+    }
 
     deleteLater();
 }
 
 
-QHash<QString, Nepomuk::Variant> Nepomuk::ResourceData::allProperties()
+QHash<QUrl, Nepomuk::Variant> Nepomuk::ResourceData::allProperties()
 {
     if( m_proxyData )
         return m_proxyData->allProperties();
 
-    Soprano::Model* model = ResourceManager::instance()->mainModel();
-    QHash<QString, Variant> props;
+    load();
 
-    if ( determineUri() ) {
-        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( m_uri, Soprano::Node(), Soprano::Node() ) );
-        while ( it.next() ) {
-            Statement statement = *it;
-            if ( props.contains( statement.predicate().toString() ) ) {
-                props[statement.predicate().toString()].append( nodeToVariant( statement.object() ) );
-            }
-            else {
-                props.insert( statement.predicate().toString(), nodeToVariant( statement.object() ) );
-            }
-        }
-        it.close();
-    }
-
-    return props;
+    return m_cache;
 }
 
 
-bool Nepomuk::ResourceData::hasProperty( const QString& uri )
+bool Nepomuk::ResourceData::hasProperty( const QUrl& uri )
 {
     if( m_proxyData )
         return m_proxyData->hasProperty( uri );
 
     if ( determineUri() ) {
-        return ResourceManager::instance()->mainModel()->containsAnyStatement( Soprano::Statement( m_uri, QUrl( uri ), Soprano::Node() ) );
+        return ResourceManager::instance()->mainModel()->containsAnyStatement( Soprano::Statement( m_uri, uri, Soprano::Node() ) );
     }
     else {
         return false;
@@ -183,22 +190,36 @@ bool Nepomuk::ResourceData::hasType( const QUrl& uri )
     if( m_proxyData )
         return m_proxyData->hasType( uri );
 
-    if ( determineUri() ) {
-        return ResourceManager::instance()->mainModel()->containsAnyStatement( Soprano::Statement( m_uri, Soprano::Vocabulary::RDF::type(), QUrl( uri ) ) );
-    }
-    else {
-        return false;
+    if ( load() ) {
+        return constHasType( uri );
     }
+
+    return false;
 }
 
 
-Nepomuk::Variant Nepomuk::ResourceData::property( const QString& uri )
+bool Nepomuk::ResourceData::constHasType( const QUrl& uri ) const
+{
+    Types::Class requestedType( uri );
+    for ( QList<QUrl>::const_iterator it = m_types.constBegin();
+          it != m_types.constEnd(); ++it ) {
+        Types::Class availType( *it );
+        if ( availType == requestedType ||
+             availType.isSubClassOf( requestedType ) ) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
+Nepomuk::Variant Nepomuk::ResourceData::property( const QUrl& uri )
 {
     if( m_proxyData )
         return m_proxyData->property( uri );
 
     if ( load() ) {
-        QHash<QUrl, Variant>::iterator it = m_cache.find( uri );
+        QHash<QUrl, Variant>::const_iterator it = m_cache.find( uri );
         if ( it == m_cache.end() ) {
             return Variant();
         }
@@ -228,35 +249,43 @@ Nepomuk::Variant Nepomuk::ResourceData::property( const QString& uri )
 
 bool Nepomuk::ResourceData::store()
 {
-    if ( determineUri() ) {
-        if ( !exists() ) {
-            QList<Statement> statements;
+    if ( !determineUri() ) {
+        // create a random URI and add us to the initialized data, i.e. make us "valid"
+        m_modificationMutex.lock();
+        m_uri = ResourceManager::instance()->generateUniqueUri();
+        initializedData()->insert( m_uri.toString(), this );
+        m_modificationMutex.unlock();
+    }
 
-            // save type
-            // FIXME: handle multiple types
-            statements.append( Statement( m_uri, Soprano::Vocabulary::RDF::type(), m_type ) );
+    if ( !exists() ) {
+        QList<Statement> statements;
 
-            // save the kickoff identifier (other identifiers are stored via setProperty)
-            if ( !m_kickoffIdentifier.isEmpty() ) {
-                statements.append( Statement( m_uri, QUrl(Resource::identifierUri()), LiteralValue(m_kickoffIdentifier) ) );
-            }
+        // save type (There should be no need to save all the types since there is only one way
+        // that m_types contains more than one element: if we loaded them)
+        statements.append( Statement( m_uri, Soprano::Vocabulary::RDF::type(), m_types.first() ) );
 
-            // HACK: make sure that files have proper fileUrl properties so long as we do not have a File class for
-            // Dolphin and co.
-            if ( QFile::exists( m_uri.toLocalFile())) {
-                statements.append( Statement( m_uri,
-                                              QUrl("http://freedesktop.org/standards/xesam/1.0/core#url"),
-                                              LiteralValue( m_uri.toLocalFile() ) ) );
-            }
+        // save the creation date
+        statements.append( Statement( m_uri, Soprano::Vocabulary::NAO::created(), Soprano::LiteralValue( QDateTime::currentDateTime() ) ) );
 
-            return ResourceManager::instance()->mainModel()->addStatements( statements ) == Soprano::Error::ErrorNone;
+        // save the kickoff identifier (other identifiers are stored via setProperty)
+        if ( !m_kickoffIdentifier.isEmpty() ) {
+            statements.append( Statement( m_uri, QUrl(Resource::identifierUri()), LiteralValue(m_kickoffIdentifier) ) );
         }
-        else {
-            return true;
+
+        // HACK: make sure that files have proper fileUrl properties so long as we do not have a File class for
+        // Dolphin and co.
+        if ( constHasType( Soprano::Vocabulary::Xesam::File() ) &&
+             QFile::exists( m_uri.toLocalFile()) ) {
+            statements.append( Statement( m_uri,
+                                          Soprano::Vocabulary::Xesam::url(),
+                                          LiteralValue( m_uri.toLocalFile() ) ) );
         }
+
+        ResourceFilterModel fm( ResourceManager::instance()->mainModel() );
+        return fm.addStatements( statements ) == Soprano::Error::ErrorNone;
     }
     else {
-        return false;
+        return true;
     }
 }
 
@@ -272,7 +301,49 @@ bool Nepomuk::ResourceData::load()
 
             while ( it.next() ) {
                 Statement statement = *it;
-                if ( statement.predicate().uri() != Soprano::Vocabulary::RDF::type() ) {
+                if ( statement.predicate().uri() == Soprano::Vocabulary::RDF::type() ) {
+                    if ( statement.object().isResource() ) {
+                        QUrl storedType = statement.object().uri();
+                        if ( !m_types.contains( storedType ) ) {
+                            m_types << storedType;
+                        }
+                        if ( m_mainType == Soprano::Vocabulary::RDFS::Resource() ) {
+                            Q_ASSERT( !storedType.isEmpty() );
+                            m_mainType = storedType;
+                        }
+                        else {
+                            Types::Class currentTypeClass = m_mainType;
+                            Types::Class storedTypeClass = storedType;
+
+                            // Keep the type that is further down the hierarchy
+                            if ( storedTypeClass.isSubClassOf( currentTypeClass ) ) {
+                                m_mainType = storedTypeClass.uri();
+                            }
+                            else {
+                                // This is a little convenience hack since the user is most likely
+                                // more interested in the file content than the actual file
+                                Types::Class xesamContentClass( Soprano::Vocabulary::Xesam::Content() );
+                                if ( m_mainType == Soprano::Vocabulary::Xesam::File() &&
+                                     ( storedTypeClass == xesamContentClass ||
+                                       storedTypeClass.isSubClassOf( xesamContentClass ) ) ) {
+                                    m_mainType = storedTypeClass.uri();
+                                }
+                                else {
+                                    // the same is true for nie:DataObject vs. nie:InformationElement
+                                    Types::Class nieInformationElementClass( Vocabulary::NIE::InformationElement() );
+                                    Types::Class nieDataObjectClass( Vocabulary::NIE::DataObject() );
+                                    if( ( currentTypeClass == nieDataObjectClass ||
+                                          currentTypeClass.isSubClassOf( nieDataObjectClass ) ) &&
+                                        ( storedTypeClass == nieInformationElementClass ||
+                                          storedTypeClass.isSubClassOf( nieInformationElementClass ) ) ) {
+                                        m_mainType = storedTypeClass.uri();
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+                else {
                     m_cache[statement.predicate().uri()].append( nodeToVariant( statement.object() ) );
                 }
             }
@@ -290,22 +361,21 @@ bool Nepomuk::ResourceData::load()
 }
 
 
-void Nepomuk::ResourceData::setProperty( const QString& uri, const Nepomuk::Variant& value )
+void Nepomuk::ResourceData::setProperty( const QUrl& uri, const Nepomuk::Variant& value )
 {
     if( m_proxyData )
         return m_proxyData->setProperty( uri, value );
 
     // step 0: make sure this resource is in the store
     if ( store() ) {
-        Soprano::Model* model = ResourceManager::instance()->mainModel();
-
-        QList<Statement> resourceStatements;
+        QList<Node> valueNodes;
 
         // make sure resource values are in the store
         if ( value.simpleType() == qMetaTypeId<Resource>() ) {
             QList<Resource> l = value.toResourceList();
             for( QList<Resource>::iterator resIt = l.begin(); resIt != l.end(); ++resIt ) {
-                (*resIt).m_data->store();
+                if ( resIt->isValid() )
+                    resIt->m_data->store();
             }
         }
 
@@ -313,59 +383,45 @@ void Nepomuk::ResourceData::setProperty( const QString& uri, const Nepomuk::Vari
 
         // one-to-one Resource
         if( value.isResource() ) {
-            resourceStatements.append( Statement( m_uri, QUrl(uri), QUrl( value.toResource().uri() ) ) );
+            valueNodes.append( value.toResource().resourceUri() );
         }
 
         // one-to-many Resource
         else if( value.isResourceList() ) {
             const QList<Resource>& l = value.toResourceList();
             for( QList<Resource>::const_iterator resIt = l.constBegin(); resIt != l.constEnd(); ++resIt ) {
-                resourceStatements.append( Statement( m_uri, QUrl(uri), QUrl( (*resIt).uri() ) ) );
+                valueNodes.append( (*resIt).resourceUri() );
             }
         }
 
         // one-to-many literals
         else if( value.isList() ) {
-            QList<Node> nl = Nepomuk::valuesToRDFNodes( value );
-            for( QList<Node>::const_iterator nIt = nl.constBegin(); nIt != nl.constEnd(); ++nIt ) {
-                resourceStatements.append( Statement( m_uri, QUrl(uri), *nIt ) );
-            }
+            valueNodes += Nepomuk::valuesToRDFNodes( value );
         }
 
         // one-to-one literal
         else {
-            resourceStatements.append( Statement( m_uri, QUrl(uri),
-                                                  Nepomuk::valueToRDFNode( value ) ) );
+            valueNodes.append( Nepomuk::valueToRDFNode( value ) );
         }
 
         // update the cache for now
         m_cache[uri] = value;
 
         // update the store
-        QList<Statement> existingStatements = model->listStatements( Soprano::Statement( m_uri, QUrl(uri), Node() ) ).allStatements();
-        for ( QList<Statement>::const_iterator it = existingStatements.constBegin();
-              it != existingStatements.constEnd(); ++it ) {
-            if ( !resourceStatements.contains( *it ) ) {
-                model->removeStatement( *it );
-            }
-        }
-        for ( QList<Statement>::const_iterator it = resourceStatements.constBegin();
-              it != resourceStatements.constEnd(); ++it ) {
-            if ( !existingStatements.contains( *it ) ) {
-                model->addStatement( *it );
-            }
-        }
+        ResourceFilterModel fm( ResourceManager::instance()->mainModel() );
+        fm.updateProperty( m_uri, uri, valueNodes );
     }
 }
 
 
-void Nepomuk::ResourceData::removeProperty( const QString& uri )
+void Nepomuk::ResourceData::removeProperty( const QUrl& uri )
 {
     if( m_proxyData )
         return m_proxyData->removeProperty( uri );
 
     if ( determineUri() ) {
-        ResourceManager::instance()->mainModel()->removeAllStatements( Statement( m_uri, QUrl(uri), Node() ) );
+        ResourceFilterModel fm( ResourceManager::instance()->mainModel() );
+        fm.removeProperty( m_uri, uri );
     }
 }
 
@@ -376,11 +432,15 @@ void Nepomuk::ResourceData::remove( bool recursive )
         return m_proxyData->remove();
 
     if ( determineUri() ) {
-        Soprano::Model* model = ResourceManager::instance()->mainModel();
-        model->removeAllStatements( Statement( m_uri, Node(), Node() ) );
+        ResourceFilterModel fm( ResourceManager::instance()->mainModel() );
+        fm.removeAllStatements( Statement( m_uri, Node(), Node() ) );
         if ( recursive ) {
-            model->removeAllStatements( Statement( Node(), Node(), m_uri ) );
+            fm.removeAllStatements( Statement( Node(), Node(), m_uri ) );
         }
+
+        // the url is invalid now
+        initializedData()->remove( m_uri.toString() );
+        m_uri = QUrl();
     }
 }
 
@@ -404,7 +464,7 @@ bool Nepomuk::ResourceData::isValid() const
         return m_proxyData->isValid();
 
     // FIXME: check namespaces and stuff
-    return( !m_type.isEmpty() );
+    return( !m_mainType.isEmpty() && ( !m_uri.isEmpty() || !m_kickoffIdentifier.isEmpty() ) );
 }
 
 
@@ -413,7 +473,7 @@ bool Nepomuk::ResourceData::determineUri()
     if( m_proxyData )
         return m_proxyData->determineUri();
 
-    if( m_uri.isEmpty() ) {
+    else if( m_uri.isEmpty() && !m_kickoffUriOrId.isEmpty() ) {
         Q_ASSERT( !m_kickoffUriOrId.isEmpty() );
 
         m_modificationMutex.lock();
@@ -425,8 +485,7 @@ bool Nepomuk::ResourceData::determineUri()
             // The kickoffUriOrId is actually a URI
             //
             m_uri = kickoffUriOrId();
-            kDebug(300004) << " kickoff identifier " << kickoffUriOrId() << " exists as a URI " << uri();
-            updateType();
+//            kDebug(300004) << " kickoff identifier " << kickoffUriOrId() << " exists as a URI " << uri();
         }
         else {
             //
@@ -452,41 +511,35 @@ bool Nepomuk::ResourceData::determineUri()
             //
             // TODO: basically it is perfectly valid to store both types in the first case
             //
-            Q_ASSERT( !m_type.isEmpty() );
+            Q_ASSERT( !m_mainType.isEmpty() );
             if ( it.next() ) {
-                const Nepomuk::Class* wantedType = Nepomuk::Class::load( m_type );
-                if ( wantedType && m_type != Soprano::Vocabulary::RDFS::Resource() ) {
+                // At this point we only have one type since without a uri we could not have loaded other types
+                // than the one used on construction
+                Types::Class wantedType( m_mainType );
+
+                if ( m_mainType != Soprano::Vocabulary::RDFS::Resource() && wantedType.isAvailable() ) {
                     do {
                         // get the type of the stored resource
                         StatementIterator resourceSl = model->listStatements( Statement( it.current().subject(),
                                                                                          Soprano::Vocabulary::RDF::type(),
                                                                                          Node() ) );
-                        if ( resourceSl.next() && resourceSl.current().object().isResource() ) {
-                            const Nepomuk::Class* storedType = Nepomuk::Class::load( resourceSl.current().object().uri() );
-                            if ( storedType ) {
-                                if ( storedType == wantedType ) {
-                                    // great. :)
-                                    m_uri = it.current().subject().uri();
-                                }
-                                else if ( wantedType->isSubClassOf( storedType ) ) {
-                                    // Keep the type that is further down the hierarchy
-                                    m_type = wantedType->uri();
-                                    m_uri = it.current().subject().uri();
-                                    break;
-                                }
-                                else if ( storedType->isSubClassOf( wantedType ) ) {
-                                    // just use the existing data with the finer grained type
+                        while ( resourceSl.next() ) {
+                            if ( resourceSl.current().object().isResource() ) {
+                                Types::Class storedType = resourceSl.current().object().uri();
+                                if ( storedType == wantedType ||
+                                     wantedType.isSubClassOf( storedType ) ||
+                                     storedType.isSubClassOf( wantedType ) ) {
+                                    // in load() The type will be updated properly
                                     m_uri = it.current().subject().uri();
                                     break;
                                 }
                             }
                         }
-                    } while ( it.next() );
+                    } while ( it.next() && m_uri.isEmpty() );
                 }
                 else {
                     m_uri = it.current().subject().uri();
-                    kDebug(300004) << k_funcinfo << " kickoff identifier " << kickoffUriOrId() << " already exists with URI " << uri();
-                    updateType();
+//                    kDebug(300004) << k_funcinfo << " kickoff identifier " << kickoffUriOrId() << " already exists with URI " << uri();
                 }
             }
 
@@ -511,7 +564,7 @@ bool Nepomuk::ResourceData::determineUri()
                     m_uri = ResourceManager::instance()->generateUniqueUri();
                 }
 
-                kDebug(300004) << " kickoff identifier " << kickoffUriOrId() << " seems fresh. Generated new URI " << m_uri;
+//                kDebug(300004) << " kickoff identifier " << kickoffUriOrId() << " seems fresh. Generated new URI " << m_uri;
             }
         }
 
@@ -519,50 +572,49 @@ bool Nepomuk::ResourceData::determineUri()
         // Move us to the final data hash now that the URI is known
         //
         if( !uri().isEmpty() && uri() != kickoffUriOrId() ) {
-            if( !initializedData()->contains( uri() ) )
-                initializedData()->insert( uri(), this );
+            QString s = uri().toString();
+            if( !initializedData()->contains( s ) ) {
+                initializedData()->insert( s, this );
+            }
             else {
-                m_proxyData = initializedData()->value( uri() );
+                m_proxyData = initializedData()->value( s );
                 m_proxyData->ref();
             }
         }
 
         m_modificationMutex.unlock();
-
-        return !uri().isEmpty();
     }
-    else
-        return true;
+
+    return !m_uri.isEmpty();
 }
 
 
-void Nepomuk::ResourceData::updateType()
-{
-    Soprano::Model* model = ResourceManager::instance()->mainModel();
+// void Nepomuk::ResourceData::updateType()
+// {
+//     Soprano::Model* model = ResourceManager::instance()->mainModel();
 
-    // get the type of the stored resource
-    StatementIterator typeStatements = model->listStatements( Statement( m_uri,
-                                                                         Soprano::Vocabulary::RDF::type(),
-                                                                         Node() ) );
-    if ( typeStatements.next() && typeStatements.current().object().isResource() ) {
-        // FIXME: handle multple types, maybe select the one type that fits best
-        QUrl storedType = typeStatements.current().object().uri();
-        if ( m_type == Soprano::Vocabulary::RDFS::Resource() ) {
-            m_type = storedType;
-        }
-        else {
-            const Nepomuk::Class* wantedTypeClass = Nepomuk::Class::load( m_type );
-            const Nepomuk::Class* storedTypeClass = Nepomuk::Class::load( storedType );
+//     // get the type of the stored resource
+//     StatementIterator typeStatements = model->listStatements( Statement( m_uri,
+//                                                                          Soprano::Vocabulary::RDF::type(),
+//                                                                          Node() ) );
+//     if ( typeStatements.next() && typeStatements.current().object().isResource() ) {
+//         // FIXME: handle multple types, maybe select the one type that fits best
+//         QUrl storedType = typeStatements.current().object().uri();
+//         if ( m_type == Soprano::Vocabulary::RDFS::Resource() ) {
+//             Q_ASSERT( !storedType.isEmpty() );
+//             m_type = storedType;
+//         }
+//         else {
+//             Types::Class wantedTypeClass = m_type;
+//             Types::Class storedTypeClass = storedType;
 
-            if ( wantedTypeClass && storedTypeClass ) {
-                // Keep the type that is further down the hierarchy
-                if ( wantedTypeClass->isSubClassOf( storedTypeClass ) ) {
-                    m_type = wantedTypeClass->uri();
-                }
-            }
-        }
-    }
-}
+//             // Keep the type that is further down the hierarchy
+//             if ( wantedTypeClass.isSubClassOf( storedTypeClass ) ) {
+//                 m_type = wantedTypeClass.uri();
+//             }
+//         }
+//     }
+// }
 
 
 bool Nepomuk::ResourceData::operator==( const ResourceData& other ) const
@@ -575,8 +627,7 @@ bool Nepomuk::ResourceData::operator==( const ResourceData& other ) const
         return true;
 
     if( that->m_uri != other.m_uri ||
-        that->m_type != other.m_type ) {
-        kDebug(300004) << "different uri or type";
+        that->m_mainType != other.m_mainType ) {
         return false;
     }
 
@@ -587,7 +638,8 @@ bool Nepomuk::ResourceData::operator==( const ResourceData& other ) const
 Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QUrl& uri, const QUrl& type )
 {
     if ( uri.isEmpty() ) {
-        return 0;
+        // return an invalid resource which may be activated by calling setProperty
+        return new ResourceData( uri, QString(), type );
     }
 
     // default to "file" scheme, i.e. we do not allow an empty scheme
@@ -603,7 +655,7 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QUrl& uri, const QUrl&
     // The uriOrId has no local representation yet -> create one
     //
     if( it == initializedData()->end() ) {
-        kDebug(300004) << "No existing ResourceData instance found for uri " << uri;
+//        kDebug(300004) << "No existing ResourceData instance found for uri " << uri;
         //
         // The actual URI is already known here
         //
@@ -624,7 +676,7 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QUrl& uri, const QUrl&
 Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QString& uriOrId, const QUrl& type )
 {
     if ( uriOrId.isEmpty() ) {
-        return 0;
+        return new ResourceData( QUrl(), QString(), type );
     }
 
     // special case: files (only absolute paths for now)
@@ -647,22 +699,13 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QString& uriOrId, cons
 
         // check if the type matches (see determineUri for details)
         if ( !type.isEmpty() && type != Soprano::Vocabulary::RDFS::Resource() ) {
-            const Nepomuk::Class* wantedType = Nepomuk::Class::load( type );
-            if ( wantedType ) {
-                while ( it != kickoffData()->end() &&
-                        it.key() == uriOrId ) {
-                    const Nepomuk::Class* storedType = Nepomuk::Class::load( it.value()->type() );
-                    if ( storedType ) {
-                        if ( storedType->isSubClassOf( wantedType ) ) {
-                            break;
-                        }
-                        else if ( wantedType->isSubClassOf( storedType ) ) {
-                            it.value()->m_type = type;
-                            break;
-                        }
-                    }
-                    ++it;
+            Types::Class wantedType = type;
+            while ( it != kickoffData()->end() &&
+                    it.key() == uriOrId ) {
+                if ( it.value()->hasType( type ) ) {
+                    break;
                 }
+                ++it;
             }
         }
 
@@ -673,7 +716,7 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QString& uriOrId, cons
     // The uriOrId has no local representation yet -> create one
     //
     if( !resFound ) {
-        kDebug(300004) << "No existing ResourceData instance found for uriOrId " << uriOrId;
+//        kDebug(300004) << "No existing ResourceData instance found for uriOrId " << uriOrId;
         //
         // Every new ResourceData object ends up in the kickoffdata since its actual URI is not known yet
         //
@@ -691,7 +734,7 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QString& uriOrId, cons
 }
 
 
-QList<Nepomuk::ResourceData*> Nepomuk::ResourceData::allResourceDataOfType( const QString& type )
+QList<Nepomuk::ResourceData*> Nepomuk::ResourceData::allResourceDataOfType( const QUrl& type )
 {
     QList<ResourceData*> l;
 
@@ -708,15 +751,15 @@ QList<Nepomuk::ResourceData*> Nepomuk::ResourceData::allResourceDataOfType( cons
 }
 
 
-QList<Nepomuk::ResourceData*> Nepomuk::ResourceData::allResourceDataWithProperty( const QString& _uri, const Variant& v )
+QList<Nepomuk::ResourceData*> Nepomuk::ResourceData::allResourceDataWithProperty( const QUrl& uri, const Variant& v )
 {
     QList<ResourceData*> l;
 
     for( ResourceDataHash::iterator rdIt = kickoffData()->begin();
          rdIt != kickoffData()->end(); ++rdIt ) {
 
-        if( rdIt.value()->hasProperty( _uri ) &&
-            rdIt.value()->property( _uri ) == v ) {
+        if( rdIt.value()->hasProperty( uri ) &&
+            rdIt.value()->property( uri ) == v ) {
             l.append( rdIt.value() );
         }
     }
