@@ -1,6 +1,6 @@
 /* 
  * This file is part of the Nepomuk KDE project.
- * Copyright (C) 2006-2007 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2006-2008 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,6 +24,7 @@
 #include "nepomuk_export.h"
 
 #include <QtCore/QObject>
+#include <QtCore/QUrl>
 
 
 namespace Soprano {
@@ -36,12 +37,15 @@ namespace Nepomuk {
     }
 
     class Resource;
-    class Ontology;
     class Variant;
     class ResourceManagerHelper;
 
     /**
+     * \class ResourceManager resourcemanager.h Nepomuk/ResourceManager
+     *
      * \brief The ResourceManager is the central \a %KMetaData configuration point.
+     *
+     * \author Sebastian Trueg <trueg@kde.org>
      */
     class NEPOMUK_EXPORT ResourceManager : public QObject
     {
@@ -51,11 +55,11 @@ namespace Nepomuk {
         static ResourceManager* instance();
 
         /**
-         * Initialize the KMetaData framework. This method will initialize the communication with
-         * the local Nepomuk-KDE services, ie. the RDF repository.
+         * Initialize the Nepomuk framework. This method will initialize the communication with
+         * the local Nepomuk-KDE services, ie. the data repository.
          *
-         * Calling init() manually is optional now. In the future we might introduce options
-         * that make calling it useful.
+         * When using multiple threads make sure to call this method in the main thread
+         * before doing anything else.
          *
          * \return 0 if all necessary components could be found and -1 otherwise.
          *
@@ -69,17 +73,24 @@ namespace Nepomuk {
          */
         bool initialized() const;
 
-        /**
-         * The Ontology instance representing the underlying Nepomuk desktop ontology.
-         */
-        Ontology* ontology() const;
-
         /**
          * Retrieve the main data storage model.
          */
         Soprano::Model* mainModel();
+        
+        /**
+         * Override the main model used for all storage. By default the main model
+         * used is the Nepomuk server main model.
+         *
+         * \param The model to use instead of the Nepomuk server or 0 to reset.
+         *
+         * \since 4.1
+         */
+        void setOverrideMainModel( Soprano::Model* model );
 
         /**
+         * \deprecated Use the Resource constructor directly.
+         *
          * Creates a Resource object representing the data referenced by \a uri.
          * The result is the same as from using the Resource::Resource( const QString&, const QString& )
          * constructor with an empty type.
@@ -87,7 +98,7 @@ namespace Nepomuk {
          * \return The Resource representing the data at \a uri or an invalid Resource object if the local
          * NEPOMUK RDF store does not contain an object with URI \a uri.
          */
-        Resource createResourceFromUri( const QString& uri );
+        KDE_DEPRECATED Resource createResourceFromUri( const QString& uri );
 
         /**
          * Remove the resource denoted by \a uri completely.
@@ -103,7 +114,12 @@ namespace Nepomuk {
          * This includes Resources that are not synced yet so it might
          * not represent exactly the state as in the RDF store.
          */
-        QList<Resource> allResourcesOfType( const QString& type );
+        QList<Resource> allResourcesOfType( const QUrl& type );
+
+        /**
+         * \deprecated Use allResourcesOfType( const QString& type )
+         */
+        KDE_DEPRECATED QList<Resource> allResourcesOfType( const QString& type );
 
         /**
          * Retrieve a list of all resources that have property \a uri defined with a value of \a v.
@@ -116,7 +132,12 @@ namespace Nepomuk {
          *            prepended.
          * \param v The value all returned resources should have set as properts \a uri.
          */
-        QList<Resource> allResourcesWithProperty( const QString& uri, const Variant& v );
+        QList<Resource> allResourcesWithProperty( const QUrl& uri, const Variant& v );
+
+        /**
+         * \deprecated Use allResourcesWithProperty( const QString& type )
+         */
+        KDE_DEPRECATED QList<Resource> allResourcesWithProperty( const QString& uri, const Variant& v );
 
         /**
          * Generates a unique URI that is not used in the store yet. This method ca be used to 
